function DeviceEngine() {
    var mAsyncInProgress;

    this.getAllVariables = function (callback) {
        return internalGetVariables([], callback, '');
    }

    this.getVariablesByName = function (names, callback) {
        return internalGetVariables(names, callback, 'name');
    }

    this.getVariablesById = function (ids, callback) {
        return internalGetVariables(ids, callback, 'id');
    }

    this.commitVariables = function (variables, callback) {
        var strURL = '/setvar.csv';
        var requestParams = [];

        for( var row in variables ) {
            var variable = variables[row];

            requestParams.push(variable.getId() + '=' + variable.getValue());
        }

        return performAsyncJob(strURL, requestParams, callback);
    }

    this.getAvailableLogs = function() {
        var logids = [];

        $.ajax({
            type: 'GET',
            url: '/getlogids.csv',
            success: function(data) {
                logids = $.parseJSON(data);
            },
            async: false,
            cache: false,
            timeout: 5000
        });

        return logids;
    }

    this.getLogs = function(id, start, stop, format, callback) {
        var logs = '';
        var result = true;
        var doAsyncRequest = false;
        var params = buildGetLogParams(id, start, stop, format);

        if( typeof(callback) !== 'undefined' ) {
            if( mAsyncInProgress ) {
                return false;
            }

            doAsyncRequest = true;
            mAsyncInProgress = true;
        }

        $.ajax({
            type: 'GET',
            url: '/getlog.csv',
            data: params,
            success: function(data) {
                logs = data;
            },
            complete: function(jqXHR, textStatus) {
                if( textStatus !== 'success' ) {
                    result = false;
                }

                if( doAsyncRequest ) {
                    callback(result, logs);

                    // in case an async call was asked
                    mAsyncInProgress = false;
                }
            },
            async: doAsyncRequest,
            cache: false,
            timeout: 600000
        });

        if( !doAsyncRequest ) {
            return logs;
        }

        return result;
    }

    this.getLogsURI = function(id, start, stop, format) {
        return '/getlog.csv?' + buildGetLogParams(id, start, stop, format);
    }

    this.getActiveAlarms = function(callback) {
        var alarms = null;
        var result = true;
        var doAsyncRequest = false;

        if( typeof(callback) !== 'undefined' ) {
            if( mAsyncInProgress ) {
                return false;
            }

            doAsyncRequest = true;
            mAsyncInProgress = true;
        }

        $.ajax({
            type: 'GET',
            url: '/alarms.cgi',
            data: { 'action': 'getActive' },
            success: function(data) {
                alarms = parseAlarmActiveCSVResult(data);
            },
            complete: function(jqXHR, textStatus) {
                if( textStatus !== 'success' ) {
                    result = false;
                }

                if( doAsyncRequest ) {
                    callback(result, alarms);

                    // in case an async call was asked
                    mAsyncInProgress = false;
                }
            },
            async: doAsyncRequest,
            cache: false,
            timeout: 5000
        });

        if( !doAsyncRequest ) {
            return alarms;
        }

        return result;
    }

    this.getAlarmsHistory = function(callback) {
        var alarmsHistory = null;
        var result = true;
        var doAsyncRequest = false;

        if( typeof(callback) !== 'undefined' ) {
            if( mAsyncInProgress ) {
                return false;
            }

            doAsyncRequest = true;
            mAsyncInProgress = true;
        }

        $.ajax({
            type: 'GET',
            url: '/alarms.cgi',
            data: { 'action': 'getHistory' },
            success: function(data) {
                alarmsHistory = parseAlarmHistoryCSVResult(data);
            },
            complete: function(jqXHR, textStatus) {
                if( textStatus !== 'success' ) {
                    result = false;
                }

                if( doAsyncRequest ) {
                    callback(result, alarmsHistory);

                    // in case an async call was asked
                    mAsyncInProgress = false;
                }
            },
            async: doAsyncRequest,
            cache: false,
            timeout: 5000
        });

        if( !doAsyncRequest ) {
            return alarmsHistory;
        }

        return result;
    }

    this.getAlarmsAll = function(callback) {
        var alarms = null;
        var alarmsHistory = null;
        var result = true;
        var doAsyncRequest = false;

        if( typeof(callback) !== 'undefined' ) {
            if( mAsyncInProgress ) {
                return false;
            }

            doAsyncRequest = true;
            mAsyncInProgress = true;
        } else {
            /* this function accepts only async calls... */
            return false;
        }

        $.ajax({
            type: 'GET',
            url: '/alarms.cgi',
            data: { 'action': 'getAll' },
            success: function(data) {
                var responseParts = data.split('\r\n\r\n');

                if (responseParts.length != 2) {
                    result = false;
                } else {
                    alarms = parseAlarmActiveCSVResult(responseParts[0]);
                    alarmsHistory = parseAlarmHistoryCSVResult(responseParts[1]);
                }
            },
            complete: function(jqXHR, textStatus) {
                if( textStatus !== 'success' ) {
                    result = false;
                }

                if( doAsyncRequest ) {
                    callback(result, alarms, alarmsHistory);

                    // in case an async call was asked
                    mAsyncInProgress = false;
                }
            },
            async: true,
            cache: false,
            timeout: 5000
        });

        return result;
    }

    this.wipeAlarmsHistory = function(callback) {
        var result = true;
        var doAsyncRequest = false;

        if( typeof(callback) !== 'undefined' ) {
            if( mAsyncInProgress ) {
                return false;
            }

            doAsyncRequest = true;
            mAsyncInProgress = true;
        }

        $.ajax({
            type: 'GET',
            url: '/alarms.cgi',
            data: { 'action': 'wipeHistory' },
            complete: function(jqXHR, textStatus) {
                if( textStatus !== 'success' ) {
                    result = false;
                }

                if( doAsyncRequest ) {
                    callback(result);

                    // in case an async call was asked
                    mAsyncInProgress = false;
                }
            },
            async: doAsyncRequest,
            cache: false,
            timeout: 5000
        });

        return result;
    }

    this.resetAlarms = function(callback) {
        return this.resetAlarm(-1, callback);
    }

    this.resetAlarm = function(id, callback) {
        var params = { 'action': 'resetAlarm' };

        if (id >= 0) {
            params['id'] = id;
        }

        return internalResetAlarm(params, callback);
    }

    function internalResetAlarm(params, callback)
    {
        var result = true;
        var doAsyncRequest = false;

        if( typeof(callback) !== 'undefined' ) {
            if( mAsyncInProgress ) {
                return false;
            }

            doAsyncRequest = true;
            mAsyncInProgress = true;
        }

        $.ajax({
            type: 'GET',
            url: '/alarms.cgi',
            data: params,
            complete: function(jqXHR, textStatus) {
                if( textStatus !== 'success' ) {
                    result = false;
                }

                if( doAsyncRequest ) {
                    callback(result);

                    // in case an async call was asked
                    mAsyncInProgress = false;
                }
            },
            async: doAsyncRequest,
            cache: false,
            timeout: 5000
        });

        return result;
    }

    function dateToProtocolString(date) {
        // for c.pCO local time, c.pCO logger date format is YYYY-MM-DDThh:mm:ss
        return date.getFullYear().toString() + '-' +
               (date.getMonth() + 1 < 10 ? '0' : '') + (date.getMonth() + 1).toString() + '-' +
               (date.getDate() < 10 ? '0' : '') + date.getDate().toString() + 'T' +
               (date.getHours() < 10 ? '0' : '') + date.getHours().toString() + ':' +
               (date.getMinutes() < 10 ? '0' : '') + date.getMinutes().toString() + ':' +
               (date.getSeconds() < 10 ? '0' : '') + date.getSeconds().toString();
    }

    function dateToProtocolStringUTC(date) {
        // for UTC local time, c.pCO logger date format is YYYY-MM-DDThh:mm:ssZ;
        // date obj is in browser local time, so it has to be converted to UTC
        return date.getUTCFullYear().toString() + '-' +
                (date.getUTCMonth() + 1 < 10 ? '0' : '') + (date.getUTCMonth() + 1).toString() + '-' +
                (date.getUTCDate() < 10 ? '0' : '') + date.getUTCDate().toString() + 'T' +
                (date.getUTCHours() < 10 ? '0' : '') + date.getUTCHours().toString() + ':' +
                (date.getUTCMinutes() < 10 ? '0' : '') + date.getUTCMinutes().toString() + ':' +
                (date.getUTCSeconds() < 10 ? '0' : '') + date.getUTCSeconds().toString() + 'Z';
    }

    function buildGetLogParams(id, start, stop, format) {
        var params = 'id=' + id;

        // format can be "device" or "utc"
        if( format == 'utc' ) {
            // request timestamps in UTC time format
            if( start !== null ) {
                params += '&start=' + dateToProtocolStringUTC(start);
            }

            if( stop !== null ) {
                params += '&stop=' + dateToProtocolStringUTC(stop);
            }
        } else {
            // request timestamps in c.pCO local time format
            if( start !== null ) {
                params += '&start=' + dateToProtocolString(start);
            }

            if( stop !== null ) {
                params += '&stop=' + dateToProtocolString(stop);
            }
        }

        return params;
    }

    function internalGetVariables(vars, callback, type) {
        var strURL = '/getvar.csv';
        var requestParams = [];

        for( var i = 0; i < vars.length; i++ ) {
            var value = vars[i];

            requestParams.push(type + '=' + value);
        }

        if( vars.length == 0 ) {
            requestParams.push('all=1');
        }

        return performAsyncJob(strURL, requestParams, callback);
    }

    function parseDeviceVariableCSVResult(data) {
        var objects = $.csv.toObjects(data);

        if( typeof(objects[0]) === 'undefined' ) {
            return null;
        }

        if( objects[0].constructor === Object ) {
            var returnVarList = [];

            for( var row in objects ) {
                var newVariable = new DeviceVariable(objects[row]);

                returnVarList.push(newVariable);
            }

            return returnVarList;
        }

        return null;
    }

    function parseAlarmActiveCSVResult(data) {
        var objects = $.csv.toObjects(data);

        if( typeof(objects) !== 'undefined' && objects.length == 0) {
            return [];
        }

        if( typeof(objects[0]) === 'undefined' ) {
            return null;
        }

        if( objects[0].constructor === Object ) {
            var returnAlarmList = [];

            for( var row in objects ) {
                var newAlarm = new AlarmActive(objects[row]);

                returnAlarmList.push(newAlarm);
            }

            return returnAlarmList;
        }

        return null;
    }

    function parseAlarmHistoryCSVResult(data) {
        var objects = $.csv.toObjects(data);

        if( typeof(objects) !== 'undefined' && objects.length == 0) {
            return [];
        }

        if( typeof(objects[0]) === 'undefined' ) {
            return null;
        }

        // TIME,ID,NAME,EVENT,VAR1,VAR2
        if( objects[0].constructor === Object ) {
            var returnAlarmSequenceList = [];

            for( var row in objects ) {
                var newRecord = new AlarmRecord(objects[row]);

                if (newRecord.isStopEvent()) {
                    returnAlarmSequenceList.push(new AlarmSequence(null, newRecord));
                } else if (newRecord.isStartEvent()) {
                    /* find a stop for this varId and check if it misses the start */
                    for (var i = 0; i < returnAlarmSequenceList.length; i++) {
                        var alarmSequence = returnAlarmSequenceList[i];

                        if (alarmSequence.getAlarmId() === newRecord.getAlarmId() &&
                            !alarmSequence.hasStart()) {
                            alarmSequence.setStart(newRecord);
                            break;
                        }
                    }

                    if (i == returnAlarmSequenceList.length) {
                        returnAlarmSequenceList.push(new AlarmSequence(newRecord, null));
                    }
                }
            }

            return returnAlarmSequenceList;
        }

        return null;
    }

    function performAsyncJob(strURL, params, callback) {
        var MAX_CHUNK_LENGTH = 240; //OS Input buffer is 256
        var result = true;
        var loadedData = [];
        var doAsyncRequest = false;

        if( typeof(callback) !== 'undefined' ) {
            if( mAsyncInProgress ) {
                return false;
            }

            doAsyncRequest = true;
            mAsyncInProgress = true;
        }

        var requestChunks = [];
        var singleChunk = '';

        for (var i = 0; i < params.length; i++) {
            var textToAppend = (singleChunk.length > 0 ? '&' : '') + params[i];

            if (singleChunk.length + textToAppend.length > MAX_CHUNK_LENGTH) {
                requestChunks.push(singleChunk);
                singleChunk = params[i];
            } else {
                singleChunk += textToAppend;
            }
        }
        // push the last singleChunk
        requestChunks.push(singleChunk);

        // The following loop is:
        // - blocking in case of a sync procedure so requests will be serialized
        //   and results returned after the loop
        // - in case of an async request this loop will start parallel requests
        //   (one per chunk) and only the last completed one will call the
        //   provided user callback
        var asyncJobsDone = 0;

        for (var i = 0; i < requestChunks.length; i++) {
            $.ajax({
                type: 'POST',
                url: strURL,
                data: requestChunks[i],
                success: function(data) {
                    loadedData = loadedData.concat(parseDeviceVariableCSVResult(data));
                },
                complete: function(jqXHR, textStatus) {
                    if( textStatus !== 'success' ) {
                        result = false;
                    }

                    if( doAsyncRequest && ++asyncJobsDone == requestChunks.length) {
                        // in case an async call was asked (and we are the last one to complete)
                        mAsyncInProgress = false;

                        callback(result, loadedData);
                    }
                },
                async: doAsyncRequest,
                cache: false,
                timeout: 5000
            });
        }

        if( !doAsyncRequest ) {
            return loadedData;
        }

        return result;
    }
}

/* Alarm record definition (start/stop record) */
function AlarmRecord(jsonData) {
    this.mRawData = jsonData || {};
}

AlarmRecord.prototype.getAlarmId = function () {
    return this.mRawData['ID'];
}

AlarmRecord.prototype.getAlarmName = function () {
    return this.mRawData['NAME'];
}

AlarmRecord.prototype.getEvent = function () {
    return this.mRawData['EVENT'];
}

AlarmRecord.prototype.getTimestamp = function () {
    return new Date(this.mRawData['TIME']);
}

AlarmRecord.prototype.getVar1 = function () {
    return this.mRawData['VAR1'];
}

AlarmRecord.prototype.getVar2 = function () {
    return this.mRawData['VAR2'];
}

AlarmRecord.prototype.isStartEvent = function () {
    return this.getEvent() === 'Start';
}

AlarmRecord.prototype.isStopEvent = function () {
    return this.getEvent() === 'Stop';
}

/* Alarm sequence definition (start-stop records sequence) */
function AlarmSequence(recordStart, recordStop) {
    this.mRecordStart = recordStart;
    this.mRecordStop = recordStop;
}

AlarmSequence.prototype.getAlarmId = function () {
    if (this.mRecordStart != null)
        return this.mRecordStart.getAlarmId();

    if (this.mRecordStop != null)
        return this.mRecordStop.getAlarmId();

    return "";
}

AlarmSequence.prototype.getAlarmName = function () {
    if (this.mRecordStart != null)
        return this.mRecordStart.getAlarmName();

    if (this.mRecordStop != null)
        return this.mRecordStop.getAlarmName();

    return "";
}

AlarmSequence.prototype.isComplete = function () {
    return (this.mRecordStart != null && this.mRecordStop != null);
}

AlarmSequence.prototype.hasStart = function () {
    return (this.mRecordStart != null);
}

AlarmSequence.prototype.hasStop = function () {
    return (this.mRecordStop != null);
}

AlarmSequence.prototype.setStart = function (recordStart) {
    this.mRecordStart = recordStart;
}

AlarmSequence.prototype.setStop = function (recordStop) {
    this.mRecordStop = recordStop;
}

AlarmSequence.prototype.getStartTime = function () {
    if (this.hasStart())
        return this.mRecordStart.getTimestamp();

    return null;
}

AlarmSequence.prototype.getStartVar1 = function () {
    if (this.hasStart())
        return this.mRecordStart.getVar1();

    return 'N/A';
}

AlarmSequence.prototype.getStartVar2 = function () {
    if (this.hasStart())
        return this.mRecordStart.getVar2();

    return 'N/A';
}

AlarmSequence.prototype.getStopTime = function () {
    if (this.hasStop())
        return this.mRecordStop.getTimestamp();

    return null;
}

AlarmSequence.prototype.getStopVar1 = function () {
    if (this.hasStop())
        return this.mRecordStop.getVar1();

    return 'N/A';
}

AlarmSequence.prototype.getStopVar2 = function () {
    if (this.hasStop())
        return this.mRecordStop.getVar2();

    return 'N/A';
}

/* Active alarm definition */
function AlarmActive(jsonData) {
    this.mRawData = jsonData || {};
}

AlarmActive.prototype.getAlarmId = function () {
    return this.mRawData['id'];
}

AlarmActive.prototype.getVarName = function () {
    return this.mRawData['name'];
}

AlarmActive.prototype.getTimestamp = function () {
    return new Date(this.mRawData['timestamp']);
}

/* Device variable definition */
function DeviceVariable(jsonData) {
    this.mRawData = jsonData || {};
}

DeviceVariable.prototype.getId = function () {
    return this.mRawData['id'];
}

DeviceVariable.prototype.getName = function () {
    return this.mRawData['name'];
}

DeviceVariable.prototype.getDesc = function () {
    return this.mRawData['desc'];
}

DeviceVariable.prototype.canBeWritten = function () {
    return this.mRawData['access'] === 'RW';
}

DeviceVariable.prototype.getValue = function () {
    return this.mRawData['val'];
}

DeviceVariable.prototype.setValue = function (value) {
    this.mRawData['val'] = value;
}

DeviceVariable.prototype.getStatus = function () {
    return this.mRawData['status'] || 'OK';
}

