// README
//  
// Javascript functions to handle dynamic logger in HTML pages
//

        function initAvailableLogs() {
            //mAvailableLogs = [ { "name": "Log 0", "id": "0", "periodic": "0" }, { "name": "Log 1", "id": "1", "periodic": "1" } ];
            mAvailableLogs = mDeviceEngine.getAvailableLogs();
            var sel = $("#selectLogs");

            sel.empty();
            for ( var i = 0; i < mAvailableLogs.length; i++ ) {
              sel.append('<option value="' + mAvailableLogs[i].id + '">' + mAvailableLogs[i].name + '</option>');
            }
        }

        function isCurrentLogPeriodic() {
            for ( var i = 0; i < mAvailableLogs.length; i++ ) {
              if( mAvailableLogs[i].id === mCurrentlyDisplayedLogId ) {
                return (mAvailableLogs[i].periodic !== '0');
              }
            }

            return false;
        }

        function parseDate(text, type) {
            var dateMs = Date.parse(text);

            if( isNaN(dateMs) ) {
                alert('Please select a valid ' + type + ' date (leave the filed empty for no date limit)');
                return null;
            }

            return new Date(dateMs);
        }

        function getSelectedLogId() {
            var selection = $("select#selectLogs");

            if (typeof(selection) === 'undefined' ) {
                alert('Please select a valid log');
                return -1;
            }

            return selection.val();
        }

        function setSelectedLogId(id) {
            $("#selectLogs").val(id);
        }

        function getSelectedStartDate() {
            var startDateText = $("input#starttime").val();

            if( startDateText !== '' ) {
                return parseDate(startDateText, 'start');
            }

            return null;
        }

        function getSelectedStopDate() {
            var stopDateText = $("input#stoptime").val();

            if( stopDateText !== '' ) {
                return parseDate(stopDateText, 'stop');
            }

            return null;
        }

        function getSelectedTimeFormat() {
            var timeFormatText = $("select#timeformat").val();
            
            if( timeFormatText !== '' ) {
                return timeFormatText;
            }

            return null;
        }

        function saveLogs() {
            var logId = getSelectedLogId();
            var startDate = getSelectedStartDate();
            var stopDate = getSelectedStopDate();
            var timeFormat = getSelectedTimeFormat();

            if( logId == null ||
                logId < 0) {
                alert('Invalid log id');
                return;
            }

            if( startDate !== null &&
                stopDate !== null &&
                startDate > stopDate ) {
                alert('Invalid date range, start > stop');
                return;
            }

            if( timeFormat == null ) {
                alert('Invalid time format');
                return;
            }

            window.location = mDeviceEngine.getLogsURI(logId, startDate, stopDate, timeFormat);
        }

        function loadLogs() {
            var logId = getSelectedLogId();
            var startDate = getSelectedStartDate();
            var stopDate = getSelectedStopDate();
            var timeFormat = getSelectedTimeFormat();

            if( logId == null ||
                logId < 0) {
                alert('Invalid log id');
                return;
            }

            if( startDate !== null &&
                stopDate !== null &&
                startDate > stopDate ) {
                alert('Invalid date range, start > stop');
                return;
            }

            if( timeFormat == null ) {
                alert('Invalid time format');
                return;
            }

            clearGraphDisplayData();

            startLogsDownload(logId, startDate, stopDate, timeFormat);
        }

        function hideWaitIcon() {
            $('#graphdiv').empty();
        }

        function showWaitIcon(text) {
            // in case it was already there...
            hideWaitIcon();

            $('#graphdiv').append(text + '<img src="imgs/ajax-loader.gif" style="vertical-align:middle"/>');
        }

        function showErrorMsg(text) {
            // in case it was already there...
            hideWaitIcon();

            $('#graphdiv').append(text);
        }

        function startLogsDownload(id, start, stop, format) {
/* Handy for test purposes
            var data;
            $.ajax({
                url: "LOG00.csv",
                success: function(d) {
                    data = d;
                },
                async: false,
                cache: false,
                dataType: "text"
            });
            mCurrentlyDisplayedLogId = id;
            logsLoaded(true, data );
            return;
*/

            if( mDeviceEngine.getLogs(id, start, stop, format, logsLoaded ) ) {
                showWaitIcon('Loading data, please wait...');

                disableTabControl(false);

                mCurrentlyDisplayedLogId = id;

                // store last loaded log id
                createCookie(DATALOGGER_SELECTED_ID_COOKIE_NAME, mCurrentlyDisplayedLogId);
            } else {
                showErrorMsg('Error loading data');
            }
        }

        function buildEmptyLoggerSerieRecord(timestamp, row) {
            //TIMESTAMP, EVENT
            var data = [timestamp, null];

            // start from DATA_OFFSET_INDEX to skip non DATA fields
            for ( var i = DATA_OFFSET_INDEX; i < row.length; i++ ) {
                data.push(null);
            }

            return data;
        }

        function buildLoggerSerieRecord(timestamp, row) {
            //TIMESTAMP, EVENT
            var data = [timestamp, null];

            // start from DATA_OFFSET_INDEX to skip non DATA fields
            for ( var i = DATA_OFFSET_INDEX; i < row.length; i++ ) {
                data.push(parseFloat(row[i]));
            }

            return data;
        }

        function processData(data) {
            var labels = [];
            var parsedData = [];
            var rows = $.csv.toArrays(data);

            if ( rows.length > 0 )
            {
                var headersRow = rows[0];

                // first row contains headers...
                for ( var i = 0; i < headersRow.length; i++ ) {
                    labels.push(headersRow[i]);
                }

                for ( var i = 1; i < rows.length; i++ ) {
                    var currentRow = rows[i];

                    if(currentRow.length !== headersRow.length) {
                        continue;
                    }

                    var time = new Date(currentRow[TIME_INDEX]);

                    if ( currentRow[EVENT_INDEX] !== '' ) {
                        parsedData.push( buildEmptyLoggerSerieRecord( time, currentRow ) );
                    }

                    parsedData.push( buildLoggerSerieRecord( time, currentRow ) );
                }
            }

            mDataloggerData = parsedData;
            mDataloggerLabels = labels;

            plotData(parsedData, labels);
        }

        function logsLoaded(result, data) {
            hideWaitIcon();

            enableTabControl();

            if( !result ) {
                showErrorMsg('Error loading data');
            } else {
                showWaitIcon('Processing downloaded data, please wait...');

                // Start an async timeout to let UI update itself (gif will not
                // animate but at least our message will be displayed)
                setTimeout( function() {
                    processData(data);
                }, 200 );
            }
        }

        function plotData(data, labels) {
            mCurrentlyDisplayedData = data;
            mCurrentlyDisplayedDataLabels = labels;

            mGraphs = new Dygraph(
                document.getElementById('graphdiv'),
                data,
                {
                    legend: 'always',
                    labelsKMB: true,
                    labelsDiv: document.getElementById("legend"),
                    labelsSeparateLines: true,

                    stepPlot: !isCurrentLogPeriodic() && !isLiveTab(),
                    drawGapEdgePoints: true,
                    drawPointCallback: Dygraph.Circles.EX,
                    pointSize: 2,

                    showRangeSelector: true,

                    highlightCircleSize: 2,
                    //strokeWidth: 1,
                    //strokeBorderWidth: 1,
                    highlightSeriesBackgroundAlpha: 0.3,

                    highlightSeriesOpts: {
                        //strokeWidth: 1,
                        //strokeBorderWidth: 1,
                        highlightCircleSize: 5,
                    },

                    //axes: {
                    //    'y2' : {
                    //        strokePattern: Dygraph.DASHED_LINE
                    //    }
                    //},

                    interactionModel: {
                        'mousedown' : downV1,
                        'mousemove' : moveV1,
                        'mouseup' : upV1,
                        'click' : clickV1,
                        'dblclick' : dblClickV1,
                        'mousewheel' : scrollV1,
                        'DOMMouseScroll' : scrollV1
                    }
                }
            );

                mGraphs.updateOptions( { 'labels': labels } );

            loadVisibilityElements();

            updateLabels();

            showGraphDisplayData();
        }

        function showGraphDisplayData() {
            $('#btnCreateImage').show();
            $('#btnRestoreZoom').show();
            $('#btnRebuildGraph').show();
        }

        function clearGraphDisplayData() {
            if ( typeof(mGraphs) !== 'undefined' &&
                 mGraphs !== null ) {
            mGraphs.destroy();
            mGraphs = null;
            }

            $('#legend').empty();
            $('#series').empty();
            $('#btnCreateImage').hide();
            $('#btnRestoreZoom').hide();
            $('#btnRebuildGraph').hide();
        }

        function loadVisibilityElements() {
            var labels = mGraphs.getLabels();
            var colors = mGraphs.getColors();
            var stepStyle = !isCurrentLogPeriodic() && !isLiveTab();
            var seriesElem = $('#series');
            // always hide EVENT column
            mGraphs.setVisibility(0, false);

            var seriesInfo = [];

            seriesInfo.push('<p><b>Series properties:</b></p>');

            seriesInfo.push('<table>');
            seriesInfo.push('<th>Name</th>');
            seriesInfo.push('<th>Visible<input id=checkall onclick=checkAll() type=checkbox checked></th>');
            seriesInfo.push('<th>Axis</th>');
            seriesInfo.push('<th>Color</th>');
            seriesInfo.push('<th>Style</th>');
//labels.length
            for( var i = 2; i < labels.length; i++ ) {
				if (i<82)
				{
					seriesInfo.push('<tr>');
					seriesInfo.push('<td>' + labels[i] + '</td>');
					seriesInfo.push('<td style="text-align: center;"><input type=checkbox name="inVis" id="' + (i - 1) + '" checked></td>');					
					seriesInfo.push('<td><select size="1" name="selAxis" id="' + (i - 1) + '">' +
											'<option value="y" selected>Primary</option>' +
											'<option value="y2">Secondary</option>' +
										 '</select></td>');
					seriesInfo.push('<td><input name="inColor" id="' + (i - 1) + '" class="color" value="' + new RGBColorParser(colors[i - 1]).toHex() + '"></td>');
					seriesInfo.push('<td><select size="1" name="selStyle" id="' + (i - 1) + '">' +
											'<option value="false" ' + (stepStyle ? '' : 'selected') + '>Line</option>' +
											'<option value="true" ' + (stepStyle ? 'selected' : '') + '>Step</option>' +
										 '</select></td>');
					seriesInfo.push('</tr>');
				}
				else
					seriesInfo.push('<td style="text-align: center; display:none;"><input type=checkbox name="inVis" id="' + (i - 1) + '" uchecked></td>');					
            }
            seriesInfo.push('</table>');
            seriesInfo.push('<input type="button" value="Apply properties" onclick="applySeriesProperties();">');

            seriesElem.empty();
            seriesElem.append(seriesInfo.join(''));			
            jscolor.bind();
			applySeriesProperties();
        }
		
		function checkAll(){
			var checkall = document.getElementById('checkall');
			var inVis = document.getElementsByName('inVis');
			for( var i = 0; i < 80; i++ ) {
				inVis[i].checked=checkall.checked;
			}
		}

        function applySeriesProperties() {
            var labels = mGraphs.getLabels();
            var newOptions = [];
            var seriesVisibility = mGraphs.visibility();

            $('input[name=inVis]').each(function( key, elem ) {
                seriesVisibility[elem.id] = elem.checked;
            });

            var axes = [];

            $('select[name=selAxis]').each(function( key, elem ) {
                var elemId = parseInt(elem.id);
                var stepStyle = $('select[id=' + elemId  + '][name=selStyle]')[0];
                var step = stepStyle.options[stepStyle.selectedIndex].value;

                // here id is labels index - 1 since the first label is always TIME
                axes.push( '"' + labels[elemId + 1] + '":' +
                         '{' +
                             '"axis": "' + elem.options[elem.selectedIndex].value + '", ' +
                             '"stepPlot": ' + step + '' +
                         '}' );
            });
            newOptions.push('"series" : {' + axes.join(',') + '}');

            // first color is always EVENT one but we hide EVENT column so put a fake value
            var colors = [ '"#000000"' ];
            $('input[name=inColor]').each(function( key, elem ) {
                colors[elem.id] = '"#' + elem.value + '"';
            });
            newOptions.push('"colors": [' + colors.join(',') + ']');

            newOptions = "{" + newOptions.join(',') + "}";

            mGraphs.updateOptions($.parseJSON(newOptions));
        }

        function createImage() {
            if ( typeof(mGraphs) === 'undefined' ||
                 mGraphs === null ) {
                alert('No graph to render, load a log graph first.');
                return;
            }

            var img = document.getElementById('renderedGraphImg');

            Dygraph.Export.asPNG(mGraphs, img);

            showExportedImage(true);
        }

        function showExportedImage(show) {
            if(show) {
                $('#export-div').show();
            } else {
                $('#export-div').hide();
            }
        }

        function getGraphTitle() {
            return $("input#graphTitle").val();
        }

        function setGraphTitle(title) {
            $("input#graphTitle").val(title);
        }

        function getPrimaryYLabel() {
            return $("input#primaryYLabel").val();
        }

        function setPrimaryYLabel(title) {
            $("input#primaryYLabel").val(title);
        }

        function getSecondayYLabel() {
            return $("input#secondaryYLabel").val();
        }

        function setSecondayYLabel(title) {
            $("input#secondaryYLabel").val(title);
        }

        function updateLabels() {
            var title = getGraphTitle();
            var ylabel = getPrimaryYLabel();
            var y2label = getSecondayYLabel();

            if ( typeof(mGraphs) === 'undefined' ||
                 mGraphs === null ) {
                alert('No graph, load a log graph first.');
                return;
            }

            mGraphs.updateOptions({
                        title: title,
                        ylabel: ylabel,
                        y2label: y2label
                    });

            storeLabelsCookie();
        }

        function restoreZoom() {
            restorePositioning(mGraphs);
        }

        function rebuildGraph() {
            clearGraphDisplayData();

            plotData(mCurrentlyDisplayedData, mCurrentlyDisplayedDataLabels);
        }

        function dateToString(targetDate, withSeconds) {
            var dateString = targetDate.getFullYear() + '/' + (targetDate.getMonth() + 1 < 10 ? '0' : '') + (targetDate.getMonth() + 1) + '/' + (targetDate.getDate() < 10 ? '0' : '') + targetDate.getDate() + ' ' +
                              (targetDate.getHours() < 10 ? '0' : '') + targetDate.getHours() + ':' + (targetDate.getMinutes() < 10 ? '0' : '') + targetDate.getMinutes();

            if ( withSeconds ) {
                dateString += ':' + (targetDate.getSeconds() < 10 ? '0' : '') + targetDate.getSeconds();
            }

            return dateString;
        }

        function quickDatesFormat(deltaElem, typeElem) {
            var targetDate = new Date();
            var delta = parseInt(deltaElem.options[deltaElem.selectedIndex].value);
            var type = typeElem.options[typeElem.selectedIndex].value;

            switch(type) {
                case 'days':
                    targetDate.setDate(targetDate.getDate() + delta);
                break;
                case 'weeks':
                    targetDate.setDate(targetDate.getDate() + (delta * 7));
                break;
                case 'months':
                    targetDate.setMonth(targetDate.getMonth() + delta);
                break;
            }

            // NOTE: this format must be consistent with NewCssCal's one define below (currently 'YYYYMMDD')
            var newStartTime = dateToString(targetDate, false);

            $("#starttime").val(newStartTime);
            $("#stoptime").val('');
        }

        function quickLoadLogs(deltaElem, typeElem) {
            quickDatesFormat(deltaElem, typeElem);

            loadLogs();
        }

        function quickSaveLogs(deltaElem, typeElem) {
            quickDatesFormat(deltaElem, typeElem);

            saveLogs();
        }

        function enableTabControl() {
            var tabContainerInputs = $('#tab-container :input');

            tabContainerInputs.prop('disabled', false);
        }

        function disableTabControl(skipLiveBtn) {
            var tabContainerInputs = $('#tab-container :input');

            tabContainerInputs.each(function() {
                if ( !skipLiveBtn || $(this).attr('id') !== 'liveStartStopBtn' ) {
                    $(this).prop('disabled', true);
                }
            });
        }


        function createCookie(name, value, days) {
            if (days) {
                var date = new Date();
                date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
                var expires = "; expires=" + date.toGMTString();
            } else var expires = "";
            document.cookie = escape(name) + "=" + escape(value) + expires + "; path=/";
        }

        function readCookie(name) {
            var nameEQ = escape(name) + "=";
            var ca = document.cookie.split(';');
            for (var i = 0; i < ca.length; i++) {
                var c = ca[i];
                while (c.charAt(0) == ' ') c = c.substring(1, c.length);
                if (c.indexOf(nameEQ) == 0) return unescape(c.substring(nameEQ.length, c.length));
            }
            return null;
        }

        function eraseCookie(name) {
            createCookie(name, "", -1);
        }

        function fillVariablesTable() {
            var idx = 0;
            var tableContent = [];

            tableContent[idx++] = '<tr>';
            tableContent[idx++] = '<th>Trace</th>';
            tableContent[idx++] = '<th>ID</th>';
            tableContent[idx++] = '<th>Name</th>';
            tableContent[idx++] = '<th>Current Value</th>';
            tableContent[idx++] = '<th>Description</th>';
            tableContent[idx++] = '</tr>';

            for( var i = 0; i < mAvailableVariables.length; i++ ) {
                var variable = mAvailableVariables[i];

                tableContent[idx++] = '<tr>';

                tableContent[idx++] = '<td><input type="checkbox" name="enTrace" id="' + i + '" onclick="updateSelectedVariablesList()"/></td>';
                tableContent[idx++] = '<td>' + variable.getId() + '</td>';
                tableContent[idx++] = '<td>' + variable.getName() + '</td>';
                tableContent[idx++] = '<td align="center">' + variable.getValue() + '</td>';
                tableContent[idx++] = '<td>' + variable.getDesc() + '</td>';

                tableContent[idx++] = '</tr>';
            }

            $('#varsTable').html(tableContent.join(''));
        }

        function toggleShowHideVarsTable() {
            var divParams = $('#varsTable');

            if ( divParams.is(":visible") ) {
                divParams.hide();
            } else {
                divParams.show();
            }
        }

        function getScanPeriod() {
            var scanPeriodText = $('#scanPeriodTextId');

            return parseInt(scanPeriodText.val()) || 0;
        }

        function setScanPeriod(value) {
            var scanPeriodText = $('#scanPeriodTextId');

            scanPeriodText.val(value);
        }

        function getTimeSpan() {
            var spanTimeText = $('#spanTimeTextId');

            return parseInt(spanTimeText.val()) || 0;
        }

        function setTimeSpan(value) {
            var spanTimeText = $('#spanTimeTextId');

            spanTimeText.val(value);
        }

        function onPeriodChanged() {
            var scanPeriod = getScanPeriod();

            if ( scanPeriod < MIN_SCAN_PERION ) {
                setScanPeriod(MIN_SCAN_PERION);
            } else if ( scanPeriod > MAX_SCAN_PERION ) {
                setScanPeriod(MAX_SCAN_PERION);
            }

            updateSpanMinMax();
        }

        function appendPeriodMinMax() {
            var periodMinMax = $('#periodMinMax');

            periodMinMax.text('(min=' + MIN_SCAN_PERION + ', max=' + MAX_SCAN_PERION + ')');
        }

        function computeMinSpan(scanPeriod) {
             return toInt((scanPeriod * MIN_POINTS_ON_GRAPH + 59) / 60);
        }

        function computeMaxSpan(scanPeriod) {
            var graphdiv = $('#graphdiv');

            return toInt((scanPeriod * graphdiv.width() + 59) / 60);
        }

        function updateSpanMinMax() {
            var spanMinMax = $('#spanMinMax');
            var scanPeriod = getScanPeriod();

            var minMinutes = computeMinSpan(scanPeriod);
            var maxMinutes = computeMaxSpan(scanPeriod);

            spanMinMax.text('(min=' + minMinutes  + ', max=' + maxMinutes + ')');

            var spanTime = getTimeSpan();

            if ( spanTime < minMinutes ) {
                setTimeSpan(minMinutes);
            } else if ( spanTime > maxMinutes ) {
                setTimeSpan(maxMinutes);
            }
        }

        function updateSelectedVariablesList() {
            var selectedVariablesText = $('#selectedVariables');

            mSelectedVarNames = [];
            mSelectedLiveVarIds = [];

            $('input[name="enTrace"]').each( function() {
                if ( $(this).is(':checked') ) {
                    var varIndex = parseInt($(this).attr('id')) || 0;

                    if ( varIndex >= 0 && varIndex < mAvailableVariables.length ) {
                        var variable = mAvailableVariables[varIndex];

                        mSelectedLiveVarIds.push(variable.getId());
                        mSelectedVarNames.push(variable.getName());
                    }
                }
            });

            if ( mSelectedVarNames.length !== selectedVariablesText.text() ) {
            if ( mSelectedVarNames.length > 0 ) {
                selectedVariablesText.text(mSelectedVarNames.join(', '));
            } else {
                selectedVariablesText.text('none');
            }

            if ( typeof(mGraphs) !== 'undefined' &&
                 mGraphs !== null ) {
                // clear previously displayed data
                clearGraphDisplayData();
            }
        }
        }

        function toInt(value) {
            return value | 0;
        }

        function startStopLive() {
            var startStopLiveBtn = $('#liveStartStopBtn');

            mLiveRunning = !mLiveRunning;

            if ( mLiveRunning ) {
                if ( mSelectedLiveVarIds.length === 0 ) {
                    alert('No variable selected, please select at least one variabile to plot');
                    mLiveRunning = false;
                    return;
                }

                if ( mLiveData.length > 0 ) {
                    if (confirm('Clear previously acquired data?')) {
                        mLiveData = [];
                    }
                }

                var divParams = $('#varsTable');

                if ( divParams.is(":visible") ) {
                    divParams.hide();
                }

                startStopLiveBtn.css({ "background": "url('imgs/stop_icon.png') no-repeat center" });
                disableTabControl(true);
                startLiveInterval();
            } else {
                startStopLiveBtn.css({ "background": "url('imgs/play_icon.png') no-repeat center" });
                enableTabControl();
                stopLiveInterval();
            }
        }

        function pushNewData(data) {
            mLiveData.push(data);

            mLiveData.shift();

            mGraphs.updateOptions( { 'file': mLiveData } );
        }

        function buildEmptySerieData() {
            var emptyData = [];

            // fill with undefined/null data
            emptyData[mSelectedLiveVarIds.length - 1] = null;

            return emptyData;
        }

        function buildLiveSerieRecord(timestamp, variables) {
            var data = [];

            //TIMESTAMP
            data.push(timestamp);
            //EVENT
            data.push(null);

            for ( var i = 0; i < variables.length; i++ ) {
                if ( typeof(variables[i]) !== "undefined" &&
                     variables[i] !== null ) {
                    data.push(parseFloat(variables[i].getValue()));
                } else {
                    data.push(null);
                }
            }

            return data;
        }

        function removeExpiredLiveData() {
            var itemsToRemove = 0;
            var now = new Date();
            var minTimestamp = now.getTime() - (getTimeSpan() * 60 * 1000);

            for ( var i = 0; i < mLiveData.length; i++) {
                if ( mLiveData[i][0] < minTimestamp ) {
                    itemsToRemove++;
                }
            }

            if ( itemsToRemove > 0 ) {
                mLiveData.splice(0, itemsToRemove);
            }
        }

        function getLiveDataToDisplay() {
            var scanPeriod = getScanPeriod();
            var numOfData = toInt((getTimeSpan() * 60) / scanPeriod) + 1;

            removeExpiredLiveData();

            if ( mLiveData.length < numOfData ) {
                var now = new Date();

                // fill with empty null items items
                for ( var i = mLiveData.length; i < numOfData; i++ ) {
                    var xTimestamp = new Date(now.getTime() - i * scanPeriod * 1000);

                    mLiveData.unshift(buildLiveSerieRecord(xTimestamp, buildEmptySerieData()));
                }
            } else if ( mLiveData.length > numOfData ) {
                // remove unnecessary items
                for ( var i = 0; i < mLiveData.length - numOfData; i++ ) {
                    mLiveData.shift();
                }
            }

            return mLiveData;
        }

        function loadLiveVarsFromCookie() {
            var cookie = readCookie(LIVE_VARS_COOKIE_NAME);

            if ( cookie !== null &&
                 typeof(cookie) !== "undefined" &&
                cookie.length > 0 ) {
                try
                {
                    var names = $.parseJSON(cookie);

                    for ( var i = 0; i < names.length; i++ ) {
                        var name = names[i];
                        var found = false;

                        for( var j = 0; j < mAvailableVariables.length; j++ ) {
                            var variable = mAvailableVariables[j];

                            if( variable.getName() === name ) {
                                $('input[name="enTrace"][id="' + j + '"]').prop("checked", true);
                                found = true;
                                break;
                            }
                        }

                        if( !found ) {
                            // reset selected variables if at least one on them is not present
                            $('input[name="enTrace"]').prop("checked", false);
                            break;
                        }
                    }

                    updateSelectedVariablesList();
                } catch (e) {
                    return;
                }
            }
        }

        function buildLiveLabels() {
            mLiveLabels = [];

            mLiveLabels.push("TIMESTAMP");
            mLiveLabels.push("EVENT");

            for ( var i = 0; i < mSelectedVarNames.length; i++) {
                mLiveLabels.push(mSelectedVarNames[i]);
            }

            return mLiveLabels;
        }

        function liveDataProbe() {
            var timestamp = new Date();
            var variables = mDeviceEngine.getVariablesById(mSelectedLiveVarIds);

            if ( typeof(variables) === "undefined" || variables === null ) {
                variables = buildEmptySerieData();
            }

            pushNewData(buildLiveSerieRecord(timestamp, variables));
        }

        function startLiveInterval() {
            var data = getLiveDataToDisplay();
            var labels = buildLiveLabels();

            // clear currently displayed log id since we are starting a live graph
            mCurrentlyDisplayedLogId = null;

            plotData(data, labels);

            // one shot now and then start an interval...
            liveDataProbe();

            mLiveInterval = setInterval(liveDataProbe, getScanPeriod() * 1000);

            // save cookies with currently selected var names (save it for 1 year) and scan info
            createCookie(LIVE_VARS_COOKIE_NAME, JSON.stringify(mSelectedVarNames), 365);
            createCookie(LIVE_PERIOD_COOKIE_NAME, getScanPeriod());
            createCookie(LIVE_SPAN_COOKIE_NAME, getTimeSpan());
        }

        function stopLiveInterval() {
            clearInterval(mLiveInterval);

            var timestamp = new Date();
            pushNewData(buildLiveSerieRecord(timestamp, buildEmptySerieData()));
        }

        function buildLiveCSV() {
            var oneLinePrinted = false;
            var lines = [];
            var headers = buildLiveLabels();

            lines.push(headers.join(','));

            for ( var i = 0; i < mLiveData.length; i++) {
                var currentRow = mLiveData[i];

                for ( var j = 1; j < currentRow.length; j++ ) {
                    if ( currentRow[j] != null ) {
                        oneLinePrinted = true;
                        break;
                    }
                }

                // print only if at least one item is not empty or we already started printing
                // (in which case the empty data serie is due to user stop/start interactions)
                if ( oneLinePrinted || j !== currentRow.length ) {
                    var fields = [];

                    fields.push(dateToString(currentRow[0], true));

                    for ( var j = 1; j < currentRow.length; j++ ) {
                        fields.push(currentRow[j] != null ? currentRow[j] : "");
                    }

                    lines.push(fields.join(','));
                }
            }

            return lines.join('\r\n');
        }

        function saveLiveData() {
            try {
                var isFileSaverSupported = !!new Blob();
            } catch(e) {
                alert('File save not supported by your browser');
                return;
            }

            var csvData = buildLiveCSV();
            var blob = new Blob([csvData], {type: "text/plain;charset=utf-8"});

            saveAs(blob, "live_data.csv");
        }

        function applyPreset(presetElem) {
            var seconds = parseInt(presetElem.options[presetElem.selectedIndex].value);

            setScanPeriod(seconds);

            onPeriodChanged();

            setTimeSpan(computeMaxSpan(seconds));
        }

        function isLiveTab() {
            return mCurrentTabName === LIVE_TABNAME;
        }

        function getGraphTitleCookieName() {
            switch(mCurrentTabName) {
                case DATALOGGER_TABNAME:
                    return DATALOGGER_TABNAME + BASE_TITLE_COOKIE_NAME + getSelectedLogId();

                case LIVE_TABNAME:
                    return LIVE_TABNAME + BASE_TITLE_COOKIE_NAME;
            }
        }

        function getPrimaryYCookieName() {
            switch(mCurrentTabName) {
                case DATALOGGER_TABNAME:
                    return DATALOGGER_TABNAME + BASE_PRIMARY_Y_COOKIE_NAME + getSelectedLogId();

                case LIVE_TABNAME:
                    return LIVE_TABNAME + BASE_PRIMARY_Y_COOKIE_NAME;
            }
        }

        function getSecondayYCookieName() {
            switch(mCurrentTabName) {
                case DATALOGGER_TABNAME:
                    return DATALOGGER_TABNAME + BASE_SECONDARY_Y_COOKIE_NAME + getSelectedLogId();

                case LIVE_TABNAME:
                    return LIVE_TABNAME + BASE_SECONDARY_Y_COOKIE_NAME;
            }
        }

        function storeLabelsCookie() {
            var title = getGraphTitle();
            var ylabel = getPrimaryYLabel();
            var y2label = getSecondayYLabel();

            createCookie(getGraphTitleCookieName(), title);
            createCookie(getPrimaryYCookieName(), ylabel);
            createCookie(getSecondayYCookieName(), y2label);
        }

        function loadGraphLabels() {
            var title = readCookie(getGraphTitleCookieName()) || '';
            var labely1 = readCookie(getPrimaryYCookieName()) || '';
            var labely2 = readCookie(getSecondayYCookieName()) || '';

            setGraphTitle(title);
            setPrimaryYLabel(labely1);
            setSecondayYLabel(labely2);
        }

        function onTimeFormatChange() {
            var timeFormat = getSelectedTimeFormat();
            if (timeFormat == null)
                return;
            // Change value of save data button and show/hide time zone warning text
            if (timeFormat == 'utc') {
                $("#btnSavelogs").val('Save UTC log data');
                $("#timezoneWarnText").show();
                
            } else {
                $("#btnSavelogs").val('Save log data');
                $("#timezoneWarnText").hide();
           }
        }

        function loadDataloggerTabData() {
            var selectedId = readCookie(DATALOGGER_SELECTED_ID_COOKIE_NAME) || '0';

            setSelectedLogId(selectedId);

            loadGraphLabels();
        }

        function loadLiveTabData() {
            var period = readCookie(LIVE_PERIOD_COOKIE_NAME) || '';
            var span = readCookie(LIVE_SPAN_COOKIE_NAME) || '';

            loadLiveVarsFromCookie();

            setTimeSpan(span);

            setScanPeriod(period);
            onPeriodChanged();

            loadGraphLabels();
        }

        function loadTabSpecificData(name) {
            mCurrentTabName = name;

            mCurrentlyDisplayedData = [];
            mCurrentlyDisplayedDataLabels = [];

            switch(name) {
                case DATALOGGER_TABNAME:
                    loadDataloggerTabData();
                    mCurrentlyDisplayedData = mDataloggerData;
                    mCurrentlyDisplayedDataLabels = mDataloggerLabels;
                break;

                case LIVE_TABNAME:
                    loadLiveTabData();
                    mCurrentlyDisplayedData = mLiveData;
                    mCurrentlyDisplayedDataLabels = mLiveLabels;
                break;
            }

            if ( mCurrentlyDisplayedData.length > 0 ) {
                if ( confirm("Replot previously acquired data?") ) {
                    rebuildGraph();
                } else {
                    // Nope, clear currently displayed info
                    mCurrentlyDisplayedData = [];
                    mCurrentlyDisplayedDataLabels = [];

                    clearGraphDisplayData();
                }
            } else {
                clearGraphDisplayData();
            }
        }

        function setupTabControl() {
            var activeTabIndex = -1;
            var tabNames = [DATALOGGER_TABNAME, LIVE_TABNAME];

            $(".tab-menu > li").click(function(e){
                if ( !mLiveRunning ) {
                for(var i = 0; i < tabNames.length; i++) {
                    if(e.target.id == tabNames[i]) {
                        activeTabIndex = i;
                    } else {
                        $("#"+tabNames[i]).removeClass("active");
                        $("#"+tabNames[i]+"-tab").css("display", "none");
                    }
                }
                $("#"+tabNames[activeTabIndex]+"-tab").fadeIn();
                $("#"+tabNames[activeTabIndex]).addClass("active");

                    if ( mCurrentTabName !== tabNames[activeTabIndex] ) {
                        loadTabSpecificData(tabNames[activeTabIndex]);
                    }
                }
                return false;
            });

            loadTabSpecificData(mCurrentTabName);
        }

