
function downV1(event, g, context) {
  context.initializeMouseDown(event, g, context);
  if (event.shiftKey || event.ctrlKey || event.button == 2) {
    Dygraph.startPan(event, g, context);
  } else if (event.altKey || event.button == 1) {
    Dygraph.startZoom(event, g, context);
  } else {
    startZoomXY(event, g, context); //Dygraph.startZoom(event, g, context);
  }
}

function moveV1(event, g, context) {
  if (context.isPanning) {
    Dygraph.movePan(event, g, context);
  } else if (context.isZooming) {
    Dygraph.moveZoom(event, g, context);
  } else if (context.isZoomingXY) {
    moveZoomXY(event, g, context); //Dygraph.moveZoom(event, g, context);
  }
}

function upV1(event, g, context) {
  if (context.isPanning) {
    Dygraph.endPan(event, g, context);
  } else if (context.isZooming) {
    Dygraph.endZoom(event, g, context);
  } else if (context.isZoomingXY) {
    endZoomXY(event, g, context); //Dygraph.endZoom(event, g, context);
  }
}

function startZoomXY(event, g, context) {
  context.isZoomingXY = true;
};

function moveZoomXY(event, g, context) {
  context.dragEndX = g.dragGetX_(event, context);
  context.dragEndY = g.dragGetY_(event, context);

  drawZoomRectXY_(
      g,
      context.dragStartX,
      context.dragEndX,
      context.dragStartY,
      context.dragEndY,
      context.prevEndX,
      context.prevEndY);

  context.prevEndX = context.dragEndX;
  context.prevEndY = context.dragEndY;
};

function endZoomXY(event, g, context) {
  context.isZoomingXY = false;
  context.dragEndX = g.dragGetX_(event, context);
  context.dragEndY = g.dragGetY_(event, context);
  var regionWidth = Math.abs(context.dragEndX - context.dragStartX);
  var regionHeight = Math.abs(context.dragEndY - context.dragStartY);

  if (regionWidth < 2 && regionHeight < 2 &&
      g.lastx_ != undefined && g.lastx_ != -1) {
    Dygraph.Interaction.treatMouseOpAsClick(g, event, context);
  }
  
  if (regionWidth >= 10 && regionHeight >= 10) {
    
	//IMPORTANT: first Y Zoom then X Zoom!
    g.doZoomY_(Math.min(context.dragStartY, context.dragEndY),
               Math.max(context.dragStartY, context.dragEndY));
    g.doZoomX_(Math.min(context.dragStartX, context.dragEndX),
               Math.max(context.dragStartX, context.dragEndX));
  } else {
    g.clearZoomRect_();
  }
  
  context.dragStartX = null;
  context.dragStartY = null;
};

function drawZoomRectXY_(g, startX, endX, startY, endY, prevEndX, prevEndY) {
  var ctx = g.canvas_ctx_;

  ctx.clearRect(Math.min(startX, prevEndX), Math.min(startY, prevEndY),
					  Math.abs(startX - prevEndX), Math.abs(startY - prevEndY));
					  
  if (endX && startX && endY && startY) {
      ctx.fillStyle = "rgba(128,128,128,0.33)";
      ctx.fillRect(Math.min(startX, endX), Math.min(startY, endY),
                   Math.abs(endX - startX), Math.abs(endY - startY));
  }
};

// Take the offset of a mouse event on the dygraph canvas and
// convert it to a pair of percentages from the bottom left. 
// (Not top left, bottom is where the lower value is.)
function offsetToPercentage(g, offsetX, offsetY) {
  // This is calculating the pixel offset of the leftmost date.
  var xOffset = g.toDomCoords(g.xAxisRange()[0], null)[0];
  var yar0 = g.yAxisRange(0);

  // This is calculating the pixel of the higest value. (Top pixel)
  var yOffset = g.toDomCoords(null, yar0[1])[1];

  // x y w and h are relative to the corner of the drawing area,
  // so that the upper corner of the drawing area is (0, 0).
  var x = offsetX - xOffset;
  var y = offsetY - yOffset;

  // This is computing the rightmost pixel, effectively defining the
  // width.
  var w = g.toDomCoords(g.xAxisRange()[1], null)[0] - xOffset;

  // This is computing the lowest pixel, effectively defining the height.
  var h = g.toDomCoords(null, yar0[0])[1] - yOffset;

  // Percentage from the left.
  var xPct = w == 0 ? 0 : (x / w);
  // Percentage from the top.
  var yPct = h == 0 ? 0 : (y / h);

  // The (1-) part below changes it from "% distance down from the top"
  // to "% distance up from the bottom".
  return [xPct, (1-yPct)];
}

function dblClickV1(event, g, context) {
  // Reducing by 20% makes it 80% the original size, which means
  // to restore to original size it must grow by 25%
  var percentages = offsetToPercentage(g, event.offsetX, event.offsetY);
  var xPct = percentages[0];
  var yPct = percentages[1];

  if (event.altKey || event.shiftKey || event.ctrlKey) {
    zoom(g, -.25, xPct, yPct);
  } else {
    zoom(g, +.2, xPct, yPct);
  }
}

var lastClickedGraph = null;

function clickV1(event, g, context) {
  lastClickedGraph = g;
  Dygraph.cancelEvent(event);
}

function scrollV1(event, g, context) {
  if (lastClickedGraph != g) {
    return;
  }
  var normal = event.detail ? event.detail * -1 : event.wheelDelta / 40;
  // For me the normalized value shows 0.075 for one click. If I took
  // that verbatim, it would be a 7.5%.
  var percentage = normal / 50;

  var percentages = offsetToPercentage(g, event.offsetX, event.offsetY);
  var xPct = percentages[0];
  var yPct = percentages[1];

  zoom(g, percentage, xPct, yPct);
  Dygraph.cancelEvent(event);
}

// Adjusts [x, y] toward each other by zoomInPercentage%
// Split it so the left/bottom axis gets xBias/yBias of that change and
// tight/top gets (1-xBias)/(1-yBias) of that change.
//
// If a bias is missing it splits it down the middle.
function zoom(g, zoomInPercentage, xBias, yBias) {
  xBias = xBias || 0.5;
  yBias = yBias || 0.5;
  function adjustAxis(axis, zoomInPercentage, bias) {
    var delta = axis[1] - axis[0];
    var increment = delta * zoomInPercentage;
    var foo = [increment * bias, increment * (1-bias)];
    return [ axis[0] + foo[0], axis[1] - foo[1] ];
  }
  var yAxes = g.yAxisRanges();
  var newYAxes = [];
  for (var i = 0; i < yAxes.length; i++) {
    newYAxes[i] = adjustAxis([g.toDomYCoord(yAxes[i][0]),g.toDomYCoord(yAxes[i][1])], zoomInPercentage, yBias);
  }
  
  var xAxes = adjustAxis([g.toDomXCoord(g.xAxisRange()[0]),g.toDomXCoord(g.xAxisRange()[1])], zoomInPercentage, xBias);
  
  g.doZoomY_(Math.min(newYAxes[0][0],newYAxes[0][1]),Math.max(newYAxes[0][0],newYAxes[0][1]));
  g.doZoomX_(Math.min(xAxes[0],xAxes[1]),Math.max(xAxes[0],xAxes[1]));
			   
  //g.updateOptions({
  //  dateWindow: adjustAxis(g.xAxisRange(), zoomInPercentage, xBias),
  //  valueRange: newYAxes[0]
  //  });
}

function restorePositioning(g) {
  g.updateOptions({
    dateWindow: null,
    valueRange: null
  });
}
