var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define("service/log.service", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Log helper.
     * See https://stackoverflow.com/a/32928812
     *
     * How to use:
     * var logHelper = new LogHelper(this);
     * LogService.Logger.log("Log msg");
     * LogService.Logger.error("Error msg");
     */
    var LogService = /** @class */ (function () {
        function LogService(context) {
            if (!context) {
                throw Error("ConsoleLogService: context is null");
            }
            this._logger = { log: function () { }, error: function () { } };
            if (console) {
                for (var consoleMode in console) {
                    if (typeof console[consoleMode] == "function" && this._logger[consoleMode]) {
                        // Override _logger function
                        var className = void 0;
                        if (typeof context === "string" || context instanceof String) {
                            className = context;
                        }
                        else {
                            className = context.constructor ? context.constructor.name : context.toString();
                        }
                        this._logger[consoleMode] = console.log.bind(window.console, className + ": ");
                    }
                }
            }
            // Acquire the function after the ovveride
            this._log = this._logger.log;
            this._error = this._logger.error;
        }
        return LogService;
    }());
    exports.LogService = LogService;
});
define("common/loggable-entity", ["require", "exports", "service/log.service"], function (require, exports, log_service_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var LoggableEntity = /** @class */ (function () {
        function LoggableEntity() {
            /** LogService instance. */
            this._logService = new log_service_1.LogService(this);
            this._log = this._logService._log;
            this._error = this._logService._error;
        }
        return LoggableEntity;
    }());
    exports.LoggableEntity = LoggableEntity;
    var LoggableEntityStatic = /** @class */ (function () {
        function LoggableEntityStatic(context) {
            this._logService = new log_service_1.LogService(context);
            this._log = this._logService._log;
            this._error = this._logService._error;
        }
        return LoggableEntityStatic;
    }());
    exports.LoggableEntityStatic = LoggableEntityStatic;
});
define("service/base.service", ["require", "exports", "common/loggable-entity"], function (require, exports, loggable_entity_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var BaseService = /** @class */ (function (_super) {
        __extends(BaseService, _super);
        function BaseService() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BaseService._singletonConstructorErrorMsg = "instantiation failed. Singleton module! Use buildInstance() instead of new";
        BaseService._invalidBaseUrl = "base url are not valid. BaseUrl = ";
        return BaseService;
    }(loggable_entity_1.LoggableEntity));
    exports.BaseService = BaseService;
});
define("dataModel/datatype.model", ["require", "exports", "common/validator"], function (require, exports, validator_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var DataTypeUtil = /** @class */ (function () {
        function DataTypeUtil() {
            if (DataTypeUtil._instance) {
                throw new Error("DataTypeUtil error: Instantiation failed. Singleton module! Use .getInstance() instead of new");
            }
            this._datatypeDictionary = {};
            this._datatypeDictionary[DataTypeEnum.Bool] = new DataType(DataTypeEnum.Bool, "BOOL");
            this._datatypeDictionary[DataTypeEnum.Sint] = new DataType(DataTypeEnum.Sint, "SINT");
            this._datatypeDictionary[DataTypeEnum.Dint] = new DataType(DataTypeEnum.Dint, "DINT");
            this._datatypeDictionary[DataTypeEnum.Time] = new DataType(DataTypeEnum.Time, "TIME");
            this._datatypeDictionary[DataTypeEnum.Real] = new DataType(DataTypeEnum.Real, "REAL");
            this._datatypeDictionary[DataTypeEnum.String] = new DataType(DataTypeEnum.String, "STRING");
            this._datatypeDictionary[DataTypeEnum.AnyStructure] = new DataType(DataTypeEnum.AnyStructure, "ANY STRUCTURE");
            this._datatypeDictionary[DataTypeEnum.Usint] = new DataType(DataTypeEnum.Usint, "USINT");
            this._datatypeDictionary[DataTypeEnum.Int] = new DataType(DataTypeEnum.Int, "INT");
            this._datatypeDictionary[DataTypeEnum.Uint] = new DataType(DataTypeEnum.Uint, "UINT");
            this._datatypeDictionary[DataTypeEnum.UdInt] = new DataType(DataTypeEnum.UdInt, "UDINT");
            this._datatypeDictionary[DataTypeEnum.Lint] = new DataType(DataTypeEnum.Lint, "LINT");
            this._datatypeDictionary[DataTypeEnum.Ulint] = new DataType(DataTypeEnum.Ulint, "ULINT");
            this._datatypeDictionary[DataTypeEnum.Date] = new DataType(DataTypeEnum.Date, "DATE");
            this._datatypeDictionary[DataTypeEnum.Lreal] = new DataType(DataTypeEnum.Lreal, "LREAL");
        }
        Object.defineProperty(DataTypeUtil.prototype, "DataTypeDictionary", {
            get: function () { return this._datatypeDictionary; },
            enumerable: true,
            configurable: true
        });
        DataTypeUtil.getInstance = function () {
            if (DataTypeUtil._instance === null) {
                DataTypeUtil._instance = new DataTypeUtil();
            }
            return DataTypeUtil._instance;
        };
        DataTypeUtil.IsNumericDataType = function (dataType) {
            switch (dataType) {
                case DataTypeEnum.Dint:
                case DataTypeEnum.Int:
                case DataTypeEnum.Lint:
                case DataTypeEnum.Lreal:
                case DataTypeEnum.Real:
                case DataTypeEnum.Sint:
                case DataTypeEnum.UdInt:
                case DataTypeEnum.Uint:
                case DataTypeEnum.Ulint:
                case DataTypeEnum.Usint:
                    return true;
                default:
                    return false;
            }
        };
        DataTypeUtil._instance = null;
        return DataTypeUtil;
    }());
    exports.DataTypeUtil = DataTypeUtil;
    var DataType = /** @class */ (function () {
        function DataType(_type, _displayName) {
            this._type = _type;
            this._displayName = _displayName;
        }
        Object.defineProperty(DataType.prototype, "Type", {
            get: function () { return this._type; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DataType.prototype, "DisplayName", {
            get: function () { return this._displayName; },
            enumerable: true,
            configurable: true
        });
        return DataType;
    }());
    exports.DataType = DataType;
    var DataTypeEnum;
    (function (DataTypeEnum) {
        DataTypeEnum[DataTypeEnum["Bool"] = 1] = "Bool";
        DataTypeEnum[DataTypeEnum["Sint"] = 2] = "Sint";
        DataTypeEnum[DataTypeEnum["Dint"] = 3] = "Dint";
        DataTypeEnum[DataTypeEnum["Time"] = 4] = "Time";
        DataTypeEnum[DataTypeEnum["Real"] = 5] = "Real";
        DataTypeEnum[DataTypeEnum["String"] = 6] = "String";
        DataTypeEnum[DataTypeEnum["AnyStructure"] = 7] = "AnyStructure";
        DataTypeEnum[DataTypeEnum["Usint"] = 8] = "Usint";
        DataTypeEnum[DataTypeEnum["Int"] = 9] = "Int";
        DataTypeEnum[DataTypeEnum["Uint"] = 10] = "Uint";
        DataTypeEnum[DataTypeEnum["UdInt"] = 11] = "UdInt";
        DataTypeEnum[DataTypeEnum["Lint"] = 12] = "Lint";
        DataTypeEnum[DataTypeEnum["Ulint"] = 13] = "Ulint";
        DataTypeEnum[DataTypeEnum["Date"] = 14] = "Date";
        DataTypeEnum[DataTypeEnum["Lreal"] = 15] = "Lreal";
    })(DataTypeEnum = exports.DataTypeEnum || (exports.DataTypeEnum = {}));
    var CoefficientData = /** @class */ (function () {
        function CoefficientData(_addCoefficient, _multCoefficient) {
            this._addCoefficient = _addCoefficient;
            this._multCoefficient = _multCoefficient;
        }
        Object.defineProperty(CoefficientData.prototype, "AddCoefficient", {
            get: function () { return this._addCoefficient; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CoefficientData.prototype, "MultCoefficient", {
            get: function () { return this._multCoefficient; },
            enumerable: true,
            configurable: true
        });
        return CoefficientData;
    }());
    exports.CoefficientData = CoefficientData;
    var UomData = /** @class */ (function (_super) {
        __extends(UomData, _super);
        function UomData(_symbol, _uomDstId, addCoefficient, multCoefficient) {
            var _this = _super.call(this, addCoefficient, multCoefficient) || this;
            _this._symbol = _symbol;
            _this._uomDstId = _uomDstId;
            return _this;
        }
        Object.defineProperty(UomData.prototype, "UomDstId", {
            get: function () { return this._uomDstId; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UomData.prototype, "Symbol", {
            get: function () { return this._symbol; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UomData.prototype, "HaveConversionCoefficient", {
            get: function () {
                return validator_1.Validator.isDefined(this._addCoefficient) && validator_1.Validator.isDefined(this._multCoefficient);
            },
            enumerable: true,
            configurable: true
        });
        return UomData;
    }(CoefficientData));
    exports.UomData = UomData;
    var SourceValue;
    (function (SourceValue) {
        SourceValue[SourceValue["Current"] = 0] = "Current";
        SourceValue[SourceValue["Imported"] = 1] = "Imported";
    })(SourceValue = exports.SourceValue || (exports.SourceValue = {}));
});
define("service/storage.service", ["require", "exports", "service/base.service", "common/validator"], function (require, exports, base_service_1, validator_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var StorageService = /** @class */ (function (_super) {
        __extends(StorageService, _super);
        function StorageService() {
            var _this = _super.call(this) || this;
            if (StorageService._instance) {
                _this._error(base_service_1.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            _this.loadSettings();
            return _this;
        }
        Object.defineProperty(StorageService.prototype, "AlarmRefreshIntevalSeconds", {
            get: function () { return this._settings.AlarmRefreshInterval; },
            set: function (value) { this._settings.AlarmRefreshInterval = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(StorageService.prototype, "VariableRefreshIntevalSeconds", {
            get: function () { return this._settings.VariableRefreshInterval; },
            set: function (value) { this._settings.VariableRefreshInterval = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(StorageService.prototype, "ExportCharSeparator", {
            get: function () { return this._settings.ExportCharSeparator; },
            set: function (value) { this._settings.ExportCharSeparator = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(StorageService.prototype, "VariableNumberOfDigits", {
            get: function () { return this._settings.VariableNumberOfDigits; },
            set: function (value) { this._settings.VariableNumberOfDigits = value; },
            enumerable: true,
            configurable: true
        });
        StorageService.getInstance = function () {
            if (StorageService._instance === null) {
                StorageService._instance = new StorageService();
            }
            return StorageService._instance;
        };
        StorageService.prototype.loadSettings = function () {
            if (localStorage) {
                StorageService._isLocalStorageEnable = true;
                // Load settings
                try {
                    var settingsContainerValue = JSON.parse(localStorage.getItem(StorageService._settingsStorageKey));
                    if (!settingsContainerValue) {
                        // Load default setting
                        this._settings = this.loadDefaultSettings();
                    }
                    else {
                        this._settings = new Settings();
                        // Validate values
                        // Alarm refresh interval
                        if (validator_2.Validator.isDefined(settingsContainerValue._alarmRefreshInterval)) {
                            var interval = validator_2.Validator.getNum(settingsContainerValue._alarmRefreshInterval, StorageService.RefreshIntervalSecondsValueDefault);
                            this._settings.AlarmRefreshInterval = interval >= 1 ? interval : StorageService.RefreshIntervalSecondsValueDefault;
                        }
                        else {
                            this._settings.AlarmRefreshInterval = StorageService.RefreshIntervalSecondsValueDefault;
                        }
                        // Variable refresh interval
                        if (validator_2.Validator.isDefined(settingsContainerValue._variableRefreshInterval)) {
                            var interval = validator_2.Validator.getNum(settingsContainerValue._variableRefreshInterval, StorageService.RefreshIntervalSecondsValueDefault);
                            this._settings.VariableRefreshInterval = interval >= 1 ? interval : StorageService.RefreshIntervalSecondsValueDefault;
                        }
                        else {
                            this._settings.VariableRefreshInterval = StorageService.RefreshIntervalSecondsValueDefault;
                        }
                        // Export char separator
                        if (validator_2.Validator.isDefined(settingsContainerValue._exportCharSeparator)) {
                            if (settingsContainerValue._exportCharSeparator.length === 1) {
                                this._settings.ExportCharSeparator = settingsContainerValue._exportCharSeparator;
                            }
                            else {
                                this._settings.ExportCharSeparator = StorageService.ExportSeparatorCharValueDefault;
                            }
                        }
                        else {
                            this._settings.ExportCharSeparator = StorageService.ExportSeparatorCharValueDefault;
                        }
                        // Number of digits
                        if (validator_2.Validator.isDefined(settingsContainerValue._numberOfDigits)) {
                            var numberOfDigits = validator_2.Validator.getNum(settingsContainerValue._variableRefreshInterval, StorageService.RefreshIntervalSecondsValueDefault);
                            this._settings.VariableNumberOfDigits = numberOfDigits > 0 && numberOfDigits < 7
                                ? numberOfDigits : StorageService.VariableNumberOfDigitsDefault;
                        }
                        else {
                            this._settings.VariableNumberOfDigits = StorageService.VariableNumberOfDigitsDefault;
                        }
                    }
                }
                catch (e) {
                    this._log("Failed to load settings from local storage. I used the default. " + e);
                    // Load default setting
                    this._settings = this.loadDefaultSettings();
                }
            }
            else {
                StorageService._isLocalStorageEnable = false;
                // Load default setting
                this._settings = this.loadDefaultSettings();
            }
        };
        StorageService.prototype.loadDefaultSettings = function () {
            var settings = new Settings();
            settings.AlarmRefreshInterval = StorageService.RefreshIntervalSecondsValueDefault;
            settings.VariableRefreshInterval = StorageService.RefreshIntervalSecondsValueDefault;
            settings.ExportCharSeparator = StorageService.ExportSeparatorCharValueDefault;
            settings.VariableNumberOfDigits = StorageService.VariableNumberOfDigitsDefault;
            return settings;
        };
        StorageService.prototype.saveSettings = function () {
            if (StorageService._isLocalStorageEnable) {
                try {
                    // Validate
                    if ((validator_2.Validator.isDefined(this._settings.AlarmRefreshInterval)
                        && this._settings.AlarmRefreshInterval >= 1) === false) {
                        // Use default
                        this._log("Skip the saving of alarm refresh interval. Value is not valid. Value=" + this._settings.AlarmRefreshInterval);
                        this._settings.AlarmRefreshInterval = StorageService.RefreshIntervalSecondsValueDefault;
                    }
                    if ((validator_2.Validator.isDefined(this._settings.VariableRefreshInterval)
                        && this._settings.VariableRefreshInterval >= 1) === false) {
                        // Use default
                        this._log("Skip the saving of variable refresh interval. Value is not valid. Value=" + this._settings.VariableRefreshInterval);
                        this._settings.VariableRefreshInterval = StorageService.RefreshIntervalSecondsValueDefault;
                    }
                    if ((validator_2.Validator.isDefined(this._settings.ExportCharSeparator)
                        && this._settings.ExportCharSeparator.length === 1) === false) {
                        // Use default
                        this._log("Skip the saving of export character separator. Value is not valid. Value=" + this._settings.ExportCharSeparator);
                        this._settings.ExportCharSeparator = StorageService.ExportSeparatorCharValueDefault;
                    }
                    if ((validator_2.Validator.isDefined(this._settings.VariableNumberOfDigits)
                        && this._settings.VariableNumberOfDigits > 0 && this._settings.VariableNumberOfDigits < 7) === false) {
                        // Use default
                        this._log("Skip the saving of digit number. Value is not valid. Value=" + this._settings.VariableNumberOfDigits);
                        this._settings.VariableNumberOfDigits = StorageService.VariableNumberOfDigitsDefault;
                    }
                    // Save
                    localStorage.setItem(StorageService._settingsStorageKey, JSON.stringify(this._settings));
                }
                catch (e) {
                    this._log("Save on localStorage is failed. " + e);
                }
            }
            else {
                this._log("Save skipped. LocalStorage isn't enable");
            }
        };
        StorageService.prototype.validateAlarmRefreshValue = function (value) {
            var interval = validator_2.Validator.getNum(value, StorageService.RefreshIntervalSecondsValueDefault);
            return interval >= 1 ? interval : StorageService.RefreshIntervalSecondsValueDefault;
        };
        StorageService._settingsStorageKey = "cFieldSettings";
        // Storage key
        StorageService.XmlConfigurationStoragekey = "cFieldXmlConfig";
        StorageService.CrcConfigurationStorageKey = "cFieldCrcConfig";
        // Default values
        StorageService.RefreshIntervalSecondsValueDefault = 5;
        StorageService.ExportSeparatorCharValueDefault = ",";
        StorageService.VariableNumberOfDigitsDefault = 2;
        StorageService._instance = null;
        return StorageService;
    }(base_service_1.BaseService));
    exports.StorageService = StorageService;
    var SettingsContainer = /** @class */ (function () {
        function SettingsContainer() {
        }
        return SettingsContainer;
    }());
    /** Use the same field name of SettingsContainer otherwise the JSON parse/stringfy doesn't works correcty */
    var Settings = /** @class */ (function () {
        function Settings() {
        }
        Object.defineProperty(Settings.prototype, "AlarmRefreshInterval", {
            get: function () { return this._alarmRefreshInterval; },
            set: function (value) { this._alarmRefreshInterval = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Settings.prototype, "VariableRefreshInterval", {
            get: function () { return this._variableRefreshInterval; },
            set: function (value) { this._variableRefreshInterval = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Settings.prototype, "ExportCharSeparator", {
            get: function () { return this._exportCharSeparator; },
            set: function (value) { this._exportCharSeparator = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Settings.prototype, "VariableNumberOfDigits", {
            get: function () { return this._variableNumberOfDigits; },
            set: function (value) { this._variableNumberOfDigits = value; },
            enumerable: true,
            configurable: true
        });
        return Settings;
    }());
    exports.Settings = Settings;
});
define("common/data-structure", ["require", "exports", "common/validator"], function (require, exports, validator_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var StringSet = /** @class */ (function () {
        function StringSet() {
            this._set = {};
            this._count = 0;
        }
        Object.defineProperty(StringSet.prototype, "Set", {
            get: function () { return this._set; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(StringSet.prototype, "Count", {
            get: function () { return this._count; },
            enumerable: true,
            configurable: true
        });
        /**
         * Insert only if there aren't conflicts
         * @param key
         */
        StringSet.prototype.addNoConflict = function (key) {
            if (validator_3.Validator.isDefined(key)) {
                if (this._set.hasOwnProperty(key) === false) {
                    this._set[key] = null;
                    this._count++;
                }
            }
        };
        StringSet.prototype.remove = function (key) {
            if (validator_3.Validator.isDefined(key)) {
                if (this._set.hasOwnProperty(key) === true) {
                    delete this._set[key];
                    this._count--;
                    return true;
                }
            }
            return false;
        };
        StringSet.prototype.contains = function (key) {
            return validator_3.Validator.isDefined(key) && this._set.hasOwnProperty(key) === true;
        };
        StringSet.prototype.getValues = function () {
            var result = [];
            for (var key in this._set) {
                result.push(key);
            }
            return result;
        };
        return StringSet;
    }());
    exports.StringSet = StringSet;
    var CustomMap = /** @class */ (function () {
        function CustomMap() {
            this._set = {};
            this._count = 0;
        }
        Object.defineProperty(CustomMap.prototype, "Set", {
            get: function () { return this._set; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CustomMap.prototype, "Count", {
            get: function () { return this._count; },
            enumerable: true,
            configurable: true
        });
        CustomMap.prototype.addNoConflict = function (key, value) {
            if (validator_3.Validator.isDefined(key)) {
                if (this._set.hasOwnProperty(key) === false) {
                    this._set[key] = value;
                    this._count++;
                    return true;
                }
            }
            return false;
        };
        CustomMap.prototype.remove = function (key) {
            if (validator_3.Validator.isDefined(key)) {
                if (this._set.hasOwnProperty(key) === true) {
                    var value = this._set[key];
                    delete this._set[key];
                    this._count--;
                    return value;
                }
            }
            return null;
        };
        // Very slow
        CustomMap.prototype.clearSlow = function () {
            for (var property in this._set) {
                delete this._set[property];
            }
            this._count = 0;
        };
        CustomMap.prototype.contains = function (key) {
            return validator_3.Validator.isDefined(key) && this._set.hasOwnProperty(key) === true;
        };
        CustomMap.prototype.getValues = function () {
            var result = [];
            for (var key in this._set) {
                result.push(this._set[key]);
            }
            return result;
        };
        CustomMap.prototype.getKeys = function () {
            var result = [];
            for (var key in this._set) {
                result.push(key);
            }
            return result;
        };
        return CustomMap;
    }());
    exports.CustomMap = CustomMap;
    var MapUtil = /** @class */ (function () {
        function MapUtil() {
        }
        /**
         * Add the value to the map. The value is inserted into an array
         * @param map
         * @param key
         * @param value
         */
        MapUtil.AddMapKeyToArrayOfValue = function (map, key, value) {
            if (validator_3.Validator.isDefined(key)) {
                if (map.hasOwnProperty(key)) {
                    // Add only id
                    map[key].push(value);
                }
                else {
                    // Create array and add id
                    map[key] = [value];
                }
            }
        };
        return MapUtil;
    }());
    exports.MapUtil = MapUtil;
    var ArrayUtil = /** @class */ (function () {
        function ArrayUtil() {
        }
        /**
         * Convert the input into an array.
         * @param inputArray Can be an array or a single object.
         * @return an array built from <code>inputArray</code>.
         */
        ArrayUtil.GetArray = function (inputArray) {
            if (inputArray) {
                var outputArray = void 0;
                if (inputArray.length) {
                    outputArray = inputArray;
                }
                else {
                    outputArray = [inputArray];
                }
                return outputArray;
            }
            return null;
        };
        ArrayUtil.mergeStringArray = function (array1, array2) {
            var result = new StringSet();
            for (var _i = 0, array1_1 = array1; _i < array1_1.length; _i++) {
                var item1 = array1_1[_i];
                result.addNoConflict(item1);
            }
            for (var _a = 0, array2_1 = array2; _a < array2_1.length; _a++) {
                var item2 = array2_1[_a];
                result.addNoConflict(item2);
            }
            return result.getValues();
        };
        return ArrayUtil;
    }());
    exports.ArrayUtil = ArrayUtil;
});
define("common/string-util", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var StringUtil = /** @class */ (function () {
        function StringUtil() {
        }
        // Work only %s
        StringUtil.sprintf = function (str) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            if (typeof str !== "undefined" && str !== null) {
                var flag_1 = true;
                var i_1 = 0;
                str = str.replace(/%s/g, function () {
                    var arg = args[i_1++];
                    if (typeof arg === "undefined" || arg === null) {
                        flag_1 = false;
                        return "";
                    }
                    return arg;
                });
                return flag_1 ? str : "";
            }
            else {
                return "";
            }
        };
        ;
        return StringUtil;
    }());
    exports.StringUtil = StringUtil;
});
define("service/translator.service", ["require", "exports", "knockout", "i18next", "i18next-xhr-backend", "i18next-browser-languagedetector", "service/cfield.service", "service/base.service", "common/data-structure", "common/string-util", "common/validator"], function (require, exports, ko, i18next, i18nextXHRBackend, i18nextBrowserLanguageDetector, cfield_service_1, base_service_2, data_structure_1, string_util_1, validator_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Translator service.
     */
    var TranslatorService = /** @class */ (function (_super) {
        __extends(TranslatorService, _super);
        /**
         * Default constructor.
         * @param baseUrl Root url of cField.
         */
        function TranslatorService(baseUrl) {
            var _this = _super.call(this) || this;
            /** <code>True</code> if translator support multiple languages. */
            _this._hasMultipleLanguages = false;
            /** Supported languages. */
            _this._availableLanguages = [];
            /** Language loading state */
            _this._languageStateLoading = {};
            _this._discardedLanguage = {};
            if (TranslatorService._instance) {
                _this._error(base_service_2.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            // Check null values
            if (baseUrl == null) {
                _this._error(base_service_2.BaseService._invalidBaseUrl + baseUrl);
                return _this;
            }
            _this._baseUrl = baseUrl;
            _this._currentLanguage = ko.observable();
            // The observable state of service is always notified (also in the case re-assign of same value)
            _this._translatorStateObservable = ko.observable(TranslatorServiceState.NotInitialized);
            _this._onChangeLanguageObservable = ko.observable();
            _this._onChangeLanguageObservable.extend({ notify: "always" });
            return _this;
        }
        Object.defineProperty(TranslatorService.prototype, "HasMultipleLanguages", {
            get: function () { return this._hasMultipleLanguages; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TranslatorService.prototype, "t", {
            get: function () {
                return this._t;
            },
            enumerable: true,
            configurable: true
        });
        TranslatorService.prototype.initPreloadLanguages = function () {
            var cFieldManifest = cfield_service_1.cFieldService.getInstance().CFieldManifest;
            if (cFieldManifest && cFieldManifest.preloadLanguages) {
                this._preloadLanguages = cFieldManifest.preloadLanguages;
            }
        };
        Object.defineProperty(TranslatorService.prototype, "CurrentLanguage", {
            get: function () { return this._currentLanguage; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TranslatorService.prototype, "SupportedLanguages", {
            get: function () { return this._availableLanguages; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(TranslatorService.prototype, "TranslatorStateObservable", {
            get: function () {
                return this._translatorStateObservable;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TranslatorService.prototype, "OnChangeLanguageObservable", {
            get: function () {
                return this._onChangeLanguageObservable;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Build instance of translator service in accord with singleton pattern.
         * @param baseUrl Root url of cField.
         * @return an instance of translator service.
         */
        TranslatorService.buildInstance = function (baseUrl) {
            if (TranslatorService._instance === null) {
                TranslatorService._instance = new TranslatorService(baseUrl);
            }
            return TranslatorService._instance;
        };
        /**
         * Get instance of translator service in accord with singleton pattern.
         * @return an instance of translator service.
         */
        TranslatorService.getInstance = function () {
            if (TranslatorService._instance === null) {
                throw new Error("Error: TranslatorService not init");
            }
            return TranslatorService._instance;
        };
        /**
         * Init the service.
         * @return the promise that initialize the service.
         */
        TranslatorService.prototype.init = function () {
            var _this = this;
            TranslatorService._setTranslationRecursionCounter = 0;
            return this.initI18next()
                .then(function (t) {
                return _this.setTranslatorParams(t);
            })
                .then(function () {
                _this.registerBootstrapLocales();
            });
        };
        /**
         * Init translator.
         */
        TranslatorService.prototype.initI18next = function () {
            return new Promise(function (resolve, reject) {
                var _this = this;
                // Init preload languages
                this.initPreloadLanguages();
                // Register a callback to manage the languages that fail the loading
                i18next.on("failedLoading", function (language, namespace, message) {
                    if (language && namespace) {
                        if (!this._languageStateLoading[language]) {
                            this._languageStateLoading[language] = {};
                        }
                        var lngObj = this._languageStateLoading[language];
                        lngObj[namespace] = false;
                    }
                }.bind(this));
                // Init translator
                i18next
                    .use(i18nextXHRBackend)
                    .use(i18nextBrowserLanguageDetector)
                    .init({
                    debug: false,
                    fallbackLng: TranslatorService.defaultFallbackLanguage,
                    preload: this._preloadLanguages,
                    keySeparator: '>',
                    ns: TranslatorService.Namespaces,
                    load: "all",
                    detection: {
                        order: ["querystring", "navigator"]
                    },
                    backend: {
                        loadPath: this._baseUrl + "/locales/{{lng}}/{{ns}}.json"
                    }
                }, function (err, t) {
                    var i18nextInitialization = _this.elaborateI18nextInitialization(err, t);
                    if (i18nextInitialization.Result === true) {
                        resolve(t);
                    }
                    else {
                        reject(i18nextInitialization.Details);
                    }
                });
            }.bind(this));
        };
        TranslatorService.prototype.elaborateI18nextInitialization = function (err, t) {
            // Get languages
            var previewLanguages = this.getLanguages();
            if (previewLanguages.length > 0) {
                if (!err) {
                    // Initialization is ok
                    this._availableLanguages = data_structure_1.ArrayUtil.mergeStringArray(this._availableLanguages, previewLanguages);
                    this.printAvailableLanguages();
                    return new I18nextInitializationResult(true);
                }
                else {
                    // Some errors
                    if (typeof previewLanguages !== "undefined" && previewLanguages !== null
                        && typeof previewLanguages.length !== "undefined" && previewLanguages.length !== null) {
                        // Remove language that I can't manage
                        for (var language in this._languageStateLoading) {
                            var currentLanguage = this._languageStateLoading[language];
                            for (var namespace in currentLanguage) {
                                var currentNamespace = currentLanguage[namespace];
                                if (currentNamespace === false && TranslatorService.IsNamespaceCritical(namespace) === true) {
                                    // Remove
                                    for (var i = 0; i < previewLanguages.length; i++) {
                                        var availableLanguage = previewLanguages[i];
                                        if (availableLanguage === language) {
                                            previewLanguages.splice(i, 1);
                                            // Save the missing language because multiple changeLanguage can return a null err if 
                                            // i18next doesn't download a missing language (because it knows that it doesn't exist)
                                            this._discardedLanguage[language] = true;
                                            this._log("The '" + language + "' has been removed because it hasn't the '" + namespace + "' namespace");
                                        }
                                    }
                                }
                            }
                        }
                        if (previewLanguages.length > 0) {
                            // Check if there is the fallback language
                            var foundDefaultLanguage = false;
                            for (var _i = 0, previewLanguages_1 = previewLanguages; _i < previewLanguages_1.length; _i++) {
                                var language = previewLanguages_1[_i];
                                if (language === TranslatorService.defaultFallbackLanguage) {
                                    foundDefaultLanguage = true;
                                    break;
                                }
                            }
                            if (foundDefaultLanguage === true) {
                                // Initialization is ok
                                this._availableLanguages = data_structure_1.ArrayUtil.mergeStringArray(this._availableLanguages, previewLanguages);
                                this.printAvailableLanguages();
                                return new I18nextInitializationResult(true);
                            }
                            else {
                                this._translatorStateObservable(TranslatorServiceState.Error);
                                return new I18nextInitializationResult(false, "TranslatorService: init(). Fallback language (" + TranslatorService.defaultFallbackLanguage +
                                    ") not found");
                            }
                        }
                        else {
                            // Not available language
                            this._translatorStateObservable(TranslatorServiceState.Error);
                            return new I18nextInitializationResult(false, "TranslatorService: init(). Not available languages (all languages have been discarded)");
                        }
                    }
                    else {
                        this._translatorStateObservable(TranslatorServiceState.Error);
                        return new I18nextInitializationResult(false, err);
                    }
                }
            }
            else {
                // No available languages
                this._translatorStateObservable(TranslatorServiceState.Error);
                return new I18nextInitializationResult(false, "TranslatorService: init(). Not available languages");
            }
        };
        /**
         * Check if translation namespace is critical.
         * @param namespace
         * @return <code>True</code> if translation namespace is critical.
         */
        TranslatorService.IsNamespaceCritical = function (namespace) {
            switch (namespace) {
                case TranslatorService.cfieldBaseTranslationFilename:
                    return true;
                default:
                    return false;
            }
        };
        /**
         * @return The available languages.
         */
        TranslatorService.prototype.getLanguages = function () {
            // Preload languages aren't included in i18next.languages;
            var loadedLanguages = i18next.languages;
            var availableLanguages = [];
            if (typeof loadedLanguages !== "undefined" && loadedLanguages !== null
                && typeof loadedLanguages.length !== "undefined" && loadedLanguages.length !== null) {
                var availableLanguagesSet = {};
                for (var _i = 0, loadedLanguages_1 = loadedLanguages; _i < loadedLanguages_1.length; _i++) {
                    var loadedLanguage = loadedLanguages_1[_i];
                    availableLanguagesSet[loadedLanguage] = null;
                }
                // Manage preload languages
                if (this._preloadLanguages) {
                    for (var _a = 0, _b = this._preloadLanguages; _a < _b.length; _a++) {
                        var preloadLanguage = _b[_a];
                        for (var _c = 0, loadedLanguages_2 = loadedLanguages; _c < loadedLanguages_2.length; _c++) {
                            var loadedLanguage = loadedLanguages_2[_c];
                            if (loadedLanguage !== preloadLanguage) {
                                availableLanguagesSet[preloadLanguage] = null;
                            }
                        }
                    }
                }
                for (var language in availableLanguagesSet) {
                    // Workaroud: i18next remembers the language that fails to load. When you change language multiple times than
                    // the err function can be null because i18next hasn't tried to download the missing language (to due to the previous
                    // attempts). So the elaborateI18nextInitialization can't remove them.
                    if (!this._discardedLanguage[language]) {
                        availableLanguages.push(language);
                    }
                }
            }
            return availableLanguages;
        };
        TranslatorService.prototype.printAvailableLanguages = function () {
            if (validator_4.Validator.isDefined(this._availableLanguages) && this._availableLanguages.length > 0) {
                this._log("Available languages: " + this._availableLanguages.join(" "));
            }
        };
        /**
         * Set params of translator service.
         */
        TranslatorService.prototype.setTranslatorParams = function (t) {
            TranslatorService._setTranslationRecursionCounter++;
            return new Promise(function (resolve, reject) {
                var _this = this;
                if (!this._availableLanguages) {
                    reject("TranslatorService setTranslatorParams(): available languages are not defined");
                }
                else {
                    var availableLanguagesCount = this._availableLanguages.length;
                    if (availableLanguagesCount > 0) {
                        if (!t) {
                            reject("TranslatorService setTranslatorParams(): t is not defined");
                        }
                        // Set param
                        this._t = t;
                        this._hasMultipleLanguages = this._availableLanguages.length > 1;
                        // Find index of detected language
                        var detectedLanguage = i18next.language;
                        for (var i = 0; i < this._availableLanguages.length; i++) {
                            if (this._availableLanguages[i] === detectedLanguage) {
                                this._currentLanguageIndex = i;
                                break;
                            }
                        }
                        if (typeof this._currentLanguageIndex === "undefined") {
                            this._log("The '" + detectedLanguage + "' is the desiderable language but it wasn't found among the available languages. Try to set the '"
                                + this._availableLanguages[0] + "' language");
                            // I don't find a valid language. I set the first language
                            i18next.changeLanguage(this._availableLanguages[0], function (err, t) {
                                var i18nextInitialization = _this.elaborateI18nextInitialization(err, t);
                                if (i18nextInitialization.Result === true) {
                                    if (TranslatorService._setTranslationRecursionCounter > 10) {
                                        // Something wrong
                                        _this._error("SetTranslatorParam exceeds the recursion threshold");
                                        reject();
                                    }
                                    else {
                                        _this.setTranslatorParams(t).then(function () {
                                            resolve();
                                        });
                                        resolve();
                                    }
                                }
                                else {
                                    reject(i18nextInitialization.Details);
                                }
                            });
                            // Promise not resolved... wait the changeLanguage callback
                        }
                        else {
                            // I find the language
                            this._currentLanguage(detectedLanguage);
                            resolve();
                        }
                    }
                    else {
                        reject("TranslatorService setTranslatorParams(): no params to set because no languages are available");
                    }
                }
            }.bind(this));
        };
        /**
         * Switch to next language.
         */
        TranslatorService.prototype.changeLanguage = function (language) {
            TranslatorService._setTranslationRecursionCounter = 0;
            return new Promise(function (resolve, reject) {
                var _this = this;
                if (this._hasMultipleLanguages) {
                    var index = void 0;
                    if (language) {
                        for (var i = 0; i < this._availableLanguages.length; i++) {
                            var lng = this._availableLanguages[i];
                            if (lng === language) {
                                index = i;
                                this._currentLanguageIndex = i;
                            }
                        }
                    }
                    else {
                        // Get the index of next language
                        index = ++this._currentLanguageIndex;
                        if (index >= this._availableLanguages.length) {
                            index = 0;
                        }
                    }
                    // Change language
                    i18next.changeLanguage(this._availableLanguages[index], function (err, t) {
                        var i18nextInitialization = _this.elaborateI18nextInitialization(err, t);
                        if (i18nextInitialization.Result === true) {
                            if (TranslatorService._setTranslationRecursionCounter > 10) {
                                // Something wrong
                                _this._error("SetTranslatorParam exceeds the recursion threshold");
                                reject();
                            }
                            else {
                                _this.setTranslatorParams(t).then(function () {
                                    _this._onChangeLanguageObservable(TranslatorEvent.LanguageChanged);
                                    resolve();
                                });
                            }
                        }
                        else {
                            reject(i18nextInitialization.Details);
                        }
                    });
                }
                else {
                    reject("TranslatorService changeLanguage(): no multi-language support");
                }
            }.bind(this));
        };
        TranslatorService.prototype.registerBootstrapLocales = function () {
            for (var _i = 0, _a = this._availableLanguages; _i < _a.length; _i++) {
                var language = _a[_i];
                $.fn.bootstrapTable.locales[language] = {
                    formatLoadingMessage: function () {
                        return this._t(TranslatorService.CfieldTranslationNs + "loadingInProgress");
                    }.bind(this),
                    formatRecordsPerPage: function (pageNumber) {
                        return string_util_1.StringUtil.sprintf(this._t(TranslatorService.CfieldTranslationNs + "bootstrapTableRowPerPage"), pageNumber);
                    }.bind(this),
                    formatShowingRows: function (pageFrom, pageTo, totalRows) {
                        return string_util_1.StringUtil.sprintf(this._t(TranslatorService.CfieldTranslationNs + "bootstrapTableShowingPageToOf"), pageFrom, pageTo, totalRows);
                    }.bind(this),
                    formatSearch: function () {
                        return this._t(TranslatorService.CfieldTranslationNs + "search");
                    }.bind(this),
                    formatNoMatches: function () {
                        return this._t(TranslatorService.CfieldTranslationNs + "noMatchingRecordFound");
                    }.bind(this),
                    formatPaginationSwitch: function () {
                        return this._t(TranslatorService.CfieldTranslationNs + "bootstrapTablePaginationSwitch");
                    }.bind(this),
                    formatRefresh: function () {
                        return this._t(TranslatorService.CfieldTranslationNs + "refresh");
                    }.bind(this),
                    formatToggle: function () {
                        return this._t(TranslatorService.CfieldTranslationNs + "toogle");
                    }.bind(this),
                    formatColumns: function () {
                        return this._t(TranslatorService.CfieldTranslationNs + "columns");
                    }.bind(this),
                    formatAllRows: function () {
                        return this._t(TranslatorService.CfieldTranslationNs + "all");
                    }.bind(this),
                    formatExport: function () {
                        return this._t(TranslatorService.CfieldTranslationNs + "exportData");
                    }.bind(this),
                    formatClearFilters: function () {
                        return this._t(TranslatorService.CfieldTranslationNs + "clearFilters");
                    }.bind(this)
                };
            }
        };
        /** Filename for cField traslations. */
        TranslatorService.cfieldBaseTranslationFilename = "cfield.base";
        TranslatorService.CfieldTranslationNs = TranslatorService.cfieldBaseTranslationFilename + ":";
        /** Filename for Io channel type translations. */
        TranslatorService.ioChannelTypeTranslationFilename = "channelTypes";
        TranslatorService.IoChannelTypeTranslationNs = TranslatorService.ioChannelTypeTranslationFilename + ":";
        /** Filename for variable translations. */
        TranslatorService.variableTranslationFilename = "variable";
        TranslatorService.VariableTranslationNs = TranslatorService.variableTranslationFilename + ":";
        /** Filename for category translations. */
        TranslatorService.categoryTranslationFilename = "category";
        TranslatorService.CategoryTranslationNs = TranslatorService.categoryTranslationFilename + ":";
        /** Translation file of namespace (to load). */
        TranslatorService.Namespaces = [
            TranslatorService.cfieldBaseTranslationFilename,
            TranslatorService.ioChannelTypeTranslationFilename,
            TranslatorService.variableTranslationFilename,
            TranslatorService.categoryTranslationFilename,
        ];
        /** Default fallback language. */
        TranslatorService.defaultFallbackLanguage = "en-US";
        /** Singleton instance */
        TranslatorService._instance = null;
        /** Counter on recursion */
        TranslatorService._setTranslationRecursionCounter = 0;
        return TranslatorService;
    }(base_service_2.BaseService));
    exports.TranslatorService = TranslatorService;
    var I18nextInitializationResult = /** @class */ (function () {
        function I18nextInitializationResult(_result, _details) {
            this._result = _result;
            this._details = _details;
        }
        Object.defineProperty(I18nextInitializationResult.prototype, "Result", {
            get: function () { return this._result; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(I18nextInitializationResult.prototype, "Details", {
            get: function () { return this._details; },
            enumerable: true,
            configurable: true
        });
        return I18nextInitializationResult;
    }());
    /**
     * State of translator service.
     */
    var TranslatorServiceState;
    (function (TranslatorServiceState) {
        TranslatorServiceState[TranslatorServiceState["NotInitialized"] = 0] = "NotInitialized";
        TranslatorServiceState[TranslatorServiceState["InitializationOnProgress"] = 1] = "InitializationOnProgress";
        TranslatorServiceState[TranslatorServiceState["Initialized"] = 2] = "Initialized";
        TranslatorServiceState[TranslatorServiceState["LanguageChanged"] = 3] = "LanguageChanged";
        TranslatorServiceState[TranslatorServiceState["Error"] = 4] = "Error";
    })(TranslatorServiceState = exports.TranslatorServiceState || (exports.TranslatorServiceState = {}));
    /**
     * State of translator service.
     */
    var TranslatorEvent;
    (function (TranslatorEvent) {
        TranslatorEvent[TranslatorEvent["LanguageChanged"] = 0] = "LanguageChanged";
    })(TranslatorEvent = exports.TranslatorEvent || (exports.TranslatorEvent = {}));
});
define("common/converter", ["require", "exports", "common/validator", "dataModel/datatype.model"], function (require, exports, validator_5, datatype_model_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var Converter = /** @class */ (function () {
        function Converter() {
        }
        Converter.BoolToIntText = function (value) {
            return value == true ? "1" : "0";
        };
        Converter.BoolToBoolText = function (value) {
            return value == true ? "TRUE" : "FALSE";
        };
        Converter.BoolTextToBool = function (value) {
            return value === "TRUE";
        };
        Converter.IntTextToBool = function (value) {
            return value === "1";
        };
        Converter.StringToFixedFloatString = function (value, numberOfDigits) {
            var floatNumber = parseFloat(value);
            if (!isNaN(floatNumber)) {
                return Number(floatNumber.toFixed(numberOfDigits)).toString();
            }
            return null;
        };
        Converter.NumberToFixedFloat = function (value, numberOfDigits) {
            if (validator_5.Validator.isDefined(value) && !isNaN(value)) {
                return Number(value.toFixed(numberOfDigits));
            }
            return null;
        };
        Converter.ConvertValueInAccordWithDataType = function (dataType, value, isBoolWriteWithOneAndZero) {
            if (dataType == datatype_model_1.DataTypeEnum.Bool) {
                if (!validator_5.Validator.isDefined(isBoolWriteWithOneAndZero)) {
                    return Converter.BoolTextToBool(value);
                }
                else {
                    return Converter.IntTextToBool(value);
                }
            }
            else if (datatype_model_1.DataTypeUtil.IsNumericDataType(dataType)) {
                var number = Number(value);
                if (validator_5.Validator.isDefined(number) && !isNaN(number)) {
                    return number.valueOf();
                }
            }
            else {
                return value;
            }
        };
        return Converter;
    }());
    exports.Converter = Converter;
});
define("service/import-export.service", ["require", "exports", "service/base.service", "dataModel/datatype.model", "common/converter", "common/validator"], function (require, exports, base_service_3, datatype_model_2, converter_1, validator_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ImportExportService = /** @class */ (function (_super) {
        __extends(ImportExportService, _super);
        function ImportExportService() {
            var _this = _super.call(this) || this;
            if (ImportExportService._instance) {
                _this._error(base_service_3.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            return _this;
        }
        ImportExportService.getInstance = function () {
            if (ImportExportService._instance === null) {
                ImportExportService._instance = new ImportExportService();
            }
            return ImportExportService._instance;
        };
        ImportExportService.prototype.parseDevParamsFile = function (file) {
            return new Promise(function (resolve, reject) {
                if (file) {
                    if (file.type.match("text.*")) {
                        var fileReader_1 = new FileReader();
                        fileReader_1.onload = function (event) {
                            var lineSeparator = "\r\n";
                            // Take result
                            var data = fileReader_1.result;
                            var result = this.parseDevParamText(data);
                            if (result.IsSuccess == true) {
                                resolve(result.NameToDevParamMap);
                                return;
                            }
                            else {
                                reject(result.Error);
                                return;
                            }
                        }.bind(this);
                        fileReader_1.readAsText(file);
                    }
                    else {
                        reject("Extension mismatch (filename= " + file.name + ",  extension type= " + file.type);
                    }
                }
                else {
                    reject("Parse dev params: file is null");
                }
            }.bind(this));
        };
        ImportExportService.prototype.parseDevParamText = function (data) {
            var recordArray = data.split(ImportExportService.LineSeparator);
            if (recordArray && recordArray.length > 2) {
                // First line
                var record = recordArray[0];
                if (record.charAt(0) === "#") {
                    this._log("Read params file with this header: " + record);
                }
                else {
                    return new DevDataParseResult(false, null, "Parse dev params: first row doesn't start with #");
                }
                // Second line
                record = recordArray[1];
                if (record.charAt(0) === "#") {
                    record = record.substr(1);
                }
                else {
                    return new DevDataParseResult(false, null, "Parse dev params: second row doesn't start with #");
                }
                // Build string and then parse
                var newData = record;
                var csvData = recordArray.slice(2);
                newData = newData + ImportExportService.LineSeparator + csvData.join(ImportExportService.LineSeparator);
                var devParamsArray = $.csv.toObjects(newData, { separator: "\t" });
                if (devParamsArray) {
                    // Convert to map
                    var nameToDevParamMap = {};
                    for (var _i = 0, devParamsArray_1 = devParamsArray; _i < devParamsArray_1.length; _i++) {
                        var devParam = devParamsArray_1[_i];
                        nameToDevParamMap[devParam.Variable] = devParam;
                    }
                    return new DevDataParseResult(true, nameToDevParamMap, null);
                }
                else {
                    return new DevDataParseResult(false, null, "Parse dev params: I don't find records");
                }
            }
            else {
                return new DevDataParseResult(false, null, "Parse dev params: # records > 2 is false");
            }
        };
        ImportExportService.prototype.writeDevParamText = function (devParamArray, writeDescription) {
            if (devParamArray && devParamArray.length > 0) {
                var exportTextArray = [];
                // Write header
                exportTextArray.push(ImportExportService.HeaderFirstLine + ImportExportService.LineSeparator);
                exportTextArray.push(ImportExportService.CommentChar +
                    ImportExportService.HeaderSecondLineVariable + ImportExportService.TabSeparator +
                    ImportExportService.HeaderSecondLineDescription + ImportExportService.TabSeparator +
                    ImportExportService.HeaderSecondLineDataType + ImportExportService.TabSeparator +
                    ImportExportService.HeaderSecondLineDefaultValue + ImportExportService.LineSeparator);
                // Write body
                for (var _i = 0, devParamArray_1 = devParamArray; _i < devParamArray_1.length; _i++) {
                    var devItem = devParamArray_1[_i];
                    var description = writeDescription ? this.getValueOrEmpty(devItem.Description, devItem.DataTypeRaw.Type) : "";
                    var valueToSet = this.getValueOrEmpty(devItem.DefaultValue, devItem.DataTypeRaw.Type);
                    exportTextArray.push(devItem.Variable + ImportExportService.TabSeparator +
                        "" + ImportExportService.TabSeparator +
                        devItem.DataType + ImportExportService.TabSeparator +
                        valueToSet + ImportExportService.LineSeparator);
                }
                return exportTextArray.join("");
            }
            return null;
        };
        ImportExportService.prototype.getValueOrEmpty = function (value, dataType) {
            var newValue;
            if (validator_6.Validator.isDefined(value)) {
                if (dataType === datatype_model_2.DataTypeEnum.Real) {
                    newValue = converter_1.Converter.StringToFixedFloatString(value, 6);
                }
                else {
                    newValue = value;
                }
            }
            else {
                newValue = "";
            }
            return newValue;
        };
        /** Singleton instance. */
        ImportExportService._instance = null;
        ImportExportService.LineSeparator = "\r\n";
        ImportExportService.TabSeparator = "\t";
        ImportExportService.CommentChar = "#";
        ImportExportService.HeaderFirstLine = "#Version 1.1 DevConfiguration (c.Field)";
        ImportExportService.HeaderSecondLineVariable = "Variable";
        ImportExportService.HeaderSecondLineDescription = "Description";
        ImportExportService.HeaderSecondLineDataType = "DataType";
        ImportExportService.HeaderSecondLineDefaultValue = "DefaultValue";
        return ImportExportService;
    }(base_service_3.BaseService));
    exports.ImportExportService = ImportExportService;
    var DevDataParseResult = /** @class */ (function () {
        function DevDataParseResult(_isSuccess, _nameToDevParamMap, _error) {
            this._isSuccess = _isSuccess;
            this._nameToDevParamMap = _nameToDevParamMap;
            this._error = _error;
        }
        Object.defineProperty(DevDataParseResult.prototype, "IsSuccess", {
            get: function () { return this._isSuccess; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DevDataParseResult.prototype, "NameToDevParamMap", {
            get: function () { return this._nameToDevParamMap; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DevDataParseResult.prototype, "Error", {
            get: function () { return this._error; },
            enumerable: true,
            configurable: true
        });
        return DevDataParseResult;
    }());
    exports.DevDataParseResult = DevDataParseResult;
    var DevParamToSet = /** @class */ (function () {
        function DevParamToSet(_variableName, _description, _dataType, _defaultValue) {
            this._variableName = _variableName;
            this._description = _description;
            this._dataType = _dataType;
            this._defaultValue = _defaultValue;
        }
        Object.defineProperty(DevParamToSet.prototype, "Variable", {
            get: function () { return this._variableName; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(DevParamToSet.prototype, "Description", {
            get: function () { return this._description; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(DevParamToSet.prototype, "DataType", {
            get: function () { return this._dataType.DisplayName; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(DevParamToSet.prototype, "DataTypeRaw", {
            get: function () { return this._dataType; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(DevParamToSet.prototype, "DefaultValue", {
            get: function () { return this._defaultValue; },
            enumerable: true,
            configurable: true
        });
        ;
        return DevParamToSet;
    }());
    exports.DevParamToSet = DevParamToSet;
});
define("dataModel/variable.model", ["require", "exports", "common/converter"], function (require, exports, converter_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var DevParamContainer = /** @class */ (function () {
        function DevParamContainer(_variable, _devParam) {
            this._variable = _variable;
            this._devParam = _devParam;
            this._devParamValue = converter_2.Converter.ConvertValueInAccordWithDataType(_variable._dataType, _devParam.DefaultValue);
        }
        Object.defineProperty(DevParamContainer.prototype, "Variable", {
            get: function () { return this._variable; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DevParamContainer.prototype, "DevParam", {
            get: function () { return this._devParam; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DevParamContainer.prototype, "DevParamValue", {
            get: function () { return this._devParamValue; },
            enumerable: true,
            configurable: true
        });
        return DevParamContainer;
    }());
    exports.DevParamContainer = DevParamContainer;
});
define("service/variable.service", ["require", "exports", "service/cfield.service", "service/device-config.service", "service/base.service", "service/import-export.service", "service/uom.service", "common/validator", "common/data-structure", "common/converter", "dataModel/variable.model"], function (require, exports, cField_service_1, device_config_service_1, base_service_4, import_export_service_1, uom_service_1, validator_7, data_structure_2, converter_3, variable_model_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var VariableService = /** @class */ (function (_super) {
        __extends(VariableService, _super);
        /**
         * Init service.
         */
        function VariableService() {
            var _this = _super.call(this) || this;
            if (VariableService._instance) {
                _this._error(base_service_4.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            _this._deviceEngine = cField_service_1.cFieldService.getInstance().DeviceEngine;
            return _this;
        }
        /**
         * Get the instance of variable service in accord with singleton pattern.
         * @return an instance of service.
         */
        VariableService.getInstance = function () {
            if (VariableService._instance === null) {
                VariableService._instance = new VariableService();
            }
            return VariableService._instance;
        };
        /**
         * @param id Array of ids to query
         */
        VariableService.prototype.getVar = function (idArray, skipSupUomRefresh) {
            var _this = this;
            return new Promise(function (resolve, reject) {
                var uomService = uom_service_1.UoMService.getInstance();
                var uomSupId;
                var canIRemoveSupId = false;
                if (idArray.length != 0 && !skipSupUomRefresh) {
                    // If sup uom zone is linked to variable than update it
                    if (uomService.IsSupZoneLinkToVariable) {
                        uomSupId = uomService.SupZoneLinkedVariableId;
                        if (validator_7.Validator.isDefined(uomSupId)) {
                            var isUomSupIdAlreadyIncluded = false;
                            // Check if uomSupId is already included in the id to updates         
                            for (var _i = 0, idArray_1 = idArray; _i < idArray_1.length; _i++) {
                                var id = idArray_1[_i];
                                if (id === uomSupId) {
                                    isUomSupIdAlreadyIncluded = true;
                                    break;
                                }
                            }
                            if (!isUomSupIdAlreadyIncluded) {
                                idArray.push(uomSupId);
                                canIRemoveSupId = true;
                            }
                        }
                    }
                }
                _this.tryGetVariableById(idArray, uomService, canIRemoveSupId, resolve, reject);
            });
        };
        VariableService.prototype.tryGetVariableById = function (idArray, uomService, canIRemoveSupId, resolve, reject) {
            var functionCallResult = this.getVariableById(idArray, uomService, canIRemoveSupId, resolve, reject);
            if (functionCallResult === false) {
                // DeviceService is busy. Retry after a timeout
                setTimeout(function () {
                    this.tryGetVariableById(idArray, uomService, canIRemoveSupId, resolve, reject);
                }.bind(this), VariableService._defaultRetryIfBusyTimeoutMs);
            }
        };
        // If deviceEngine.js is busy then it return false
        VariableService.prototype.getVariableById = function (idArray, uomService, canIRemoveSupId, resolve, reject) {
            return this._deviceEngine.getVariablesById(idArray, function (result, loadedData) {
                if (result === true) {
                    var isSuccess = this.updateVariableValues(loadedData, false);
                    // Update sup uom id if needed
                    if (isSuccess && validator_7.Validator.isDefined(loadedData) && uomService.IsSupZoneLinkToVariable) {
                        var n = loadedData.length;
                        var i = 0;
                        var uomSupId = uomService.SupZoneLinkedVariableId;
                        while (i < n) {
                            if (loadedData[i].getId() == uomSupId) {
                                // Update value
                                uom_service_1.UoMService.getInstance().SupZoneId = loadedData[i].getValue();
                                break;
                            }
                            i++;
                        }
                        if (i < n && canIRemoveSupId) {
                            // Remove from response
                            loadedData.splice(i, 1);
                        }
                    }
                    resolve(new GetSetVarResponse(loadedData, isSuccess));
                }
                else {
                    reject("VariableService. GetVar() is failed (idArray=" + idArray + ")");
                }
            }.bind(this));
        };
        /**
         * Send a getVarAll CGI
         */
        VariableService.prototype.getAllVar = function () {
            return this.getVar([]);
        };
        VariableService.prototype.getVarEfficient = function (idArray) {
            // Get CGI
            if (idArray.length > 150) {
                return this.getAllVar();
            }
            else {
                return this.getVar(idArray);
            }
        };
        /**
         * Send a setVar CGI.
         * @param variableArray Varaibles to set.
         * @param newValueArray Value of variable to set (position base corrispondence).
         */
        VariableService.prototype.setVar = function (variableArray, newValueArray) {
            if (variableArray.length && newValueArray.length && variableArray.length === newValueArray.length) {
                return new Promise(function (resolve, reject) {
                    // Build parameters array
                    var requestParams = [];
                    for (var i = 0; i < variableArray.length; i++) {
                        var variable = variableArray[i];
                        var newValue = newValueArray[i];
                        requestParams.push(variable._id + '=' + newValue);
                    }
                    this._deviceEngine.performAsyncJobWrap("/setvar.csv", requestParams, function (result, loadedData) {
                        if (result === true) {
                            var isSuccess = this.updateVariableValues(loadedData, true);
                            resolve(new GetSetVarResponse(loadedData, isSuccess));
                        }
                        else {
                            reject("VariableService. SetVar() is failed");
                        }
                    }.bind(this));
                }.bind(this));
            }
            else {
                return Promise.resolve(new GetSetVarResponse([], false));
            }
        };
        VariableService.prototype.getParameters = function (inputPromise) {
            var _this = this;
            var promise = validator_7.Validator.isDefined(inputPromise) ? inputPromise : VariableService.getInstance().getDevParams();
            return promise
                .then(function (devParamMap) {
                var devParamContainerArray = _this.combineParametersWithVariable(devParamMap);
                return Promise.resolve(devParamContainerArray);
            });
        };
        VariableService.prototype.combineParametersWithVariable = function (devParamMap) {
            var devParamContainerArray = [];
            // Extract variable associated to dev
            var mainVariableDictionary = device_config_service_1.DeviceConfigService.getInstance().VariablesDictionary;
            for (var id in mainVariableDictionary) {
                var variable = mainVariableDictionary[id];
                if (variable._familyType === device_config_service_1.DeviceConfigService.VarTypeVariable && variable._flags && variable._flags._isDev) {
                    // Search name
                    var devParam = devParamMap[variable._name];
                    if (devParam) {
                        // Save
                        devParamContainerArray.push(new variable_model_1.DevParamContainer(variable, devParam));
                    }
                }
            }
            return devParamContainerArray;
        };
        /**
         * Get initial values (dev) using CGI.
         */
        VariableService.prototype.getDevParams = function () {
            var cgiPromise = new Promise(function (resolve, reject) {
                $.ajax({
                    url: "/get_isai_params.cgi",
                    success: function (data, textStatus, jqXHR) {
                        resolve(jqXHR.responseText);
                    },
                    error: function (jqXHR, textStatus, errorThrown) {
                        reject("get_isai_params.cgi error: " + textStatus + "  " + errorThrown);
                    },
                    cache: false,
                    dataType: "text",
                    timeout: 60000
                });
            })
                .then(function (body) {
                // Check error
                try {
                    var resultJson = JSON.parse(body);
                    if (resultJson && resultJson.hasOwnProperty("result")) {
                        var errorCode = resultJson.result;
                        return Promise.reject(errorCode);
                    }
                }
                catch (Error) {
                    // Body isn't json than server doesn't reply with an error code
                }
                // Try parse body
                var result = import_export_service_1.ImportExportService.getInstance().parseDevParamText(body);
                if (result.IsSuccess == true) {
                    return Promise.resolve(result.NameToDevParamMap);
                }
                else {
                    return Promise.reject(result.Error);
                }
            });
            return cgiPromise;
        };
        /**
         * Set initial values (dev) using CGI.
         */
        VariableService.prototype.setDevVars = function (text) {
            return new Promise(function (resolve, reject) {
                // Build file
                //let file = new File([text], "DevParams.txt", {type: "text/plain;charset=utf-8"});
                var file = new Blob([text], { type: "text/plain;charset=utf-8" });
                // Create form
                var form = new FormData();
                form.append("data", file, "DevParams.txt");
                // Send CGI
                $.ajax({
                    type: "POST",
                    url: "/set_isai_params.cgi",
                    enctype: 'multipart/form-data',
                    cache: false,
                    timeout: 120000,
                    contentType: false,
                    processData: false,
                    data: form,
                    success: function (data, textStatus, jqXHR) {
                        resolve(data);
                    },
                    error: function (jqXHR, textStatus, errorThrown) {
                        this._log("Error: " + textStatus + "  " + errorThrown);
                        reject("Error during the set of CGI");
                    }
                });
            }.bind(this));
        };
        /**
         * Scan all variables, extract the min/max limits linked to variable and update their values using getVar CGI.
         */
        VariableService.prototype.updateVariableLimits = function (onlyDev) {
            // Get ids of variable to update
            var ids = this.getIdsOfMinMaxVariables(onlyDev);
            if (ids.length == 0) {
                return Promise.resolve();
            }
            // Update
            return this.getVarEfficient(ids);
        };
        /**
         *
         * @param onlyDev If <code>true</code>, only dev variables are scanned.
         */
        VariableService.prototype.getIdsOfMinMaxVariables = function (onlyDev) {
            // So when the CGI updates the value of variable with id 1 then I must to check the variable 5, 6, 18 and update the link to variable that uses the id 1
            var mainVariableDictionary = device_config_service_1.DeviceConfigService.getInstance().VariablesDictionary;
            var set = new data_structure_2.StringSet();
            for (var id in mainVariableDictionary) {
                var variable = mainVariableDictionary[id];
                // variable._flags can be not defined
                if (onlyDev || (variable._familyType === device_config_service_1.DeviceConfigService.VarTypeVariable && variable._flags && variable._flags._isDev)) {
                    // Init and Dev cannot be link to variable
                    set.addNoConflict(variable._minId);
                    set.addNoConflict(variable._maxId);
                }
            }
            return set.getValues();
        };
        /**
         * Update the value of variable if the response of setVar CGI is OK.
         * @param variables
         * @param checkStatus If <code>true</code> than check the status of reply for each
         * @return <code>True</code> if success.
         */
        VariableService.prototype.updateVariableValues = function (variables, checkStatus) {
            var isSuccess = true;
            if (variables && variables.length > 0) {
                // Update _currentValue of variables
                var mainVariableDictionary = device_config_service_1.DeviceConfigService.getInstance().VariablesDictionary;
                for (var _i = 0, variables_1 = variables; _i < variables_1.length; _i++) {
                    var variable = variables_1[_i];
                    if (!checkStatus || variable.getStatus() === "OK") {
                        var id = variable.getId();
                        var variableToUpdate = mainVariableDictionary[id];
                        if (validator_7.Validator.isDefined(variableToUpdate)) {
                            variableToUpdate._currentValue = converter_3.Converter.ConvertValueInAccordWithDataType(variableToUpdate._dataType, variable.getValue(), true);
                        }
                        else {
                            this._log("VariableService updateVariableValues(). The variable with id " + id + " can be update because it isn't been found on variable dictionary of cField");
                            isSuccess = false;
                        }
                    }
                }
            }
            return isSuccess;
        };
        VariableService._defaultRetryIfBusyTimeoutMs = 250;
        /** Singleton instance. */
        VariableService._instance = null;
        return VariableService;
    }(base_service_4.BaseService));
    exports.VariableService = VariableService;
    var GetSetVarResponse = /** @class */ (function () {
        function GetSetVarResponse(_deviceVariableArray, _isSuccess) {
            this._deviceVariableArray = _deviceVariableArray;
            this._isSuccess = _isSuccess;
        }
        return GetSetVarResponse;
    }());
    exports.GetSetVarResponse = GetSetVarResponse;
});
define("service/uom.service", ["require", "exports", "knockout", "service/base.service", "service/device-config.service", "service/variable.service", "dataModel/datatype.model", "common/validator", "common/data-structure"], function (require, exports, ko, base_service_5, device_config_service_2, variable_service_1, datatype_model_3, validator_8, data_structure_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var UoMServiceState;
    (function (UoMServiceState) {
        UoMServiceState[UoMServiceState["UnInit"] = 0] = "UnInit";
        /** UoM is always the empty string. */
        UoMServiceState[UoMServiceState["InitCompleteNonActive"] = 1] = "InitCompleteNonActive";
        /** UoM is extracted in accord with zone. */
        UoMServiceState[UoMServiceState["InitCompleteActive"] = 2] = "InitCompleteActive";
    })(UoMServiceState = exports.UoMServiceState || (exports.UoMServiceState = {}));
    var UoMService = /** @class */ (function (_super) {
        __extends(UoMService, _super);
        /**
         * Init service.
         */
        function UoMService(uomConfiguration, uomZoneResultAlarm, uomZoneResultLog, uomZoneResultSup) {
            var _this = _super.call(this) || this;
            // Id of UoM zone
            _this._alarmZoneId = null;
            _this._logZoneId = null;
            _this._supZoneId = null;
            if (UoMService._instance) {
                _this._error(base_service_5.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            if ((uomConfiguration && uomZoneResultAlarm && uomZoneResultLog && uomZoneResultSup)) {
                _this._uoMConfiguration = uomConfiguration;
                _this._uomZoneResultAlarm = uomZoneResultAlarm;
                _this._uomZoneResultLog = uomZoneResultLog;
                _this._uomZoneResultSup = uomZoneResultSup;
                _this._uomServiceStateObservable = ko.observable(UoMServiceState.UnInit);
            }
            else {
                _this._uomServiceStateObservable = ko.observable(UoMServiceState.InitCompleteNonActive);
            }
            return _this;
        }
        Object.defineProperty(UoMService.prototype, "UomServiceStateObservable", {
            /** Get the observable of uom service state. */
            get: function () {
                return this._uomServiceStateObservable;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UoMService.prototype, "SupZoneId", {
            set: function (value) { this._supZoneId = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UoMService.prototype, "IsAlarmZoneLinkToVariable", {
            get: function () {
                return this._uomZoneResultAlarm && this._uomZoneResultAlarm._isSuccessfull && this._uomZoneResultAlarm._data._linkToVariable;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UoMService.prototype, "AlarmZoneLinkedVariableId", {
            get: function () {
                if (this._uomZoneResultAlarm && this._uomZoneResultAlarm._data) {
                    return this._uomZoneResultAlarm._data._id;
                }
                else {
                    return null;
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UoMService.prototype, "IsLogZoneLinkToVariable", {
            get: function () {
                return this._uomZoneResultLog && this._uomZoneResultLog._isSuccessfull && this._uomZoneResultLog._data._linkToVariable;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UoMService.prototype, "IsSupZoneLinkToVariable", {
            get: function () {
                return this._uomZoneResultSup && this._uomZoneResultSup._isSuccessfull && this._uomZoneResultSup._data._linkToVariable;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UoMService.prototype, "SupZoneLinkedVariableId", {
            get: function () {
                if (this._uomZoneResultSup && this._uomZoneResultSup._data) {
                    return this._uomZoneResultSup._data._id;
                }
                else {
                    return null;
                }
            },
            enumerable: true,
            configurable: true
        });
        UoMService.buildInstance = function (uomConfiguration, uomZoneResultAlarm, uomZoneResultLog, uomZoneResultSup) {
            if (UoMService._instance === null) {
                UoMService._instance = new UoMService(uomConfiguration, uomZoneResultAlarm, uomZoneResultLog, uomZoneResultSup);
            }
        };
        UoMService.getInstance = function () {
            if (UoMService._instance !== null) {
                return UoMService._instance;
            }
            UoMService._instance._error(base_service_5.BaseService._singletonConstructorErrorMsg);
        };
        UoMService.prototype.init = function () {
            var _this = this;
            return this.getUoMZone(this._uomZoneResultAlarm, this._uomZoneResultLog, this._uomZoneResultSup)
                .then(function () {
                _this._uomServiceStateObservable(UoMServiceState.InitCompleteActive);
            })
                .catch(function (reason) {
                _this._log("The initializaion of the zone for alarm, log and sup is failed");
                _this._uomServiceStateObservable(UoMServiceState.InitCompleteNonActive);
            });
        };
        UoMService.prototype.getUoMZone = function (uomZoneResultAlarm, uomZoneResultLog, uomZoneResultSup) {
            // Map where key is the id of uom linked to variable while the array contains the uom type (alarm, log, sup) associated to
            // this id
            var idToUomZoneTypeMap = {};
            // Alarm      
            if (uomZoneResultAlarm && uomZoneResultAlarm._isSuccessfull && uomZoneResultAlarm._data) {
                if (uomZoneResultAlarm._data._linkToVariable === false) {
                    this._alarmZoneId = uomZoneResultAlarm._data._uoMZoneId;
                }
                else {
                    data_structure_3.MapUtil.AddMapKeyToArrayOfValue(idToUomZoneTypeMap, uomZoneResultAlarm._data._id, UomTarget.Alarm);
                }
            }
            else {
                Promise.reject("uomZoneResultAlarm is not valid");
            }
            // Log
            if (uomZoneResultLog && uomZoneResultLog._isSuccessfull && uomZoneResultLog._data) {
                if (uomZoneResultLog._data._linkToVariable === false) {
                    this._logZoneId = uomZoneResultLog._data._uoMZoneId;
                }
                else {
                    data_structure_3.MapUtil.AddMapKeyToArrayOfValue(idToUomZoneTypeMap, uomZoneResultLog._data._id, UomTarget.Log);
                }
            }
            else {
                Promise.reject("uomZoneResultLog is not valid");
            }
            // Sup
            if (uomZoneResultSup && uomZoneResultSup._isSuccessfull && uomZoneResultSup._data) {
                if (uomZoneResultSup._data._linkToVariable === false) {
                    this._supZoneId = uomZoneResultSup._data._uoMZoneId;
                }
                else {
                    data_structure_3.MapUtil.AddMapKeyToArrayOfValue(idToUomZoneTypeMap, uomZoneResultSup._data._id, UomTarget.Sup);
                }
            }
            else {
                Promise.reject("uomZoneResultSup is not valid");
            }
            return this.updateZoneId(idToUomZoneTypeMap);
        };
        /**
         *
         * @param varIdArray Id to query
         * @param varNameArray Identifier of uom (alarm, log or sup)
         */
        UoMService.prototype.updateZoneId = function (idToUomZoneTypeMap) {
            var _this = this;
            // Get id to update
            var varIdToUpdate = [];
            for (var id in idToUomZoneTypeMap) {
                varIdToUpdate.push(id);
            }
            // Get var if needed
            if (varIdToUpdate.length > 0) {
                return variable_service_1.VariableService.getInstance().getVar(varIdToUpdate, true)
                    .then(function (getSetVarResponse) {
                    var result = getSetVarResponse._deviceVariableArray;
                    if (result.length > 0) {
                        for (var _i = 0, result_1 = result; _i < result_1.length; _i++) {
                            var deviceVariable = result_1[_i];
                            var id = deviceVariable.getId();
                            if (validator_8.Validator.isDefined(id)) {
                                var uomTypeArray = idToUomZoneTypeMap[id];
                                if (validator_8.Validator.isDefined(uomTypeArray)) {
                                    for (var _a = 0, uomTypeArray_1 = uomTypeArray; _a < uomTypeArray_1.length; _a++) {
                                        var uomType = uomTypeArray_1[_a];
                                        switch (uomType) {
                                            case UomTarget.Alarm:
                                                _this._alarmZoneId = deviceVariable.getValue();
                                                break;
                                            case UomTarget.Log:
                                                _this._logZoneId = deviceVariable.getValue();
                                                break;
                                            case UomTarget.Sup:
                                                _this._supZoneId = deviceVariable.getValue();
                                                break;
                                        }
                                    }
                                }
                            }
                            else {
                                _this._log("Invalid id return by getVar during updateZoneId()");
                            }
                        }
                    }
                    // Check if all variable are been set
                    if (_this._alarmZoneId == null) {
                        _this._log("Alarm id UoM zone isn't set");
                    }
                    if (_this._logZoneId == null) {
                        _this._log("Log id UoM zone isn't set");
                    }
                    if (_this._supZoneId == null) {
                        _this._log("Sup id UoM zone isn't set");
                    }
                    return Promise.resolve();
                });
            }
            else {
                return Promise.resolve();
            }
        };
        UoMService.prototype.getAlarmUom = function (uomId) {
            return this.getUomConversionData(uomId, this._alarmZoneId);
        };
        UoMService.prototype.getLogUom = function (uomId) {
            return this.getUomConversionData(uomId, this._logZoneId);
        };
        UoMService.prototype.getSupUom = function (uomId) {
            return this.getUomConversionData(uomId, this._supZoneId);
        };
        UoMService.prototype.getNoZoneUom = function (uomId) {
            return this.getUomConversionData(uomId, UoMService.NoZoneId);
        };
        /**
         *
         * @param uomId Source uom id
         * @param dstZoneId Destination zone
         */
        UoMService.prototype.getUomConversionData = function (uomId, dstZoneId) {
            if (this._uomServiceStateObservable() === UoMServiceState.InitCompleteActive) {
                var uomSrcData = this._uoMConfiguration.UoM[uomId];
                if (validator_8.Validator.isDefined(uomSrcData)) {
                    var convZone = uomSrcData.uomToConvZone;
                    if (validator_8.Validator.isDefined(convZone) && convZone.hasOwnProperty(dstZoneId)) {
                        var uomConversionData = convZone[dstZoneId];
                        if (validator_8.Validator.isDefined(uomConversionData)) {
                            var uomDstId = uomConversionData.uomDstId;
                            if (validator_8.Validator.isDefined(uomDstId)) {
                                // Manage uom symbol
                                var symbol = void 0;
                                var uomDstData = this._uoMConfiguration.UoM[uomDstId];
                                if (validator_8.Validator.isDefined(uomDstData)) {
                                    symbol = uomDstData.desc;
                                }
                                // Manage coefficient
                                var addCoefficient = void 0;
                                var multCoefficient = void 0;
                                if (uomConversionData.hasConversionCoeff) {
                                    var uomDstIdMap = this._uoMConfiguration.UoMCoefficient[uomId];
                                    if (validator_8.Validator.isDefined(uomDstIdMap)) {
                                        var coefficient = uomDstIdMap[uomDstId];
                                        if (validator_8.Validator.isDefined(coefficient)) {
                                            addCoefficient = coefficient.add;
                                            multCoefficient = coefficient.mult;
                                        }
                                    }
                                }
                                return new datatype_model_3.UomData(symbol, uomDstId, addCoefficient, multCoefficient);
                            }
                        }
                        else {
                            return new datatype_model_3.UomData(this._uoMConfiguration.UoM[uomId].desc, uomId);
                        }
                    }
                    else {
                        return new datatype_model_3.UomData(this._uoMConfiguration.UoM[uomId].desc, uomId);
                    }
                }
                else {
                    this._log._error("UoM not found (new UoM?). Uom Id = ", uomId);
                }
            }
            return new datatype_model_3.UomData("", uomId);
        };
        UoMService.prototype.getUomConversionCoefficient = function (uomSrcId, uomDstId) {
            if (this._uomServiceStateObservable() === UoMServiceState.InitCompleteActive) {
                var uomSrcData = this._uoMConfiguration.UoMCoefficient[uomSrcId];
                if (validator_8.Validator.isDefined(uomSrcData)) {
                    var uomDstData = uomSrcData[uomDstId];
                    if (validator_8.Validator.isDefined(uomDstData) && validator_8.Validator.isDefined(uomDstData.add) && validator_8.Validator.isDefined(uomDstData.mult)) {
                        return new datatype_model_3.CoefficientData(uomDstData.add, uomDstData.mult);
                    }
                }
            }
            return null;
        };
        UoMService.prototype.updateAlarmZone = function () {
            return new Promise(function (resolve, reject) {
                var _this = this;
                if (this.IsAlarmZoneLinkToVariable) {
                    var id_1 = this.AlarmZoneLinkedVariableId;
                    if (validator_8.Validator.isDefined(id_1)) {
                        // Refresh the value
                        variable_service_1.VariableService.getInstance().getVar([id_1])
                            .then(function (response) {
                            var zoneIdVariable = device_config_service_2.DeviceConfigService.getInstance().VariablesDictionary[id_1];
                            if (validator_8.Validator.isDefined(zoneIdVariable)) {
                                _this._alarmZoneId = zoneIdVariable._currentValue;
                                resolve();
                            }
                            else {
                                reject("Variable with id=" + id_1 + " has not been found in the dictionary");
                            }
                        })
                            .catch(function (reason) {
                            reject(reason);
                        });
                    }
                    else {
                        reject("The zone of alarm is link to variable but the id of this variable is null");
                    }
                }
                else {
                    resolve();
                }
            }.bind(this));
        };
        UoMService.NoZoneId = "0";
        /** Singleton instance. */
        UoMService._instance = null;
        return UoMService;
    }(base_service_5.BaseService));
    exports.UoMService = UoMService;
    var UomTarget = /** @class */ (function () {
        function UomTarget() {
        }
        UomTarget.Default = "0";
        UomTarget.Alarm = "1";
        UomTarget.Log = "2";
        UomTarget.Sup = "3";
        return UomTarget;
    }());
    exports.UomTarget = UomTarget;
});
define("common/formatter", ["require", "exports", "dataModel/datatype.model", "common/converter", "common/validator"], function (require, exports, datatype_model_4, converter_4, validator_9) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var Formatter = /** @class */ (function () {
        function Formatter() {
        }
        Formatter.FormatNumber = function (value, datatypeEnum, numberOfDigitis) {
            if (!validator_9.Validator.isDefined(value)) {
                return null;
            }
            switch (datatypeEnum) {
                case datatype_model_4.DataTypeEnum.Lreal:
                case datatype_model_4.DataTypeEnum.Real:
                    return converter_4.Converter.NumberToFixedFloat(value, validator_9.Validator.isDefined(numberOfDigitis) ? numberOfDigitis : 2);
                default:
                    return value;
            }
        };
        return Formatter;
    }());
    exports.Formatter = Formatter;
});
define("component/provider/category", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("dataModel/account.model", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var Account = /** @class */ (function () {
        function Account(user, profile) {
            this._user = user;
            switch (profile) {
                case "admin":
                    this._profile = Profile.Admin;
                    break;
                default:
                    this._profile = Profile.User;
                    break;
            }
        }
        return Account;
    }());
    exports.Account = Account;
    var Profile;
    (function (Profile) {
        Profile[Profile["User"] = 0] = "User";
        Profile[Profile["Admin"] = 1] = "Admin";
    })(Profile || (Profile = {}));
});
define("service/login.service", ["require", "exports", "knockout", "service/cfield.service", "service/base.service", "dataModel/account.model"], function (require, exports, ko, cfield_service_2, base_service_6, account_model_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var UserLoginKey = "cFieldLoginUsername";
    var LoginService = /** @class */ (function (_super) {
        __extends(LoginService, _super);
        /**
         * Init service.
         * @param cFieldServiceInstance CField service.
         */
        function LoginService() {
            var _this = _super.call(this) || this;
            /** <code>True</code> if login is on progress. */
            _this._isLoginOnProgress = false;
            /** <code>True</code> if logout is on progress. */
            _this._isLogoutOnProgress = false;
            if (LoginService._instance) {
                _this._error(base_service_6.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            var baseUrl = cfield_service_2.cFieldService.getInstance().BaseUrl;
            LoginService.LoginCgiUrl = baseUrl + "/assets/cgi/mock_login.json";
            LoginService.LogoutCgiUrl = baseUrl + "/assets/cgi_logout/no_delete.html";
            _this._isLoginEnabled = ko.observable(false);
            _this._isLoginRequired = ko.observable(true);
            return _this;
        }
        Object.defineProperty(LoginService.prototype, "IsLoginEnabled", {
            get: function () { return this._isLoginEnabled; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LoginService.prototype, "IsLoginRequired", {
            get: function () { return this._isLoginRequired; },
            enumerable: true,
            configurable: true
        });
        /**
         * Get instance of login service in accord with singleton pattern.
         */
        LoginService.getInstance = function () {
            if (LoginService._instance === null) {
                LoginService._instance = new LoginService();
            }
            return LoginService._instance;
        };
        /**
         * Try to login.
         */
        LoginService.prototype.login = function (user, password) {
            if (this._isLoginOnProgress) {
                return this._loginPromise;
            }
            this._loginPromise = new Promise(function (resolve, reject) {
                this._isLoginOnProgress = true;
                $.ajax({
                    url: LoginService.LoginCgiUrl,
                    cache: false,
                    dataType: "json",
                    username: user,
                    password: password,
                    success: function (data) {
                        // Build account
                        if (data) {
                            var userOption = void 0;
                            if (data.hasOwnProperty("profile")) {
                                userOption = data;
                            }
                            if (userOption) {
                                this._isLoginRequired(false);
                                resolve(new account_model_1.Account(user, userOption.profile));
                                return;
                            }
                        }
                        this._isLoginRequired(true);
                        reject("Data not valid");
                    }.bind(this),
                    error: function (jqXHR, textStatus, errorThrown) {
                        this._isLoginRequired(true);
                        reject(textStatus + ", " + errorThrown);
                    }.bind(this),
                    complete: function (jqXHR, textStatus) {
                        this._isLoginOnProgress = false;
                    }.bind(this)
                });
            }.bind(this));
            return this._loginPromise;
        };
        /**
         * Logout.
         * @return a promise about the logout operation.
         */
        LoginService.prototype.logout = function () {
            if (this._isLogoutOnProgress) {
                return this._logoutPromise;
            }
            this._logoutPromise = new Promise(function (resolve, reject) {
                this._isLogoutOnProgress = true;
                $.ajax({
                    url: LoginService.LogoutCgiUrl,
                    cache: false,
                    username: "wrong_user",
                    password: "wrong_pwd",
                    success: function (data) {
                        // Credentials are valid. The logout is failed
                        reject("Don't create the user 'wrong_user' with password 'wrong_pwd'");
                    }.bind(this),
                    error: function (jqXHR, textStatus, errorThrown) {
                        // Credentials aren't valid. The logout is been successfull
                        this._isLoginRequired(true);
                        resolve();
                    }.bind(this),
                    complete: function (jqXHR, textStatus) {
                        this._isLogoutOnProgress = false;
                    }.bind(this)
                });
            }.bind(this));
            return this._logoutPromise;
        };
        /** Singleton instance. */
        LoginService._instance = null;
        return LoginService;
    }(base_service_6.BaseService));
    exports.LoginService = LoginService;
});
define("service/timer.service", ["require", "exports", "knockout", "service/base.service"], function (require, exports, ko, base_service_7) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Timer helper.
     */
    var TimerService = /** @class */ (function (_super) {
        __extends(TimerService, _super);
        /**
         * Init helper.
         * @param currentTimezone. Current timezone.
         */
        function TimerService(currentTimezone) {
            var _this = _super.call(this) || this;
            if (TimerService._instance) {
                _this._error(base_service_7.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            _this._currentTimezone = ko.observable(currentTimezone);
            return _this;
        }
        Object.defineProperty(TimerService.prototype, "CurrentTimezoneObservable", {
            get: function () {
                return this._currentTimezone;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TimerService.prototype, "CurrentTimezone", {
            set: function (timezone) {
                this._currentTimezone(timezone);
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Switch timezone (Local/Remote).
         */
        TimerService.prototype.toogleTimezone = function () {
            if (this._currentTimezone() === Timezone.Local) {
                this._currentTimezone(Timezone.Remote);
            }
            else {
                this._currentTimezone(Timezone.Local);
            }
        };
        /**
         * Get instance of timer service in accord with singleton pattern.
         * @return an instance of timer service.
         */
        TimerService.getInstance = function () {
            if (TimerService._instance === null) {
                throw new Error("Error: TimerService not init");
            }
            return TimerService._instance;
        };
        /**
         * Get instance of timer service in accord with singleton pattern.
         * @param currentTimezone. Current timezone.
         * @return an instance of timer service.
         */
        TimerService.buildInstance = function (currentTimezone) {
            if (TimerService._instance === null) {
                TimerService._instance = new TimerService(currentTimezone);
            }
            return TimerService._instance;
        };
        /**
         * Convert date to string in accord with current timezone.
         * @param targetDate. Date to convert.
         * @param timezoneOffsetMinutes. Offset (in minutes) to add to utc timezone to obtain the remote date.
         */
        TimerService.prototype.dateToString = function (targetDate, timezoneOffsetMinutes) {
            // Check null
            if (targetDate == null || (typeof targetDate === "undefined")) {
                return "-";
            }
            if (this._currentTimezone() == Timezone.Local) {
                // Format the date using local machine offset
                return moment(targetDate, "YYYY-MM-DDTHH:mm:ssZ").format(TimerService.DateFormat);
            }
            else {
                // Get date using local machine offset
                var dateMoment = moment(targetDate);
                // Add minutes to convert the date with the offset of pco
                dateMoment.utcOffset(timezoneOffsetMinutes);
                return dateMoment.format(TimerService.DateFormat);
            }
        };
        TimerService.addHour = function (inputDate, hourOffset) {
            return moment(inputDate).add(hourOffset, "m").toDate();
        };
        TimerService.getMinuteOffset = function (inputDateString) {
            return moment.parseZone(inputDateString).utcOffset();
        };
        /** Singleton instance */
        TimerService._instance = null;
        /** Custom mask to format date. */
        TimerService.DateFormat = "DD/MM/YYYY HH:mm:ss";
        return TimerService;
    }(base_service_7.BaseService));
    exports.TimerService = TimerService;
    /**
     * Timezone.
     */
    var Timezone;
    (function (Timezone) {
        /** Local timezone. */
        Timezone[Timezone["Local"] = 0] = "Local";
        /** Remote timezone. */
        Timezone[Timezone["Remote"] = 1] = "Remote";
    })(Timezone = exports.Timezone || (exports.Timezone = {}));
});
define("component/viewModel/base-error-component.viewmodel", ["require", "exports", "common/loggable-entity"], function (require, exports, loggable_entity_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var BaseErrorComponentViewModel = /** @class */ (function (_super) {
        __extends(BaseErrorComponentViewModel, _super);
        function BaseErrorComponentViewModel() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BaseErrorComponentViewModel.prototype.showError = function (reason) {
            // Don't use here the this._log() funtion otherwise it doesn't catch the number of line
        };
        return BaseErrorComponentViewModel;
    }(loggable_entity_2.LoggableEntity));
    exports.BaseErrorComponentViewModel = BaseErrorComponentViewModel;
});
define("component/viewModel/base-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/login.service", "service/timer.service", "service/translator.service", "component/viewModel/base-error-component.viewmodel"], function (require, exports, ko, cfield_service_3, login_service_1, timer_service_1, translator_service_1, base_error_component_viewmodel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Common utility for the components that require Cfield service initialization.
     */
    var BaseComponentViewModel = /** @class */ (function (_super) {
        __extends(BaseComponentViewModel, _super);
        function BaseComponentViewModel(hideExtraToolbar, registerObserverOnScreenSizeChange, registerObserverOnTraslationChange, registerObserverOnTimezoneChange) {
            var _this = _super.call(this) || this;
            /** Listener to manage the screen resize. */
            _this._screenSizeChangedListener = function (mediaQueryList) {
                _this._isMobileScreen(mediaQueryList.matches == false);
            };
            _this._hideExtraToolbar = hideExtraToolbar;
            _this._cFieldServiceInstance = cfield_service_3.cFieldService.getInstance();
            // Add listener to detected when the screen becomes small
            var screenSizeMediaQuery = window.matchMedia("(min-width: 768px)");
            _this._isMobileScreen = ko.observable(screenSizeMediaQuery.matches == false);
            screenSizeMediaQuery.addListener(_this._screenSizeChangedListener);
            // Register observers
            _this._registeredObserverArray = [];
            _this._registeredObserverArray.push(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable.subscribe(_this.onChangeCFieldState, _this));
            if (registerObserverOnScreenSizeChange === true) {
                _this._registeredObserverArray.push(_this._isMobileScreen.subscribe(_this.onChangeScreenSize, _this));
            }
            if (registerObserverOnTraslationChange === true) {
                _this._registeredObserverArray.push(translator_service_1.TranslatorService.getInstance().OnChangeLanguageObservable.subscribe(_this.onChangeLanguage, _this));
            }
            if (registerObserverOnTimezoneChange === true) {
                _this._registeredObserverArray.push(timer_service_1.TimerService.getInstance().CurrentTimezoneObservable.subscribe(_this.onChangeTimezone, _this));
            }
            return _this;
        }
        Object.defineProperty(BaseComponentViewModel.prototype, "IsMobileScreen", {
            get: function () {
                return this._isMobileScreen;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseComponentViewModel.prototype, "HideExtraToolbar", {
            get: function () { return this._hideExtraToolbar; },
            enumerable: true,
            configurable: true
        });
        /**
         * Dispose
         */
        BaseComponentViewModel.prototype.dispose = function () {
            // Unregister observers
            for (var _i = 0, _a = this._registeredObserverArray; _i < _a.length; _i++) {
                var observer = _a[_i];
                observer.dispose();
            }
        };
        /**
         * Manage the common cField state.
         * @param newState cField service state.
         */
        BaseComponentViewModel.prototype.onChangeCFieldState = function (newState) {
            switch (newState) {
                case cfield_service_3.CfieldServiceState.Initialized:
                    this.onSuccessfulInitializedConf();
                    break;
            }
        };
        /**
         * Default callback invoked when cField service is initialized.
         */
        BaseComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            // No operation
        };
        /**
         * Default translator observer.
         * @param newState Translator state.
         */
        BaseComponentViewModel.prototype.onChangeLanguage = function (newState) {
            // No operation
        };
        /**
         * Observer to monitor if the screen size is small (i.e. compatible with smartphone).
         * @param isMobileScreen If <code>true</code> the screen is small.
         */
        BaseComponentViewModel.prototype.onChangeScreenSize = function (isMobileScreen) {
            // No operation
        };
        /**
         * Refresh the view model in accord with local or remote timezone.
         * @param newTimeZone The new timezone.
         */
        BaseComponentViewModel.prototype.onChangeTimezone = function (newTimeZone) {
            // No operation
        };
        /**
         * Refresh configuration.
         */
        BaseComponentViewModel.prototype.refreshConfiguration = function () {
            this._cFieldServiceInstance.releaseResourceOnLogout();
            this._cFieldServiceInstance.initService();
        };
        /**
         * Toogle timezone.
         */
        BaseComponentViewModel.prototype.toggleTimezone = function () {
            timer_service_1.TimerService.getInstance().toogleTimezone();
        };
        /**
         * Change language.
         */
        BaseComponentViewModel.prototype.changeLanguage = function () {
            var _this = this;
            var translatorService = translator_service_1.TranslatorService.getInstance();
            var changeLanguagePromise = translatorService.changeLanguage();
            changeLanguagePromise
                .then(function () {
                _this._log(translatorService.CurrentLanguage());
            })
                .catch(function (reason) {
                _this._log(reason);
            });
        };
        /**
         * Logout.
         */
        BaseComponentViewModel.prototype.logout = function () {
            var _this = this;
            var loginService = login_service_1.LoginService.getInstance();
            if (!loginService.IsLoginEnabled) {
                return;
            }
            var logoutPromise = loginService.logout();
            logoutPromise
                .then(function () {
                _this._cFieldServiceInstance.releaseResourceOnLogout();
            })
                .catch(function (reason) {
                _this._log(reason);
            });
        };
        return BaseComponentViewModel;
    }(base_error_component_viewmodel_1.BaseErrorComponentViewModel));
    exports.BaseComponentViewModel = BaseComponentViewModel;
});
define("component/viewModel/base-table-component.viewmodel", ["require", "exports", "component/viewModel/base-component.viewmodel"], function (require, exports, base_component_viewmodel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var BaseTableComponentViewModel = /** @class */ (function (_super) {
        __extends(BaseTableComponentViewModel, _super);
        function BaseTableComponentViewModel(hideExtraToolbar, registerObserverOnScreenSizeChange, registerObserverOnTraslationChange, registerObserverOnTimezoneChange) {
            var _this = _super.call(this, hideExtraToolbar, registerObserverOnScreenSizeChange, registerObserverOnTraslationChange, registerObserverOnTimezoneChange) || this;
            // Set layout in accord with screen size
            if (_this._isMobileScreen()) {
                _this._dataPageSize = 10;
                _this._enableCardView = true;
            }
            else {
                _this._dataPageSize = 25;
                _this._enableCardView = false;
            }
            return _this;
        }
        Object.defineProperty(BaseTableComponentViewModel.prototype, "DataPageSize", {
            get: function () {
                return this._dataPageSize;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseTableComponentViewModel.prototype, "EnableCardView", {
            get: function () {
                return this._enableCardView;
            },
            enumerable: true,
            configurable: true
        });
        /** Common icon for bootstrap-table. */
        BaseTableComponentViewModel.toolbarIconPak = {
            paginationSwitchDown: "fa-caret-square-o-down",
            paginationSwitchUp: "fa-caret-square-o-up",
            refresh: "fa-refresh",
            toggle: "fa-list-alt",
            columns: "fa-check-square-o",
            detailOpen: "fa-plus",
            detailClose: "fa-minus"
        };
        return BaseTableComponentViewModel;
    }(base_component_viewmodel_1.BaseComponentViewModel));
    exports.BaseTableComponentViewModel = BaseTableComponentViewModel;
});
define("component/viewModel/base-variable-header-component.viewmodel", ["require", "exports", "knockout", "service/translator.service", "component/viewModel/base-table-component.viewmodel"], function (require, exports, ko, translator_service_2, base_table_component_viewmodel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Manage the common header tile associated to the variables (name, description, uom..)
     */
    var BaseVariableHeaderComponentViewModel = /** @class */ (function (_super) {
        __extends(BaseVariableHeaderComponentViewModel, _super);
        function BaseVariableHeaderComponentViewModel(hideExtraToolbar, registerObservableOnScreenSizeChange, registerObservableOnTraslationChange, registerObservableOnTimezoneChange) {
            var _this = _super.call(this, hideExtraToolbar, registerObservableOnScreenSizeChange, registerObservableOnTraslationChange, registerObservableOnTimezoneChange) || this;
            _this.VariableNameHeader = ko.observable();
            _this.VariableDescriptionHeader = ko.observable();
            _this.VariableTypeHeader = ko.observable();
            _this.VariableUoMHeader = ko.observable();
            _this.VariableMinValueHeader = ko.observable();
            _this.VariableMaxValueHeader = ko.observable();
            _this.VariableValueHeader = ko.observable();
            _this.translateHeader();
            return _this;
        }
        /**
         * Default translator observer.
         * @param newState Translator state.
         */
        BaseVariableHeaderComponentViewModel.prototype.onChangeLanguage = function (newState) {
            _super.prototype.onChangeLanguage.call(this, newState);
            if (newState === translator_service_2.TranslatorEvent.LanguageChanged) {
                this.translateHeader();
            }
        };
        BaseVariableHeaderComponentViewModel.prototype.translateHeader = function () {
            var translator = translator_service_2.TranslatorService.getInstance().t;
            this.VariableNameHeader(translator(translator_service_2.TranslatorService.CfieldTranslationNs + "name"));
            this.VariableDescriptionHeader(translator(translator_service_2.TranslatorService.CfieldTranslationNs + "description"));
            this.VariableTypeHeader(translator(translator_service_2.TranslatorService.CfieldTranslationNs + "type"));
            this.VariableUoMHeader(translator(translator_service_2.TranslatorService.CfieldTranslationNs + "UoM"));
            this.VariableMinValueHeader(translator(translator_service_2.TranslatorService.CfieldTranslationNs + "min"));
            this.VariableMaxValueHeader(translator(translator_service_2.TranslatorService.CfieldTranslationNs + "max"));
            this.VariableValueHeader(translator(translator_service_2.TranslatorService.CfieldTranslationNs + "value"));
        };
        return BaseVariableHeaderComponentViewModel;
    }(base_table_component_viewmodel_1.BaseTableComponentViewModel));
    exports.BaseVariableHeaderComponentViewModel = BaseVariableHeaderComponentViewModel;
});
define("component/viewModel/base-category-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/device-config.service", "service/translator.service", "component/viewModel/base-variable-header-component.viewmodel", "component/viewModel/item/category-item.viewmodel", "common/validator"], function (require, exports, ko, cfield_service_4, device_config_service_3, translator_service_3, base_variable_header_component_viewmodel_1, category_item_viewmodel_1, validator_10) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Model the category section.
     */
    var BaseCategoryComponentViewModel = /** @class */ (function (_super) {
        __extends(BaseCategoryComponentViewModel, _super);
        function BaseCategoryComponentViewModel(hideExtraToolbar, registerObserverOnScreenSizeChange, registerObserverOnTraslationChange, registerObserverOnTimezoneChange, _tableId) {
            var _this = _super.call(this, hideExtraToolbar, registerObserverOnScreenSizeChange, registerObserverOnTraslationChange, registerObserverOnTimezoneChange) || this;
            _this._tableId = _tableId;
            _this._baseCategoryViewModelHelper = new BaseCategoryViewModelHelper();
            _this._categoryItemViewModelArray = ko.observableArray();
            _this._categoryDictionary = {};
            _this._categoryHeader = ko.observable();
            _this.translateCategoryHeader();
            return _this;
        }
        Object.defineProperty(BaseCategoryComponentViewModel.prototype, "CategoryDictionary", {
            get: function () { return this._categoryDictionary; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseCategoryComponentViewModel.prototype, "BaseCategoryViewModelHelper", {
            get: function () { return this._baseCategoryViewModelHelper; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseCategoryComponentViewModel.prototype, "TableIdToUpdate", {
            // Html elemt id of the table to update
            get: function () { return this._tableId; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseCategoryComponentViewModel.prototype, "CategoryHeader", {
            get: function () { return this._categoryHeader; },
            enumerable: true,
            configurable: true
        });
        // override
        BaseCategoryComponentViewModel.prototype.onChangeCFieldState = function (newState) {
            _super.prototype.onChangeCFieldState.call(this, newState);
            // Manage ad-hoc state
            switch (newState) {
                case cfield_service_4.CfieldServiceState.RefreshConfig:
                    // Clear state 		
                    this._categoryItemViewModelArray = ko.observableArray();
                    this._categoryDictionary = {};
                    break;
            }
        };
        // override
        BaseCategoryComponentViewModel.prototype.onChangeLanguage = function (newState) {
            _super.prototype.onChangeLanguage.call(this, newState);
            if (newState === translator_service_3.TranslatorEvent.LanguageChanged) {
                this.translateCategoryHeader();
                this.translateCategories();
            }
        };
        // override
        BaseCategoryComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            _super.prototype.onSuccessfulInitializedConf.call(this);
            // Get category dictionary
            var categoriesDictionary = device_config_service_3.DeviceConfigService.getInstance().VariableCategoryDictionary;
            // Init category helper
            this.initCategoryHelper(categoriesDictionary);
            // Build category viewmodels    
            for (var idCategory in categoriesDictionary) {
                this._categoryItemViewModelArray.push(new category_item_viewmodel_1.CategoryItemViewModel(this, idCategory, categoriesDictionary[idCategory]));
            }
            // Add "Without category"
            this._categoryItemViewModelArray.push(new category_item_viewmodel_1.CategoryItemViewModel(this));
            // Build a category dictionary
            ko.utils.arrayForEach(this._categoryItemViewModelArray(), function (categoryViewModel) {
                this._categoryDictionary[categoryViewModel.Id] = categoryViewModel;
            }.bind(this));
        };
        BaseCategoryComponentViewModel.prototype.initCategoryHelper = function (categoryDictionary) {
            var categoryCount = 0;
            if (validator_10.Validator.isDefined(categoryDictionary)) {
                for (var category in categoryDictionary) {
                    categoryCount++;
                }
            }
            this._baseCategoryViewModelHelper.Init(categoryCount);
        };
        BaseCategoryComponentViewModel.prototype.getCatagories = function (variable) {
            // Get categories
            var categoryViewModels = [];
            if (validator_10.Validator.isDefined(variable._idCatArray)) {
                for (var _i = 0, _a = variable._idCatArray; _i < _a.length; _i++) {
                    var categoryId = _a[_i];
                    categoryViewModels.push(this._categoryDictionary[categoryId]);
                }
            }
            else {
                categoryViewModels.push(this._categoryDictionary[category_item_viewmodel_1.CategoryItemViewModel.NoId]);
            }
            return categoryViewModels;
        };
        Object.defineProperty(BaseCategoryComponentViewModel.prototype, "Categories", {
            get: function () { return this._categoryItemViewModelArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseCategoryComponentViewModel.prototype, "TableIdToFilter", {
            // override
            get: function () {
                return this._tableId;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseCategoryComponentViewModel.prototype, "ListOfActiveCategories", {
            // override
            get: function () {
                var activeCategoryItemViewModelArray = [];
                ko.utils.arrayForEach(this._categoryItemViewModelArray(), function (categoryItemViewModel) {
                    if (categoryItemViewModel.IsActive() === true) {
                        activeCategoryItemViewModelArray.push(categoryItemViewModel);
                    }
                });
                return activeCategoryItemViewModelArray;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseCategoryComponentViewModel.prototype, "NextCategoryClass", {
            // override
            get: function () {
                return this._baseCategoryViewModelHelper.GetNextCategotyClass();
            },
            enumerable: true,
            configurable: true
        });
        // override
        BaseCategoryComponentViewModel.prototype.UnselectAllCategories = function () {
            // Set state of categories
            ko.utils.arrayForEach(this._categoryItemViewModelArray(), function (categoryItemViewModel) {
                categoryItemViewModel.resetCategory();
            });
            // Filter table
            $(this._tableId).bootstrapTable("arrayFilterBy", {
                // "Categories" is the property of EditableVariableItemViewModel
                CategoryId: []
            });
        };
        BaseCategoryComponentViewModel.prototype.translateCategoryHeader = function () {
            this._categoryHeader(translator_service_3.TranslatorService.getInstance().t(translator_service_3.TranslatorService.CfieldTranslationNs + "categories"));
        };
        BaseCategoryComponentViewModel.prototype.translateCategories = function () {
            ko.utils.arrayForEach(this._categoryItemViewModelArray(), function (categoryItemViewModel) {
                categoryItemViewModel.translate();
            }.bind(this));
        };
        return BaseCategoryComponentViewModel;
    }(base_variable_header_component_viewmodel_1.BaseVariableHeaderComponentViewModel));
    exports.BaseCategoryComponentViewModel = BaseCategoryComponentViewModel;
    var CategoriesGenerationMode;
    (function (CategoriesGenerationMode) {
        CategoriesGenerationMode[CategoriesGenerationMode["Dark"] = 0] = "Dark";
        CategoriesGenerationMode[CategoriesGenerationMode["DarkMedium"] = 1] = "DarkMedium";
        CategoriesGenerationMode[CategoriesGenerationMode["DarkMediumLight"] = 2] = "DarkMediumLight";
    })(CategoriesGenerationMode || (CategoriesGenerationMode = {}));
    /**
     * Helper to generate style for the category.
     */
    var BaseCategoryViewModelHelper = /** @class */ (function () {
        function BaseCategoryViewModelHelper() {
            this._isInit = false;
            this._categotyStyleClassSelector = 0;
            this._indexes = [0, 0, 0];
            this._counter = 0;
        }
        BaseCategoryViewModelHelper.prototype.Init = function (categoriesCount) {
            if (validator_10.Validator.isDefined(categoriesCount) == false || categoriesCount <= 7) {
                this._categoriesGenerationMode = CategoriesGenerationMode.Dark;
            }
            else if (categoriesCount <= 14) {
                this._categoriesGenerationMode = CategoriesGenerationMode.DarkMedium;
            }
            else {
                this._categoriesGenerationMode = CategoriesGenerationMode.DarkMediumLight;
            }
        };
        /**
         * Z-order algorithm to choose the css class for the category: dark->medium->light->dark->medium ...
         */
        BaseCategoryViewModelHelper.prototype.GetNextCategotyClass = function () {
            var index;
            // Eventually reset and adjust selector
            switch (this._categoriesGenerationMode) {
                case CategoriesGenerationMode.Dark:
                    if (this._indexes[0] === BaseCategoryViewModelHelper.CategoryCssClassArray[0].length) {
                        // Reset
                        this._indexes[0] = 0;
                    }
                    index = this._indexes[0]++;
                    break;
                case CategoriesGenerationMode.DarkMedium:
                    this._categotyStyleClassSelector = this._counter % 2;
                    if (this._indexes[this._categotyStyleClassSelector] === BaseCategoryViewModelHelper.CategoryCssClassArray[this._categotyStyleClassSelector].length) {
                        // Reset
                        this._indexes[this._categotyStyleClassSelector] = 0;
                    }
                    index = this._indexes[this._categotyStyleClassSelector]++;
                    break;
                case CategoriesGenerationMode.DarkMediumLight:
                    this._categotyStyleClassSelector = this._counter % 3;
                    if (this._indexes[this._categotyStyleClassSelector] === BaseCategoryViewModelHelper.CategoryCssClassArray[this._categotyStyleClassSelector].length) {
                        // Reset
                        this._indexes[this._categotyStyleClassSelector] = 0;
                    }
                    index = this._indexes[this._categotyStyleClassSelector]++;
                    break;
            }
            // Get class
            var categoryClass = BaseCategoryViewModelHelper.CategoryCssClassArray[this._categotyStyleClassSelector][index];
            this._counter++;
            return categoryClass;
        };
        BaseCategoryViewModelHelper.UnusedCategoryCssClass = "unusedCategoryStyle";
        // Don't change the names of categories. They are used into css file
        BaseCategoryViewModelHelper.CategoryCssClassArray = [
            // 0 index
            [
                "paletteDark0",
                "paletteDark1",
                "paletteDark2",
                "paletteDark3",
                "paletteDark4",
                "paletteDark5",
                "paletteDark6"
            ],
            // 1 index
            [
                "paletteMedium0",
                "paletteMedium1",
                "paletteMedium2",
                "paletteMedium3",
                "paletteMedium4",
                "paletteMedium5",
                "paletteMedium6"
            ],
            // 2 index
            [
                "paletteLight0",
                "paletteLight1",
                "paletteLight2",
                "paletteLight3",
                "paletteLight4",
                "paletteLight5",
                "paletteLight6"
            ]
        ];
        BaseCategoryViewModelHelper.CategoryNoClickedCssClass = "categoryNoClickedStyle";
        return BaseCategoryViewModelHelper;
    }());
    exports.BaseCategoryViewModelHelper = BaseCategoryViewModelHelper;
});
define("component/viewModel/item/category-item.viewmodel", ["require", "exports", "knockout", "service/translator.service", "common/validator", "component/viewModel/base-category-component.viewmodel"], function (require, exports, ko, translator_service_4, validator_11, base_category_component_viewmodel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var CategoryItemViewModel = /** @class */ (function () {
        function CategoryItemViewModel(_categoryHelper, _id, _translationKey) {
            this._categoryHelper = _categoryHelper;
            this._id = _id;
            this._translationKey = _translationKey;
            this._categoryName = ko.observable();
            if (validator_11.Validator.isDefined(_id)) {
                // With category
                if (_translationKey) {
                    this._categoryCssClass = this._categoryHelper.NextCategoryClass;
                    this._isActive = ko.observable(true);
                }
                else {
                    // Manage error
                    this._categoryCssClass = base_category_component_viewmodel_1.BaseCategoryViewModelHelper.UnusedCategoryCssClass;
                    this._translationKey = CategoryItemViewModel.NoCategoryKey;
                    this._isActive = ko.observable(true);
                }
            }
            else {
                // Without categoty
                this._id = CategoryItemViewModel.NoId;
                this._translationKey = CategoryItemViewModel.NoCategoryKey;
                this._categoryCssClass = base_category_component_viewmodel_1.BaseCategoryViewModelHelper.UnusedCategoryCssClass;
                this._isActive = ko.observable(true);
            }
            // Common code
            if (this._isActive()) {
                this._currentCategoryCssClass = ko.observable(this._categoryCssClass);
            }
            else {
                this._currentCategoryCssClass = ko.observable(base_category_component_viewmodel_1.BaseCategoryViewModelHelper.CategoryNoClickedCssClass);
            }
            this.translate();
        }
        Object.defineProperty(CategoryItemViewModel.prototype, "Id", {
            get: function () { return this._id; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(CategoryItemViewModel.prototype, "TranslationKey", {
            get: function () { return this._translationKey; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CategoryItemViewModel.prototype, "CategoryName", {
            get: function () { return this._categoryName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CategoryItemViewModel.prototype, "CurrentCategoryCssClass", {
            get: function () { return this._currentCategoryCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CategoryItemViewModel.prototype, "IsActive", {
            get: function () { return this._isActive; },
            enumerable: true,
            configurable: true
        });
        CategoryItemViewModel.prototype.translate = function () {
            // Ad hoc manage for noCategory
            if (this._translationKey === CategoryItemViewModel.NoCategoryKey) {
                // Read from cfieldbase namespace
                this._categoryName(CategoryItemViewModel._translator(translator_service_4.TranslatorService.CfieldTranslationNs + this._translationKey));
            }
            else {
                // Read from category namespace
                this._categoryName(CategoryItemViewModel._translator(translator_service_4.TranslatorService.CategoryTranslationNs + this._translationKey));
            }
        };
        CategoryItemViewModel.prototype.onCategoryClicked = function () {
            // Manage Ui
            if (this._isActive() === true) {
                // Reset
                this.resetCategory();
            }
            else {
                // Change color
                this._isActive(true);
                this._currentCategoryCssClass(this._categoryCssClass);
            }
            // Filter table
            var activeCategoryViewModels = this._categoryHelper.ListOfActiveCategories;
            var categoryIdQuery = [];
            for (var _i = 0, activeCategoryViewModels_1 = activeCategoryViewModels; _i < activeCategoryViewModels_1.length; _i++) {
                var activeCategoryViewModel = activeCategoryViewModels_1[_i];
                categoryIdQuery.push(activeCategoryViewModel.Id);
            }
            $(this._categoryHelper.TableIdToFilter).bootstrapTable("arrayFilterBy", {
                // "Categories" is the property of EditableVariableItemViewModel
                CategoryId: categoryIdQuery
            });
        };
        CategoryItemViewModel.prototype.resetCategory = function () {
            this._isActive(false);
            this._currentCategoryCssClass(base_category_component_viewmodel_1.BaseCategoryViewModelHelper.CategoryNoClickedCssClass);
        };
        CategoryItemViewModel._translator = translator_service_4.TranslatorService.getInstance().t;
        CategoryItemViewModel.NoId = "-1";
        CategoryItemViewModel.NoCategoryKey = "noCategory";
        return CategoryItemViewModel;
    }());
    exports.CategoryItemViewModel = CategoryItemViewModel;
});
define("component/presenter/update-item-set", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var UpdateItemSetHelper = /** @class */ (function () {
        function UpdateItemSetHelper(_pendingUpdateSet, _numberOfPendingUpdate) {
            this._pendingUpdateSet = _pendingUpdateSet;
            this._numberOfPendingUpdate = _numberOfPendingUpdate;
        }
        Object.defineProperty(UpdateItemSetHelper.prototype, "PendingUpdates", {
            get: function () { return this._pendingUpdateSet.getValues(); },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UpdateItemSetHelper.prototype, "NumberOfPendingUpdate", {
            get: function () { return this._numberOfPendingUpdate; },
            enumerable: true,
            configurable: true
        });
        UpdateItemSetHelper.prototype.add = function (variable) {
            // Update the counter of pending updates
            this._pendingUpdateSet.addNoConflict(variable.Variable._id, variable);
            this._numberOfPendingUpdate(this._pendingUpdateSet.Count);
        };
        UpdateItemSetHelper.prototype.remove = function (variable) {
            // Remove from set 
            this._pendingUpdateSet.remove(variable._id);
            this._numberOfPendingUpdate(this._pendingUpdateSet.Count);
        };
        UpdateItemSetHelper.prototype.clear = function () {
            if (this._numberOfPendingUpdate() > 0) {
                this._pendingUpdateSet.clearSlow();
                this._numberOfPendingUpdate(this._pendingUpdateSet.Count);
            }
        };
        return UpdateItemSetHelper;
    }());
    exports.UpdateItemSetHelper = UpdateItemSetHelper;
});
define("common/style", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var StyleClass = /** @class */ (function () {
        function StyleClass() {
        }
        StyleClass.RemeberToSaveRowClass = "warning";
        StyleClass.CurrentValueNotEqualToEditValueClass = "warning";
        StyleClass.InvalidInputError = "error";
        StyleClass.DangerRow = "danger";
        StyleClass.ImportedNotEqualToCurrentCssClass = "notEqual";
        return StyleClass;
    }());
    exports.StyleClass = StyleClass;
});
define("component/presenter/variable-item.presenter", ["require", "exports", "common/style", "common/validator"], function (require, exports, style_1, validator_12) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var RowStyleEditableVariableItemPresenter = /** @class */ (function () {
        function RowStyleEditableVariableItemPresenter() {
        }
        // interface implementation
        RowStyleEditableVariableItemPresenter.prototype.onValidationOk = function (jquerySelector, event, value, variable, index) {
            this.removeCssClassError(jquerySelector);
            this.removeErrorTootip(jquerySelector);
        };
        // interface implementation
        RowStyleEditableVariableItemPresenter.prototype.onModifyValidValue = function (jquerySelector, event, value, variable, index) {
            // Add style for row
            var row = $("table tbody tr[data-index='" + index + "']");
            if (validator_12.Validator.isDefined(row)) {
                row.addClass(style_1.StyleClass.CurrentValueNotEqualToEditValueClass);
            }
        };
        // interface implementation
        RowStyleEditableVariableItemPresenter.prototype.onNotModifyValidValue = function (jquerySelector, event, value, variable, index) {
            this.removeRowStyle(index);
        };
        // interface implementation
        RowStyleEditableVariableItemPresenter.prototype.onInvalidValue = function (jquerySelector, event, value, variable, index) {
            this.addCssClassError(jquerySelector);
            this.addErrorTootip(jquerySelector, variable);
            this.removeRowStyle(index);
        };
        // interface implementation
        RowStyleEditableVariableItemPresenter.prototype.onFocus = function () { };
        // interface implementation
        RowStyleEditableVariableItemPresenter.prototype.clearInternalState = function () { };
        RowStyleEditableVariableItemPresenter.prototype.addCssClassError = function (jquerySelector) {
            $(jquerySelector).addClass(style_1.StyleClass.InvalidInputError);
        };
        RowStyleEditableVariableItemPresenter.prototype.removeCssClassError = function (jquerySelector) {
            $(jquerySelector).removeClass(style_1.StyleClass.InvalidInputError);
        };
        RowStyleEditableVariableItemPresenter.prototype.addErrorTootip = function (jquerySelector, variable) {
            var titleText = variable.CurrentValueValidationMsg;
            $(jquerySelector).attr("title", titleText);
        };
        RowStyleEditableVariableItemPresenter.prototype.removeErrorTootip = function (jquerySelector) {
            $(jquerySelector).removeAttr("title");
        };
        RowStyleEditableVariableItemPresenter.prototype.removeRowStyle = function (index) {
            var row = $("table tbody tr[data-index='" + index + "']");
            row.removeClass(style_1.StyleClass.CurrentValueNotEqualToEditValueClass);
        };
        return RowStyleEditableVariableItemPresenter;
    }());
    exports.RowStyleEditableVariableItemPresenter = RowStyleEditableVariableItemPresenter;
    var RowStyleWithMemoryEditableVariableItemPresenter = /** @class */ (function (_super) {
        __extends(RowStyleWithMemoryEditableVariableItemPresenter, _super);
        function RowStyleWithMemoryEditableVariableItemPresenter(_updateItemSetHelper) {
            var _this = _super.call(this) || this;
            _this._updateItemSetHelper = _updateItemSetHelper;
            return _this;
        }
        // override
        RowStyleWithMemoryEditableVariableItemPresenter.prototype.onModifyValidValue = function (jquerySelector, event, value, variable, index) {
            this.addToUpdateSet(variable);
            _super.prototype.onModifyValidValue.call(this, jquerySelector, event, value, variable, index);
        };
        // override
        RowStyleWithMemoryEditableVariableItemPresenter.prototype.onNotModifyValidValue = function (jquerySelector, event, value, variable, index) {
            this.removeFromUpdateSet(variable.Variable);
            _super.prototype.onNotModifyValidValue.call(this, jquerySelector, event, value, variable, index);
        };
        // override
        RowStyleWithMemoryEditableVariableItemPresenter.prototype.onInvalidValue = function (jquerySelector, event, value, variable, index) {
            this.removeFromUpdateSet(variable.Variable);
            _super.prototype.onInvalidValue.call(this, jquerySelector, event, value, variable, index);
        };
        // override
        RowStyleWithMemoryEditableVariableItemPresenter.prototype.clearInternalState = function () {
            this._updateItemSetHelper.clear();
        };
        RowStyleWithMemoryEditableVariableItemPresenter.prototype.addToUpdateSet = function (variable) {
            this._updateItemSetHelper.add(variable);
        };
        RowStyleWithMemoryEditableVariableItemPresenter.prototype.removeFromUpdateSet = function (variable) {
            this._updateItemSetHelper.remove(variable);
        };
        return RowStyleWithMemoryEditableVariableItemPresenter;
    }(RowStyleEditableVariableItemPresenter));
    exports.RowStyleWithMemoryEditableVariableItemPresenter = RowStyleWithMemoryEditableVariableItemPresenter;
});
define("component/viewModel/item/variable-item.viewmodel", ["require", "exports", "service/device-config.service", "service/translator.service", "service/uom.service", "common/loggable-entity", "common/validator", "common/formatter", "dataModel/datatype.model"], function (require, exports, device_config_service_4, translator_service_5, uom_service_2, loggable_entity_3, validator_13, formatter_1, datatype_model_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * There aren't obserable fields because the bootstrap-table doesn't support them. So the table is re-draw when data changes.
     */
    var EditableVariableItemViewModel = /** @class */ (function () {
        function EditableVariableItemViewModel(_variableCurrentValue, _variable, _currentValueItemPresenter, _categoryViewModels) {
            this._variableCurrentValue = _variableCurrentValue;
            this._variable = _variable;
            this._currentValueItemPresenter = _currentValueItemPresenter;
            this._categoryViewModels = _categoryViewModels;
            this._currentValueValidationState = null;
            this._variableEditedValue = null;
            // Problem: I can't init these fields if the cFieldService isn't in "Initialized" state. I try to register an obserber
            // to cFieldState in a static method of this class but I receive a null reference to cField service..
            if (EditableVariableItemViewModel.isStaticFieldInitialize == false) {
                EditableVariableItemViewModel._translator = translator_service_5.TranslatorService.getInstance().t;
                EditableVariableItemViewModel._uomService = uom_service_2.UoMService.getInstance();
                EditableVariableItemViewModel._variableDictionary = device_config_service_4.DeviceConfigService.getInstance().VariablesDictionary;
                EditableVariableItemViewModel.isStaticFieldInitialize = true;
            }
            // Validate
            if (!validator_13.Validator.isDefined(_variable)) {
                EditableVariableItemViewModel._log._error("Variable is not defined");
                return;
            }
            if (!validator_13.Validator.isDefined(_variable._dataType)) {
                EditableVariableItemViewModel._log._error("Datatype for variable " + _variable._id + " is not defined");
                return;
            }
            var dataType = EditableVariableItemViewModel._dataTypeDictionary[this._variable._dataType];
            if (!validator_13.Validator.isDefined(dataType)) {
                EditableVariableItemViewModel._log._error("Type of datatype for variable " + _variable._id + " is not defined");
                return;
            }
            if (!validator_13.Validator.isDefined(_variable._uom)) {
                EditableVariableItemViewModel._log._error("Uom for variable " + _variable._id + " is not defined");
                return;
            }
            // Don't check the scope: the parameters haven't scope.
            // Data is valid
            this._dataType = dataType;
            this._isNumeric = datatype_model_5.DataTypeUtil.IsNumericDataType(this._dataType.Type);
            this._isEditable = validator_13.Validator.isDefined(_variable._scope) ? _variable._scope._webWrite : true;
            // Format value
            if (this._isNumeric) {
                this._variableFormattedCurrentValue = this.formatNumberValue(_variableCurrentValue);
                this.initMin();
                this.initMax();
            }
            if (validator_13.Validator.isDefined(_categoryViewModels)) {
                this._categoryIds = [];
                for (var _i = 0, _categoryViewModels_1 = _categoryViewModels; _i < _categoryViewModels_1.length; _i++) {
                    var categoryViewModel = _categoryViewModels_1[_i];
                    this._categoryIds.push(categoryViewModel.Id);
                }
            }
        }
        Object.defineProperty(EditableVariableItemViewModel.prototype, "Variable", {
            get: function () { return this._variable; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableName", {
            get: function () { return this._variable._name; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableDescription", {
            get: function () {
                return EditableVariableItemViewModel._translator(translator_service_5.TranslatorService.VariableTranslationNs + this._variable._name, { defaultValue: this._variable._description || null });
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "IsNumeric", {
            get: function () { return this._isNumeric; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "Accuracy", {
            get: function () { return EditableVariableItemViewModel.numberMaxVariableAccuracy; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableFullPrecisionValue", {
            get: function () {
                if (this._isEditable && this.IsVariableModified) {
                    return this._variableEditedValue;
                }
                else {
                    return this._variableCurrentValue;
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableCurrentValue", {
            get: function () {
                if (this._isEditable && this.IsVariableModified) {
                    return this._variableEditedValue;
                }
                else if (this._isNumeric) {
                    return this._variableFormattedCurrentValue;
                }
                else {
                    return this._variableCurrentValue;
                }
            },
            enumerable: true,
            configurable: true
        });
        EditableVariableItemViewModel.prototype.composeCurrentNumberValueUom = function (uomData) {
            if (this._isEditable && this.IsVariableModified) {
                return this.formatNumberValueUom(this._variableEditedValue, uomData.Symbol);
            }
            else {
                return this.formatNumberValueUom(this._variableFormattedCurrentValue, uomData.Symbol);
            }
        };
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableCurrentValueUom", {
            // It use the sup uom target
            get: function () {
                return this.composeCurrentNumberValueUom(this.VariableSupUom);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "DataType", {
            get: function () { return this._dataType.Type; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableType", {
            get: function () { return this._dataType.DisplayName || null; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableSupUom", {
            // Can be changed dinamically..
            get: function () { return EditableVariableItemViewModel._uomService.getSupUom(this._variable._uom); },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableNoZoneUom", {
            get: function () { return EditableVariableItemViewModel._uomService.getNoZoneUom(this._variable._uom); },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableFormattedMinValue", {
            get: function () { return this._variableFormattedMinValue; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableMinValueUom", {
            get: function () {
                return this.formatNumberValueUom(this._variableFormattedMinValue, this.VariableSupUom.Symbol);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableFormattedMaxValue", {
            get: function () { return this._variableFormattedMaxValue; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableMaxValueUom", {
            get: function () {
                return this.formatNumberValueUom(this._variableFormattedMaxValue, this.VariableSupUom.Symbol);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "CurrentValueValidationMsg", {
            get: function () {
                switch (this._currentValueValidationState) {
                    case validator_13.RangeValidationResult.NotValid:
                        return EditableVariableItemViewModel._translator(translator_service_5.TranslatorService.CfieldTranslationNs + "validationFailed");
                    case validator_13.RangeValidationResult.OutOfRange:
                        return EditableVariableItemViewModel._translator(translator_service_5.TranslatorService.CfieldTranslationNs + "outOfRange");
                    default:
                        return "";
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "IsEditable", {
            get: function () { return this._isEditable; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableEditedValue", {
            get: function () { return this._variableEditedValue; },
            set: function (newValue) { this._variableEditedValue = newValue; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "ValidationRegex", {
            get: function () { return validator_13.Validator.GetVariableValidationRegex(this._dataType.Type); },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "IsVariableEditedValid", {
            get: function () {
                this._currentValueValidationState = validator_13.Validator.ValidateVariableValue(this, this._variableEditedValue, this._variableFormattedMinValue, this._variableFormattedMaxValue);
                return this._variableEditedValue != null
                    && (this._currentValueValidationState === validator_13.RangeValidationResult.Valid
                        || this._currentValueValidationState === validator_13.RangeValidationResult.NotEvalueated);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "IsVariableModified", {
            get: function () {
                if (validator_13.Validator.isDefined((this._variableEditedValue))) {
                    if (this._isNumeric) {
                        return this._variableFormattedCurrentValue != this._variableEditedValue;
                    }
                    else {
                        return this._variableCurrentValue != this._variableEditedValue;
                    }
                }
                return false;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "IsVariableToUpdate", {
            get: function () { return this.IsVariableEditedValid && this.IsVariableModified; },
            enumerable: true,
            configurable: true
        });
        EditableVariableItemViewModel.prototype.composeVariableEditedValueUom = function (uomData) {
            var firstPart = validator_13.Validator.isDefined(this._variableEditedValue) ? this._variableEditedValue : "";
            var thirdPart = validator_13.Validator.isDefined(uomData.Symbol) ? uomData.Symbol : "";
            var secondPart = thirdPart.length > 0 ? " " : "";
            return firstPart + secondPart + thirdPart;
        };
        Object.defineProperty(EditableVariableItemViewModel.prototype, "VariableEditedValueUom", {
            get: function () {
                return this.composeVariableEditedValueUom(this.VariableSupUom);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "CurrentValueItemPresenter", {
            get: function () { return this._currentValueItemPresenter; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(EditableVariableItemViewModel.prototype, "CategoryViewModels", {
            get: function () { return this._categoryViewModels; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditableVariableItemViewModel.prototype, "CategoryId", {
            get: function () { return this._categoryIds; },
            enumerable: true,
            configurable: true
        });
        EditableVariableItemViewModel.prototype.initMin = function () {
            var isMinValueDefined = validator_13.Validator.isDefined(this._variable._minValue);
            var isMinIdDefined = validator_13.Validator.isDefined(this._variable._minId);
            if (!isMinValueDefined && !isMinIdDefined) {
                return null;
            }
            if (isMinValueDefined) {
                var numericValue = Number(this._variable._minValue);
                if (!isNaN(numericValue)) {
                    // Convert
                    var valueContainer = this.convertValueToOtherZone(this.VariableNoZoneUom, numericValue, this.VariableSupUom.UomDstId, uom_service_2.UomTarget.Sup);
                    if (validator_13.Validator.isDefined(valueContainer)) {
                        this._variableFormattedMinValue = this.formatNumberValue(valueContainer.Value);
                    }
                    else {
                        // No conversion needed
                        this._variableFormattedMinValue = this.formatNumberValue(numericValue);
                    }
                }
            }
            else if (isMinIdDefined) {
                var minVariable = EditableVariableItemViewModel._variableDictionary[this._variable._minId];
                if (validator_13.Validator.isDefined(minVariable)) {
                    // Use sup uom
                    this._variableFormattedMinValue = this.formatNumberValue(minVariable._currentValue);
                }
            }
        };
        EditableVariableItemViewModel.prototype.initMax = function () {
            var isMaxValueDefined = validator_13.Validator.isDefined(this._variable._maxValue);
            var isMaxIdDefined = validator_13.Validator.isDefined(this._variable._maxId);
            if (!isMaxValueDefined && !isMaxIdDefined) {
                return null;
            }
            if (isMaxValueDefined) {
                var numericValue = Number(this._variable._maxValue);
                if (!isNaN(numericValue)) {
                    // Convert
                    var valueContainer = this.convertValueToOtherZone(this.VariableNoZoneUom, numericValue, this.VariableSupUom.UomDstId, uom_service_2.UomTarget.Sup);
                    if (validator_13.Validator.isDefined(valueContainer)) {
                        this._variableFormattedMaxValue = this.formatNumberValue(valueContainer.Value);
                    }
                    else {
                        // No conversion needed
                        this._variableFormattedMaxValue = this.formatNumberValue(numericValue);
                    }
                }
            }
            else if (isMaxIdDefined) {
                var maxVariable = EditableVariableItemViewModel._variableDictionary[this._variable._maxId];
                if (validator_13.Validator.isDefined(maxVariable)) {
                    // Use sup uom
                    this._variableFormattedMaxValue = this.formatNumberValue(maxVariable._currentValue);
                }
            }
        };
        EditableVariableItemViewModel.prototype.refresh = function (resetEditing, variableCurrentValue) {
            if (resetEditing) {
                this._variableEditedValue = null;
            }
            this._variableCurrentValue = variableCurrentValue;
            // Format value
            if (this._isNumeric) {
                this._variableFormattedCurrentValue = this.formatNumberValue(variableCurrentValue);
                this.initMin();
                this.initMax();
            }
        };
        /**
         *
         * @param value Must be already formatted to the desiderable precision
         * @param uom
         */
        EditableVariableItemViewModel.prototype.formatNumberValueUom = function (value, uom) {
            // Format value + uom
            if (!validator_13.Validator.isDefined(value)) {
                return null;
            }
            var third = validator_13.Validator.isDefined(uom) ? uom : "";
            var second = third.length > 0 ? " " : "";
            return this.formatNumberValue(value) + second + third;
        };
        EditableVariableItemViewModel.prototype.formatNumberValue = function (value) {
            return formatter_1.Formatter.FormatNumber(value, this._dataType.Type, EditableVariableItemViewModel.numberMaxVariableAccuracy);
        };
        EditableVariableItemViewModel.prototype.getUomData = function (variableId, target) {
            var variable = EditableVariableItemViewModel._variableDictionary[variableId];
            if (validator_13.Validator.isDefined(variable)) {
                // Get current value
                if (!validator_13.Validator.isDefined(variable._currentValue)) {
                    EditableVariableItemViewModel._log._error("getSupUomData(): value not defined.");
                    return null;
                }
                var value = Number(variable._currentValue);
                if (isNaN(value)) {
                    EditableVariableItemViewModel._log._error("getSupUomData(): value isn't a number. Value=" + value);
                    return null;
                }
                // Get uom
                var uomData = void 0;
                switch (target) {
                    case uom_service_2.UomTarget.Sup:
                        uomData = EditableVariableItemViewModel._uomService.getSupUom(variable._uom);
                        break;
                    case uom_service_2.UomTarget.Default:
                        uomData = EditableVariableItemViewModel._uomService.getNoZoneUom(variable._uom);
                        break;
                    default:
                        EditableVariableItemViewModel._log._error("Uom target not supported. Target=" + target);
                        return;
                }
                // Convert
                return new ValueContainer(value.valueOf(), uomData);
            }
            else {
                EditableVariableItemViewModel._log._error("getSupUomData(): variable not found");
                return null;
            }
        };
        EditableVariableItemViewModel.prototype.convertDefaultToSupZone = function (srcValue, uomSrcData, uomDstId) {
            return this.formatConvertedValue(srcValue, uomSrcData, this.convertValueToOtherZone(uomSrcData, srcValue, uomDstId, uom_service_2.UomTarget.Sup));
        };
        EditableVariableItemViewModel.prototype.convertSupToDefaultZone = function (srcValue, uomSrcData, uomDstId) {
            return this.formatConvertedValue(srcValue, uomSrcData, this.convertValueToOtherZone(uomSrcData, srcValue, uomDstId, uom_service_2.UomTarget.Default));
        };
        /**
         *
         * @param uomSrcData Where I start
         * @param srcValue Where I start
         * @param uomDstId  Where I want to arrive
         * @param uomTarget Target of destination
         */
        EditableVariableItemViewModel.prototype.convertValueToOtherZone = function (uomSrcData, srcValue, uomDstId, uomTarget) {
            // Validate
            var uomSrcId = uomSrcData.UomDstId;
            if (!(validator_13.Validator.isDefined(srcValue) && validator_13.Validator.isDefined(uomSrcData) && validator_13.Validator.isDefined(uomSrcData.UomDstId)
                && validator_13.Validator.isDefined(uomDstId) && validator_13.Validator.isDefined(uomTarget))) {
                EditableVariableItemViewModel._log._error("convertValueToOtherZone(): params are nulls. uomSrcId=" + uomSrcId
                    + ", srcValue=" + srcValue + ", uomDstId=" + uomDstId + ", uomTarget=" + uomTarget);
                return null;
            }
            // Get dst target uom data
            var targetUomData;
            switch (uomTarget) {
                case uom_service_2.UomTarget.Sup:
                    targetUomData = EditableVariableItemViewModel._uomService.getSupUom(uomDstId);
                    break;
                case uom_service_2.UomTarget.Default:
                    targetUomData = EditableVariableItemViewModel._uomService.getNoZoneUom(uomDstId);
                    break;
                default:
                    EditableVariableItemViewModel._log._error("Uom target not supported. Target=" + uomTarget);
                    return null;
            }
            // targetUomData is always defined. Skip the check
            if (uomSrcId === targetUomData.UomDstId) {
                // No conversion needed
                return null;
            }
            // Get coefficients
            var dstValue;
            var coefficient = EditableVariableItemViewModel._uomService.getUomConversionCoefficient(uomSrcId, targetUomData.UomDstId);
            if (validator_13.Validator.isDefined(coefficient)) {
                // Convert
                dstValue = srcValue.valueOf() * coefficient.MultCoefficient + coefficient.AddCoefficient;
                dstValue = dstValue.valueOf();
                return new ValueContainer(dstValue, targetUomData);
            }
            else {
                return null;
            }
        };
        EditableVariableItemViewModel.prototype.formatConvertedValue = function (srcValue, uomSrcData, convertedValue) {
            if (validator_13.Validator.isDefined(convertedValue)) {
                return this.formatNumberValueUom(convertedValue.Value, convertedValue.UomData.Symbol);
            }
            else {
                return this.formatNumberValueUom(srcValue, uomSrcData.Symbol);
            }
        };
        //  Static field without dependency on object that are initialized after "CfieldServiceState.Initialized" state
        EditableVariableItemViewModel._log = new loggable_entity_3.LoggableEntityStatic("EditableVariableItemViewModel");
        EditableVariableItemViewModel._dataTypeDictionary = datatype_model_5.DataTypeUtil.getInstance().DataTypeDictionary;
        EditableVariableItemViewModel.isStaticFieldInitialize = false;
        EditableVariableItemViewModel.numberMaxVariableAccuracy = 2;
        return EditableVariableItemViewModel;
    }());
    exports.EditableVariableItemViewModel = EditableVariableItemViewModel;
    var ParameterItemViewModel = /** @class */ (function (_super) {
        __extends(ParameterItemViewModel, _super);
        function ParameterItemViewModel(_parameterImporterHelper, variableCurrentValue, variable, _variableInitValue, _variableDevValue, currentValueItemPresenter, _importedValueItemPresenter, categoryViewModels) {
            var _this = _super.call(this, variableCurrentValue, variable, currentValueItemPresenter, categoryViewModels) || this;
            _this._parameterImporterHelper = _parameterImporterHelper;
            _this._variableInitValue = _variableInitValue;
            _this._variableDevValue = _variableDevValue;
            _this._importedValueItemPresenter = _importedValueItemPresenter;
            _this._variableImportedValueValidationState = null;
            _this.IsVariableImportedValueEdited = false;
            // Dev and init are not formatted
            // override
            _this._isEditable = true;
            return _this;
        }
        Object.defineProperty(ParameterItemViewModel.prototype, "Accuracy", {
            get: function () { return ParameterItemViewModel.numberMaxParameterAccuracy; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "IsParameterImportDone", {
            get: function () { return this._parameterImporterHelper.IsImportParameterDone(); },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "VariableCurrentValueUom", {
            // It use the no zone uom target
            // override
            get: function () {
                return this.composeCurrentNumberValueUom(this.VariableNoZoneUom);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "ValidationRegex", {
            // override
            get: function () { return validator_13.Validator.GetParameterValidationRegex(this._dataType.Type); },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "VariableInitValue", {
            get: function () { return this._variableInitValue; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "VariableInitValueUom", {
            get: function () {
                // Use uom of no converion zone
                return this.formatNumberValueUom(this._variableInitValue, this.VariableNoZoneUom.Symbol);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "VariableDevValue", {
            get: function () { return this._variableDevValue; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "VariableDevValueUom", {
            get: function () {
                // Use uom of no converion zone
                return this.formatNumberValueUom(this._variableDevValue, this.VariableNoZoneUom.Symbol);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "VariableImportedValue", {
            get: function () { return this._variableImportedValue; },
            set: function (value) {
                if (!validator_13.Validator.isDefined(value)) {
                    this._variableImportedValue = null;
                }
                else {
                    this._variableImportedValue = value;
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "IsVariableImportedValueSet", {
            get: function () { return validator_13.Validator.isDefined(this._variableImportedValue); },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "VariableImportedValueUom", {
            get: function () {
                // Use uom of no converion zone
                return this.formatNumberValueUom(this._variableImportedValue, this.VariableNoZoneUom.Symbol);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "IsCurrentNotEqualToImportedValue", {
            get: function () {
                if (validator_13.Validator.isDefined(this._variableImportedValue)) {
                    var value = void 0;
                    if (validator_13.Validator.isDefined(this._variableEditedValue)) {
                        value = this._variableEditedValue;
                    }
                    else if (validator_13.Validator.isDefined(this._variableCurrentValue)) {
                        value = this._variableCurrentValue;
                    }
                    else {
                        var name_1 = validator_13.Validator.isDefined(this._variable) ? this._variable._name : "undefined";
                        EditableVariableItemViewModel._log._error("Variable IsCurrentNotEqualToImportedValue. Both current and edit variable value are not defined. VarName=" + name_1);
                        return false;
                    }
                    if (datatype_model_5.DataTypeUtil.IsNumericDataType(this._dataType.Type) && value !== "" && this._variableImportedValue !== "") {
                        return !validator_13.Validator.CompareValues(value, this._variableImportedValue, ParameterItemViewModel.numberMaxVariableAccuracy);
                    }
                    else {
                        return value != this._variableImportedValue;
                    }
                }
                else {
                    return false;
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "VariableImportedValueValidationMsg", {
            get: function () {
                switch (this._variableImportedValueValidationState) {
                    case validator_13.RangeValidationResult.NotValid:
                        return EditableVariableItemViewModel._translator(translator_service_5.TranslatorService.CfieldTranslationNs + "validationFailed");
                    default:
                        return "";
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "IsVariableImportedValueRangeValid", {
            // For imported value the range validator doen't invalidate the imported value
            get: function () {
                this._variableImportedValueValidationState = validator_13.Validator.ValidateVariableValue(this, this._variableImportedValue, this._variableFormattedMinValue, this._variableFormattedMaxValue);
                return this._variableImportedValueValidationState === validator_13.RangeValidationResult.Valid
                    || this._variableImportedValueValidationState === validator_13.RangeValidationResult.NotEvalueated;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "IsVariableImportedValueValid", {
            get: function () {
                if (this.IsVariableImportedValueSet && validator_13.Validator.isDefined(this._variableImportedValue)) {
                    this._variableImportedValueValidationState = validator_13.RangeValidationResult.Valid;
                    return true;
                }
                else {
                    this._variableImportedValueValidationState = validator_13.RangeValidationResult.NotValid;
                    return false;
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "VariableMinValueUom", {
            // override
            get: function () {
                return this.formatNumberValueUom(this._variableFormattedMinValue, this.VariableNoZoneUom.Symbol);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "VariableMaxValueUom", {
            // override
            get: function () {
                return this.formatNumberValueUom(this._variableFormattedMaxValue, this.VariableNoZoneUom.Symbol);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "VariableEditedValueUom", {
            // override
            get: function () {
                return this.composeVariableEditedValueUom(this.VariableNoZoneUom);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterItemViewModel.prototype, "ImportedValueItemPresenter", {
            get: function () {
                return this._importedValueItemPresenter;
            },
            enumerable: true,
            configurable: true
        });
        ;
        ParameterItemViewModel.prototype.refresh = function (resetEditing, variableCurrentValue, variableInitValue, variableDevValue) {
            _super.prototype.refresh.call(this, resetEditing, variableCurrentValue);
            this._variableInitValue = variableInitValue;
            this._variableDevValue = variableDevValue;
            // Reset imported value
            this.resetImportedValue();
        };
        ParameterItemViewModel.prototype.resetImportedValue = function () {
            this._variableImportedValue = null;
            this._variableImportedValueValidationState = null;
            this.IsVariableImportedValueEdited = false;
        };
        // override
        ParameterItemViewModel.prototype.initMin = function () {
            var isMinValueDefined = validator_13.Validator.isDefined(this._variable._minValue);
            var isMinIdDefined = validator_13.Validator.isDefined(this._variable._minId);
            if (!isMinValueDefined && !isMinIdDefined) {
                return null;
            }
            if (isMinValueDefined) {
                // Use default uom
                this._variableFormattedMinValue = this.formatNumberValue(this._variable._minValue);
            }
            else if (isMinIdDefined) {
                var minVariable = EditableVariableItemViewModel._variableDictionary[this._variable._minId];
                if (validator_13.Validator.isDefined(minVariable)) {
                    var numericValue = Number(minVariable._currentValue);
                    if (!isNaN(numericValue)) {
                        // Convert
                        var valueContainer = this.convertValueToOtherZone(this.VariableSupUom, numericValue, this.VariableNoZoneUom.UomDstId, uom_service_2.UomTarget.Default);
                        if (validator_13.Validator.isDefined(valueContainer)) {
                            this._variableFormattedMinValue = this.formatNumberValue(valueContainer.Value);
                        }
                        else {
                            // No conversion needed
                            this._variableFormattedMinValue = this.formatNumberValue(numericValue);
                        }
                    }
                }
            }
        };
        // override
        ParameterItemViewModel.prototype.initMax = function () {
            var isMaxValueDefined = validator_13.Validator.isDefined(this._variable._maxValue);
            var isMaxIdDefined = validator_13.Validator.isDefined(this._variable._maxId);
            if (!isMaxValueDefined && !isMaxIdDefined) {
                return null;
            }
            if (isMaxValueDefined) {
                // Use default uom
                this._variableFormattedMaxValue = this.formatNumberValue(this._variable._maxValue);
            }
            else if (isMaxIdDefined) {
                var maxVariable = EditableVariableItemViewModel._variableDictionary[this._variable._maxId];
                if (validator_13.Validator.isDefined(maxVariable)) {
                    var numericValue = Number(maxVariable._currentValue);
                    if (!isNaN(numericValue)) {
                        // Convert
                        var valueContainer = this.convertValueToOtherZone(this.VariableSupUom, numericValue, this.VariableNoZoneUom.UomDstId, uom_service_2.UomTarget.Default);
                        if (validator_13.Validator.isDefined(valueContainer)) {
                            this._variableFormattedMaxValue = this.formatNumberValue(valueContainer.Value);
                        }
                        else {
                            // No conversion needed
                            this._variableFormattedMaxValue = this.formatNumberValue(numericValue);
                        }
                    }
                }
            }
        };
        // override
        ParameterItemViewModel.prototype.formatNumberValue = function (value) {
            return formatter_1.Formatter.FormatNumber(value, this._dataType.Type, ParameterItemViewModel.numberMaxParameterAccuracy);
        };
        // override
        ParameterItemViewModel._log = new loggable_entity_3.LoggableEntityStatic("ParameterItemViewModel");
        ParameterItemViewModel.numberMaxParameterAccuracy = 2;
        return ParameterItemViewModel;
    }(EditableVariableItemViewModel));
    exports.ParameterItemViewModel = ParameterItemViewModel;
    var ValueContainer = /** @class */ (function () {
        function ValueContainer(_value, _uomData) {
            this._value = _value;
            this._uomData = _uomData;
        }
        Object.defineProperty(ValueContainer.prototype, "Value", {
            get: function () { return this._value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ValueContainer.prototype, "UomData", {
            get: function () { return this._uomData; },
            enumerable: true,
            configurable: true
        });
        return ValueContainer;
    }());
});
define("common/validator", ["require", "exports", "dataModel/datatype.model", "component/viewModel/item/variable-item.viewmodel", "ipaddr"], function (require, exports, datatype_model_6, variable_item_viewmodel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var Validator = /** @class */ (function () {
        function Validator() {
        }
        /**
         * <code>True</code> if variable is valid
         * @param variableVm
         * @param newValue
         */
        Validator.ValidateVariableValue = function (variable, newValue, minValue, maxValue) {
            var number;
            switch (variable.DataType) {
                case datatype_model_6.DataTypeEnum.Int:
                case datatype_model_6.DataTypeEnum.Uint:
                case datatype_model_6.DataTypeEnum.Sint:
                case datatype_model_6.DataTypeEnum.Usint:
                case datatype_model_6.DataTypeEnum.Dint:
                case datatype_model_6.DataTypeEnum.UdInt:
                case datatype_model_6.DataTypeEnum.Lint:
                case datatype_model_6.DataTypeEnum.Ulint:
                    // Validate the new value
                    var intValue = Validator.GetInt(newValue);
                    if (Validator.isDefined(intValue)) {
                        // New value is valid, check min and max
                        return Validator.RangeValidator(intValue, minValue, maxValue);
                    }
                    break;
                case datatype_model_6.DataTypeEnum.Real:
                case datatype_model_6.DataTypeEnum.Lreal: {
                    // Validate the new value. Which is the type of newValue???
                    var realValue = Validator.GetReal(newValue);
                    if (Validator.isDefined(realValue)) {
                        // New value is valid, check min and max
                        return Validator.RangeValidator(realValue, minValue, maxValue);
                    }
                    break;
                }
                case datatype_model_6.DataTypeEnum.Bool:
                    if (typeof newValue === "boolean") {
                        return RangeValidationResult.Valid;
                    }
                    break;
            }
            return RangeValidationResult.NotValid;
        };
        Validator.GetInt = function (value) {
            if (Validator.isDefined(value) && value !== "") {
                var number = Number(value);
                if (isNaN(number) === false) {
                    var intNumber = parseInt(value, 10);
                    if (number.valueOf() === intNumber) {
                        return intNumber;
                    }
                }
            }
            return null;
        };
        Validator.GetReal = function (value) {
            if (Validator.isDefined(value) && value !== "") {
                var number = Number(value);
                if (isNaN(number) === false) {
                    return number.valueOf();
                }
            }
            return null;
        };
        Validator.RangeValidator = function (value, minValue, maxValue) {
            var isMinValidationRequired = Validator.isDefined(minValue);
            var isMaxValidationRequired = Validator.isDefined(maxValue);
            var isLowerBoundValid = false;
            if (isMinValidationRequired) {
                isLowerBoundValid = value >= minValue;
            }
            var isUpperBoundValid = false;
            if (isMaxValidationRequired) {
                isUpperBoundValid = value <= maxValue;
            }
            // Check all cases
            if (isMinValidationRequired && isMaxValidationRequired) {
                if (isLowerBoundValid && isUpperBoundValid) {
                    return RangeValidationResult.Valid;
                }
                else {
                    return RangeValidationResult.OutOfRange;
                }
            }
            else if (isMinValidationRequired) {
                if (isLowerBoundValid) {
                    return RangeValidationResult.Valid;
                }
                else {
                    return RangeValidationResult.OutOfRange;
                }
            }
            else if (isMaxValidationRequired) {
                if (isUpperBoundValid) {
                    return RangeValidationResult.Valid;
                }
                else {
                    return RangeValidationResult.OutOfRange;
                }
            }
            return RangeValidationResult.NotEvalueated;
        };
        Validator.ValidateHostname = function (value) {
            return Validator.isDefined(value)
                && value.length > 1
                && value.length <= 24
                && Validator._hostnameRegexValidator.test(value);
        };
        Validator.ValidateIpv4 = function (ipAddress) {
            return ipaddr.IPv4.isValid(ipAddress);
            //return (/^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/.test(ipAddress)) ? true : false;
        };
        Validator.ValidateIpv6 = function (ipAddress) {
            return ipaddr.IPv6.isValid(ipAddress);
            //return (/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$|^(([a-zA-Z]|[a-zA-Z][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z]|[A-Za-z][A-Za-z0-9\-]*[A-Za-z0-9])$|^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$/.test(ipAddress)) ? true : false;
        };
        Validator.Ipv4ToString = function (ipAddress) {
            if (ipaddr.IPv4.isValid(ipAddress)) {
                var ip = ipaddr.IPv4.parse(ipAddress);
                if (Validator.isDefined(ip)) {
                    return ip.toString();
                }
            }
            return null;
        };
        Validator.Ipv6ToString = function (ipAddress) {
            if (ipaddr.IPv6.isValid(ipAddress)) {
                var ip = ipaddr.IPv6.parse(ipAddress);
                if (Validator.isDefined(ip)) {
                    return ip.toNormalizedPaddingString();
                }
            }
            return null;
        };
        Validator.CompareValues = function (a, b, accuracy) {
            // Poor logic
            // let delta: number = Math.abs(getValueNumber - setValueNumber);
            // return (setValueNumber / 100 < delta) 
            if (Validator.isDefined(a) && Validator.isDefined(b)) {
                return a.toFixed(accuracy) == b.toFixed(accuracy);
            }
            else {
                return false;
            }
        };
        Validator.getNum = function (value, defaultValue) {
            var num = parseInt(value);
            if (Validator.isDefined(num) && isNaN(num) === false) {
                return num;
            }
            else {
                return defaultValue;
            }
        };
        Validator.isDefined = function (value) {
            return typeof value !== "undefined" && value !== null;
        };
        Validator.GetVariableValidationRegex = function (dataTypeEnum) {
            switch (dataTypeEnum) {
                // Signed real        
                case datatype_model_6.DataTypeEnum.Real:
                case datatype_model_6.DataTypeEnum.Lreal:
                    return "^[-]?[0-9]+[.,]?[0-9]{0," + variable_item_viewmodel_1.EditableVariableItemViewModel.numberMaxVariableAccuracy + "}$";
                // Unsigned int
                case datatype_model_6.DataTypeEnum.Usint:
                case datatype_model_6.DataTypeEnum.Uint:
                case datatype_model_6.DataTypeEnum.UdInt:
                case datatype_model_6.DataTypeEnum.Ulint:
                    return "^[0-9]+$";
                // Signed int
                case datatype_model_6.DataTypeEnum.Sint:
                case datatype_model_6.DataTypeEnum.Int:
                case datatype_model_6.DataTypeEnum.Lint:
                case datatype_model_6.DataTypeEnum.Dint:
                    return "^[-]?[0-9]+";
                default:
                    return null;
            }
        };
        Validator.GetParameterValidationRegex = function (dataTypeEnum) {
            switch (dataTypeEnum) {
                // Signed real        
                case datatype_model_6.DataTypeEnum.Real:
                case datatype_model_6.DataTypeEnum.Lreal:
                    return "^[-]?[0-9]+[.,]?[0-9]{0," + variable_item_viewmodel_1.ParameterItemViewModel.numberMaxParameterAccuracy + "}$";
                // Unsigned int
                case datatype_model_6.DataTypeEnum.Usint:
                case datatype_model_6.DataTypeEnum.Uint:
                case datatype_model_6.DataTypeEnum.UdInt:
                case datatype_model_6.DataTypeEnum.Ulint:
                    return "^[0-9]+$";
                // Signed int
                case datatype_model_6.DataTypeEnum.Sint:
                case datatype_model_6.DataTypeEnum.Int:
                case datatype_model_6.DataTypeEnum.Lint:
                case datatype_model_6.DataTypeEnum.Dint:
                    return "^[-]?[0-9]+";
                default:
                    return null;
            }
        };
        Validator._hostnameRegexValidator = new RegExp("^([A-Za-z]|[A-Za-z][A-Za-z0-9\.\-]*[A-Za-z0-9])$");
        return Validator;
    }());
    exports.Validator = Validator;
    var RangeValidationResult;
    (function (RangeValidationResult) {
        RangeValidationResult[RangeValidationResult["NotValid"] = 0] = "NotValid";
        RangeValidationResult[RangeValidationResult["NotEvalueated"] = 1] = "NotEvalueated";
        RangeValidationResult[RangeValidationResult["OutOfRange"] = 2] = "OutOfRange";
        RangeValidationResult[RangeValidationResult["Valid"] = 3] = "Valid";
    })(RangeValidationResult = exports.RangeValidationResult || (exports.RangeValidationResult = {}));
    var IntValidationResult = /** @class */ (function () {
        function IntValidationResult(_isBoolean, _isInt, _intValue) {
            this._isBoolean = _isBoolean;
            this._isInt = _isInt;
            this._intValue = _intValue;
        }
        return IntValidationResult;
    }());
    var RealValidationResult = /** @class */ (function () {
        function RealValidationResult(_isBoolean, _isReal, _realValue) {
            this._isBoolean = _isBoolean;
            this._isReal = _isReal;
            this._realValue = _realValue;
        }
        return RealValidationResult;
    }());
});
define("dataModel/system-info.model", ["require", "exports", "common/validator"], function (require, exports, validator_14) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var AppMemoryInfo = /** @class */ (function () {
        function AppMemoryInfo(_ram, _rom, _code, _data, _vm, _chng, _conf, _prot) {
            this._ram = _ram;
            this._rom = _rom;
            this._code = _code;
            this._data = _data;
            this._vm = _vm;
            this._chng = _chng;
            this._conf = _conf;
            this._prot = _prot;
        }
        Object.defineProperty(AppMemoryInfo.prototype, "Ram", {
            get: function () { return this._ram; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppMemoryInfo.prototype, "Rom", {
            get: function () { return this._rom; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppMemoryInfo.prototype, "Code", {
            get: function () { return this._code; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppMemoryInfo.prototype, "Data", {
            get: function () { return this._data; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppMemoryInfo.prototype, "Vm", {
            get: function () { return this._vm; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppMemoryInfo.prototype, "Chng", {
            get: function () { return this._chng; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppMemoryInfo.prototype, "Conf", {
            get: function () { return this._conf; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppMemoryInfo.prototype, "Prot", {
            get: function () { return this._prot; },
            enumerable: true,
            configurable: true
        });
        return AppMemoryInfo;
    }());
    exports.AppMemoryInfo = AppMemoryInfo;
    var AppCycleTimeInfo = /** @class */ (function () {
        function AppCycleTimeInfo(_current, _max) {
            this._current = _current;
            this._max = _max;
        }
        Object.defineProperty(AppCycleTimeInfo.prototype, "Current", {
            get: function () { return this._current; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppCycleTimeInfo.prototype, "Max", {
            get: function () { return this._max; },
            enumerable: true,
            configurable: true
        });
        return AppCycleTimeInfo;
    }());
    exports.AppCycleTimeInfo = AppCycleTimeInfo;
    var AppInfo = /** @class */ (function () {
        function AppInfo(_appMemoryInfo, _appCycleTymeInfo) {
            this._appMemoryInfo = _appMemoryInfo;
            this._appCycleTymeInfo = _appCycleTymeInfo;
        }
        Object.defineProperty(AppInfo.prototype, "AppMemoryInfo", {
            get: function () { return this._appMemoryInfo; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppInfo.prototype, "AppCycleTymeInfo", {
            get: function () { return this._appCycleTymeInfo; },
            enumerable: true,
            configurable: true
        });
        return AppInfo;
    }());
    exports.AppInfo = AppInfo;
    var MemoryInfo = /** @class */ (function () {
        function MemoryInfo(_used, _free) {
            this._used = _used;
            this._free = _free;
        }
        Object.defineProperty(MemoryInfo.prototype, "Used", {
            get: function () { return this._used; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MemoryInfo.prototype, "Free", {
            get: function () { return this._free; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MemoryInfo.prototype, "Size", {
            get: function () { return this._used + this._free; },
            enumerable: true,
            configurable: true
        });
        return MemoryInfo;
    }());
    exports.MemoryInfo = MemoryInfo;
    var BaseMemory = /** @class */ (function () {
        function BaseMemory(_size, _free) {
            this._size = _size;
            this._free = _free;
        }
        Object.defineProperty(BaseMemory.prototype, "Used", {
            get: function () { return this._size - this._free; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseMemory.prototype, "Free", {
            get: function () { return this._free; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseMemory.prototype, "Size", {
            get: function () { return this._size; },
            enumerable: true,
            configurable: true
        });
        return BaseMemory;
    }());
    var Nand0Info = /** @class */ (function (_super) {
        __extends(Nand0Info, _super);
        function Nand0Info(_size, _free) {
            return _super.call(this, _size, _free) || this;
        }
        return Nand0Info;
    }(BaseMemory));
    exports.Nand0Info = Nand0Info;
    var Nand1Info = /** @class */ (function (_super) {
        __extends(Nand1Info, _super);
        function Nand1Info(_size, _free) {
            return _super.call(this, _size, _free) || this;
        }
        return Nand1Info;
    }(BaseMemory));
    exports.Nand1Info = Nand1Info;
    var Msd0Info = /** @class */ (function (_super) {
        __extends(Msd0Info, _super);
        function Msd0Info(_size, _free) {
            return _super.call(this, _size, _free) || this;
        }
        return Msd0Info;
    }(BaseMemory));
    exports.Msd0Info = Msd0Info;
    var Mcc0Info = /** @class */ (function (_super) {
        __extends(Mcc0Info, _super);
        function Mcc0Info(_size, _free) {
            return _super.call(this, _size, _free) || this;
        }
        return Mcc0Info;
    }(BaseMemory));
    exports.Mcc0Info = Mcc0Info;
    var FileSystemInfo = /** @class */ (function () {
        function FileSystemInfo(_nand0, _nand1, _msd0, _mcc0) {
            this._nand0 = _nand0;
            this._nand1 = _nand1;
            this._msd0 = _msd0;
            this._mcc0 = _mcc0;
        }
        Object.defineProperty(FileSystemInfo.prototype, "Nand0Info", {
            get: function () { return this._nand0; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileSystemInfo.prototype, "Nand1Info", {
            get: function () { return this._nand1; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileSystemInfo.prototype, "Msd0Info", {
            get: function () { return this._msd0; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileSystemInfo.prototype, "Mcc0Info", {
            get: function () { return this._mcc0; },
            enumerable: true,
            configurable: true
        });
        return FileSystemInfo;
    }());
    exports.FileSystemInfo = FileSystemInfo;
    var SystemInfo = /** @class */ (function () {
        function SystemInfo(_bootVersion, _osVersion, _coreType, _boardType, _pcoType, _macAddress, _uid, _tera, _appInfo, _memoryInfo, _fileSystemInfo, _hasBacnetServerClientLicense, _hasBacnetServerLicense, _hasNetLicense, _hasSnmpLicense, _hasDigitalSignature) {
            this._bootVersion = _bootVersion;
            this._osVersion = _osVersion;
            this._coreType = _coreType;
            this._boardType = _boardType;
            this._pcoType = _pcoType;
            this._macAddress = _macAddress;
            this._uid = _uid;
            this._tera = _tera;
            this._appInfo = _appInfo;
            this._memoryInfo = _memoryInfo;
            this._fileSystemInfo = _fileSystemInfo;
            this._hasBacnetServerClientLicense = _hasBacnetServerClientLicense;
            this._hasBacnetServerLicense = _hasBacnetServerLicense;
            this._hasNetLicense = _hasNetLicense;
            this._hasSnmpLicense = _hasSnmpLicense;
            this._hasDigitalSignature = _hasDigitalSignature;
        }
        Object.defineProperty(SystemInfo.prototype, "BootVersion", {
            get: function () { return this._bootVersion; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "OsVersion", {
            get: function () { return this._osVersion; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "CoreType", {
            get: function () { return this._coreType; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "BoardType", {
            get: function () { return this._boardType; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "PcoType", {
            get: function () { return this._pcoType; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "MacAddress", {
            get: function () { return this._macAddress; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "Uid", {
            get: function () { return this._uid; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "Tera", {
            get: function () { return this._tera; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "AppInfo", {
            get: function () { return this._appInfo; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "MemoryInfo", {
            get: function () { return this._memoryInfo; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "FileSystemInfo", {
            get: function () { return this._fileSystemInfo; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "HasBacnetServerClientLicense", {
            get: function () { return this._hasBacnetServerClientLicense; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "HasBacnetServerLicense", {
            get: function () { return this._hasBacnetServerLicense; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "HasNetLicense", {
            get: function () { return this._hasNetLicense; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "HasSnmpLicense", {
            get: function () { return this._hasSnmpLicense; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfo.prototype, "HasDigitalSignature", {
            get: function () { return this._hasDigitalSignature; },
            enumerable: true,
            configurable: true
        });
        return SystemInfo;
    }());
    exports.SystemInfo = SystemInfo;
    var SystemInfoUtil = /** @class */ (function () {
        function SystemInfoUtil() {
        }
        SystemInfoUtil.GetPcoTypeDescription = function (pcoTypeId) {
            var pcoTypeItem = (SystemInfoUtil._pcoInfoMap[pcoTypeId]);
            if (validator_14.Validator.isDefined(pcoTypeItem)) {
                return pcoTypeItem.name;
            }
            else {
                return null;
            }
        };
        SystemInfoUtil.GetBoardTypeDescription = function (pcoTypeId, boardTypeId) {
            var pcoTypeItem = (SystemInfoUtil._pcoInfoMap[pcoTypeId]);
            if (validator_14.Validator.isDefined(pcoTypeItem)) {
                var boardTypeItem = pcoTypeItem.boardType[boardTypeId];
                if (validator_14.Validator.isDefined(boardTypeItem)) {
                    return boardTypeItem.name;
                }
            }
            return null;
        };
        /**
         * Scan the <code>_pcoInfoMap</code> and extract the deviceFamilyId and modelId in accord to <code>deviceType</code>.
         * These values are used to select the entry in the device.json.
         * @param deviceType
         */
        SystemInfoUtil.GetDeviceFamilyAndModelForDeviceJson = function (deviceType) {
            var deviceFamilyEntry = SystemInfoUtil._pcoInfoMap[deviceType.PcoType];
            if (validator_14.Validator.isDefined(deviceFamilyEntry)) {
                var boardTypeEntry = deviceFamilyEntry.boardType[deviceType.BoardType];
                if (validator_14.Validator.isDefined(boardTypeEntry)) {
                    return [deviceFamilyEntry.deviceFamilyId, boardTypeEntry.modelId];
                }
            }
            return null;
        };
        SystemInfoUtil._pcoInfoMap = {
            // c.pco type key
            "12": {
                "name": "c.pCO",
                "deviceFamilyId": "pco5plus",
                "boardType": {
                    // Board type key
                    "10": {
                        "name": "Large",
                        "modelId": "large" // used in device.json (second level)
                    },
                    "11": {
                        "name": "Medium",
                        "modelId": "medium"
                    },
                    "12": {
                        "name": "Small",
                        "modelId": "small"
                    },
                    "13": {
                        "name": "XL",
                        "modelId": "extraLarge"
                    }
                }
            },
            // c.pco oem
            "13": {
                "name": "c.pco oem",
                "deviceFamilyId": "pcoOem",
                "boardType": {
                    "10": {
                        "name": "Small",
                        "modelId": "small"
                    },
                    "11": {
                        "name": "Medium",
                        "modelId": "medium"
                    }
                }
            },
            // c.pco mini
            "14": {
                "name": "c.pCOmini",
                "deviceFamilyId": "pcoMini",
                "boardType": {
                    "20": {
                        "name": "Basic",
                        "modelId": "miniBasic"
                    },
                    "21": {
                        "name": "Enhanced",
                        "modelId": "miniEnhanced"
                    },
                    "22": {
                        "name": "High end",
                        "modelId": "miniHighEnd"
                    }
                }
            },
            // upc3
            "17": {
                "name": "uPC3",
                "deviceFamilyId": "uPC3",
                "boardType": {
                    "10": {
                        "name": "Medium",
                        "modelId": "medium"
                    },
                    "11": {
                        "name": "Small",
                        "modelId": "small"
                    },
                    "12": {
                        "name": "Extra small",
                        "modelId": "extraSmall"
                    },
                    "13": {
                        "name": "Super medium",
                        "modelId": "superMedium"
                    }
                }
            },
            // c.pHc
            "18": {
                "name": "c.pHC",
                "deviceFamilyId": "pHC",
                "boardType": {
                    "10": {
                        "name": "Extended",
                        "modelId": "extended"
                    },
                    "11": {
                        "name": "Basic",
                        "modelId": "basic"
                    }
                }
            }
        };
        return SystemInfoUtil;
    }());
    exports.SystemInfoUtil = SystemInfoUtil;
});
define("dataModel/network-info.model", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var NetworkInfo = /** @class */ (function () {
        function NetworkInfo(
            // Common
            _hostname, _isMdnsActive, 
            // IPv4
            _ipv4IsEnable, _ipv4IsDhcpActive, _ipv4Ip, _ipv4Mask, _ipv4Gateway, _ipv4Dns, 
            // Ipv6
            Ipv6IsEnable, Ipv6IsDhcpActive, Ipv6LinkLocal, Ipv6Ip, Ipv6Gateway, Ipv6Dns) {
            this._hostname = _hostname;
            this._isMdnsActive = _isMdnsActive;
            this._ipv4IsEnable = _ipv4IsEnable;
            this._ipv4IsDhcpActive = _ipv4IsDhcpActive;
            this._ipv4Ip = _ipv4Ip;
            this._ipv4Mask = _ipv4Mask;
            this._ipv4Gateway = _ipv4Gateway;
            this._ipv4Dns = _ipv4Dns;
            this.Ipv6IsEnable = Ipv6IsEnable;
            this.Ipv6IsDhcpActive = Ipv6IsDhcpActive;
            this.Ipv6LinkLocal = Ipv6LinkLocal;
            this.Ipv6Ip = Ipv6Ip;
            this.Ipv6Gateway = Ipv6Gateway;
            this.Ipv6Dns = Ipv6Dns;
        }
        Object.defineProperty(NetworkInfo.prototype, "Hostname", {
            get: function () { return this._hostname; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkInfo.prototype, "IsMdnsActive", {
            get: function () { return this._isMdnsActive; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkInfo.prototype, "Ipv4IsEnable", {
            get: function () { return this._ipv4IsEnable; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkInfo.prototype, "Ipv4IsDhcpActive", {
            get: function () { return this._ipv4IsDhcpActive; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkInfo.prototype, "Ipv4Ip", {
            get: function () { return this._ipv4Ip; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkInfo.prototype, "Ipv4Mask", {
            get: function () { return this._ipv4Mask; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkInfo.prototype, "Ipv4Gateway", {
            get: function () { return this._ipv4Gateway; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkInfo.prototype, "Ipv4Dns", {
            get: function () { return this._ipv4Dns; },
            enumerable: true,
            configurable: true
        });
        return NetworkInfo;
    }());
    exports.NetworkInfo = NetworkInfo;
    var NetworkCommonInfo = /** @class */ (function () {
        function NetworkCommonInfo(_hostname, _isMdnsActive) {
            this._hostname = _hostname;
            this._isMdnsActive = _isMdnsActive;
        }
        Object.defineProperty(NetworkCommonInfo.prototype, "Hostname", {
            get: function () { return this._hostname; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkCommonInfo.prototype, "IsMdnsActive", {
            get: function () { return this._isMdnsActive; },
            enumerable: true,
            configurable: true
        });
        return NetworkCommonInfo;
    }());
    exports.NetworkCommonInfo = NetworkCommonInfo;
    var NetworkIpv4Info = /** @class */ (function () {
        function NetworkIpv4Info(_ipv4IsEnable, _ipv4IsDhcpActive, _ipv4Ip, _ipv4Mask, _ipv4Gateway, _ipv4Dns) {
            this._ipv4IsEnable = _ipv4IsEnable;
            this._ipv4IsDhcpActive = _ipv4IsDhcpActive;
            this._ipv4Ip = _ipv4Ip;
            this._ipv4Mask = _ipv4Mask;
            this._ipv4Gateway = _ipv4Gateway;
            this._ipv4Dns = _ipv4Dns;
        }
        Object.defineProperty(NetworkIpv4Info.prototype, "Ipv4IsEnable", {
            get: function () { return this._ipv4IsEnable; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkIpv4Info.prototype, "Ipv4IsDhcpActive", {
            get: function () { return this._ipv4IsDhcpActive; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkIpv4Info.prototype, "Ipv4Ip", {
            get: function () { return this._ipv4Ip; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkIpv4Info.prototype, "Ipv4Mask", {
            get: function () { return this._ipv4Mask; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkIpv4Info.prototype, "Ipv4Gateway", {
            get: function () { return this._ipv4Gateway; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkIpv4Info.prototype, "Ipv4Dns", {
            get: function () { return this._ipv4Dns; },
            enumerable: true,
            configurable: true
        });
        return NetworkIpv4Info;
    }());
    exports.NetworkIpv4Info = NetworkIpv4Info;
    var NetworkIpv6Info = /** @class */ (function () {
        function NetworkIpv6Info(_ipv6IsEnable, _ipv6IsDhcpActive, _ipv6Ip, _ipv6Gateway, _ipv6Dns) {
            this._ipv6IsEnable = _ipv6IsEnable;
            this._ipv6IsDhcpActive = _ipv6IsDhcpActive;
            this._ipv6Ip = _ipv6Ip;
            this._ipv6Gateway = _ipv6Gateway;
            this._ipv6Dns = _ipv6Dns;
        }
        Object.defineProperty(NetworkIpv6Info.prototype, "Ipv6IsEnable", {
            get: function () { return this._ipv6IsEnable; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkIpv6Info.prototype, "Ipv6IsDhcpActive", {
            get: function () { return this._ipv6IsDhcpActive; } // it's slaac flag
            ,
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkIpv6Info.prototype, "Ipv6Ip", {
            get: function () { return this._ipv6Ip; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkIpv6Info.prototype, "Ipv6Gateway", {
            get: function () { return this._ipv6Gateway; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NetworkIpv6Info.prototype, "Ipv6Dns", {
            get: function () { return this._ipv6Dns; },
            enumerable: true,
            configurable: true
        });
        return NetworkIpv6Info;
    }());
    exports.NetworkIpv6Info = NetworkIpv6Info;
});
define("service/info.service", ["require", "exports", "dataModel/system-info.model", "dataModel/network-info.model", "service/base.service", "common/validator"], function (require, exports, system_info_model_1, network_info_model_1, base_service_8, validator_15) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var InfoService = /** @class */ (function (_super) {
        __extends(InfoService, _super);
        function InfoService() {
            var _this = _super.call(this) || this;
            _this._cachedSystemInfo = null;
            _this.onSuccessSystemInfoPromise = function (data, textStatus, jqXHR) {
                // Create object
                var response = data;
                // Extract properties
                var boot, os, coreType, boardType, macAddress, uid, tera, pcotype;
                var appMemRam, appMemRom, appMemCode, appMemData, appMemVm, appMemChng, appMemConf, appMemProt, appCycleTimeCur, appCycleTimeMax, memUsed, memFree, fsNand0Size, fsNand0Free, fsNand1Size, fsNand1Free, fsMsd0Size, fsMsd0Free, fsMcc0Size, fsMcc0Free;
                var hasBacnetClientServerLicense, hasBacnetServerLicense, hasNetLicense, hasDigitalSignature, hasSnmpLicense;
                if (response.hasOwnProperty("bootv")) {
                    boot = response.bootv;
                }
                if (response.hasOwnProperty("osv")) {
                    os = response.osv;
                }
                if (response.hasOwnProperty("coretype")) {
                    coreType = response.coretype;
                }
                if (response.hasOwnProperty("boardtype")) {
                    boardType = response.boardtype;
                }
                if (response.hasOwnProperty("mac")) {
                    macAddress = response.mac;
                }
                if (response.hasOwnProperty("uid")) {
                    uid = response.uid;
                }
                if (response.hasOwnProperty("tera")) {
                    tera = response.tera;
                }
                if (response.hasOwnProperty("pcotype")) {
                    pcotype = response.pcotype;
                }
                if (response.hasOwnProperty("app")) {
                    var app = response.app;
                    if (app.hasOwnProperty("memory")) {
                        var memory = app.memory;
                        if (memory.hasOwnProperty("ram")) {
                            appMemRam = memory.ram;
                        }
                        if (memory.hasOwnProperty("rom")) {
                            appMemRom = memory.rom;
                        }
                        if (memory.hasOwnProperty("code")) {
                            appMemCode = memory.code;
                        }
                        if (memory.hasOwnProperty("data")) {
                            appMemData = memory.data;
                        }
                        if (memory.hasOwnProperty("vm")) {
                            appMemVm = memory.vm;
                        }
                        if (memory.hasOwnProperty("chng")) {
                            appMemChng = memory.chng;
                        }
                        if (memory.hasOwnProperty("conf")) {
                            appMemConf = memory.conf;
                        }
                        if (memory.hasOwnProperty("prot")) {
                            appMemProt = memory.prot;
                        }
                    }
                    if (app.hasOwnProperty("cycletime")) {
                        var cycletime = app.cycletime;
                        if (cycletime.hasOwnProperty("cur")) {
                            appCycleTimeCur = cycletime.cur;
                        }
                        if (cycletime.hasOwnProperty("max")) {
                            appCycleTimeMax = cycletime.max;
                        }
                    }
                }
                if (response.hasOwnProperty("mem")) {
                    var mem = response.mem;
                    if (mem.hasOwnProperty("used")) {
                        memUsed = mem.used;
                    }
                    if (mem.hasOwnProperty("free")) {
                        memFree = mem.free;
                    }
                }
                if (response.hasOwnProperty("fs")) {
                    var fs = response.fs;
                    if (fs.hasOwnProperty("nand:0:")) {
                        var nand0 = fs["nand:0:"];
                        if (nand0.hasOwnProperty("size")) {
                            fsNand0Size = nand0.size;
                        }
                        if (nand0.hasOwnProperty("free")) {
                            fsNand0Free = nand0.free;
                        }
                    }
                    if (fs.hasOwnProperty("nand:1:")) {
                        var nand1 = fs["nand:1:"];
                        if (nand1.hasOwnProperty("size")) {
                            fsNand1Size = nand1.size;
                        }
                        if (nand1.hasOwnProperty("free")) {
                            fsNand1Free = nand1.free;
                        }
                    }
                    if (fs.hasOwnProperty("msd:0:")) {
                        var msd0 = fs["msd:0:"];
                        if (msd0.hasOwnProperty("size")) {
                            fsMsd0Size = msd0.size;
                        }
                        if (msd0.hasOwnProperty("free")) {
                            fsMsd0Free = msd0.free;
                        }
                    }
                    if (fs.hasOwnProperty("mmc:0:")) {
                        var mcc0 = fs["mmc:0:"];
                        if (mcc0.hasOwnProperty("size")) {
                            fsMcc0Size = mcc0.size;
                        }
                        if (mcc0.hasOwnProperty("free")) {
                            fsMcc0Free = mcc0.free;
                        }
                    }
                }
                if (response.hasOwnProperty("plugins")) {
                    var pluginsProperty = response.plugins;
                    if (pluginsProperty.hasOwnProperty("bacnet")) {
                        hasBacnetClientServerLicense = true;
                    }
                    if (pluginsProperty.hasOwnProperty("snmp")) {
                        hasSnmpLicense = true;
                    }
                    if (pluginsProperty.hasOwnProperty("net")) {
                        hasNetLicense = true;
                    }
                    if (pluginsProperty.hasOwnProperty("digsgn")) {
                        hasDigitalSignature = true;
                    }
                }
                // Finish
                _this._cachedResolveSystemInfoPromise(new system_info_model_1.SystemInfo(boot, os, coreType, boardType, pcotype, macAddress, uid, tera, new system_info_model_1.AppInfo(new system_info_model_1.AppMemoryInfo(appMemRam, appMemRom, appMemCode, appMemData, appMemVm, appMemChng, appMemConf, appMemProt), new system_info_model_1.AppCycleTimeInfo(appCycleTimeCur, appCycleTimeMax)), new system_info_model_1.MemoryInfo(memUsed, memFree), new system_info_model_1.FileSystemInfo(new system_info_model_1.Nand0Info(fsNand0Size, fsNand0Free), new system_info_model_1.Nand1Info(fsNand1Size, fsNand1Free), new system_info_model_1.Msd0Info(fsMcc0Size, fsMsd0Free), new system_info_model_1.Mcc0Info(fsMcc0Size, fsMcc0Free)), hasBacnetClientServerLicense, hasBacnetServerLicense, hasNetLicense, hasSnmpLicense, hasDigitalSignature));
            };
            if (InfoService._instance) {
                _this._error(base_service_8.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            return _this;
        }
        InfoService.getInstance = function () {
            if (InfoService._instance === null) {
                InfoService._instance = new InfoService();
            }
            return InfoService._instance;
        };
        InfoService.prototype.getSystemInfo = function (force) {
            if (!force && this._cachedSystemInfo != null) {
                return Promise.resolve(this._cachedSystemInfo);
            }
            return new Promise(function (resolve, reject) {
                this._cachedResolveSystemInfoPromise = resolve;
                $.ajax({
                    type: "GET",
                    url: "/get_sys_info.cgi",
                    dataType: "json",
                    success: this.onSuccessSystemInfoPromise,
                    error: function (jqXHR, textStatus, errorThrown) {
                        reject("get SystemInfo (/get_sys_info.cgi) rejected. " + textStatus + ", " + errorThrown);
                    },
                    cache: false,
                    timeout: 10000
                });
            }.bind(this));
        };
        InfoService.prototype.getNetworkInfo = function (force) {
            return new Promise(function (resolve, reject) {
                $.ajax({
                    type: "GET",
                    url: "/get_net_info.cgi",
                    dataType: "json",
                    success: function (response, textStatus, jqXHR) {
                        this.onSuccessGetNetworkInfoPromise(response, resolve, reject);
                    }.bind(this),
                    error: function (jqXHR, textStatus, errorThrown) {
                        reject("get NetInfo rejected (/get_net_info.cgi). " + textStatus + ", " + errorThrown);
                    },
                    cache: false,
                    timeout: 10000
                });
            }.bind(this));
        };
        InfoService.prototype.onSuccessGetNetworkInfoPromise = function (response, resolve, reject) {
            if (validator_15.Validator.isDefined(response)) {
                // Extract common properties
                var hostname = void 0, isMdnsActive = void 0;
                if (validator_15.Validator.isDefined(response.common)) {
                    if (validator_15.Validator.isDefined(response.common.hostname)) {
                        hostname = response.common.hostname;
                    }
                    if (validator_15.Validator.isDefined(response.common.mdns)) {
                        isMdnsActive = response.common.mdns == 1;
                    }
                }
                // Extract ipv4. Mdns is a common property
                var ipv4IsEnabled = void 0, ipv4IsDhcpActive = void 0;
                var ipv4Ip = void 0, ipv4Mask = void 0, ipv4Gateway = void 0, ipv4Dns = void 0;
                if (validator_15.Validator.isDefined(response.ipv4)) {
                    if (validator_15.Validator.isDefined(response.ipv4.enable)) {
                        ipv4IsEnabled = response.ipv4.enable == 1;
                    }
                    if (validator_15.Validator.isDefined(response.ipv4.dhcp)) {
                        ipv4IsDhcpActive = response.ipv4.dhcp == 1;
                    }
                    if (validator_15.Validator.isDefined(response.ipv4.ip)) {
                        ipv4Ip = response.ipv4.ip.replace(/:/g, ".");
                    }
                    if (validator_15.Validator.isDefined(response.ipv4.mask)) {
                        ipv4Mask = response.ipv4.mask.replace(/:/g, ".");
                    }
                    if (validator_15.Validator.isDefined(response.ipv4.gw)) {
                        ipv4Gateway = response.ipv4.gw.replace(/:/g, ".");
                    }
                    if (validator_15.Validator.isDefined(response.ipv4.dns)) {
                        ipv4Dns = response.ipv4.dns.replace(/:/g, ".");
                    }
                }
                // Extract ipv6
                var ipv6IsEnabled = void 0, ipv6IsDhcpActive = void 0;
                var ipv6IpLinkLocal = void 0, ipv6Ip = void 0, ipv6Gateway = void 0, ipv6Dns = void 0;
                if (validator_15.Validator.isDefined(response.ipv6)) {
                    if (validator_15.Validator.isDefined(response.ipv6.enablev6)) {
                        ipv6IsEnabled = response.ipv6.enablev6 == 1;
                    }
                    if (validator_15.Validator.isDefined(response.ipv6.dhcpv6)) {
                        ipv6IsDhcpActive = response.ipv6.dhcpv6 == 1;
                    }
                    if (validator_15.Validator.isDefined(response.ipv6.ipv6_ll)) {
                        ipv6IpLinkLocal = response.ipv6.ipv6_ll;
                    }
                    if (validator_15.Validator.isDefined(response.ipv6.ipv6)) {
                        ipv6Ip = response.ipv6.ipv6;
                    }
                    if (validator_15.Validator.isDefined(response.ipv6.gwv6)) {
                        ipv6Gateway = response.ipv6.gwv6;
                    }
                    if (validator_15.Validator.isDefined(response.ipv6.dnsv6)) {
                        ipv6Dns = response.ipv6.dnsv6;
                    }
                }
                resolve(new network_info_model_1.NetworkInfo(hostname, isMdnsActive, ipv4IsEnabled, ipv4IsDhcpActive, ipv4Ip, ipv4Mask, ipv4Gateway, ipv4Dns, ipv6IsEnabled, ipv6IsDhcpActive, ipv6IpLinkLocal, ipv6Ip, ipv6Gateway, ipv6Dns));
            }
            else {
                reject("InfoService onSuccessGetNetworkInfoPromise(): no info network data");
            }
        };
        InfoService.prototype.setNetworkCommonParam = function (networkInfo) {
            if (validator_15.Validator.isDefined(networkInfo)) {
                var data = {};
                var atLeastAValidValue = false;
                if (validator_15.Validator.isDefined(networkInfo.Hostname)) {
                    data["hostname"] = networkInfo.Hostname;
                    atLeastAValidValue = true;
                }
                if (validator_15.Validator.isDefined(networkInfo.IsMdnsActive)) {
                    data["mdns"] = networkInfo.IsMdnsActive ? 1 : 0;
                    atLeastAValidValue = true;
                }
                if (atLeastAValidValue) {
                    this.setNetworkInfo(data);
                }
                else {
                    this._error("InfoService setNetworkCommonParam(): not init data");
                }
            }
            else {
                this._error("InfoService setNetworkCommonParam(): not valid data");
            }
        };
        InfoService.prototype.setNetworkIpv4Param = function (networkInfo) {
            if (validator_15.Validator.isDefined(networkInfo)) {
                var data = {};
                if (validator_15.Validator.isDefined(networkInfo.Ipv4IsEnable)) {
                    data["enable"] = networkInfo.Ipv4IsEnable ? 1 : 0;
                }
                if (networkInfo.Ipv4IsEnable === false) {
                    // ipv4 off
                    this.setNetworkInfo(data);
                }
                else {
                    if (validator_15.Validator.isDefined(networkInfo.Ipv4IsDhcpActive)) {
                        data["dhcp"] = networkInfo.Ipv4IsDhcpActive ? 1 : 0;
                    }
                    if (networkInfo.Ipv4IsDhcpActive) {
                        // ipv4 auto
                        this.setNetworkInfo(data);
                    }
                    else {
                        // ipv4 manual
                        var atLeastAValidValue = false;
                        if (validator_15.Validator.isDefined(networkInfo.Ipv4Ip)) {
                            data["ip"] = networkInfo.Ipv4Ip.replace(/:/g, "\.");
                            atLeastAValidValue = true;
                        }
                        if (validator_15.Validator.isDefined(networkInfo.Ipv4Mask)) {
                            data["mask"] = networkInfo.Ipv4Mask.replace(/:/g, "\.");
                            atLeastAValidValue = true;
                        }
                        if (validator_15.Validator.isDefined(networkInfo.Ipv4Gateway)) {
                            data["gw"] = networkInfo.Ipv4Gateway.replace(/:/g, "\.");
                            atLeastAValidValue = true;
                        }
                        if (validator_15.Validator.isDefined(networkInfo.Ipv4Dns)) {
                            data["dns"] = networkInfo.Ipv4Dns.replace(/:/g, "\.");
                            atLeastAValidValue = true;
                        }
                        if (atLeastAValidValue) {
                            this.setNetworkInfo(data);
                        }
                        else {
                            this._error("InfoService setNetworkIpv4Param(): not init data");
                        }
                    }
                }
            }
            else {
                this._error("InfoService setNetworkIpv4Param(): not valid data");
            }
        };
        InfoService.prototype.setNetworkIpv6Param = function (networkInfo) {
            if (validator_15.Validator.isDefined(networkInfo)) {
                var data = {};
                if (validator_15.Validator.isDefined(networkInfo.Ipv6IsEnable)) {
                    data["enablev6"] = networkInfo.Ipv6IsEnable ? 1 : 0;
                }
                if (networkInfo.Ipv6IsEnable === false) {
                    // ipv6 off
                    this.setNetworkInfo(data);
                }
                else {
                    if (validator_15.Validator.isDefined(networkInfo.Ipv6IsDhcpActive)) {
                        data["dhcpv6"] = networkInfo.Ipv6IsDhcpActive ? 1 : 0;
                    }
                    if (networkInfo.Ipv6IsDhcpActive) {
                        // ipv6 slaac
                        this.setNetworkInfo(data);
                    }
                    else {
                        // ipv6 manual
                        var atLeastAValidValue = false;
                        if (validator_15.Validator.isDefined(networkInfo.Ipv6Ip)) {
                            data["ipv6"] = networkInfo.Ipv6Ip;
                            atLeastAValidValue = true;
                        }
                        if (validator_15.Validator.isDefined(networkInfo.Ipv6Gateway)) {
                            data["gwv6"] = networkInfo.Ipv6Gateway;
                            atLeastAValidValue = true;
                        }
                        if (validator_15.Validator.isDefined(networkInfo.Ipv6Dns)) {
                            data["dnsv6"] = networkInfo.Ipv6Dns;
                            atLeastAValidValue = true;
                        }
                        if (atLeastAValidValue) {
                            this.setNetworkInfo(data);
                        }
                        else {
                            this._error("InfoService setNetworkIpv6Param(): not init data");
                        }
                    }
                }
            }
            else {
                this._error("InfoService setNetworkIpv6Param(): not valid data");
            }
        };
        InfoService.prototype.setNetworkInfo = function (dataToSend) {
            if (validator_15.Validator.isDefined(dataToSend)) {
                // Set apply
                dataToSend["apply"] = 1;
                // Send request
                $.ajax({
                    //type: "GET",
                    type: "POST",
                    url: "/set_net_info.cgi",
                    data: dataToSend,
                    success: function (response, textStatus, jqXHR) {
                        // Potentially I can't receive this answer if the network configuration changes
                        console.log("setNetworkInfo success");
                    }.bind(this),
                    error: function (jqXHR, textStatus, errorThrown) {
                        // Potentially I can't receive this answer if the network configuration changes
                        console.log("setNetworkInfo error. Details: " + errorThrown);
                    },
                    timeout: 10000,
                });
            }
            else {
                this._error("InfoService setNetworkInfo(): not valid data");
            }
        };
        /** Singleton instance. */
        InfoService._instance = null;
        return InfoService;
    }(base_service_8.BaseService));
    exports.InfoService = InfoService;
});
define("service/version/version-helper", ["require", "exports", "common/loggable-entity", "common/validator"], function (require, exports, loggable_entity_4, validator_16) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var VersionHelper = /** @class */ (function (_super) {
        __extends(VersionHelper, _super);
        function VersionHelper() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        VersionHelper.PrintCfieldVersion = function () {
            var revision;
            if (validator_16.Validator.isDefined(window)) {
                // Hardcoded property
                revision = window["cFieldRevision"];
            }
            VersionHelper._instance._log("cFieldRevision=" + revision);
        };
        VersionHelper.IsCfieldVersionCompatible = function (configurationXmlSchemaVersion, configurationCfieldVersion) {
            // Print version
            VersionHelper._instance._log("Configuration: xmlSchemaVersion=" + configurationXmlSchemaVersion +
                ", cFieldVersion=" + configurationCfieldVersion + "\
            Web pages: xmlSchemaVersion=" + VersionHelper.WebPageXmlSchemaVersion +
                ", cFieldVersion=" + VersionHelper.WebPageCfieldVersion);
            // Xml schema check
            if (VersionHelper.WebPageXmlSchemaVersion !== configurationXmlSchemaVersion) {
                VersionHelper._instance._log("VersionHelper: mismatch between the xml schema version of configuration and cField. Details: configurationXmlSchemaVersion: "
                    + configurationXmlSchemaVersion + ", webPageXmlSchemaVersion: " + VersionHelper.WebPageXmlSchemaVersion);
            }
            // Cfield version check
            if (!configurationCfieldVersion) {
                VersionHelper.PrintConfigurationCfieldVersionError(configurationCfieldVersion);
                return false;
            }
            var configurationCfieldVersionNumbers = configurationCfieldVersion.split(".");
            var webPageCfieldVersionNumbers = VersionHelper.WebPageCfieldVersion.split(".");
            if (configurationCfieldVersionNumbers.length != 3) {
                VersionHelper.PrintConfigurationCfieldVersionError(configurationCfieldVersion);
                return false;
            }
            for (var _i = 0, configurationCfieldVersionNumbers_1 = configurationCfieldVersionNumbers; _i < configurationCfieldVersionNumbers_1.length; _i++) {
                var versionNumber = configurationCfieldVersionNumbers_1[_i];
                if (validator_16.Validator.isDefined(versionNumber) == false) {
                    VersionHelper.PrintConfigurationCfieldVersionError(configurationCfieldVersion);
                    return false;
                }
            }
            // Check major release
            if (webPageCfieldVersionNumbers[0] != configurationCfieldVersionNumbers[0]) {
                VersionHelper._instance._log("VersionHelper: mismatch between the major release. Details: configurationMajorRelease: "
                    + configurationCfieldVersionNumbers[0] + ", webPageMajorRelease: " + webPageCfieldVersionNumbers[0]);
                return false;
            }
            // Check minor release
            var minorWebPageCfieldVersion = parseInt(webPageCfieldVersionNumbers[1], 10);
            var minorConfigurationCfieldVersion = parseInt(configurationCfieldVersionNumbers[1], 10);
            if (minorWebPageCfieldVersion > minorConfigurationCfieldVersion) {
                // Than the web page could n't have enough data 
                VersionHelper._instance._log("VersionHelper: cField minor release of configuration is bigger than webpage. Details: configurationMinorRelease: "
                    + configurationCfieldVersionNumbers[1] + ", webPageMinorRelease: " + webPageCfieldVersionNumbers[1]);
                return false;
            }
            return true;
        };
        /**
         * @return <code>False</code> only for pgd touch browser
         */
        VersionHelper.IsAgentSupported = function () {
            if (navigator) {
                var userAgent = navigator.userAgent;
                if (userAgent) {
                    var upperCaseUserAgent = userAgent.toUpperCase();
                    if (upperCaseUserAgent.includes("CAREL")) {
                        VersionHelper._instance._log("Browser not supported: " + userAgent);
                        return false;
                    }
                }
            }
            return true;
        };
        VersionHelper.PrintConfigurationCfieldVersionError = function (configurationCfieldVersion) {
            VersionHelper._instance._log("VersionHelper: the cfield version of configuration isn't valid. Details: configurationCfieldVersion: "
                + configurationCfieldVersion);
        };
        VersionHelper.CfieldVersion = "1.0.0";
        VersionHelper.WebPageXmlSchemaVersion = "1.1.001";
        VersionHelper.WebPageCfieldVersion = "1.0.000";
        VersionHelper._instance = new VersionHelper();
        return VersionHelper;
    }(loggable_entity_4.LoggableEntity));
    exports.VersionHelper = VersionHelper;
});
define("dataModel/application-info.model", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ApplicationInfo = /** @class */ (function () {
        function ApplicationInfo(_softwareName, _softwareVersion, _configurationName) {
            this._softwareName = _softwareName;
            this._softwareVersion = _softwareVersion;
            this._configurationName = _configurationName;
        }
        Object.defineProperty(ApplicationInfo.prototype, "SoftwareName", {
            get: function () { return this._softwareName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ApplicationInfo.prototype, "SoftwareVersion", {
            get: function () { return this._softwareVersion; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ApplicationInfo.prototype, "ConfigurationName", {
            get: function () { return this._configurationName; },
            enumerable: true,
            configurable: true
        });
        return ApplicationInfo;
    }());
    exports.ApplicationInfo = ApplicationInfo;
});
define("dataModel/io.model", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var IoBase = /** @class */ (function () {
        function IoBase(_pinId, _pinDisplayName, _isActive, _direction, _variableId, _channelTypeKey) {
            this._pinId = _pinId;
            this._pinDisplayName = _pinDisplayName;
            this._isActive = _isActive;
            this._direction = _direction;
            this._variableId = _variableId;
            this._channelTypeKey = _channelTypeKey;
        }
        Object.defineProperty(IoBase.prototype, "Pin", {
            get: function () { return this._pinId; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoBase.prototype, "PinDisplayName", {
            get: function () { return this._pinDisplayName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoBase.prototype, "IsActive", {
            get: function () { return this._isActive; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoBase.prototype, "VariableId", {
            get: function () { return this._variableId; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoBase.prototype, "Direction", {
            get: function () { return this._direction; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoBase.prototype, "ChannelTypeKey", {
            get: function () { return this._channelTypeKey; },
            enumerable: true,
            configurable: true
        });
        return IoBase;
    }());
    var IoUniversal = /** @class */ (function (_super) {
        __extends(IoUniversal, _super);
        function IoUniversal(_pinId, _pinDisplayName, _isActive, _direction, _variableId, _channelTypeKey, _minVariableId, _maxVariableId) {
            var _this = _super.call(this, _pinId, _pinDisplayName, _isActive, _direction, _variableId, _channelTypeKey) || this;
            _this._minVariableId = _minVariableId;
            _this._maxVariableId = _maxVariableId;
            return _this;
        }
        Object.defineProperty(IoUniversal.prototype, "MinVariableId", {
            get: function () { return this._minVariableId; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoUniversal.prototype, "MaxVariableId", {
            get: function () { return this._maxVariableId; },
            enumerable: true,
            configurable: true
        });
        return IoUniversal;
    }(IoBase));
    exports.IoUniversal = IoUniversal;
    var IoAnalogOutput = /** @class */ (function (_super) {
        __extends(IoAnalogOutput, _super);
        function IoAnalogOutput(_pinId, _pinDisplayName, _isActive, _variableId, _channelTypeKey) {
            return _super.call(this, _pinId, _pinDisplayName, _isActive, IoDirection.AOUT, _variableId, _channelTypeKey) || this;
        }
        return IoAnalogOutput;
    }(IoBase));
    exports.IoAnalogOutput = IoAnalogOutput;
    /**
     * Its direction is DIN_Standard (always). As consequence the unique channel type (or probe) is AIN_OnOff (see in the AIN list)
     * To set DIN_COUNT or DIN_FREQ you must use Universal pins
     */
    var IoDigitalInput = /** @class */ (function (_super) {
        __extends(IoDigitalInput, _super);
        function IoDigitalInput(_pinId, _pinDisplayName, _isActive, _variableId, _channelTypeKey) {
            return _super.call(this, _pinId, _pinDisplayName, _isActive, IoDirection.DI, _variableId, _channelTypeKey) || this;
        }
        return IoDigitalInput;
    }(IoBase));
    exports.IoDigitalInput = IoDigitalInput;
    var IoDigitalOutput = /** @class */ (function (_super) {
        __extends(IoDigitalOutput, _super);
        function IoDigitalOutput(_pinId, _pinDisplayName, _isActive, _variableId, _channelTypeKey) {
            return _super.call(this, _pinId, _pinDisplayName, _isActive, IoDirection.DOUT, _variableId, _channelTypeKey) || this;
        }
        return IoDigitalOutput;
    }(IoBase));
    exports.IoDigitalOutput = IoDigitalOutput;
    var IoConfiguration = /** @class */ (function () {
        function IoConfiguration(_ioUniversalArray, _ioAnalogOutputArray, _ioDigitalInputArray, _ioDigitalOutputArray) {
            this._ioUniversalArray = _ioUniversalArray;
            this._ioAnalogOutputArray = _ioAnalogOutputArray;
            this._ioDigitalInputArray = _ioDigitalInputArray;
            this._ioDigitalOutputArray = _ioDigitalOutputArray;
        }
        Object.defineProperty(IoConfiguration.prototype, "IoUniversalArray", {
            get: function () { return this._ioUniversalArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoConfiguration.prototype, "IoAnalogOutputArray", {
            get: function () { return this._ioAnalogOutputArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoConfiguration.prototype, "IoDigitalInputArray", {
            get: function () { return this._ioDigitalInputArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoConfiguration.prototype, "IoDigitalOutputArray", {
            get: function () { return this._ioDigitalOutputArray; },
            enumerable: true,
            configurable: true
        });
        return IoConfiguration;
    }());
    exports.IoConfiguration = IoConfiguration;
    var IoDirection;
    (function (IoDirection) {
        IoDirection[IoDirection["AOUT"] = 0] = "AOUT";
        IoDirection[IoDirection["DOUT"] = 1] = "DOUT";
        IoDirection[IoDirection["AIN"] = 2] = "AIN";
        IoDirection[IoDirection["DI"] = 3] = "DI";
        IoDirection[IoDirection["DI_COUNT"] = 4] = "DI_COUNT";
        IoDirection[IoDirection["DI_FREQ"] = 5] = "DI_FREQ";
    })(IoDirection = exports.IoDirection || (exports.IoDirection = {}));
    var IODirectionUtil = /** @class */ (function () {
        function IODirectionUtil() {
        }
        IODirectionUtil.GetIoDirectionString = function (ioDirection) {
            switch (ioDirection) {
                case IoDirection.AOUT:
                    return "AOUT";
                case IoDirection.DOUT:
                    return "DOUT";
                case IoDirection.AIN:
                    return "AIN";
                case IoDirection.DI:
                    return "DI";
                case IoDirection.DI_COUNT:
                    return "DI_COUNT";
                case IoDirection.DI_FREQ:
                    return "DI_FREQ";
            }
            throw Error("GetIoDirectionText(): input doesn't match the IoDirection" + ioDirection);
        };
        return IODirectionUtil;
    }());
    exports.IODirectionUtil = IODirectionUtil;
});
define("dataModel/log.model", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var BaseLogDefinitionModel = /** @class */ (function () {
        function BaseLogDefinitionModel(_name, _recordNumber, _loggedVariableIdArray) {
            this._name = _name;
            this._recordNumber = _recordNumber;
            this._loggedVariableIdArray = _loggedVariableIdArray;
        }
        Object.defineProperty(BaseLogDefinitionModel.prototype, "Id", {
            get: function () { return this._id; },
            set: function (value) { this._id = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseLogDefinitionModel.prototype, "Name", {
            get: function () { return this._name; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseLogDefinitionModel.prototype, "RecordNumber", {
            get: function () { return this._recordNumber; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseLogDefinitionModel.prototype, "LoggedVariableIdArray", {
            get: function () { return this._loggedVariableIdArray; },
            enumerable: true,
            configurable: true
        });
        return BaseLogDefinitionModel;
    }());
    exports.BaseLogDefinitionModel = BaseLogDefinitionModel;
    var PeriodicLogDefinitionModel = /** @class */ (function (_super) {
        __extends(PeriodicLogDefinitionModel, _super);
        function PeriodicLogDefinitionModel(name, _sampleVariableId, recordNumber, loggedVariableIdArray) {
            var _this = _super.call(this, name, recordNumber, loggedVariableIdArray) || this;
            _this._sampleVariableId = _sampleVariableId;
            return _this;
        }
        Object.defineProperty(PeriodicLogDefinitionModel.prototype, "SampleVariableId", {
            get: function () { return this._sampleVariableId; },
            enumerable: true,
            configurable: true
        });
        return PeriodicLogDefinitionModel;
    }(BaseLogDefinitionModel));
    exports.PeriodicLogDefinitionModel = PeriodicLogDefinitionModel;
    var EventLogDefinitionModel = /** @class */ (function (_super) {
        __extends(EventLogDefinitionModel, _super);
        function EventLogDefinitionModel(name, recordNumber, _eventVariableIdArray, loggedVariableIdArray) {
            var _this = _super.call(this, name, recordNumber, loggedVariableIdArray) || this;
            _this._eventVariableIdArray = _eventVariableIdArray;
            return _this;
        }
        return EventLogDefinitionModel;
    }(BaseLogDefinitionModel));
    exports.EventLogDefinitionModel = EventLogDefinitionModel;
    var LogDefinitionModel = /** @class */ (function () {
        function LogDefinitionModel() {
            this._periodicLogMap = {};
            this._eventLogMap = {};
        }
        Object.defineProperty(LogDefinitionModel.prototype, "PeriodicLogMap", {
            get: function () { return this._periodicLogMap; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogDefinitionModel.prototype, "EventLogMap", {
            get: function () { return this._eventLogMap; },
            enumerable: true,
            configurable: true
        });
        LogDefinitionModel.prototype.addPeriodicLog = function (periodicLogDefinition) {
            this._periodicLogMap[periodicLogDefinition.Name] = periodicLogDefinition;
        };
        LogDefinitionModel.prototype.addEventLog = function (eventLogDefinition) {
            this._eventLogMap[eventLogDefinition.Name] = eventLogDefinition;
        };
        return LogDefinitionModel;
    }());
    exports.LogDefinitionModel = LogDefinitionModel;
    var LogModel = /** @class */ (function () {
        function LogModel(_id, _startDate, _stopDate, _timezone, _data) {
            this._id = _id;
            this._startDate = _startDate;
            this._stopDate = _stopDate;
            this._timezone = _timezone;
            this._data = _data;
            this._logItemArray = [];
        }
        Object.defineProperty(LogModel.prototype, "LabelArray", {
            get: function () { return this._labelArray; },
            set: function (value) { this._labelArray = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogModel.prototype, "LogItemArray", {
            get: function () { return this._logItemArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogModel.prototype, "Id", {
            get: function () { return this._id; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogModel.prototype, "Timezone", {
            get: function () { return this._timezone; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogModel.prototype, "OriginalOffsetMinuteFromUtc", {
            get: function () { return this._originalOffsetMinuteFromUtc; },
            set: function (value) { this._originalOffsetMinuteFromUtc = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogModel.prototype, "RawData", {
            get: function () { return this._data; },
            enumerable: true,
            configurable: true
        });
        LogModel.prototype.addLogItem = function (item) {
            this._logItemArray.push(item);
        };
        return LogModel;
    }());
    exports.LogModel = LogModel;
    var LogItemModel = /** @class */ (function () {
        function LogItemModel(_logDate, _event, _logValueMap) {
            this._logDate = _logDate;
            this._event = _event;
            this._logValueMap = _logValueMap;
        }
        Object.defineProperty(LogItemModel.prototype, "LogDate", {
            get: function () { return this._logDate; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(LogItemModel.prototype, "LogValueMap", {
            get: function () { return this._logValueMap; },
            enumerable: true,
            configurable: true
        });
        ;
        return LogItemModel;
    }());
    exports.LogItemModel = LogItemModel;
});
define("service/device-config.service", ["require", "exports", "knockout", "x2js", "service/base.service", "service/info.service", "service/storage.service", "service/uom.service", "service/variable.service", "service/version/version-helper", "dataModel/application-info.model", "dataModel/datatype.model", "dataModel/system-info.model", "dataModel/io.model", "dataModel/log.model", "common/data-structure", "common/validator"], function (require, exports, ko, x2js, base_service_9, info_service_1, storage_service_1, uom_service_3, variable_service_2, version_helper_1, application_info_model_1, datatype_model_7, system_info_model_2, io_model_1, log_model_1, data_structure_4, validator_17) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Model the state of configuration initialization. Remember to update also the <code>ConfigService#isInitRunning</code> method.
     */
    var DeviceConfigServiceState;
    (function (DeviceConfigServiceState) {
        /** Uninit configuration. */
        DeviceConfigServiceState[DeviceConfigServiceState["UnInit"] = 0] = "UnInit";
        /** The initialization is started. */
        DeviceConfigServiceState[DeviceConfigServiceState["StartInit"] = 1] = "StartInit";
        /** Before the download the configuration. */
        DeviceConfigServiceState[DeviceConfigServiceState["OnDownloadConfig"] = 2] = "OnDownloadConfig";
        /** Before the building of the configuration (download is been completed). */
        DeviceConfigServiceState[DeviceConfigServiceState["OnBuildConfig"] = 3] = "OnBuildConfig";
        /** Configuration completed. */
        DeviceConfigServiceState[DeviceConfigServiceState["InitCompleted"] = 4] = "InitCompleted";
        /** Crc is changed. */
        DeviceConfigServiceState[DeviceConfigServiceState["CrcChanged"] = 5] = "CrcChanged";
        /** cField not supported. */
        DeviceConfigServiceState[DeviceConfigServiceState["CfieldNotSupported"] = 6] = "CfieldNotSupported";
        /** cField version not compatible. */
        DeviceConfigServiceState[DeviceConfigServiceState["CfieldVersionNotCompatible"] = 7] = "CfieldVersionNotCompatible";
        /** Error. */
        DeviceConfigServiceState[DeviceConfigServiceState["Error"] = 8] = "Error";
    })(DeviceConfigServiceState = exports.DeviceConfigServiceState || (exports.DeviceConfigServiceState = {}));
    // Save the state of reject promise. If you don't set the _state, the init method set it to Error
    var ErrorDeviceConfigContainer = /** @class */ (function () {
        function ErrorDeviceConfigContainer(_details, _state) {
            this._details = _details;
            this._state = _state;
        }
        return ErrorDeviceConfigContainer;
    }());
    /**
     * Service for the configuration of device.
     */
    var DeviceConfigService = /** @class */ (function (_super) {
        __extends(DeviceConfigService, _super);
        /**
         * Init service.
         * @param baseUrl Url of cfield root directory.
         */
        function DeviceConfigService(baseUrl) {
            var _this = _super.call(this) || this;
            _this._crcCount = 0;
            /** Dictionary of variable. */
            _this._mainVariableDictionary = {};
            _this._variableCategoryDictionary = {};
            /** Dictionary of alarms. */
            _this._alarmDictionary = {};
            _this._deviceInfo = null;
            _this._deviceConfiguration = null;
            _this.periodicCrcCheck = function () {
                var crcPromise = _this.getConfigurationCrc();
                crcPromise
                    .then(function (currentCrc) {
                    // When deviceConfigService is refreshed than this._crcConfig == null
                    if (_this._crcConfig != null && _this._crcConfig !== currentCrc) {
                        if (_this._crcCount > DeviceConfigService._crcThreshold) {
                            // Configuration changed
                            _this._deviceConfigServiceStateObservable(DeviceConfigServiceState.CrcChanged);
                            _this._crcCount = 0;
                        }
                        else {
                            _this._crcCount++;
                            _this._error("DeviceConfigService periodicCrcCheck(). Crc check is failed. Number of failed attempts: " + _this._crcCount);
                            // Retry
                            setTimeout(_this.periodicCrcCheck, DeviceConfigService.PeriodicCheckCrcDelayMs);
                        }
                    }
                    else {
                        _this._crcCount = 0;
                        // Retry
                        setTimeout(_this.periodicCrcCheck, DeviceConfigService.PeriodicCheckCrcDelayMs);
                    }
                })
                    .catch(function (reason) {
                    _this._error("DeviceConfigurationService periodicCrcCheck() error + " + reason);
                    _this._deviceConfigServiceStateObservable(DeviceConfigServiceState.Error);
                });
            };
            if (DeviceConfigService._instance) {
                _this._error(base_service_9.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            _this._baseUrl = baseUrl;
            _this._deviceConfigServiceStateObservable = ko.observable(DeviceConfigServiceState.UnInit);
            _this._isDeviceConfigurationSuddenlyChanged = ko.observable(false);
            return _this;
        }
        Object.defineProperty(DeviceConfigService.prototype, "DeviceConfigServiceStateObservable", {
            /** Get the observable of config service state. */
            get: function () {
                return this._deviceConfigServiceStateObservable;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DeviceConfigService.prototype, "IsDeviceConfigurationSuddenlyChanged", {
            get: function () { return this._isDeviceConfigurationSuddenlyChanged; },
            enumerable: true,
            configurable: true
        });
        /**
         * Build the instance of config service in accord with singleton pattern.
         * @param baseUrl Url of cfield root directory.
         * @return an instance of config service.
         */
        DeviceConfigService.buildInstance = function (baseUrl) {
            if (DeviceConfigService._instance === null) {
                DeviceConfigService._instance = new DeviceConfigService(baseUrl);
            }
            return DeviceConfigService._instance;
        };
        /**
         * Get the instance of config service in accord with singleton pattern.
         * @return an instance of config service.
         */
        DeviceConfigService.getInstance = function () {
            if (DeviceConfigService._instance === null) {
                throw new Error("DeviceConfigService: the instance is null");
            }
            return DeviceConfigService._instance;
        };
        /**
         * Reset state.
         */
        DeviceConfigService.prototype.resetState = function () {
            this.clearCache();
            this._deviceConfigServiceStateObservable(DeviceConfigServiceState.UnInit);
            this._isDeviceConfigurationSuddenlyChanged(false);
            this._xmlConfig = null;
            this._crcConfig = null;
            this._crcCount = 0;
            this._mainVariableDictionary = {};
            this._variableCategoryDictionary = {};
            this._alarmDictionary = {};
            this._applicationInfo = null;
            this._deviceInfo = null;
            this._deviceConfiguration = null;
            this._staticDeviceConfigurationPromise = null; // TODO: refactor to remove it
            this._ioConfigurationPromise = null;
            this._logDefinitionModel = null;
        };
        /**
         * Clear cache.
         */
        DeviceConfigService.prototype.clearCache = function () {
            if (localStorage) {
                localStorage.removeItem(storage_service_1.StorageService.CrcConfigurationStorageKey);
                localStorage.removeItem(storage_service_1.StorageService.XmlConfigurationStoragekey);
            }
        };
        Object.defineProperty(DeviceConfigService.prototype, "VariablesDictionary", {
            get: function () {
                return this._mainVariableDictionary;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DeviceConfigService.prototype, "VariableCategoryDictionary", {
            get: function () {
                return this._variableCategoryDictionary;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DeviceConfigService.prototype, "AlarmDictionary", {
            get: function () {
                return this._alarmDictionary;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DeviceConfigService.prototype, "ApplicationInfo", {
            get: function () { return this._applicationInfo; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DeviceConfigService.prototype, "LogDefinitionModel", {
            get: function () { return this._logDefinitionModel; },
            enumerable: true,
            configurable: true
        });
        /**
         * Init the configuration.
         * @return a promise for the configuration initialization.
         */
        DeviceConfigService.prototype.initialize = function () {
            // Change state
            this._deviceConfigServiceStateObservable(DeviceConfigServiceState.StartInit);
            // Build promise
            return new Promise(function (resolve, reject) {
                var _this = this;
                // Change state
                this._deviceConfigServiceStateObservable(DeviceConfigServiceState.OnDownloadConfig);
                // 1) Get current crc
                return this.getConfigurationCrc()
                    .then(function (currentCrc) {
                    // 2) Get configuration (online or cached). Include the getXml() CGI
                    _this._crcConfig = currentCrc;
                    return _this.getXmlConfiguration(currentCrc);
                })
                    .then(function (xmlConfig) {
                    // 3) Init the configuration
                    _this._deviceConfigServiceStateObservable(DeviceConfigServiceState.OnBuildConfig);
                    _this._xmlConfig = xmlConfig;
                    return _this.initConfig(xmlConfig);
                })
                    .then(function () {
                    // 4) Cache xml if necessary
                    if (localStorage) {
                        try {
                            if (_this._crcConfig && _this._crcConfig !== "null" && _this._xmlConfig && _this._xmlConfig !== "null") {
                                localStorage.setItem(storage_service_1.StorageService.CrcConfigurationStorageKey, _this._crcConfig);
                                localStorage.setItem(storage_service_1.StorageService.XmlConfigurationStoragekey, JSON.stringify(_this._xmlConfig));
                            }
                            else {
                                // Clear residual
                                localStorage.removeItem(storage_service_1.StorageService.CrcConfigurationStorageKey);
                                localStorage.removeItem(storage_service_1.StorageService.XmlConfigurationStoragekey);
                            }
                        }
                        catch (err) {
                            _this._error("DeviceConfigService failed to manage local storage: " + err.message);
                        }
                    }
                    // Periodic monitor of crc changing
                    setTimeout(function () {
                        this.periodicCrcCheck();
                    }.bind(_this), DeviceConfigService.PeriodicCheckCrcDelayMs);
                    // Resolve main promise
                    resolve();
                    _this._deviceConfigServiceStateObservable(DeviceConfigServiceState.InitCompleted);
                })
                    .catch(function (reason) {
                    var error;
                    var state;
                    if (reason instanceof ErrorDeviceConfigContainer) {
                        error = reason._details;
                        state = validator_17.Validator.isDefined(reason._state) ? reason._state : DeviceConfigServiceState.Error;
                    }
                    else {
                        error = reason;
                        state = DeviceConfigServiceState.Error;
                    }
                    _this._error(reason);
                    _this._deviceConfigServiceStateObservable(state);
                });
            }.bind(this));
        };
        /**
         * Get current crc
         */
        DeviceConfigService.prototype.getConfigurationCrc = function () {
            return new Promise(function (resolve, reject) {
                $.ajax({
                    type: "GET",
                    url: "/get_xml_conf.cgi?type=c",
                    success: function (data, textStatus, jqXHR) {
                        resolve(jqXHR.responseText);
                    },
                    error: function (jqXHR, textStatus, errorThrown) {
                        reject("get_xml_conf.cgi error: " + textStatus + "  " + errorThrown);
                    },
                    cache: false,
                    timeout: 10000
                });
            });
        };
        /**
         * Get current configuration
         * @param currentCrc Current crc
         */
        DeviceConfigService.prototype.getXmlConfiguration = function (currentCrc) {
            // Check if xml is cached
            if (localStorage) {
                var crcLocalStorage = localStorage.getItem(storage_service_1.StorageService.CrcConfigurationStorageKey);
                var xmlLocalStorage = localStorage.getItem(storage_service_1.StorageService.XmlConfigurationStoragekey);
                if (crcLocalStorage && xmlLocalStorage) {
                    var skip = false;
                    // Ensure that "null" string value aren't saved
                    if (crcLocalStorage === "null") {
                        // Remove
                        localStorage.removeItem(storage_service_1.StorageService.CrcConfigurationStorageKey);
                        skip = true;
                    }
                    if (xmlLocalStorage === "null") {
                        // Remove
                        localStorage.removeItem(storage_service_1.StorageService.XmlConfigurationStoragekey);
                        skip = true;
                    }
                    if (skip === false) {
                        // Compare crc
                        if (crcLocalStorage == currentCrc) {
                            // Return from storage
                            return Promise.resolve(JSON.parse(xmlLocalStorage));
                        }
                    }
                }
            }
            return new Promise(function (resolve, reject) {
                $.ajax({
                    type: "GET",
                    url: "/get_xml_conf.cgi?type=x",
                    success: function (data, textStatus, jqXHR) {
                        resolve(jqXHR.responseText);
                    },
                    error: function (jqXHR, textStatus, errorThrown) {
                        reject("get_xml_conf.cgi error: " + textStatus + "  " + errorThrown);
                    },
                    cache: false,
                    timeout: 120000
                });
            }.bind(this))
                .then(function (xmlText) {
                // Request 2: Covert xml to javascript object
                var parser = new x2js();
                return Promise.resolve(parser.xml2js(xmlText));
            });
        };
        /**
         * Build the configuration.
         * @return the promise for the configuration.
         */
        DeviceConfigService.prototype.initConfig = function (xmlConfig) {
            var _this = this;
            // Check null
            if (xmlConfig === null || (xmlConfig != null && xmlConfig.isai == null)) {
                this._deviceConfigServiceStateObservable(DeviceConfigServiceState.Error);
                return Promise.reject("xmlConfig is null");
            }
            // Check if exist cField section
            if (!xmlConfig.isai.cfield) {
                return Promise.reject(new ErrorDeviceConfigContainer("cField data not found", DeviceConfigServiceState.CfieldNotSupported));
            }
            // Check if cField version is compatible
            if (!version_helper_1.VersionHelper.IsCfieldVersionCompatible(xmlConfig.isai._version, xmlConfig.isai.cfield._version)) {
                return Promise.reject(new ErrorDeviceConfigContainer("cField version not compatible", DeviceConfigServiceState.CfieldVersionNotCompatible));
            }
            // Monitor performance
            var tStart = new Date();
            // Build model
            return this.initData(xmlConfig)
                .then(function () {
                // Init UomService
                return uom_service_3.UoMService.getInstance().init();
            })
                .then(function () {
                if (xmlConfig.isai.cfield) {
                    // Build Application info
                    _this._applicationInfo = new application_info_model_1.ApplicationInfo(xmlConfig.isai.cfield._swName, xmlConfig.isai.cfield._swVers, xmlConfig.isai.cfield._confName);
                    // Build log info
                    _this._logDefinitionModel = _this.buildLogInfo();
                }
                else {
                    _this._log("cField section is not defined");
                    _this._applicationInfo = new application_info_model_1.ApplicationInfo(null, null, null);
                }
                // I finish. Resolve main promise
                var tEnd = new Date();
                _this._log("Required time to build map of variables (in ms):" + (tEnd.getTime() - tStart.getTime()));
                return Promise.resolve();
            });
        };
        /**
         * Build data model used by cField.
         * @param pcoConfig Configuration (Json object).
         */
        DeviceConfigService.prototype.initData = function (pcoConfig) {
            return new Promise(function (resolve, reject) {
                // Check if web worker is supported
                if (typeof (Worker) !== "undefined") {
                    // Define web worker
                    var worker = new Worker(this._baseUrl + "/lib/cfieldWebWorker.js");
                    // Callback for result
                    worker.onmessage = function (event) {
                        var isError = false;
                        // Print error
                        var errorMsgArray = event.data.errorMsg;
                        if (validator_17.Validator.isDefined(errorMsgArray)) {
                            for (var _i = 0, errorMsgArray_1 = errorMsgArray; _i < errorMsgArray_1.length; _i++) {
                                var msg = errorMsgArray_1[_i];
                                this._log("Error: " + msg);
                            }
                            if (errorMsgArray.length > 0) {
                                isError = true;
                            }
                        }
                        // Print log 
                        var logMsgArray = event.data.logMsg;
                        if (validator_17.Validator.isDefined(logMsgArray)) {
                            for (var _a = 0, logMsgArray_1 = logMsgArray; _a < logMsgArray_1.length; _a++) {
                                var msg = logMsgArray_1[_a];
                                this._log("Log: " + msg);
                            }
                        }
                        if (isError == true) {
                            reject("Error on cField web worker.");
                            return;
                        }
                        // Retrieve data
                        this._mainVariableDictionary = event.data.mainVarDictionary;
                        this._alarmDictionary = event.data.alarmDictionary;
                        this._variableCategoryDictionary = event.data.variableCategoryDictionary;
                        var uomConfiguration = event.data.uoMConfiguration;
                        var uoMZoneAlarmResult = event.data.uoMZoneAlarmResult;
                        var uoMZoneLogResult = event.data.uoMZoneLogResult;
                        var uoMZoneSupResult = event.data.uoMZoneSupResult;
                        // Validate
                        if (!validator_17.Validator.isDefined(this._mainVariableDictionary)
                            || !validator_17.Validator.isDefined(this._alarmDictionary)
                            || !validator_17.Validator.isDefined(this._variableCategoryDictionary)
                            || !validator_17.Validator.isDefined(uomConfiguration)
                            || !validator_17.Validator.isDefined(uoMZoneAlarmResult)
                            || !validator_17.Validator.isDefined(uoMZoneLogResult)
                            || !validator_17.Validator.isDefined(uoMZoneSupResult)) {
                            reject("Error on cField web worker. Invalid returned data");
                            return;
                        }
                        // Build uom service
                        uom_service_3.UoMService.buildInstance(uomConfiguration, uoMZoneAlarmResult, uoMZoneLogResult, uoMZoneSupResult);
                        // Release resource
                        worker.terminate();
                        worker = undefined;
                        // Resolve promise
                        resolve();
                    }.bind(this);
                    // Callback for the error 
                    worker.onerror = function (event) {
                        this._deviceConfigServiceStateObservable(DeviceConfigServiceState.Error);
                        reject("Error on cField web worker. Details: " + event.message + " (" + event.filename + ":" + event.lineno + ")");
                    }.bind(this);
                    // Get data type dictionary
                    var datatypeDictionary = datatype_model_7.DataTypeUtil.getInstance().DataTypeDictionary;
                    // Start work
                    worker.postMessage({
                        pcoConfig: pcoConfig,
                        baseUrl: this._baseUrl,
                        datatypeDictionary: datatypeDictionary
                    });
                }
                else {
                    this._deviceConfigServiceStateObservable(DeviceConfigServiceState.Error);
                    reject("Web worker isn't supported");
                }
            }.bind(this));
        };
        // IO    
        /**
         * Get io configuration.
         */
        DeviceConfigService.prototype.getIoConfiguration = function () {
            var _this = this;
            var deviceStaticConfigurationCached;
            // 1) Get static device configuration
            var staticConfigurationDevice = this.getStaticDeviceConfiguration();
            this._ioConfigurationPromise = staticConfigurationDevice
                .then(function (deviceStaticConfiguration) {
                deviceStaticConfigurationCached = deviceStaticConfiguration;
                // 2) Update IO variables
                return _this.updateIoVariables();
            })
                .then(function (getSetVarResponse) {
                var variables = getSetVarResponse._deviceVariableArray;
                // Build cField data model
                // Universal
                var ioUniversalModelArray = [];
                var universalStaticPinArray = deviceStaticConfigurationCached.channels.universal;
                if (validator_17.Validator.isDefined(universalStaticPinArray)) {
                    var universalIsaiPinArray = data_structure_4.ArrayUtil.GetArray(_this._xmlConfig.isai.isai_io.u);
                    for (var _i = 0, universalStaticPinArray_1 = universalStaticPinArray; _i < universalStaticPinArray_1.length; _i++) {
                        var universalStaticPin = universalStaticPinArray_1[_i];
                        ioUniversalModelArray.push(_this.buildUniversalPin(deviceStaticConfigurationCached, universalIsaiPinArray, universalStaticPin));
                    }
                }
                // Digital input
                var ioDigitalInputModelArray = [];
                var digitalInputStaticPinArray = deviceStaticConfigurationCached.channels.digitalInput;
                if (digitalInputStaticPinArray) {
                    var digitalInputIsaiPinArray = data_structure_4.ArrayUtil.GetArray(_this._xmlConfig.isai.isai_io.di);
                    for (var _a = 0, digitalInputStaticPinArray_1 = digitalInputStaticPinArray; _a < digitalInputStaticPinArray_1.length; _a++) {
                        var digitalInputStaticPin = digitalInputStaticPinArray_1[_a];
                        ioDigitalInputModelArray.push(_this.buildDigitalInputPin(deviceStaticConfigurationCached, digitalInputIsaiPinArray, digitalInputStaticPin));
                    }
                }
                // Digital output
                var ioDigitalOutputModelArray = [];
                var digitalOutputStaticPinArray = deviceStaticConfigurationCached.channels.digitalOutput;
                if (digitalOutputStaticPinArray) {
                    var digitalOutputIsaiPinArray = data_structure_4.ArrayUtil.GetArray(_this._xmlConfig.isai.isai_io.do);
                    for (var _b = 0, digitalOutputStaticPinArray_1 = digitalOutputStaticPinArray; _b < digitalOutputStaticPinArray_1.length; _b++) {
                        var digitalOutputStaticPin = digitalOutputStaticPinArray_1[_b];
                        ioDigitalOutputModelArray.push(_this.buildDigitalOutputPin(deviceStaticConfigurationCached, digitalOutputIsaiPinArray, digitalOutputStaticPin));
                    }
                }
                // Analog output
                var ioAnalogOutputModelArray = [];
                var analogOutputStaticPinArray = deviceStaticConfigurationCached.channels.analogOutput;
                if (analogOutputStaticPinArray) {
                    var analogOutputIsaiPinArray = data_structure_4.ArrayUtil.GetArray(_this._xmlConfig.isai.isai_io.y);
                    for (var _c = 0, analogOutputStaticPinArray_1 = analogOutputStaticPinArray; _c < analogOutputStaticPinArray_1.length; _c++) {
                        var analogOutputStaticPin = analogOutputStaticPinArray_1[_c];
                        ioAnalogOutputModelArray.push(_this.buildAnalogOutputPin(deviceStaticConfigurationCached, analogOutputIsaiPinArray, analogOutputStaticPin));
                    }
                }
                // Packing and return
                return Promise.resolve(new io_model_1.IoConfiguration(ioUniversalModelArray, ioAnalogOutputModelArray, ioDigitalInputModelArray, ioDigitalOutputModelArray));
            });
            //this._ioConfigurationPromise = <any>staticConfigurationDevice;
            return this._ioConfigurationPromise;
        };
        /**
         * Get static device configuration.
         */
        DeviceConfigService.prototype.getStaticDeviceConfiguration = function () {
            var _this = this;
            if (this._staticDeviceConfigurationPromise == null) {
                var deviceEntryCache_1;
                // Init promise. It's type is "any" because the final retured value comes from getDeviceStaticInfo()
                var resultPromise = this.getDeviceManifest();
                // See http://stackoverflow.com/questions/29111626/javascript-promise-then-ordering
                resultPromise = resultPromise
                    .then(function (deviceEntry) {
                    deviceEntryCache_1 = deviceEntry;
                    return _this.getDeviceModel();
                })
                    .then(function (deviceModel) {
                    // Get the device configuation
                    return _this.getDeviceStaticInfo(deviceEntryCache_1, deviceModel);
                });
                this._staticDeviceConfigurationPromise = resultPromise;
            }
            return this._staticDeviceConfigurationPromise;
        };
        /**
         * Get device manifest.
         */
        DeviceConfigService.prototype.getDeviceManifest = function () {
            return new Promise(function (resolve, reject) {
                var baseUrl = this._baseUrl;
                $.ajax({
                    dataType: "json",
                    url: baseUrl + "/" + DeviceConfigService.DeviceManifestPath,
                    success: function (data, textStatus, jqXHR) {
                        resolve(data);
                    },
                    error: function (jqXHR, textStatus, errorThrown) {
                        reject("Failed to load " + DeviceConfigService.DeviceManifestPath + " file. " + textStatus + "  " + errorThrown);
                    },
                    timeout: 5000
                });
            }.bind(this));
        };
        /**
         * Get device model.
         */
        DeviceConfigService.prototype.getDeviceModel = function () {
            var infoService = info_service_1.InfoService.getInstance();
            return infoService.getSystemInfo();
        };
        /**
         * Get the device static configuration.
         */
        DeviceConfigService.prototype.getDeviceStaticInfo = function (supportedDevice, deviceType) {
            var deviceQueryParams = system_info_model_2.SystemInfoUtil.GetDeviceFamilyAndModelForDeviceJson(deviceType);
            if (deviceQueryParams == null) {
                return Promise.reject("Device not found in the admissible map");
            }
            var deviceFamilyId = deviceQueryParams[0];
            var modelId = deviceQueryParams[1];
            // Search entry in the device manifest
            var isFound = false;
            for (var _i = 0, supportedDevice_1 = supportedDevice; _i < supportedDevice_1.length; _i++) {
                var deviceFamily = supportedDevice_1[_i];
                if (deviceFamily.deviceFamilyId === deviceFamilyId) {
                    for (var _a = 0, _b = deviceFamily.devices; _a < _b.length; _a++) {
                        var device = _b[_a];
                        if (device.modelId === modelId) {
                            if (device.path) {
                                this._deviceInfo = new DeviceInfo(deviceFamilyId, modelId, device.path);
                                isFound = true;
                                break;
                            }
                        }
                    }
                }
                if (isFound) {
                    break;
                }
            }
            if (!isFound) {
                // Device not found
                return Promise.reject("Device not found in the manifest");
            }
            // Download file 
            return new Promise(function (resolve, reject) {
                $.ajax({
                    dataType: "json",
                    url: this._baseUrl + "/" + this._deviceInfo.FilePath,
                    success: function (data, textStatus, jqXHR) {
                        if (data) {
                            this._deviceConfiguration = data;
                            resolve(data);
                        }
                        else {
                            var msg = "The data into " + this._baseUrl + "/" + this._deviceInfo.FilePath + " is null";
                            reject(msg);
                        }
                    }.bind(this),
                    error: function (jqXHR, textStatus, errorThrown) {
                        var msg = reject("Failed to load " + this._baseUrl + "/" + this._deviceInfo.FilePath + " file. " + textStatus + "  " + errorThrown);
                        reject(msg);
                    }.bind(this),
                    timeout: 5000
                });
            }.bind(this));
        };
        DeviceConfigService.prototype.updateIoVariables = function () {
            var idSet = new data_structure_4.StringSet();
            var mainVariableDictionary = DeviceConfigService.getInstance().VariablesDictionary;
            // Universal
            var universalPinsArray = data_structure_4.ArrayUtil.GetArray(this._xmlConfig.isai.isai_io.u);
            if (universalPinsArray) {
                for (var _i = 0, universalPinsArray_1 = universalPinsArray; _i < universalPinsArray_1.length; _i++) {
                    var pin = universalPinsArray_1[_i];
                    this.insertIdIfRequireCGI(mainVariableDictionary, pin._ioId, idSet);
                    this.insertIdIfRequireCGI(mainVariableDictionary, pin._univPinDirId, idSet);
                    this.insertIdIfRequireCGI(mainVariableDictionary, pin._modes, idSet);
                    this.insertIdIfRequireCGI(mainVariableDictionary, pin._minScaleId, idSet);
                    this.insertIdIfRequireCGI(mainVariableDictionary, pin._maxScaleId, idSet);
                }
            }
            // Digital input
            var digitalInputArray = data_structure_4.ArrayUtil.GetArray(this._xmlConfig.isai.isai_io.di);
            if (digitalInputArray) {
                for (var _a = 0, digitalInputArray_1 = digitalInputArray; _a < digitalInputArray_1.length; _a++) {
                    var pin = digitalInputArray_1[_a];
                    this.insertIdIfRequireCGI(mainVariableDictionary, pin._ioId, idSet);
                }
            }
            // Digital output
            var digitalOutputArray = data_structure_4.ArrayUtil.GetArray(this._xmlConfig.isai.isai_io.do);
            if (digitalOutputArray) {
                for (var _b = 0, digitalOutputArray_1 = digitalOutputArray; _b < digitalOutputArray_1.length; _b++) {
                    var pin = digitalOutputArray_1[_b];
                    this.insertIdIfRequireCGI(mainVariableDictionary, pin._ioId, idSet);
                    this.insertIdIfRequireCGI(mainVariableDictionary, pin._modes, idSet);
                }
            }
            // Analog output
            var analogOutputArray = data_structure_4.ArrayUtil.GetArray(this._xmlConfig.isai.isai_io.y);
            if (analogOutputArray) {
                for (var _c = 0, analogOutputArray_1 = analogOutputArray; _c < analogOutputArray_1.length; _c++) {
                    var pin = analogOutputArray_1[_c];
                    this.insertIdIfRequireCGI(mainVariableDictionary, pin._ioId, idSet);
                    this.insertIdIfRequireCGI(mainVariableDictionary, pin._modes, idSet);
                }
            }
            // Now idSet contains all id of variable neeeded to compute IO entry
            var ioVariableIdToRefreshValueList = idSet.getValues();
            if (ioVariableIdToRefreshValueList.length > 0) {
                // Get CGI
                var variableService = variable_service_2.VariableService.getInstance();
                var variableValuesPromise = variableService.getVar(ioVariableIdToRefreshValueList);
                return variableValuesPromise;
            }
            else {
                return Promise.reject("DeviceConfigurationService: the list of variable ids to ask is empty (during the IO extraction)");
            }
        };
        DeviceConfigService.prototype.insertIdIfRequireCGI = function (mainVariableDictionary, id, idSet) {
            if (validator_17.Validator.isDefined(id)) {
                var variable = mainVariableDictionary[id];
                if (validator_17.Validator.isDefined(variable)) {
                    if (variable._familyType == DeviceConfigService.VarTypeVariable) {
                        // Require using CGI
                        idSet.addNoConflict(id);
                    }
                }
            }
        };
        /**
         * It doesn't manage the universal pin
         */
        DeviceConfigService.prototype.getPinData = function (deviceStaticConfigurationCached, isaiArray, staticPin, direction, channelTypeIdForced) {
            var mainVariableDictionary = this.VariablesDictionary;
            // Check if this pin is also configured into ISAI xml
            if (isaiArray) {
                var activeIsaiPin = void 0; // It should be 'IsaiIoBase' (except when 'channelTypeIdForced' is defined)
                // Inefficient search (but the IO are few) 
                for (var _i = 0, isaiArray_1 = isaiArray; _i < isaiArray_1.length; _i++) {
                    var isaiPin = isaiArray_1[_i];
                    if (staticPin.pin === isaiPin._pin) {
                        // The io is active
                        activeIsaiPin = isaiPin;
                        break;
                    }
                }
                if (activeIsaiPin) {
                    var directionIdString = io_model_1.IODirectionUtil.GetIoDirectionString(direction);
                    var channelTypeId = void 0;
                    if (channelTypeIdForced) {
                        channelTypeId = channelTypeIdForced;
                    }
                    else {
                        channelTypeId = mainVariableDictionary[activeIsaiPin._modes]._currentValue;
                    }
                    var channelTypeKey = this.getSubPropertyValue(deviceStaticConfigurationCached.channelTypes, directionIdString, channelTypeId);
                    if (channelTypeKey) {
                        return new IoPinData(activeIsaiPin._ioId, channelTypeKey);
                    }
                }
            }
            // Pin is not connected
            return null;
        };
        /**
         * Get channel type from <code>map.property.subProperty</code>.
         */
        DeviceConfigService.prototype.getSubPropertyValue = function (map, property, subProperty) {
            if (map && map.hasOwnProperty(property)) {
                var propertyValue = map[property];
                if (propertyValue && propertyValue.hasOwnProperty(subProperty)) {
                    // Found
                    return propertyValue[subProperty];
                }
            }
        };
        DeviceConfigService.prototype.buildAnalogOutputPin = function (deviceStaticConfigurationCached, analogOutputIsaiPinArray, analogOutputStaticPin) {
            var pinData = this.getPinData(deviceStaticConfigurationCached, analogOutputIsaiPinArray, analogOutputStaticPin, io_model_1.IoDirection.AOUT);
            if (pinData) {
                // Pin is connected
                return new io_model_1.IoAnalogOutput(analogOutputStaticPin.pin, analogOutputStaticPin.displayName, true, pinData._idVar, pinData._channelTypeKey);
            }
            else {
                // Pin is not connected
                return new io_model_1.IoAnalogOutput(analogOutputStaticPin.pin, analogOutputStaticPin.displayName, false);
            }
        };
        DeviceConfigService.prototype.buildDigitalOutputPin = function (deviceStaticConfigurationCached, digitalOutputIsaiPinArray, digitalOutputStaticPin) {
            var pinData = this.getPinData(deviceStaticConfigurationCached, digitalOutputIsaiPinArray, digitalOutputStaticPin, io_model_1.IoDirection.DOUT);
            if (pinData) {
                // Pin is connected
                return new io_model_1.IoDigitalOutput(digitalOutputStaticPin.pin, digitalOutputStaticPin.displayName, true, pinData._idVar, pinData._channelTypeKey);
            }
            else {
                // Pin is not connected
                return new io_model_1.IoDigitalOutput(digitalOutputStaticPin.pin, digitalOutputStaticPin.displayName, false);
            }
        };
        DeviceConfigService.prototype.buildDigitalInputPin = function (deviceStaticConfigurationCached, digitalInputIsaiPinArray, digitalInputStaticPin) {
            // Force the channel type
            var pinData = this.getPinData(deviceStaticConfigurationCached, digitalInputIsaiPinArray, digitalInputStaticPin, io_model_1.IoDirection.DI, DeviceConfigService.DigitalInputIsaiChannelTypeDefault);
            if (pinData) {
                // Pin is connected
                return new io_model_1.IoDigitalInput(digitalInputStaticPin.pin, digitalInputStaticPin.displayName, true, pinData._idVar, pinData._channelTypeKey);
            }
            else {
                // Pin is not connected
                return new io_model_1.IoDigitalInput(digitalInputStaticPin.pin, digitalInputStaticPin.displayName, false);
            }
        };
        /**
         * Ad hoc manage.
         * Algorithm:
         * if DIRECTION is out
         *      then ANALOG (universal can't be digital output)
         * else
         *     try to find channelType in:
         *          - DI
         *          - DI_COUNT
         *          - ANALOG Input
         * Note that the DI with channel type = standard (the only available) and the AI with channel type 5 - ON...OFF have the same code and they
         * aren't distinguishable
         */
        DeviceConfigService.prototype.buildUniversalPin = function (deviceStaticConfigurationCached, universalIsaiPinArray, universalStaticPin) {
            var mainVariableDictionary = this.VariablesDictionary;
            // Check if this pin is also configured into ISAI xml
            if (validator_17.Validator.isDefined(universalIsaiPinArray)) {
                var universalActiveIsaiPin = void 0;
                // Inefficient search (but the IO are few)
                for (var _i = 0, universalIsaiPinArray_1 = universalIsaiPinArray; _i < universalIsaiPinArray_1.length; _i++) {
                    var universalIsaiPin = universalIsaiPinArray_1[_i];
                    if (universalStaticPin.pin === universalIsaiPin._pin) {
                        // The io is active
                        universalActiveIsaiPin = universalIsaiPin;
                        break;
                    }
                }
                if (validator_17.Validator.isDefined(universalActiveIsaiPin)) {
                    var directionChannelTypesMap = deviceStaticConfigurationCached.channelTypes;
                    var direction = void 0;
                    var directionId = void 0;
                    var channelTypeId = void 0;
                    var channelTypeKey = void 0;
                    var varDirection = mainVariableDictionary[universalActiveIsaiPin._univPinDirId];
                    var currentValueNumber = varDirection._currentValue;
                    if (currentValueNumber === UnivPinDirIdValue.Output) {
                        channelTypeId = mainVariableDictionary[universalActiveIsaiPin._modes]._currentValue;
                        // Output Universal can be only output analog
                        directionId = io_model_1.IODirectionUtil.GetIoDirectionString(io_model_1.IoDirection.AOUT);
                        channelTypeKey = this.getSubPropertyValue(directionChannelTypesMap, directionId, channelTypeId);
                        direction = io_model_1.IoDirection.AOUT;
                        return new io_model_1.IoUniversal(universalStaticPin.pin, universalStaticPin.displayName, true, direction, universalActiveIsaiPin._ioId, channelTypeKey, universalActiveIsaiPin._minScaleId, universalActiveIsaiPin._maxScaleId);
                    }
                    else if (currentValueNumber === UnivPinDirIdValue.Input) {
                        // Try to extract the channel direction using the info about the connected probe (channelType)                               
                        channelTypeId = mainVariableDictionary[universalActiveIsaiPin._modes]._currentValue;
                        // Try DI
                        directionId = io_model_1.IODirectionUtil.GetIoDirectionString(io_model_1.IoDirection.DI);
                        channelTypeKey = this.getSubPropertyValue(directionChannelTypesMap, directionId, channelTypeId);
                        if (channelTypeKey) {
                            return new io_model_1.IoUniversal(universalStaticPin.pin, universalStaticPin.displayName, true, io_model_1.IoDirection.DI, universalActiveIsaiPin._ioId, channelTypeKey, universalActiveIsaiPin._minScaleId, universalActiveIsaiPin._maxScaleId);
                        }
                        // Try find into AIN (note that AIN with 5 - ON...OFF can be never found)
                        directionId = io_model_1.IODirectionUtil.GetIoDirectionString(io_model_1.IoDirection.AIN);
                        channelTypeKey = this.getSubPropertyValue(directionChannelTypesMap, directionId, channelTypeId);
                        if (channelTypeKey) {
                            return new io_model_1.IoUniversal(universalStaticPin.pin, universalStaticPin.displayName, true, io_model_1.IoDirection.AIN, universalActiveIsaiPin._ioId, channelTypeKey, universalActiveIsaiPin._minScaleId, universalActiveIsaiPin._maxScaleId);
                        }
                        // Try DI_COUNT
                        directionId = io_model_1.IODirectionUtil.GetIoDirectionString(io_model_1.IoDirection.DI_COUNT);
                        channelTypeKey = this.getSubPropertyValue(directionChannelTypesMap, directionId, channelTypeId);
                        if (channelTypeKey) {
                            return new io_model_1.IoUniversal(universalStaticPin.pin, universalStaticPin.displayName, true, io_model_1.IoDirection.DI_COUNT, universalActiveIsaiPin._ioId, channelTypeKey, universalActiveIsaiPin._minScaleId, universalActiveIsaiPin._maxScaleId);
                        }
                        // Try DI_FREQ
                        directionId = io_model_1.IODirectionUtil.GetIoDirectionString(io_model_1.IoDirection.DI_FREQ);
                        channelTypeKey = this.getSubPropertyValue(directionChannelTypesMap, directionId, channelTypeId);
                        if (channelTypeKey) {
                            return new io_model_1.IoUniversal(universalStaticPin.pin, universalStaticPin.displayName, true, io_model_1.IoDirection.DI_FREQ, universalActiveIsaiPin._ioId, channelTypeKey, universalActiveIsaiPin._minScaleId, universalActiveIsaiPin._maxScaleId);
                        }
                        this._log("DeviceConfigurationService buildIoModel(): I can't find the channel type for id=" + channelTypeId);
                    }
                }
            }
            // Pin is not connected
            return new io_model_1.IoUniversal(universalStaticPin.pin, universalStaticPin.displayName, false);
        };
        // LOG
        DeviceConfigService.prototype.buildLogInfo = function () {
            var logDefinitionModel = new log_model_1.LogDefinitionModel();
            this.getPeriodicLog(logDefinitionModel);
            this.getEventLog(logDefinitionModel);
            return logDefinitionModel;
        };
        DeviceConfigService.prototype.getPeriodicLog = function (logDefinitionModel) {
            if (this._xmlConfig.isai.isai_log.pLog) {
                var pLogArray = data_structure_4.ArrayUtil.GetArray(this._xmlConfig.isai.isai_log.pLog);
                if (pLogArray != null) {
                    for (var _i = 0, pLogArray_1 = pLogArray; _i < pLogArray_1.length; _i++) {
                        var pLogItem = pLogArray_1[_i];
                        var name_2 = pLogItem._name;
                        if (typeof name_2 === "undefined" || name_2 == null) {
                            continue;
                        }
                        this.insertPeriodicLogVars(logDefinitionModel, pLogItem);
                    }
                }
            }
        };
        DeviceConfigService.prototype.getEventLog = function (logDefinitionModel) {
            if (this._xmlConfig.isai.isai_log.eLog) {
                var eLogArray = data_structure_4.ArrayUtil.GetArray(this._xmlConfig.isai.isai_log.eLog);
                if (eLogArray != null) {
                    for (var _i = 0, eLogArray_1 = eLogArray; _i < eLogArray_1.length; _i++) {
                        var eLogItem = eLogArray_1[_i];
                        var name_3 = eLogItem._name;
                        if (typeof name_3 === "undefined" || name_3 == null) {
                            continue;
                        }
                        this.insertEventLogVars(logDefinitionModel, eLogItem);
                    }
                }
            }
        };
        DeviceConfigService.prototype.insertPeriodicLogVars = function (logDefinitionModel, pLogItem) {
            if (pLogItem) {
                // Manage logVars tag
                var pLogVarsArray = data_structure_4.ArrayUtil.GetArray(pLogItem.logVars);
                if (pLogVarsArray != null) {
                    for (var _i = 0, pLogVarsArray_1 = pLogVarsArray; _i < pLogVarsArray_1.length; _i++) {
                        var pLogVarsItem = pLogVarsArray_1[_i];
                        var vArray = data_structure_4.ArrayUtil.GetArray(pLogVarsItem.v);
                        if (vArray != null) {
                            // Take only the id
                            var varIdArray = [];
                            for (var _a = 0, vArray_1 = vArray; _a < vArray_1.length; _a++) {
                                var vItem = vArray_1[_a];
                                varIdArray.push(vItem._varId);
                            }
                            logDefinitionModel.addPeriodicLog(new log_model_1.PeriodicLogDefinitionModel(pLogItem._name, pLogItem._sampSId, pLogItem._recNr, // TODO: cast?
                            varIdArray));
                        }
                    }
                }
            }
        };
        DeviceConfigService.prototype.insertEventLogVars = function (logDefinitionModel, eLogItem) {
            if (eLogItem) {
                var logVarsArray = data_structure_4.ArrayUtil.GetArray(eLogItem.logVars);
                var evtVarsArray = data_structure_4.ArrayUtil.GetArray(eLogItem.evtVars);
                var evtVarIdArray = [];
                var loggedVarIdArray = [];
                if (evtVarsArray != null) {
                    for (var _i = 0, evtVarsArray_1 = evtVarsArray; _i < evtVarsArray_1.length; _i++) {
                        var evtVarsItem = evtVarsArray_1[_i];
                        var vArray = data_structure_4.ArrayUtil.GetArray(evtVarsItem.v);
                        if (vArray != null) {
                            // Take only the id
                            for (var _a = 0, vArray_2 = vArray; _a < vArray_2.length; _a++) {
                                var vItem = vArray_2[_a];
                                evtVarIdArray.push(vItem._varId);
                            }
                        }
                        // TODO: for?? It should be only 1 item
                        break;
                    }
                }
                else {
                    evtVarIdArray = [];
                }
                if (logVarsArray != null) {
                    for (var _b = 0, logVarsArray_1 = logVarsArray; _b < logVarsArray_1.length; _b++) {
                        var pLogVarsItem = logVarsArray_1[_b];
                        var vArray = data_structure_4.ArrayUtil.GetArray(pLogVarsItem.v);
                        if (vArray != null) {
                            // Take only the id
                            for (var _c = 0, vArray_3 = vArray; _c < vArray_3.length; _c++) {
                                var vItem = vArray_3[_c];
                                loggedVarIdArray.push(vItem._varId);
                            }
                        }
                        // TODO: for?? It should be only 1 item
                        break;
                    }
                }
                else {
                    loggedVarIdArray = [];
                }
                logDefinitionModel.addEventLog(new log_model_1.EventLogDefinitionModel(eLogItem._name, eLogItem._recNr, evtVarIdArray, loggedVarIdArray));
            }
        };
        DeviceConfigService.PeriodicCheckCrcDelayMs = 10000;
        /** Singleton instance. */
        DeviceConfigService._instance = null;
        // Crc management
        DeviceConfigService._crcThreshold = 2;
        // Type of variable (in accord with webworker)
        DeviceConfigService.VarTypeConstant = 0;
        DeviceConfigService.VarTypeVariable = 1;
        DeviceConfigService.varTypeUi = 2;
        // Type of alarm (in accord with webworker)
        DeviceConfigService.AlarmTypeDisabled = 0;
        DeviceConfigService.AlarmTypeManual = 1;
        DeviceConfigService.AlarmTypeAutoReset = 2;
        DeviceConfigService.AlarmTypeAutoResetCounter = 3;
        // Device static configuration and device manifest
        DeviceConfigService.DeviceManifestPath = "device.json";
        // IO typechannel hardcoded for digital input (they are not write into xml file)
        DeviceConfigService.DigitalInputIsaiChannelTypeDefault = "5";
        return DeviceConfigService;
    }(base_service_9.BaseService));
    exports.DeviceConfigService = DeviceConfigService;
    var DeviceInfo = /** @class */ (function () {
        function DeviceInfo(_familyId, _deviceId, _filePath) {
            this._familyId = _familyId;
            this._deviceId = _deviceId;
            this._filePath = _filePath;
        }
        Object.defineProperty(DeviceInfo.prototype, "DeviceFamily", {
            get: function () { return this._familyId; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(DeviceInfo.prototype, "DeviceId", {
            get: function () { return this._deviceId; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(DeviceInfo.prototype, "FilePath", {
            get: function () { return this._filePath; },
            enumerable: true,
            configurable: true
        });
        ;
        return DeviceInfo;
    }());
    var UnivPinDirIdValue;
    (function (UnivPinDirIdValue) {
        UnivPinDirIdValue[UnivPinDirIdValue["Input"] = 0] = "Input";
        UnivPinDirIdValue[UnivPinDirIdValue["Output"] = 1] = "Output";
    })(UnivPinDirIdValue || (UnivPinDirIdValue = {}));
    var IoPinData = /** @class */ (function () {
        function IoPinData(_idVar, _channelTypeKey) {
            this._idVar = _idVar;
            this._channelTypeKey = _channelTypeKey;
        }
        return IoPinData;
    }());
});
define("service/alarm.service", ["require", "exports", "service/cfield.service", "service/base.service", "service/uom.service"], function (require, exports, cField_service_2, base_service_10, uom_service_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var AlarmService = /** @class */ (function (_super) {
        __extends(AlarmService, _super);
        /**
         * Init service.
         */
        function AlarmService() {
            var _this = _super.call(this) || this;
            if (AlarmService._instance) {
                _this._error(base_service_10.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            _this._deviceEngine = cField_service_2.cFieldService.getInstance().DeviceEngine;
            return _this;
        }
        Object.defineProperty(AlarmService.prototype, "Alarms", {
            get: function () {
                return this._alarms;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Get the instance of alarm service in accord with singleton pattern.
         * @return an instance of service.
         */
        AlarmService.getInstance = function () {
            if (AlarmService._instance === null) {
                AlarmService._instance = new AlarmService();
            }
            return AlarmService._instance;
        };
        AlarmService.prototype.getAlarms = function () {
            var _this = this;
            // 1 - If alarm uom zone is linked to variable than query it
            return uom_service_4.UoMService.getInstance().updateAlarmZone() //this.updateUomZoneLinkToVariable()
                .then(function (response) {
                // 2 - Get all alarms
                return _this.getAllAlarms();
            })
                .then(function (alarmBundle) {
                // 3 - Merge data
                var alarmSequenceArray = _this.checkActiveAlarmsPresence(alarmBundle.ActiveAlarms, alarmBundle.HistoricalAlarms);
                return Promise.resolve(new GetAlarmsResult(alarmSequenceArray, alarmBundle.RawData));
            });
            /* Unused code because reset of auto-reset-counter is not supported
            // Cache
            alarmBundleCache = alarmBundle;
    
            // 2- Update variable of auto-reset alarms
            let deviceConfigService = DeviceConfigService.getInstance();
            let alarmDictionary = deviceConfigService.AlarmDictionary;
            let variableDictionary = deviceConfigService.VariablesDictionary;
            let varIdToUpdate: StringSet = new StringSet();
            for (let alarmEntry in alarmDictionary) {
                let alarmDefinition = alarmDictionary[alarmEntry];
                if (alarmDefinition._alarmType === DeviceConfigService.AlarmTypeAutoResetCounter) {
                    if (typeof alarmDefinition._threId !== "undefined" && alarmDefinition._threId != null) {
                        let variable = variableDictionary[alarmDefinition._threId];
                        if (typeof variable !== "undefined" && variable != null ) {
                            if (variable._typeVar === DeviceConfigService.VarTypeConstant) {
                                varIdToUpdate.addNoConflict(alarmDefinition._threId);
                            }
                        }
                        else {
                            this._log("update variables of auto-reset alarm: the variable is not been found in the dictionary (id=" + alarmDefinition._threId + "");
                        }
                    }
                }
            }
    
            if (varIdToUpdate.Count > 0) {
                return <any>VariableService.getInstance().getVarEfficient(varIdToUpdate.getValues());
            }
            else {
                return Promise.resolve();
            }
            */
        };
        // private updateUomZoneLinkToVariable(): Promise<void> {
        //     return new Promise<void>(function(resolve, reject) {
        //         let uomService = UoMService.getInstance();
        //         if (uomService.IsAlarmZoneLinkToVariable) {
        //             let id = uomService.AlarmZoneLinkedVariableId;
        //             if (Validator.isDefined(id)) {
        //                 // Refresh the value
        //                 VariableService.getInstance().getVar([id])
        //                     .then(() => {
        //                         resolve();
        //                     })
        //                     .catch((reason) => {
        //                         reject(reason)
        //                     });
        //             }
        //             else {
        //                 reject("The zone of alarm is link to variable but the id of this variable is null");
        //             }    
        //         }
        //         else {
        //             resolve();
        //         }
        //     });
        // }
        /**
         * Retrieve all alarms using DeviceEngine.js
         */
        AlarmService.prototype.getAllAlarms = function () {
            return new Promise(function (resolve, reject) {
                this._deviceEngine.getAlarmsAll(function (result, activeAlarms, historicalAlarms, rawData) {
                    if (result === true) {
                        var alarmBundle = new AlarmBundle(activeAlarms, historicalAlarms, rawData);
                        resolve(alarmBundle);
                    }
                    else {
                        reject("The reading of alarms is failed");
                    }
                });
            }.bind(this));
        };
        /**
         * Merge the alarms lists and return a new list common format.
         * @param activeAlarms
         * @param historicAlarms
         * @return the union of list.
         */
        AlarmService.prototype.checkActiveAlarmsPresence = function (activeAlarms, historicAlarms) {
            if (!activeAlarms && !historicAlarms) {
                return [];
            }
            if (!activeAlarms) {
                return historicAlarms;
            }
            if (!historicAlarms) {
                this._log("checkActiveAlarmsPresence(): active alarms without historic alarms");
                return [];
            }
            var activeAlarmRecords = [];
            for (var i = 0; i < activeAlarms.length; i++) {
                var activeAlarm = activeAlarms[i];
                var timestampMilliSeconds = activeAlarm.getTimestamp().getTime();
                var j = void 0;
                for (j = 0; j < historicAlarms.length; j++) {
                    var historicAlarm = historicAlarms[j];
                    if (historicAlarm.hasStart() && !historicAlarm.hasStop() &&
                        // Same alarm (id and time)
                        activeAlarm.getAlarmId() === historicAlarm.getAlarmId() && timestampMilliSeconds === historicAlarm.getStartTime().getTime()) {
                        // Active alarm is both in active alarm list and history list but its history hasn't the stop event
                        break;
                    }
                }
                if (j === historicAlarms.length) {
                    /* create a fake alarm sequence history record */
                    var newRecord = new AlarmRecord({
                        ID: activeAlarm.getAlarmId(),
                        NAME: activeAlarm.getVarName(),
                        EVENT: "Start",
                        TIME: activeAlarm.getTimestamp(),
                        VAR1: "",
                        VAR2: "",
                    });
                    var newAlarm = new AlarmSequence(newRecord, null);
                    newAlarm.setIsStillActive(true);
                    activeAlarmRecords.push(newAlarm);
                }
                else {
                    // Active alarm
                    historicAlarms[j].setIsStillActive(true);
                    // Insert the alarm at the end of active alarms array
                    activeAlarmRecords.push(historicAlarms[j]);
                    // Remove the from historical alarm
                    historicAlarms.splice(j, 1); // splice: adds/removes items to/from an array, and returns the removed item(s). This method changes the original array.
                }
            }
            return activeAlarmRecords.concat(historicAlarms);
        };
        /**
         * Refresh the alarms.
         * @return the update sequence of alarms (request to device).
         */
        AlarmService.prototype.wipeAlarms = function () {
            var _this = this;
            var promise = new Promise(function (resolve, reject) {
                // Wipe alarm
                _this._deviceEngine.wipeAlarmsHistory(function (result) {
                    if (result === true) {
                        // Refresh list of alarms after some delay. In this way the server will can update the alarm states.
                        setTimeout(function () {
                            resolve();
                        }.bind(_this), 500);
                    }
                    else {
                        reject(AlarmService.WipeErrorMsg);
                    }
                });
            }).then(function () {
                return _this.getAlarms();
            });
            return promise;
        };
        /**
         * Reset a single alarm.
         * @return the update sequence of alarms (request to device).
         */
        AlarmService.prototype.resetAlarm = function (id) {
            var _this = this;
            var promise = new Promise(function (resolve, reject) {
                // Reset alarm
                _this._deviceEngine.resetAlarm(id, function (result) {
                    if (result === true) {
                        // Refresh list of alarms after some delay. In this way the server will can update the alarm states.
                        setTimeout(function () {
                            resolve();
                        }.bind(_this), 500);
                    }
                    else {
                        reject(AlarmService.ResetErrorMsg);
                    }
                });
            }).then(function () {
                return _this.getAlarms();
            });
            return promise;
        };
        AlarmService.prototype.resetAllAlarms = function () {
            var _this = this;
            var promise = new Promise(function (resolve, reject) {
                // Reset alarm
                _this._deviceEngine.resetAlarms(function (result) {
                    if (result === true) {
                        // Wait before resolve the promise. In this way the device can complete the reset
                        setTimeout(function () {
                            resolve();
                        }.bind(_this), 500);
                    }
                    else {
                        reject(AlarmService.ResetErrorMsg);
                    }
                });
            }).then(function () {
                return _this.getAlarms();
            });
            return promise;
        };
        AlarmService.ResetErrorMsg = "Reset is failed";
        AlarmService.WipeErrorMsg = "Clear is failed";
        /** Singleton instance. */
        AlarmService._instance = null;
        return AlarmService;
    }(base_service_10.BaseService));
    exports.AlarmService = AlarmService;
    var GetAlarmsResult = /** @class */ (function () {
        function GetAlarmsResult(_alarmSequenceArray, _rawData) {
            this._alarmSequenceArray = _alarmSequenceArray;
            this._rawData = _rawData;
        }
        Object.defineProperty(GetAlarmsResult.prototype, "AlarmSequence", {
            get: function () { return this._alarmSequenceArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(GetAlarmsResult.prototype, "RawData", {
            get: function () {
                if (this._rawData && this._rawData.length > 0) {
                    var firstPart = this._rawData[0] ? this._rawData[0] : "";
                    var secondPart = this._rawData[1] ? this._rawData[1] : "";
                    return secondPart ? firstPart + "\r\n\r\n" + secondPart : firstPart;
                }
                else {
                    return "";
                }
            },
            enumerable: true,
            configurable: true
        });
        return GetAlarmsResult;
    }());
    exports.GetAlarmsResult = GetAlarmsResult;
    var AlarmBundle = /** @class */ (function () {
        function AlarmBundle(_activeAlarms, _historicalAlarms, _rawData) {
            this._activeAlarms = _activeAlarms;
            this._historicalAlarms = _historicalAlarms;
            this._rawData = _rawData;
        }
        Object.defineProperty(AlarmBundle.prototype, "ActiveAlarms", {
            get: function () { return this._activeAlarms; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmBundle.prototype, "HistoricalAlarms", {
            get: function () { return this._historicalAlarms; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmBundle.prototype, "RawData", {
            get: function () { return this._rawData; },
            enumerable: true,
            configurable: true
        });
        return AlarmBundle;
    }());
});
define("component/viewModel/box-component.viewmodel", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var BoxComponentViewModel = /** @class */ (function () {
        function BoxComponentViewModel(_hideExtraToolbar) {
            this._hideExtraToolbar = _hideExtraToolbar;
        }
        Object.defineProperty(BoxComponentViewModel.prototype, "HideExtraToolbar", {
            get: function () { return this._hideExtraToolbar; },
            enumerable: true,
            configurable: true
        });
        BoxComponentViewModel.GetCreateViewModelInstanceForKo = function () {
            return {
                createViewModel: function (params, componentInfo) {
                    var hideExtraToolbar = (params && params.HideExtraToolbar) || false;
                    var newViewModel = new BoxComponentViewModel(hideExtraToolbar);
                    return newViewModel;
                }
            };
        };
        return BoxComponentViewModel;
    }());
    exports.BoxComponentViewModel = BoxComponentViewModel;
});
define("component/viewModel/iframe-component.viewmodel", ["require", "exports", "component/viewModel/base-component.viewmodel"], function (require, exports, base_component_viewmodel_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var IFrameComponentViewModel = /** @class */ (function (_super) {
        __extends(IFrameComponentViewModel, _super);
        function IFrameComponentViewModel(src, params) {
            var _this = _super.call(this, true, false, false, false) || this;
            _this.src = src;
            if (params != null) {
                _this.style = params.style;
            }
            else {
                // Check if I'm in c.field main component and force the size of iframe
                var wrapper = $("#page-content-wrapper");
                if (wrapper) {
                    var h = wrapper.height();
                    var w = wrapper.width();
                    if (h && w) {
                        _this.style = 'position: fixed; height: ' + h + 'px !important; width: ' + w + 'px !important; border:none;';
                    }
                }
                else {
                    _this.style = 'position: fixed; height: 99%; width: 99%; border:none;';
                }
            }
            return _this;
        }
        return IFrameComponentViewModel;
    }(base_component_viewmodel_2.BaseComponentViewModel));
    exports.IFrameComponentViewModel = IFrameComponentViewModel;
});
define("component/viewModel/item/io-pin.viewmodel", ["require", "exports", "service/translator.service", "common/loggable-entity", "component/viewModel/item/variable-item.viewmodel"], function (require, exports, translator_service_6, loggable_entity_5, variable_item_viewmodel_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var IoPinViewModel = /** @class */ (function (_super) {
        __extends(IoPinViewModel, _super);
        function IoPinViewModel(_pinName, _probeTypeKey, variableCurrentValue, variable, categoryViewModels) {
            var _this = _super.call(this, variableCurrentValue, variable, null, categoryViewModels) || this;
            _this._pinName = _pinName;
            _this._probeTypeKey = _probeTypeKey;
            return _this;
        }
        Object.defineProperty(IoPinViewModel.prototype, "PinName", {
            get: function () { return this._pinName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoPinViewModel.prototype, "ChannelType", {
            get: function () {
                return variable_item_viewmodel_2.EditableVariableItemViewModel._translator(translator_service_6.TranslatorService.IoChannelTypeTranslationNs + this._probeTypeKey, { defaultValue: this._probeTypeKey || "" });
            },
            enumerable: true,
            configurable: true
        });
        // override
        IoPinViewModel._log = new loggable_entity_5.LoggableEntityStatic("ReadableVariableItem");
        return IoPinViewModel;
    }(variable_item_viewmodel_2.EditableVariableItemViewModel));
    exports.IoPinViewModel = IoPinViewModel;
});
define("component/viewModel/item/log-definition-item.viewmodel", ["require", "exports", "knockout", "service/translator.service"], function (require, exports, ko, translator_service_7) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var BaseLogDefinitionItemViewModel = /** @class */ (function () {
        function BaseLogDefinitionItemViewModel(_id, _name, _nRecord, _serieArray, _logComponentViewModel) {
            this._id = _id;
            this._name = _name;
            this._nRecord = _nRecord;
            this._serieArray = _serieArray;
            this._logComponentViewModel = _logComponentViewModel;
            this._isSelected = false;
        }
        Object.defineProperty(BaseLogDefinitionItemViewModel.prototype, "LogId", {
            get: function () { return this._id; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseLogDefinitionItemViewModel.prototype, "LogName", {
            get: function () { return this._name; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseLogDefinitionItemViewModel.prototype, "LogNrecord", {
            get: function () { return this._nRecord; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseLogDefinitionItemViewModel.prototype, "LogSerieArray", {
            get: function () { return this._serieArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseLogDefinitionItemViewModel.prototype, "IsSelected", {
            get: function () { return this._isSelected; },
            set: function (value) {
                this._isSelected = value;
                if (value == true) {
                    this._logComponentViewModel.EnableLoadLog(true);
                }
            },
            enumerable: true,
            configurable: true
        });
        return BaseLogDefinitionItemViewModel;
    }());
    exports.BaseLogDefinitionItemViewModel = BaseLogDefinitionItemViewModel;
    var PeriodicLogDefinitionItemViewModel = /** @class */ (function (_super) {
        __extends(PeriodicLogDefinitionItemViewModel, _super);
        function PeriodicLogDefinitionItemViewModel(id, name, nRecord, _sampledVariableValue, serieArray, logComponentViewModel) {
            var _this = _super.call(this, id, name, nRecord, serieArray, logComponentViewModel) || this;
            _this._sampledVariableValue = _sampledVariableValue;
            return _this;
        }
        Object.defineProperty(PeriodicLogDefinitionItemViewModel.prototype, "LogSampledVariableValue", {
            get: function () { return this._sampledVariableValue; },
            enumerable: true,
            configurable: true
        });
        return PeriodicLogDefinitionItemViewModel;
    }(BaseLogDefinitionItemViewModel));
    exports.PeriodicLogDefinitionItemViewModel = PeriodicLogDefinitionItemViewModel;
    var EventLogDefinitionItemViewModel = /** @class */ (function (_super) {
        __extends(EventLogDefinitionItemViewModel, _super);
        function EventLogDefinitionItemViewModel(id, name, nRecord, serieArray, logComponentViewModel) {
            return _super.call(this, id, name, nRecord, serieArray, logComponentViewModel) || this;
        }
        return EventLogDefinitionItemViewModel;
    }(BaseLogDefinitionItemViewModel));
    exports.EventLogDefinitionItemViewModel = EventLogDefinitionItemViewModel;
    var LogSerieItemViewModel = /** @class */ (function () {
        function LogSerieItemViewModel(_name, _description, _isVisible, yAxisType, _color, style) {
            this._name = _name;
            this._description = _description;
            this._isVisible = _isVisible;
            this._color = _color;
            this._yAxisTypeViewModel = new YAxisTypeViewModel(yAxisType);
            this._chartLineStyleViewModel = new ChartLineStyleViewModel(style);
            this._onChange = ko.observable();
            this._onChange.extend({ notify: "always" });
        }
        Object.defineProperty(LogSerieItemViewModel.prototype, "SerieName", {
            get: function () { return this._name; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogSerieItemViewModel.prototype, "SerieDescription", {
            get: function () { return this._description; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogSerieItemViewModel.prototype, "IsSerieVisible", {
            get: function () { return this._isVisible; },
            set: function (value) {
                if (value != this._isVisible) {
                    this._isVisible = value;
                    this._onChange.notifySubscribers();
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogSerieItemViewModel.prototype, "YAxisType", {
            get: function () { return this._yAxisTypeViewModel.Type; },
            set: function (value) {
                if (value != this._yAxisTypeViewModel.Type) {
                    this._yAxisTypeViewModel.Type = value;
                    this._onChange.notifySubscribers();
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogSerieItemViewModel.prototype, "YAxisName", {
            get: function () { return this._yAxisTypeViewModel.Name; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogSerieItemViewModel.prototype, "ColorName", {
            get: function () { return this._color; },
            set: function (value) {
                if (value != this._color) {
                    this._color = value;
                    this._onChange.notifySubscribers();
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogSerieItemViewModel.prototype, "ChartLineStyle", {
            get: function () { return this._chartLineStyleViewModel.Type; },
            set: function (value) {
                if (value != this._chartLineStyleViewModel.Type) {
                    this._chartLineStyleViewModel.Type = value;
                    this._onChange.notifySubscribers();
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogSerieItemViewModel.prototype, "ChartLineStyleName", {
            get: function () { return this._chartLineStyleViewModel.Name; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogSerieItemViewModel.prototype, "OnChange", {
            get: function () { return this._onChange; },
            enumerable: true,
            configurable: true
        });
        return LogSerieItemViewModel;
    }());
    exports.LogSerieItemViewModel = LogSerieItemViewModel;
    var VariableSerieItemViewModel = /** @class */ (function (_super) {
        __extends(VariableSerieItemViewModel, _super);
        function VariableSerieItemViewModel(_id, name, description, isVisible, yAxisType, _color, style) {
            var _this = _super.call(this, name, description, isVisible, yAxisType, _color, style) || this;
            _this._id = _id;
            return _this;
        }
        Object.defineProperty(VariableSerieItemViewModel.prototype, "Id", {
            get: function () { return this._id; },
            enumerable: true,
            configurable: true
        });
        return VariableSerieItemViewModel;
    }(LogSerieItemViewModel));
    exports.VariableSerieItemViewModel = VariableSerieItemViewModel;
    var YAxisTypeViewModel = /** @class */ (function () {
        function YAxisTypeViewModel(_type) {
            this._type = _type;
            if (_type === YAxisType.Primary)
                this._translationKey = "primary";
            else if (_type === YAxisType.Secondary)
                this._translationKey = "secondary";
        }
        Object.defineProperty(YAxisTypeViewModel.prototype, "Type", {
            get: function () { return this._type; },
            set: function (type) { this._type = type; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(YAxisTypeViewModel.prototype, "Name", {
            get: function () { return translator_service_7.TranslatorService.getInstance().t(translator_service_7.TranslatorService.CfieldTranslationNs + this._translationKey); },
            enumerable: true,
            configurable: true
        });
        return YAxisTypeViewModel;
    }());
    exports.YAxisTypeViewModel = YAxisTypeViewModel;
    var ChartLineStyleViewModel = /** @class */ (function () {
        function ChartLineStyleViewModel(_type) {
            this._type = _type;
            if (_type === ChartLineStyle.Line)
                this._translationKey = "line";
            else if (_type === ChartLineStyle.Step)
                this._translationKey = "step";
        }
        Object.defineProperty(ChartLineStyleViewModel.prototype, "Type", {
            get: function () { return this._type; },
            set: function (type) { this._type = type; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ChartLineStyleViewModel.prototype, "Name", {
            get: function () { return translator_service_7.TranslatorService.getInstance().t(translator_service_7.TranslatorService.CfieldTranslationNs + this._translationKey); },
            enumerable: true,
            configurable: true
        });
        return ChartLineStyleViewModel;
    }());
    exports.ChartLineStyleViewModel = ChartLineStyleViewModel;
    var YAxisType;
    (function (YAxisType) {
        YAxisType[YAxisType["Primary"] = 0] = "Primary";
        YAxisType[YAxisType["Secondary"] = 1] = "Secondary";
    })(YAxisType = exports.YAxisType || (exports.YAxisType = {}));
    var ChartLineStyle;
    (function (ChartLineStyle) {
        ChartLineStyle[ChartLineStyle["Line"] = 0] = "Line";
        ChartLineStyle[ChartLineStyle["Step"] = 1] = "Step";
    })(ChartLineStyle = exports.ChartLineStyle || (exports.ChartLineStyle = {}));
    var UtilSerie = /** @class */ (function () {
        function UtilSerie() {
        }
        UtilSerie.IndexToYAxisTypeConverter = function (index) {
            if (index >= 0 && index <= this._yAxisType.length) {
                return this._yAxisType[index];
            }
            else {
                throw Error("Index of array is outs of limits: " + index);
            }
        };
        UtilSerie.IndexToChartLineStyleConverter = function (index) {
            if (index >= 0 && index <= this._chartLineStyle.length) {
                return this._chartLineStyle[index];
            }
            else {
                throw Error("Index of array is outs of limits: " + index);
            }
        };
        UtilSerie._yAxisType = [YAxisType.Primary, YAxisType.Secondary];
        UtilSerie._chartLineStyle = [ChartLineStyle.Line, ChartLineStyle.Step];
        return UtilSerie;
    }());
    exports.UtilSerie = UtilSerie;
});
define("component/viewModel/item/alarm-item.viewmodel", ["require", "exports", "service/device-config.service", "service/timer.service", "service/translator.service", "service/uom.service", "dataModel/datatype.model", "common/view"], function (require, exports, device_config_service_5, timer_service_2, translator_service_8, uom_service_5, datatype_model_8, view_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var AlarmItemViewModel = /** @class */ (function () {
        /**
         * Init view model.
         * @param currentAlarm. Alarm.
         * @param cFieldServiceInstance. <code>cField</code> service instance.
         */
        function AlarmItemViewModel(currentAlarm) {
            // Get timer helper
            var timerHelper = timer_service_2.TimerService.getInstance();
            // Get dictionaries
            var deviceConfigService = device_config_service_5.DeviceConfigService.getInstance();
            var variableDictionary = deviceConfigService.VariablesDictionary;
            var alarmDictionary = deviceConfigService.AlarmDictionary;
            // Get alarm
            var alarmDefinition = alarmDictionary[currentAlarm.getAlarmId()];
            if (!alarmDefinition) {
                throw "Alarm not defined";
            }
            // Init	
            this._currentAlarm = currentAlarm;
            // getAlarmName return the alarmName.active of alarm
            this._translationKey = currentAlarm.getAlarmName();
            this._startTimeTzOffsetMinutes = currentAlarm.getStartTimeTzOffsetMinutes();
            this._stopTimeTzOffsetMinutes = currentAlarm.getStopTimeTzOffsetMinutes();
            this._startTime = timerHelper.dateToString(currentAlarm.getStartTime(), this._startTimeTzOffsetMinutes);
            this._stopTime = timerHelper.dateToString(currentAlarm.getStopTime(), this._stopTimeTzOffsetMinutes);
            this._defaultDescription = alarmDefinition._description;
            this._id = currentAlarm.getAlarmId();
            this._stateBundle = new AlarmViewModelStateBundle(currentAlarm._isStillActive, currentAlarm.getStartTime(), currentAlarm.getStopTime());
            this._alarmType = alarmDefinition._alarmType;
            if (typeof alarmDefinition._sampId1 !== "undefined" && alarmDefinition._sampId1 !== null) {
                this._sampled1Variable = variableDictionary[alarmDefinition._sampId1];
            }
            if (typeof alarmDefinition._sampId2 !== "undefined" && alarmDefinition._sampId2 !== null) {
                this._sampled2Variable = variableDictionary[alarmDefinition._sampId2];
            }
            // Set translation
            this.translate();
        }
        Object.defineProperty(AlarmItemViewModel.prototype, "StartTime", {
            get: function () { return this._startTime; },
            set: function (value) { this._startTime = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "StopTime", {
            get: function () { return this._stopTime; },
            set: function (value) { this._stopTime = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "Id", {
            get: function () { return this._id; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "Name", {
            /** Active name of alarm. */
            get: function () { return this._currentAlarm.getAlarmName(); },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "Description", {
            get: function () { return this._description; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "Var1", {
            get: function () {
                return this._var1;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "Var2", {
            get: function () {
                return this._var2;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "TranslationKey", {
            get: function () { return this._translationKey; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "StateBundle", {
            get: function () {
                return this._stateBundle;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "IsAlarmActive", {
            /** Exhibit the flag about the state of alarm (active or not active). */
            get: function () {
                return this._stateBundle.IsAlarmActive;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "StartDate", {
            get: function () {
                return this._stateBundle.StartDate;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "StopDate", {
            get: function () {
                return this._stateBundle.StopDate;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "StartTimeTzOffsetMinutes", {
            get: function () {
                return this._startTimeTzOffsetMinutes;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "StopTimeTzOffsetMinutes", {
            get: function () {
                return this._startTimeTzOffsetMinutes;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmItemViewModel.prototype, "IsResetEnabled", {
            get: function () {
                return this._alarmType === device_config_service_5.DeviceConfigService.AlarmTypeManual ||
                    this._alarmType == device_config_service_5.DeviceConfigService.AlarmTypeAutoResetCounter;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Translate the current view model.
         */
        AlarmItemViewModel.prototype.translate = function () {
            this._description = translator_service_8.TranslatorService.getInstance().t(translator_service_8.TranslatorService.VariableTranslationNs + this._translationKey, { defaultValue: this._defaultDescription || "" });
            this._var1 = this.getSampledVariableText(this._currentAlarm.getStartVar1(), this._sampled1Variable);
            this._var2 = this.getSampledVariableText(this._currentAlarm.getStartVar2(), this._sampled2Variable);
        };
        AlarmItemViewModel.prototype.getSampledVariableText = function (value, variable) {
            if (value === "") {
                return "";
            }
            if (variable) {
                // Manage description
                var translator = translator_service_8.TranslatorService.getInstance().t;
                var descriptionText = translator(translator_service_8.TranslatorService.VariableTranslationNs + variable._name, { defaultValue: variable._description });
                // Eventually add a space
                if (descriptionText.length > 0) {
                    descriptionText = descriptionText + " ";
                }
                // Manage value
                var valueText = void 0;
                var datatype = datatype_model_8.DataTypeUtil.getInstance().DataTypeDictionary[variable._dataType];
                valueText = view_1.DataFormatter.FormatValue(value, datatype.Type, 2);
                // Manage uom
                var uomService = uom_service_5.UoMService.getInstance();
                var uomText = uomService.getAlarmUom(variable._uom).Symbol;
                // Build text
                var result = void 0;
                if (uomText) {
                    return descriptionText + valueText + " " + uomText;
                }
                else {
                    return descriptionText + valueText + uomText;
                }
            }
            // Return only the value
            return value;
        };
        return AlarmItemViewModel;
    }());
    exports.AlarmItemViewModel = AlarmItemViewModel;
    var AlarmViewModelStateBundle = /** @class */ (function () {
        /**
         * Init.
         * @param isAlarmActive Flag to indicate if alarm is active.
         * @param startDate Start date: it is used to order the records of the table and for date conversion.
         * @param stopDate Stop date: it is used for date conversion.
         */
        function AlarmViewModelStateBundle(isAlarmActive, startDate, stopDate) {
            this._isAlarmActive = isAlarmActive;
            this._startDate = startDate;
            this._stopDate = stopDate;
        }
        Object.defineProperty(AlarmViewModelStateBundle.prototype, "IsAlarmActive", {
            get: function () {
                return this._isAlarmActive;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmViewModelStateBundle.prototype, "StartDate", {
            get: function () {
                return this._startDate;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmViewModelStateBundle.prototype, "StopDate", {
            get: function () {
                return this._stopDate;
            },
            enumerable: true,
            configurable: true
        });
        return AlarmViewModelStateBundle;
    }());
});
define("common/event", ["require", "exports", "common/view", "common/validator", "dataModel/datatype.model", "common/formatter"], function (require, exports, view_2, validator_18, datatype_model_9, formatter_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var EventHandler = /** @class */ (function () {
        function EventHandler() {
        }
        EventHandler.ValidateWithRegex = function (regex, value, valueString, datatype, accuracy, variablePresenter, regexFailCallback, event, context) {
            // Regex is already validate
            var matcher = new RegExp(regex);
            if (!valueString.match(matcher)) {
                event.preventDefault();
                if (validator_18.Validator.isDefined(variablePresenter) && variablePresenter.FromPaste === true) {
                    variablePresenter.FromPaste = false;
                    // Paste operation. Try to recude decimals
                    var formatterResult = formatter_2.Formatter.FormatNumber(value, datatype, accuracy);
                    if (validator_18.Validator.isDefined(formatterResult)) {
                        valueString = formatterResult.toString();
                        $(context).val(valueString);
                        return formatterResult;
                    }
                }
                else {
                    if (validator_18.Validator.isDefined(regexFailCallback)) {
                        regexFailCallback();
                    }
                }
            }
            else {
                return value;
            }
            return null;
        };
        /**
         * Focus in handler
         */
        EventHandler.EditableInputFocusInHandler = function (e, value, variable, index, sourceValue, context) {
            // Skip for boolean
            if (variable.DataType === datatype_model_9.DataTypeEnum.Bool) {
                return;
            }
            var inputBox = view_2.Formatter.GetInputHtml(e, context);
            var variablePresenter;
            // Split logic in accord with source
            if (sourceValue === datatype_model_9.SourceValue.Imported) {
                // Parameter
                inputBox.value = variable.VariableImportedValue;
                variablePresenter = variable.ImportedValueItemPresenter;
            }
            else {
                // Current value
                inputBox.value = variable.VariableCurrentValue;
                variablePresenter = variable.CurrentValueItemPresenter;
            }
            if (validator_18.Validator.isDefined(variablePresenter)) {
                variablePresenter.onFocus();
                variablePresenter.IsInFocusInputCalled = true;
            }
        };
        /**
         * Focus out handler
         */
        EventHandler.EditableInputFocusOutHandler = function (e, value, variable, index, sourceValue, context) {
            // Get value
            var inputBox = view_2.Formatter.GetInputHtml(e, context);
            var newValue = view_2.Formatter.GetValue(variable, inputBox);
            var formattedValue;
            var variablePresenter;
            // Split logic in accord with source
            if (sourceValue === datatype_model_9.SourceValue.Imported) {
                variable.VariableImportedValue = newValue;
                formattedValue = variable.VariableImportedValueUom;
                variablePresenter = variable.ImportedValueItemPresenter;
            }
            else {
                variable.VariableEditedValue = newValue;
                formattedValue = variable.VariableEditedValueUom;
                variablePresenter = variable.CurrentValueItemPresenter;
            }
            // Update view (with UoM)
            if (variable.DataType !== datatype_model_9.DataTypeEnum.Bool) {
                inputBox.value = formattedValue;
            }
            if (validator_18.Validator.isDefined(variablePresenter)) {
                variablePresenter.IsInFocusInputCalled = false;
            }
        };
        EventHandler.EditableInputChangeHandler = function (e, value, variable, index, sourceValue, context) {
            // Get value
            var inputBox = view_2.Formatter.GetInputHtml(e, context);
            var newValue = view_2.Formatter.GetValue(variable, inputBox);
            var variablePresenter;
            // IE11 workaroud: if I init an input html element with special character (i.e. °C) than IE11 call 'input' event
            // on this element (even if the user doesn't press any button). Do manage this behaviour I mark the variable with
            // a flag to identify if the user has clicked into the input. In this case I chekck if 'focusin' event has been 
            // called
            var isParameter = sourceValue === datatype_model_9.SourceValue.Imported;
            if (isParameter) {
                variablePresenter = variable.ImportedValueItemPresenter;
            }
            else {
                variablePresenter = variable.CurrentValueItemPresenter;
            }
            if (validator_18.Validator.isDefined(variablePresenter) && variablePresenter.IsInFocusInputCalled === false) {
                return;
            }
            // Extract regex
            var regex;
            var newValueString;
            if (validator_18.Validator.isDefined(newValue)) {
                newValueString = newValue.toString();
                if (newValueString.length > 0) {
                    regex = variable.ValidationRegex;
                }
            }
            // Split logic in accord with source
            if (isParameter) {
                // Validate only if imported is edited
                if (!variable.IsVariableImportedValueEdited && validator_18.Validator.isDefined(regex)) {
                    newValue = EventHandler.ValidateWithRegex(regex, newValue, newValueString, variable.DataType, variable.Accuracy, variablePresenter, function () {
                        // Force old value
                        $(context).val(variable.VariableImportedValue);
                    }, e, context);
                }
            }
            else {
                if (validator_18.Validator.isDefined(regex)) {
                    newValue = EventHandler.ValidateWithRegex(regex, newValue, newValueString, variable.DataType, variable.Accuracy, variablePresenter, function () {
                        // Force old value
                        if (validator_18.Validator.isDefined(variable.VariableEditedValue)) {
                            $(context).val(variable.VariableEditedValue);
                        }
                        else {
                            $(context).val(variable.VariableCurrentValue);
                        }
                    }, e, context);
                }
            }
            if (!validator_18.Validator.isDefined(newValue)) {
                return;
            }
            if (validator_18.Validator.isDefined(variablePresenter)) {
                // Split logic in accord with source
                if (isParameter) {
                    variable.VariableImportedValue = newValue;
                    if (variable.IsVariableImportedValueValid && variable.IsVariableImportedValueRangeValid) {
                        variablePresenter.onValidationOk(inputBox, e, newValue, variable, index);
                    }
                    else {
                        variablePresenter.onInvalidValue(inputBox, e, newValue, variable, index);
                    }
                }
                else {
                    variable.VariableEditedValue = newValue;
                    if (variable.IsVariableEditedValid) {
                        variablePresenter.onValidationOk(inputBox, e, newValue, variable, index);
                        if (variable.IsVariableModified) {
                            variablePresenter.onModifyValidValue(inputBox, e, newValue, variable, index);
                        }
                        else {
                            variablePresenter.onNotModifyValidValue(inputBox, e, newValue, variable, index);
                        }
                    }
                    else {
                        variablePresenter.onInvalidValue(inputBox, e, newValue, variable, index);
                    }
                }
            }
        };
        /**
         * Checkbox click handler
         */
        EventHandler.EditableInputClickCheckboxHandler = function (e, value, variable, index, sourceValue, context) {
            // Get value
            var inputBox = view_2.Formatter.GetInputHtml(e, context);
            var newValue = inputBox.checked;
            var variablePresenter;
            // Split logic in accord with source
            if (sourceValue === datatype_model_9.SourceValue.Imported) {
                variable.VariableImportedValue = newValue;
                variablePresenter = variable.ImportedValueItemPresenter;
            }
            else {
                variable.VariableEditedValue = newValue;
                variablePresenter = variable.CurrentValueItemPresenter;
            }
            // Update view
            if (validator_18.Validator.isDefined(variablePresenter)) {
                variablePresenter.onFocus();
                if (variable.IsVariableModified) {
                    variablePresenter.onModifyValidValue(inputBox, e, value, variable, index);
                }
                else {
                    variablePresenter.onNotModifyValidValue(inputBox, e, value, variable, index);
                }
            }
        };
        EventHandler.EditableInputPasteHandler = function (e, value, variable, index, sourceValue) {
            var variablePresenter;
            // Split logic in accord with source
            if (sourceValue === datatype_model_9.SourceValue.Imported) {
                variablePresenter = variable.ImportedValueItemPresenter;
            }
            else {
                variablePresenter = variable.CurrentValueItemPresenter;
            }
            if (validator_18.Validator.isDefined(variablePresenter)) {
                variablePresenter.FromPaste = true;
            }
        };
        return EventHandler;
    }());
    exports.EventHandler = EventHandler;
});
define("common/view", ["require", "exports", "service/translator.service", "common/validator", "common/string-util", "common/style", "dataModel/datatype.model", "component/viewModel/item/log-definition-item.viewmodel", "common/event"], function (require, exports, translator_service_9, validator_19, string_util_2, style_2, datatype_model_10, log_definition_item_viewmodel_1, event_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var Formatter = /** @class */ (function () {
        function Formatter() {
        }
        Formatter.RegisterFormatter = function () {
            if (Formatter._isFormatterRegistered === false) {
                Formatter._dataTypeDictionary = datatype_model_10.DataTypeUtil.getInstance().DataTypeDictionary;
                Formatter.RegisterReadOnlyVariableFormatter();
                Formatter.RegisterWritableVariableFormatter();
                Formatter.RegisterAlarmFormatter();
                Formatter.RegisterChartSerieFormatter();
                Formatter.RegisterRowStyleFormatter();
                Formatter.RegisterColumnStyleFormatter();
                Formatter.RegisterSortFunction();
                Formatter._isFormatterRegistered = true;
            }
        };
        /**
         * Format a column of bootstrap-table in accord with data type.
         * @param dataType
         * @param numericFormattedValue Value used if data type is numeric.
         * @param booleanValue Value used if data type is boolean.
         */
        Formatter.NotEditableValueFormatter = function (dataType, numericFormattedValue, booleanValue) {
            if (validator_19.Validator.isDefined(numericFormattedValue) && datatype_model_10.DataTypeUtil.IsNumericDataType(dataType)) {
                return string_util_2.StringUtil.sprintf(Formatter.NotEditableUomVariableTemplate, numericFormattedValue);
            }
            else if (validator_19.Validator.isDefined(booleanValue) && dataType === datatype_model_10.DataTypeEnum.Bool) {
                return string_util_2.StringUtil.sprintf(Formatter.InputCheckboxTemplate, booleanValue === true ? "checked" : "", "disabled");
            }
            return Formatter.NoValueTemplate;
        };
        Formatter.EditableValueFormatter = function (variableViewModel, setValidationTooltip, numericFormattedValue, booleanValue, forceReadOnlyFormatter) {
            if (validator_19.Validator.isDefined(numericFormattedValue) && datatype_model_10.DataTypeUtil.IsNumericDataType(variableViewModel.DataType)) {
                if (variableViewModel.IsEditable && !forceReadOnlyFormatter) {
                    // Manage error style
                    if ((variableViewModel.IsVariableModified && !variableViewModel.IsVariableEditedValid)) {
                        var errorText = setValidationTooltip ? variableViewModel.CurrentValueValidationMsg : "";
                        return string_util_2.StringUtil.sprintf(Formatter.EditableUomVariableTemplate, style_2.StyleClass.InvalidInputError, numericFormattedValue, errorText.length > 0 ? 'value="' + errorText + '"' : "");
                    }
                    else {
                        return string_util_2.StringUtil.sprintf(Formatter.EditableUomVariableTemplate, "", numericFormattedValue, "");
                    }
                }
                else {
                    return string_util_2.StringUtil.sprintf(Formatter.NotEditableUomVariableTemplate, variableViewModel.VariableCurrentValueUom);
                }
            }
            else if (validator_19.Validator.isDefined(booleanValue) && variableViewModel.DataType === datatype_model_10.DataTypeEnum.Bool) {
                if (variableViewModel.IsEditable && !forceReadOnlyFormatter) {
                    return string_util_2.StringUtil.sprintf(Formatter.InputCheckboxTemplate, booleanValue === true ? "checked" : "", "");
                }
                else {
                    return string_util_2.StringUtil.sprintf(Formatter.InputCheckboxTemplate, booleanValue === true ? "checked" : "", "disabled");
                }
            }
            return Formatter.NoValueTemplate;
        };
        Formatter.EditableParameterValueFormatter = function (parameterViewModel, numericFormattedValue, booleanValue) {
            if (parameterViewModel.IsVariableImportedValueSet) {
                if (validator_19.Validator.isDefined(numericFormattedValue) && datatype_model_10.DataTypeUtil.IsNumericDataType(parameterViewModel.DataType)) {
                    return string_util_2.StringUtil.sprintf(Formatter.EditableUomVariableTemplate, "", numericFormattedValue, "");
                }
                else if (validator_19.Validator.isDefined(booleanValue) && parameterViewModel.DataType === datatype_model_10.DataTypeEnum.Bool) {
                    return string_util_2.StringUtil.sprintf(Formatter.InputCheckboxTemplate, booleanValue === true ? "checked" : "", "");
                }
            }
            return Formatter.NoValueTemplate;
        };
        Formatter.RegisterReadOnlyVariableFormatter = function () {
            // Value + UoM
            window["readOnlyVariableValueFormatter"] = function (value, variableViewModel, index) {
                return Formatter.NotEditableValueFormatter(variableViewModel.DataType, variableViewModel.VariableCurrentValueUom, variableViewModel.VariableCurrentValue //TODO FIX
                );
            };
            window["readOnlyPinMinValueFormatter"] = function (value, variableViewModel, index) {
                return Formatter.NotEditableValueFormatter(variableViewModel.DataType, variableViewModel.VariableMinValueUom);
            };
            window["readOnlyPinMaxValueFormatter"] = function (value, variableViewModel, index) {
                return Formatter.NotEditableValueFormatter(variableViewModel.DataType, variableViewModel.VariableMaxValueUom);
            };
            window["readOnlyMinValueFormatter"] = function (value, variableViewModel, index) {
                return Formatter.NotEditableValueFormatter(variableViewModel.DataType, variableViewModel.VariableMinValueUom);
            };
            window["readOnlyMaxValueFormatter"] = function (value, variableViewModel, index) {
                return Formatter.NotEditableValueFormatter(variableViewModel.DataType, variableViewModel.VariableMaxValueUom);
            };
            window["readOnlyParameterInitValueFormatter"] = function (value, variableViewModel, index) {
                return Formatter.NotEditableValueFormatter(variableViewModel.DataType, variableViewModel.VariableInitValueUom, variableViewModel.VariableInitValue);
            };
            window["readOnlyParameterDevValueFormatter"] = function (value, variableViewModel, index) {
                return Formatter.NotEditableValueFormatter(variableViewModel.DataType, variableViewModel.VariableDevValueUom, variableViewModel.VariableDevValue);
            };
            // No UoM
            window["readOnlyVariableValueNoUomFormatter"] = function (value, variableVm, index) {
                var dataType = Formatter._dataTypeDictionary[variableVm.DataType];
                switch (dataType.Type) {
                    case datatype_model_10.DataTypeEnum.Bool: {
                        return string_util_2.StringUtil.sprintf(Formatter.InputCheckboxTemplate, value === true ? "checked" : "", "disabled");
                    }
                    case datatype_model_10.DataTypeEnum.Int:
                    case datatype_model_10.DataTypeEnum.Uint:
                    case datatype_model_10.DataTypeEnum.Sint:
                    case datatype_model_10.DataTypeEnum.Usint:
                    case datatype_model_10.DataTypeEnum.Dint:
                    case datatype_model_10.DataTypeEnum.UdInt:
                    case datatype_model_10.DataTypeEnum.Lint:
                    case datatype_model_10.DataTypeEnum.Ulint:
                    case datatype_model_10.DataTypeEnum.Real:
                    case datatype_model_10.DataTypeEnum.Lreal: {
                        return string_util_2.StringUtil.sprintf(Formatter.NotEditableUomVariableTemplate, variableVm.VariableCurrentValue);
                    }
                    default:
                        return value;
                }
            };
            // No UoM and no view for boolean
            window["readOnlyVariableValueNoUomNoBoolFormatter"] = function (value, variableVm, index) {
                var dataType = Formatter._dataTypeDictionary[variableVm.DataType];
                switch (dataType.Type) {
                    case datatype_model_10.DataTypeEnum.Bool: {
                        return null;
                    }
                    default:
                        return value;
                }
            };
            window["readOnlyBooleanValueFormatter"] = function (value, variableVm, index) {
                return string_util_2.StringUtil.sprintf(Formatter.InputCheckboxTemplate, value === true ? "checked" : "", "disabled");
            };
        };
        Formatter.RegisterWritableVariableFormatter = function () {
            // Value + UoM
            window["writableVariableValueFormatter"] = function (value, variableViewModel, index) {
                return Formatter.EditableValueFormatter(variableViewModel, true, variableViewModel.VariableCurrentValueUom, variableViewModel.VariableCurrentValue);
            };
            window["writableParameterValueFormatter"] = function (value, parameterViewModel, index) {
                return Formatter.EditableValueFormatter(parameterViewModel, true, parameterViewModel.VariableCurrentValueUom, parameterViewModel.VariableCurrentValue, parameterViewModel.IsParameterImportDone);
            };
            window["writableParameterImportedValueFormatter"] = function (value, parameterViewModel, index) {
                if (parameterViewModel.IsVariableImportedValueSet) {
                    var formattedValue = parameterViewModel.VariableImportedValueUom;
                    if (validator_19.Validator.isDefined(formattedValue)) {
                        return Formatter.EditableParameterValueFormatter(parameterViewModel, formattedValue, parameterViewModel.VariableImportedValue);
                    }
                }
                return Formatter.NoValueTemplate;
            };
            // Events for "Value + UoM" formatter
            window["writableVariableValueFormatterEvent"] = {
                // Focus in event
                "focusin input": function (e, value, variable, index) {
                    event_1.EventHandler.EditableInputFocusInHandler(e, value, variable, index, datatype_model_10.SourceValue.Current, this);
                },
                // Focus out event
                "focusout input": function (e, value, variable, index) {
                    event_1.EventHandler.EditableInputFocusOutHandler(e, value, variable, index, datatype_model_10.SourceValue.Current, this);
                },
                // Event input -> trigger when value change. 
                // Event keypress -> trigger when user press a key. Here I can prevent the insertion of new value
                // Event keydown -> trigger when user press a key. Here I can prevent the insertion of new value. Diffenet behaviuor than
                //                  keypress in case of 'backspace'..
                // Event keyup -> trigger when user release a key
                "input .editableValue": function (e, value, variable, index) {
                    event_1.EventHandler.EditableInputChangeHandler(e, value, variable, index, datatype_model_10.SourceValue.Current, this);
                },
                "paste input": function (e, value, variable, index) {
                    event_1.EventHandler.EditableInputPasteHandler(e, value, variable, index, datatype_model_10.SourceValue.Current);
                },
                "click input:checkbox": function (e, value, variable, index) {
                    event_1.EventHandler.EditableInputClickCheckboxHandler(e, value, variable, index, datatype_model_10.SourceValue.Current, this);
                }
            };
            // Events for "Value + UoM" formatter
            window["writableParameterImportedValueFormatterEvent"] = {
                // Focus in event
                "focusin input": function (e, value, parameter, index) {
                    event_1.EventHandler.EditableInputFocusInHandler(e, value, parameter, index, datatype_model_10.SourceValue.Imported, this);
                },
                // Focus out event
                "focusout input": function (e, value, parameter, index) {
                    event_1.EventHandler.EditableInputFocusOutHandler(e, value, parameter, index, datatype_model_10.SourceValue.Imported, this);
                },
                // Event input -> trigger when value change. 
                // Event keypress -> trigger when user press a key. Here I can prevent the insertion of new value
                // Event keydown -> trigger when user press a key. Here I can prevent the insertion of new value. Diffenet behaviuor than
                //                  keypress in case of 'backspace'..
                // Event keyup -> trigger when user release a key
                "input .editableValue": function (e, value, parameter, index) {
                    event_1.EventHandler.EditableInputChangeHandler(e, value, parameter, index, datatype_model_10.SourceValue.Imported, this);
                },
                "paste input": function (e, value, variable, index) {
                    event_1.EventHandler.EditableInputPasteHandler(e, value, variable, index, datatype_model_10.SourceValue.Imported);
                },
                "click input:checkbox": function (e, value, parameter, index) {
                    event_1.EventHandler.EditableInputClickCheckboxHandler(e, value, parameter, index, datatype_model_10.SourceValue.Imported, this);
                }
            };
            window["writableBooleanValueFormatter"] = function (value, variableVm, index) {
                return string_util_2.StringUtil.sprintf(Formatter.InputCheckboxTemplate, value === true ? "checked" : "", "");
            };
        };
        Formatter.RegisterAlarmFormatter = function () {
            window["alarmHistoryOperationFormatter"] = function (value, row, index) {
                if (row.IsAlarmActive == true && row.IsResetEnabled) {
                    var resetTranslation = translator_service_9.TranslatorService.getInstance().t(translator_service_9.TranslatorService.CfieldTranslationNs + "reset");
                    return '<button class="btn btn-primary reset-alarm">' + resetTranslation + '</button>';
                }
                else
                    return "";
            }.bind(this);
        };
        Formatter.RegisterChartSerieFormatter = function () {
            window["serieVisibleCheckboxFormatter"] = function (value, vm, index) {
                return string_util_2.StringUtil.sprintf(Formatter.InputCheckboxTemplate, vm.IsSerieVisible === true ? "checked" : "", "");
            };
            window["serieVisibleCheckboxFormatterEvent"] = {
                "click input:checkbox": function (e, value, vm, index) {
                    var inputBox = Formatter.GetInputHtml(e, this);
                    // Get value
                    var newValue = inputBox.checked;
                    vm.IsSerieVisible = newValue;
                }
            };
            window["serieSelectAxisFormatter"] = function (value, vm, index) {
                var translator = translator_service_9.TranslatorService.getInstance().t;
                var htmlNode = '<div id="axis-type"><select class="selectpicker" data-width="auto"><option %s>%s</option><option %s>%s</option></select></div>';
                htmlNode = string_util_2.StringUtil.sprintf(htmlNode, 
                // Is First selected?
                vm.YAxisType === log_definition_item_viewmodel_1.YAxisType.Primary ? Formatter.OptionSelectedTemplate : "", 
                // Text of first item
                translator(translator_service_9.TranslatorService.CfieldTranslationNs + "primary"), 
                // Is Second selected?
                vm.YAxisType === log_definition_item_viewmodel_1.YAxisType.Secondary ? Formatter.OptionSelectedTemplate : "", 
                // Text of second item
                translator(translator_service_9.TranslatorService.CfieldTranslationNs + "secondary"));
                return htmlNode;
            };
            window["serieSelectAxisFormatterEvent"] = {
                "change .selectpicker": function (e, value, vm, index) {
                    var target = $(event.target)[0];
                    if (target) {
                        var index_1 = target.selectedIndex;
                        if (typeof index_1 !== "undefined" && index_1 != null) {
                            // Convert to type
                            vm.YAxisType = log_definition_item_viewmodel_1.UtilSerie.IndexToYAxisTypeConverter(index_1);
                        }
                    }
                },
            };
            window["serieSelectStyleFormatter"] = function (value, vm, index) {
                var translator = translator_service_9.TranslatorService.getInstance().t;
                var htmlNode = '<div id="serie-style"><select class="selectpicker" data-width="auto"><option %s>%s</option><option %s>%s</option></select></div>';
                htmlNode = string_util_2.StringUtil.sprintf(htmlNode, 
                // Is First selected?
                vm.ChartLineStyle === log_definition_item_viewmodel_1.ChartLineStyle.Line ? Formatter.OptionSelectedTemplate : "", 
                // Text of first item
                translator(translator_service_9.TranslatorService.CfieldTranslationNs + "line"), 
                // Is Second selected?
                vm.ChartLineStyle === log_definition_item_viewmodel_1.ChartLineStyle.Step ? Formatter.OptionSelectedTemplate : "", 
                // Text of second item
                translator(translator_service_9.TranslatorService.CfieldTranslationNs + "step"));
                return htmlNode;
            };
            window["serieSelectStyleFormatterEvent"] = {
                "change .selectpicker": function (e, value, vm, index) {
                    var target = $(event.target)[0];
                    if (target) {
                        var index_2 = target.selectedIndex;
                        if (typeof index_2 !== "undefined" && index_2 != null) {
                            // Convert to type
                            vm.ChartLineStyle = log_definition_item_viewmodel_1.UtilSerie.IndexToChartLineStyleConverter(index_2);
                        }
                    }
                },
            };
            window["serieColorPickerFormatter"] = function (value, vm, index) {
                /*
                let htmlNode = StringUtil.sprintf(
                    '<div class="input-group colorpicker-component"><input type="text" value="%s" class="form-control"/><span class="input-group-addon"><i></i></span></div>',
                    vm.ColorName
                 );
                 */
                var htmlNode = string_util_2.StringUtil.sprintf('<div class="colorpicker-component" data-color=%s><span class="input-group-addon"><i></i></span></div>', vm.ColorName);
                return htmlNode;
            };
            window["serieColorPickerFormatterEvent"] = {
                /*
                "change .form-control":  function (e, value, vm: LogSerieItemViewModel, index: number) {
                    let inputBox = Formatter.GetInputHtml(e, this);
                    let newValue = inputBox.value;
                    if (typeof newValue !== "undefined" && newValue != null) {
                        vm.ColorName = newValue;
                    }
                },
                */
                "changeColor .colorpicker-component": function (e, value, vm, index) {
                    var newValue = e.color.toHex();
                    if (typeof newValue !== "undefined" && newValue != null) {
                        vm.ColorName = newValue;
                    }
                }
            };
        };
        Formatter.RegisterRowStyleFormatter = function () {
            window["alarmHistoryRowStyle"] = function (row, index) {
                if (row.IsAlarmActive) {
                    return {
                        classes: style_2.StyleClass.DangerRow
                    };
                }
                return {};
            };
            window["variableViewRowStyle"] = function (variable, index) {
                if (variable.IsVariableToUpdate == true) {
                    return {
                        classes: style_2.StyleClass.RemeberToSaveRowClass
                    };
                }
                return {};
            };
        };
        Formatter.RegisterColumnStyleFormatter = function () {
            // Categories formatter
            window["categoryColumnFormatter"] = function (value, editableVariableItemViewModel, index) {
                // TODO: improve with binding?
                var listHtml = [];
                for (var _i = 0, _a = editableVariableItemViewModel.CategoryViewModels; _i < _a.length; _i++) {
                    var variableCategoryItemViewModel = _a[_i];
                    var liHtml = string_util_2.StringUtil.sprintf(Formatter.CategoryRecordLiTemplate, variableCategoryItemViewModel.CurrentCategoryCssClass(), variableCategoryItemViewModel.CategoryName());
                    listHtml.push(liHtml);
                }
                var innerHtml = listHtml.join(" ");
                return string_util_2.StringUtil.sprintf(Formatter.CategoryRecordUlTemplate, innerHtml);
            };
            window["descriptionColumnFormatter"] = function (value, editableVariableItemViewModel, index) {
                if (validator_19.Validator.isDefined(editableVariableItemViewModel.VariableDescription)) {
                    return editableVariableItemViewModel.VariableDescription;
                }
                else {
                    return Formatter.NoValueTemplate;
                }
            };
            window["readOnlyParameterImportedValueCellStyle"] = function (value, parameterViewModel, index, field) {
                if (parameterViewModel.IsCurrentNotEqualToImportedValue) {
                    return {
                        classes: style_2.StyleClass.ImportedNotEqualToCurrentCssClass
                    };
                }
                return "";
            };
        };
        Formatter.RegisterSortFunction = function () {
            window["alarmHistorySortStartTimeTable"] = function (item1, item2) {
                if (item1.IsAlarmActive && !item2.IsAlarmActive) {
                    return 1;
                }
                if (!item1.IsAlarmActive && item2.IsAlarmActive) {
                    return -1;
                }
                if (item1.StartDate != null && item2.StartDate != null) {
                    return item1.StartDate.getTime() - item2.StartDate.getTime();
                }
                if (item1.StartDate != null) {
                    return 1;
                }
                else {
                    return -1;
                }
            };
            window["alarmHistorySortStopTimeTable"] = function (item1, item2) {
                if (item1.IsAlarmActive && !item2.IsAlarmActive) {
                    return 1;
                }
                if (!item1.IsAlarmActive && item2.IsAlarmActive) {
                    return -1;
                }
                if (item1.StopDate != null && item2.StopDate != null) {
                    return item1.StopDate.getTime() - item2.StopDate.getTime();
                }
                if (item1.StopDate != null) {
                    return 1;
                }
                else {
                    return -1;
                }
            };
            window["numericSorter"] = function (item1, item2) {
                if (item1 && !item2) {
                    return 1;
                }
                if (!item1 && item2) {
                    return -1;
                }
                var num1 = parseInt(item1.replace(/^\D+/g, ''));
                var num2 = parseInt(item2.replace(/^\D+/g, ''));
                if (validator_19.Validator.isDefined(num1) && validator_19.Validator.isDefined(num2) && isNaN(num1) === false && isNaN(num2) === false)
                    return num1 - num2;
                return 0;
            };
        };
        Formatter.GetInputHtml = function (event, that) {
            var target = $(event.target)[0];
            if (target) {
                return target;
            }
            else {
                return that;
            }
        };
        // For BOOL datatye then it read the "checked" field, for other datatype it read the "value" field
        Formatter.GetValue = function (variable, inputBox) {
            if (variable.DataType === datatype_model_10.DataTypeEnum.Bool) {
                return inputBox.checked;
            }
            else {
                if (datatype_model_10.DataTypeUtil.IsNumericDataType(variable.DataType)) {
                    var number = Number(inputBox.value);
                    if (validator_19.Validator.isDefined(number) && !isNaN(number)) {
                        return number.valueOf();
                    }
                }
                return inputBox.value;
            }
        };
        // Html template
        Formatter.InputCheckboxTemplate = '<div class="centerText"><input type="checkbox" %s %s></div>';
        Formatter.EditableUomVariableTemplate = '<div class="editableValueContainer"><input class="editableValue form-control %s" type="text" value="%s" %s></div>';
        Formatter.NotEditableUomVariableTemplate = '<div class="notEditableValue">%s</div>';
        Formatter.CategoryRecordUlTemplate = '<ul>%s</ul>';
        Formatter.CategoryRecordLiTemplate = '<li><div class="category-circle %s" title="%s"></div></li>';
        Formatter.NoValueTemplate = '<div class="centerText">-</div>';
        Formatter.OptionSelectedTemplate = 'selected="selected"';
        Formatter._isFormatterRegistered = false;
        return Formatter;
    }());
    exports.Formatter = Formatter;
    var DataFormatter = /** @class */ (function () {
        function DataFormatter() {
        }
        DataFormatter.FormatValue = function (value, datatypeEnum, numberOfDigitis) {
            if (!validator_19.Validator.isDefined(value)) {
                return null;
            }
            switch (datatypeEnum) {
                case datatype_model_10.DataTypeEnum.Bool:
                    return value && (value.toLowerCase() === "true" || value.toLowerCase() === "1") ? true : false;
                case datatype_model_10.DataTypeEnum.Lreal:
                case datatype_model_10.DataTypeEnum.Real:
                    return DataFormatter.FloatToString(value, numberOfDigitis);
                default:
                    return value;
            }
        };
        /**
         * Parse a string to a float, round to <code>precision</code> decimal and parse it to string again.
         * @param strFloat String to parse.
         * @param precision Number of digits.
         * @return the <code>strFloat</code> round to <code>precision</code> decimal.
         */
        DataFormatter.FloatToString = function (strFloat, precision) {
            if (validator_19.Validator.isDefined(strFloat) && strFloat.length > 0) {
                // Parse
                var floatVal = parseFloat(strFloat);
                // Round
                return DataFormatter.ToFixed(floatVal, precision);
            }
            return null;
        };
        /**
         * Round a float to <code>precision</code> decimal.
         * @param value Value to round.
         * @param precision Number of digits.
         * @return the round <code>value</code> at <code>precision</code> decimal.
         */
        DataFormatter.ToFixed = function (value, precision) {
            var power = Math.pow(10, precision || 0);
            return String(Math.round(value * power) / power);
        };
        return DataFormatter;
    }());
    exports.DataFormatter = DataFormatter;
    var ViewLogic = /** @class */ (function () {
        function ViewLogic() {
        }
        // Add logic to minimize dialog. See http://www.jquery-az.com/jquery/demo.php?ex=58.0_1
        ViewLogic.ModalMinimizeLogic = function (jquerySelector, modalClass) {
            var dialogContainerClass = "." + modalClass;
            var $dialogContainer = $(jquerySelector).closest(dialogContainerClass);
            var dialogContainerId = $dialogContainer.attr("id");
            var dialogContainerSelector = "#" + dialogContainerId;
            $(".modal-backdrop").addClass("display-none");
            $(dialogContainerSelector).toggleClass("min");
            if ($(dialogContainerSelector).hasClass("min")) {
                $(".minmaxCon").append($dialogContainer);
                $(jquerySelector).find("i").toggleClass("fa-minus").toggleClass("fa-clone");
            }
            else {
                $(".chartContainerDialog").append($dialogContainer);
                $(jquerySelector).find("i").toggleClass("fa-clone").toggleClass("fa-minus");
            }
            ;
        };
        // Add logic to minimize dialog. See http://www.jquery-az.com/jquery/demo.php?ex=58.0_1
        ViewLogic.ModalCloseMinimizeLogic = function (jquerySelector, modalClass) {
            var dialogContainerClass = "." + modalClass;
            var $dialogContainer = $(jquerySelector).closest(dialogContainerClass);
            $(this).closest(dialogContainerClass).removeClass("min");
            $(".chartContainerDialog").removeClass($dialogContainer);
            $(this).next('.modalMinimize').find("i").removeClass("fa fa-clone").addClass("fa fa-minus");
        };
        ViewLogic.ModalForceMaximazeClass = function (jquerySelector, modalClass) {
            var dialogContainerClass = "." + modalClass;
            var $dialogContainer = $(jquerySelector).closest(dialogContainerClass);
            var dialogContainerId = $dialogContainer.attr("id");
            var dialogContainerSelector = "#" + dialogContainerId;
            if ($(dialogContainerSelector).hasClass("min")) {
                $(dialogContainerSelector).toggleClass("min");
                $(jquerySelector).find("i").toggleClass("fa-clone").toggleClass("fa-minus");
            }
        };
        return ViewLogic;
    }());
    exports.ViewLogic = ViewLogic;
    var View = /** @class */ (function () {
        function View() {
        }
        View.InitColorPicker = function () {
            var colorPicker = $(".colorpicker-component");
            var colorPickerId = "colorPickerId";
            var _loop_1 = function (i) {
                var item = colorPicker.eq(i);
                item.colorpicker({
                    template: '<div class="colorpicker dropdown-menu">' +
                        '<div class="colorpicker-saturation"><i><b></b></i></div>' +
                        '<div class="colorpicker-hue"><i></i></div>' +
                        '<div class="colorpicker-alpha"></i></div>' +
                        '<div class="colorpicker-color"><div/></div>' +
                        '<input id="' + colorPickerId + i + '" type="text">'
                });
                var query = '#' + colorPickerId + i;
                item.colorpicker().on("changeColor", function (e) {
                    $(query).val(e.color.toHex());
                });
            };
            for (var i = 0; i < colorPicker.length; i++) {
                _loop_1(i);
            }
        };
        return View;
    }());
    exports.View = View;
});
define("common/binding", ["require", "exports", "knockout", "service/translator.service"], function (require, exports, ko, translator_service_10) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var Bindings = /** @class */ (function () {
        function Bindings() {
        }
        Bindings.RegisterBinding = function () {
            Bindings.registerDataPickerBinding();
        };
        Bindings.registerDataPickerBinding = function () {
            if (Bindings.IsDataPickerBinding == false) {
                var currentLocale_1 = translator_service_10.TranslatorService.getInstance().CurrentLanguage();
                ko.bindingHandlers.dateTimePicker = {
                    init: function (element, valueAccessor, allBindingsAccessor) {
                        //initialize datepicker with some optional options
                        var options = allBindingsAccessor().dateTimePickerOptions ||
                            {
                                icons: {
                                    time: "fa fa-clock-o",
                                    date: "fa fa-calendar",
                                    up: "fa fa-arrow-up",
                                    down: "fa fa-arrow-down",
                                    previous: 'fa fa-arrow-left',
                                    next: 'fa fa-arrow-right',
                                    today: 'fa fa-circle',
                                    clear: 'fa fa-trash-o',
                                    close: 'fa fa-remove'
                                },
                                locale: currentLocale_1
                            };
                        $(element).datetimepicker(options);
                        //when a user changes the date, update the view model
                        ko.utils.registerEventHandler(element, "dp.change", function (event) {
                            var value = valueAccessor();
                            if (ko.isObservable(value)) {
                                if (event.date != null && !(event.date instanceof Date)) {
                                    if (event.date === false) {
                                        // Reset
                                        value(new Date());
                                    }
                                    else {
                                        value(event.date.toDate());
                                    }
                                }
                                else {
                                    value(event.date);
                                }
                            }
                        });
                        ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                            var picker = $(element).data("DateTimePicker");
                            if (picker) {
                                picker.destroy();
                            }
                        });
                    },
                    update: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
                        var picker = $(element).data("DateTimePicker");
                        //when the view model is updated, update the widget
                        if (picker) {
                            var koDate = ko.utils.unwrapObservable(valueAccessor());
                            //in case return from server datetime i am get in this form for example /Date(93989393)/ then fomat this
                            koDate = (typeof (koDate) !== 'object') ? new Date(parseFloat(koDate.replace(/[^0-9]/g, ''))) : koDate;
                            picker.date(koDate);
                        }
                    }
                };
                Bindings.IsDataPickerBinding = true;
            }
        };
        Bindings.IsDataPickerBinding = false;
        return Bindings;
    }());
    exports.Bindings = Bindings;
});
define("service/cfield.service", ["require", "exports", "knockout", "service/base.service", "service/device-config.service", "service/alarm.service", "service/login.service", "service/storage.service", "service/translator.service", "service/variable.service", "service/version/version-helper", "service/timer.service", "component/viewModel/box-component.viewmodel", "component/viewModel/iframe-component.viewmodel", "common/view", "common/binding", "text"], function (require, exports, ko, base_service_11, device_config_service_6, alarm_service_1, login_service_2, storage_service_2, translator_service_11, variable_service_3, version_helper_2, timer_service_3, box_component_viewmodel_1, iframe_component_viewmodel_1, view_3, binding_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Cfield service state.
     */
    var CfieldServiceState;
    (function (CfieldServiceState) {
        /** Not initialized. */
        CfieldServiceState[CfieldServiceState["NotInit"] = 0] = "NotInit";
        /** The reading of cField manigest is in progress. */
        CfieldServiceState[CfieldServiceState["ReadCfieldManifestOnProgress"] = 1] = "ReadCfieldManifestOnProgress";
        /** The cField manifest is initialized. */
        CfieldServiceState[CfieldServiceState["CfieldManifestInitialized"] = 2] = "CfieldManifestInitialized";
        /** The reading of cField manifest is failed. */
        CfieldServiceState[CfieldServiceState["CfieldManifestError"] = 3] = "CfieldManifestError";
        /** Translator initialization on progress. */
        CfieldServiceState[CfieldServiceState["TranslatorOnProgress"] = 4] = "TranslatorOnProgress";
        /** Translator initialized. */
        CfieldServiceState[CfieldServiceState["TranslatorInitialized"] = 5] = "TranslatorInitialized";
        /** Translator error. */
        CfieldServiceState[CfieldServiceState["TranslatorError"] = 6] = "TranslatorError";
        /** Device configuration not initialized. */
        CfieldServiceState[CfieldServiceState["DeviceConfigUnInit"] = 7] = "DeviceConfigUnInit";
        /** The initialization of device confifuration is started. */
        CfieldServiceState[CfieldServiceState["DeviceConfigStartInit"] = 8] = "DeviceConfigStartInit";
        /** The download of configuritation is on progress. */
        CfieldServiceState[CfieldServiceState["DeviceConfigOnDownloadConfig"] = 9] = "DeviceConfigOnDownloadConfig";
        /** The build of configuritation is on progress. */
        CfieldServiceState[CfieldServiceState["DeviceConfigOnBuildConfig"] = 10] = "DeviceConfigOnBuildConfig";
        /** The configuritation is initilized. */
        CfieldServiceState[CfieldServiceState["DeviceConfigInitCompleted"] = 11] = "DeviceConfigInitCompleted";
        /** CFieldService is initialized. */
        CfieldServiceState[CfieldServiceState["Initialized"] = 12] = "Initialized";
        /** Refresh. */
        CfieldServiceState[CfieldServiceState["RefreshConfig"] = 13] = "RefreshConfig";
        /** Error. */
        CfieldServiceState[CfieldServiceState["Error"] = 14] = "Error";
        /** Crc is changed. */
        CfieldServiceState[CfieldServiceState["CrcChanged"] = 15] = "CrcChanged";
        /** cField not supported. */
        CfieldServiceState[CfieldServiceState["CfieldNotSupported"] = 16] = "CfieldNotSupported";
        /** cField not compatible. */
        CfieldServiceState[CfieldServiceState["CFieldVersionNotCompatible"] = 17] = "CFieldVersionNotCompatible";
    })(CfieldServiceState = exports.CfieldServiceState || (exports.CfieldServiceState = {}));
    /**
     * Model the state of the service initialization.
     */
    var CfieldServiceInitializationStateMonitor = /** @class */ (function () {
        /**
         * Init the state monitor.
         */
        function CfieldServiceInitializationStateMonitor() {
            var _this = this;
            this.translatorServiceStateObserver = function (newState) {
                switch (newState) {
                    case translator_service_11.TranslatorServiceState.InitializationOnProgress:
                        _this._cFieldStateObservable(CfieldServiceState.TranslatorOnProgress);
                        break;
                    case translator_service_11.TranslatorServiceState.Initialized:
                        _this._cFieldStateObservable(CfieldServiceState.TranslatorInitialized);
                        break;
                    case translator_service_11.TranslatorServiceState.Error:
                        _this._cFieldStateObservable(CfieldServiceState.Error);
                        break;
                }
            };
            /**
             * Map the state of device configuration to cField service state.
             * @param newState State of device configuration.
             */
            this.deviceConfiServiceStateObserver = function (newState) {
                switch (newState) {
                    case device_config_service_6.DeviceConfigServiceState.UnInit:
                        _this._cFieldStateObservable(CfieldServiceState.DeviceConfigUnInit);
                        break;
                    case device_config_service_6.DeviceConfigServiceState.StartInit:
                        _this._cFieldStateObservable(CfieldServiceState.DeviceConfigStartInit);
                        break;
                    case device_config_service_6.DeviceConfigServiceState.OnDownloadConfig:
                        _this._cFieldStateObservable(CfieldServiceState.DeviceConfigOnDownloadConfig);
                        break;
                    case device_config_service_6.DeviceConfigServiceState.OnBuildConfig:
                        _this._cFieldStateObservable(CfieldServiceState.DeviceConfigOnBuildConfig);
                        break;
                    case device_config_service_6.DeviceConfigServiceState.InitCompleted:
                        _this._cFieldStateObservable(CfieldServiceState.DeviceConfigInitCompleted);
                        break;
                    case device_config_service_6.DeviceConfigServiceState.CrcChanged:
                        _this._cFieldStateObservable(CfieldServiceState.CrcChanged);
                        break;
                    case device_config_service_6.DeviceConfigServiceState.CfieldNotSupported:
                        _this._cFieldStateObservable(CfieldServiceState.CfieldNotSupported);
                        break;
                    case device_config_service_6.DeviceConfigServiceState.CfieldVersionNotCompatible:
                        _this._cFieldStateObservable(CfieldServiceState.CFieldVersionNotCompatible);
                        break;
                    case device_config_service_6.DeviceConfigServiceState.Error:
                        _this._cFieldStateObservable(CfieldServiceState.Error);
                        break;
                }
            };
            // Init state
            this._cFieldStateObservable = ko.observable(CfieldServiceState.NotInit);
            // Set observer
            this.registerObserver();
        }
        Object.defineProperty(CfieldServiceInitializationStateMonitor.prototype, "CfieldStateObservable", {
            get: function () {
                return this._cFieldStateObservable;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CfieldServiceInitializationStateMonitor.prototype, "BootPromise", {
            get: function () {
                return this._bootPromise;
            },
            set: function (bootPromise) {
                this._bootPromise = bootPromise;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CfieldServiceInitializationStateMonitor.prototype, "InitPromise", {
            get: function () {
                return this._initPromise;
            },
            set: function (initPromise) {
                this._initPromise = initPromise;
            },
            enumerable: true,
            configurable: true
        });
        CfieldServiceInitializationStateMonitor.prototype.registerObserver = function () {
            this._observerArray = [];
            this._observerArray.push(translator_service_11.TranslatorService.getInstance().TranslatorStateObservable.subscribe(this.translatorServiceStateObserver));
            this._observerArray.push(device_config_service_6.DeviceConfigService.getInstance().DeviceConfigServiceStateObservable.subscribe(this.deviceConfiServiceStateObserver));
        };
        CfieldServiceInitializationStateMonitor.prototype.unregisterObserver = function () {
            for (var _i = 0, _a = this._observerArray; _i < _a.length; _i++) {
                var observer = _a[_i];
                observer.dispose();
            }
        };
        return CfieldServiceInitializationStateMonitor;
    }());
    exports.CfieldServiceInitializationStateMonitor = CfieldServiceInitializationStateMonitor;
    exports.ContainerComponentName = "container-component";
    // Every component has 2-level nested component
    // First level: Container component
    // Second level: Subcomponent (alarm, variable ecc..)
    // Then html of the (0-level) component are the same execept for the name of subcomponent passed as parameter to the container component
    var ComponentTemplateFirstPart = '<div data-bind=\'component: {name: "' + exports.ContainerComponentName + '", params:{ subComponentName: "';
    var ComponentTemplateSecondPart = '", hideExtraToolbar: HideExtraToolbar }}\'"></div>';
    /**
     * Main service of web application.
     */
    var cFieldService = /** @class */ (function (_super) {
        __extends(cFieldService, _super);
        /**
         * Init service.
         * @param baseUrl Root url of cField.
         */
        function cFieldService(baseUrl) {
            var _this = _super.call(this) || this;
            /** Device engine used to execute CGI. */
            _this._deviceEngine = null;
            version_helper_2.VersionHelper.PrintCfieldVersion();
            if (cFieldService._instance) {
                _this._error(base_service_11.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            _this._deviceEngine = new DeviceEngine();
            // Set paths
            if (baseUrl === "/")
                baseUrl = "";
            _this._baseUrl = baseUrl;
            _this._basePageViewUrl = baseUrl + "/view/page";
            _this._baseComponentViewUrl = baseUrl + "/view/component";
            // Set view formatter
            view_3.Formatter.RegisterFormatter();
            return _this;
        }
        Object.defineProperty(cFieldService.prototype, "CFieldManifest", {
            get: function () { return this._cFieldManifest; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(cFieldService.prototype, "StateMonitor", {
            /** Get the state of service initialization. */
            get: function () {
                return this._stateMonitor;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(cFieldService.prototype, "BaseUrl", {
            get: function () {
                return this._baseUrl;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(cFieldService.prototype, "BasePageViewUrl", {
            get: function () {
                return this._basePageViewUrl;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(cFieldService.prototype, "BaseComponentViewUrl", {
            get: function () {
                return this._baseComponentViewUrl;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Release resource on logout.
         */
        cFieldService.prototype.releaseResourceOnLogout = function () {
            this._deviceConfigService.resetState();
            // Fire the refresh state: the components can observer this state to clear view/viewmodel 
            this._stateMonitor.CfieldStateObservable(CfieldServiceState.RefreshConfig);
            // cField is not initialized. Reset to CfieldManifestInitialized
            this._stateMonitor.CfieldStateObservable(CfieldServiceState.CfieldManifestInitialized);
        };
        Object.defineProperty(cFieldService.prototype, "DeviceEngine", {
            get: function () { return this._deviceEngine; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(cFieldService.prototype, "AlarmHistoryComponentViewModelCached", {
            get: function () {
                return this._alarmHistoryComponentViewModelCached;
            },
            set: function (value) {
                this._alarmHistoryComponentViewModelCached = value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(cFieldService.prototype, "ParameterComponenetViewModelCached", {
            get: function () {
                return this._parameterComponenetViewModelCached;
            },
            set: function (value) {
                this._parameterComponenetViewModelCached = value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(cFieldService.prototype, "VariableComponenetViewModelCached", {
            get: function () {
                return this._variableComponenetViewModelCached;
            },
            set: function (value) {
                this._variableComponenetViewModelCached = value;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Dispose cached <code>AlarmHistoryComponentViewModel</code> instance.
         */
        cFieldService.prototype.disposeAlarmHistoryComponentViewModelCached = function () {
            this._alarmHistoryComponentViewModelCached = null;
        };
        cFieldService.prototype.disposeParameterComponentViewModelCached = function () {
            this._parameterComponenetViewModelCached = null;
        };
        cFieldService.prototype.disposeVariableComponentViewModelCached = function () {
            this._variableComponenetViewModelCached = null;
        };
        /**
         * Build an instance of cField service in accord with singleton pattern.
         * @param baseUrl Root url of cField.
         * @return an instance of cField service.
         */
        cFieldService.buildInstance = function (baseUrl) {
            if (cFieldService._instance === null) {
                cFieldService._instance = new cFieldService(baseUrl);
                // Init (in the constructor cFieldService._instance is not initialized and then the other service can't retrieve a valid reference)
                cFieldService._instance.initServices();
                // Register componenets
                cFieldService._instance.registerComponents(cFieldService._instance._baseComponentViewUrl);
                // Boot service
                cFieldService._instance.bootService();
            }
            return cFieldService._instance;
        };
        /**
         * Get an instance of cField service in accord with singleton pattern.
         * @return an instance of cField service.
         */
        cFieldService.getInstance = function () {
            if (!cFieldService._instance) {
                throw new Error("cFieldService error: CFieldService is not init");
            }
            return cFieldService._instance;
        };
        /**
         * Create services.
         */
        cFieldService.prototype.initServices = function () {
            // Init state
            this._deviceConfigService = device_config_service_6.DeviceConfigService.buildInstance(this._baseUrl);
            this._loginService = login_service_2.LoginService.getInstance();
            this._alarmService = alarm_service_1.AlarmService.getInstance();
            this._variableService = variable_service_3.VariableService.getInstance();
            this._translatorService = translator_service_11.TranslatorService.buildInstance(this._baseUrl);
            this._timerHelper = timer_service_3.TimerService.buildInstance(timer_service_3.Timezone.Remote);
            this._storageService = storage_service_2.StorageService.getInstance();
            // Monitor the state of service
            this._stateMonitor = new CfieldServiceInitializationStateMonitor();
        };
        /**
         * Register the knockout components.
         */
        cFieldService.prototype.registerComponents = function (baseComponentViewUrl) {
            // Empty component. Used to initialize main view
            ko.components.register("empty-component", { template: " " });
            ko.components.register(exports.ContainerComponentName, { require: "component/viewModel/container-component.viewmodel" });
            ko.components.register("alarm-history-component", {
                viewModel: box_component_viewmodel_1.BoxComponentViewModel.GetCreateViewModelInstanceForKo(),
                template: this.getComponentTemplate("alarm-history-view-component")
            });
            ko.components.register("alarm-history-view-component", { require: "component/viewModel/alarm-history-component.viewmodel" });
            ko.components.register("alarm-history-toolbar-component", { require: "component/viewModel/alarm-history-toolbar-component.viewmodel" });
            ko.components.register("init-progress-component", { require: "component/viewModel/init-progress-component.viewmodel" });
            ko.components.register("log-component", {
                viewModel: box_component_viewmodel_1.BoxComponentViewModel.GetCreateViewModelInstanceForKo(),
                template: this.getComponentTemplate("log-view-component")
            });
            ko.components.register("log-view-component", { require: "component/viewModel/log-component.viewmodel" });
            ko.components.register("login-form-component", { require: "component/viewModel/login-form-component.viewmodel" });
            ko.components.register("io-direction-type-component", { template: this.getComponentTemplate("io-direction-type-view-component") });
            ko.components.register("io-direction-type-view-component", { require: "component/viewModel/io-direction-type-component.viewmodel" });
            ko.components.register("parameter-component", {
                viewModel: box_component_viewmodel_1.BoxComponentViewModel.GetCreateViewModelInstanceForKo(),
                template: this.getComponentTemplate("parameter-view-component")
            });
            ko.components.register("parameter-view-component", { require: "component/viewModel/parameter-component.viewmodel" });
            ko.components.register("parameter-toolbar-component", { require: "component/viewModel/parameter-toolbar-component.viewmodel" });
            ko.components.register("pgd1-component", {
                viewModel: box_component_viewmodel_1.BoxComponentViewModel.GetCreateViewModelInstanceForKo(),
                template: this.getComponentTemplate("pgd1-view-component")
            });
            ko.components.register("pgd1-view-component", { require: "component/viewModel/pgd1-component.viewmodel" });
            ko.components.register("system-network-component", {
                viewModel: box_component_viewmodel_1.BoxComponentViewModel.GetCreateViewModelInstanceForKo(),
                template: this.getComponentTemplate("system-network-view-component")
            });
            ko.components.register("system-network-view-component", { require: "component/viewModel/system-network-component.viewmodel" });
            ko.components.register("upgrade-component", {
                viewModel: box_component_viewmodel_1.BoxComponentViewModel.GetCreateViewModelInstanceForKo(),
                template: this.getComponentTemplate("upgrade-view-component")
            });
            ko.components.register("upgrade-view-component", { require: "component/viewModel/upgrade-component.viewmodel" });
            ko.components.register("upload-component", {
                viewModel: box_component_viewmodel_1.BoxComponentViewModel.GetCreateViewModelInstanceForKo(),
                template: this.getComponentTemplate("upload-view-component")
            });
            ko.components.register("upload-view-component", { require: "component/viewModel/upload-component.viewmodel" });
            ko.components.register("variable-component", {
                viewModel: box_component_viewmodel_1.BoxComponentViewModel.GetCreateViewModelInstanceForKo(),
                template: this.getComponentTemplate("variable-view-component")
            });
            ko.components.register("variable-view-component", { require: "component/viewModel/variable-component.viewmodel" });
            ko.components.register("variable-toolbar-component", { require: "component/viewModel/variable-toolbar-component.viewmodel" });
            ko.components.register("category-component", { require: "component/viewModel/category-component.viewmodel" });
            ko.components.register("cweb-page", {
                viewModel: {
                    createViewModel: function (params, componentInfo) {
                        return new iframe_component_viewmodel_1.IFrameComponentViewModel("/index.htm", params);
                    }.bind(this)
                },
                template: { require: "text!" + this._baseComponentViewUrl + "/iframe.html" }
            });
        };
        /** Boot service (translation + manifest) */
        cFieldService.prototype.bootService = function () {
            this._stateMonitor.BootPromise = new Promise(function (resolve, reject) {
                setTimeout(function () {
                    var _this = this;
                    // Request 1: get cField manifest
                    var bootPromise = this.getCfieldManifestPromise()
                        .then(function () {
                        // Request 2: init translator
                        return translator_service_11.TranslatorService.getInstance().init();
                    })
                        .then(function () {
                        // Register binding and extenders for Knockout.js
                        binding_1.Bindings.RegisterBinding();
                        resolve();
                    })
                        .catch(function (reason) {
                        _this._error(reason);
                        reject(reason);
                    });
                }.bind(this), 5);
            }.bind(this));
        };
        /**
         * Init CField service (device configuration)
         */
        cFieldService.prototype.initService = function () {
            var _this = this;
            if (this._stateMonitor.CfieldStateObservable() !== CfieldServiceState.CfieldManifestInitialized) {
                throw Error("CfieldService error. Init service called before the completion of the boot");
            }
            if (this._isInitServiceInProgress == true) {
                return;
            }
            // Init state monitor
            this._stateMonitor.InitPromise = new Promise(function (resolve, reject) {
                this._isInitServiceInProgress = true;
                setTimeout(function () {
                    var _this = this;
                    var initDevicePromise = this._deviceConfigService.initialize();
                    initDevicePromise
                        .then(function () {
                        _this._stateMonitor.CfieldStateObservable(CfieldServiceState.Initialized);
                        resolve();
                        _this._isInitServiceInProgress = false;
                    })
                        .catch(function (reason) {
                        _this._error(reason);
                        _this._stateMonitor.CfieldStateObservable(CfieldServiceState.Error);
                        reject();
                        _this._isInitServiceInProgress = false;
                    });
                }.bind(this), 5);
            }.bind(this));
            this._stateMonitor.InitPromise.catch(function (reason) {
                _this._error("Init of cField is aborted");
            });
        };
        cFieldService.prototype.getCfieldManifestPromise = function () {
            return new Promise(function (resolve, reject) {
                $.ajax({
                    dataType: "json",
                    url: this._baseUrl + "/" + cFieldService.CfieldManifestPath,
                    success: function (cFieldManifest, textStatus, jqXHR) {
                        // Set if login must be enabled
                        this._loginService.IsLoginEnabled(cFieldManifest && cFieldManifest.properties && cFieldManifest.properties.isLoginEnabled === true);
                        // Resolve
                        this._cFieldManifest = cFieldManifest;
                        this._stateMonitor.CfieldStateObservable(CfieldServiceState.CfieldManifestInitialized);
                        resolve(cFieldManifest);
                    }.bind(this),
                    error: function (jqXHR, textStatus, errorThrown) {
                        cFieldService.getInstance().StateMonitor.CfieldStateObservable(CfieldServiceState.CfieldManifestError);
                        reject("Failed to load " + cFieldService.CfieldManifestPath + " file. " + textStatus + "  " + errorThrown);
                    },
                    timeout: 5000
                });
            }.bind(this));
        };
        /**
         * Get the html template of 0-level component.
         * @param subcomponentName Name of component.
         */
        cFieldService.prototype.getComponentTemplate = function (subcomponentName) {
            return ComponentTemplateFirstPart + subcomponentName + ComponentTemplateSecondPart;
        };
        /** Path of cField manifest. */
        cFieldService.CfieldManifestPath = "cfield.json";
        /** Singleton instance. */
        cFieldService._instance = null;
        return cFieldService;
    }(base_service_11.BaseService));
    exports.cFieldService = cFieldService;
});
define("service/route.service", ["require", "exports", "knockout", "crossroads", "service/cfield.service", "service/base.service"], function (require, exports, ko, crossroads, cfield_service_5, base_service_12) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Route service.
     */
    var RouteService = /** @class */ (function (_super) {
        __extends(RouteService, _super);
        /**
         * Init service.
         */
        function RouteService() {
            var _this = _super.call(this) || this;
            if (RouteService._instance) {
                _this._error(base_service_12.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            _this._componentName = ko.observable("empty-component");
            _this._routeEntryInfoArray = [];
            _this._defaultRoute = "";
            return _this;
        }
        Object.defineProperty(RouteService.prototype, "ComponentName", {
            get: function () { return this._componentName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RouteService.prototype, "RouteEntryInfoArray", {
            get: function () { return this._routeEntryInfoArray; },
            enumerable: true,
            configurable: true
        });
        /**
         * Get the instance of route service in accord with singleton pattern.
         * @return an instance of route service.
         */
        RouteService.getInstance = function () {
            if (RouteService._instance === null) {
                RouteService._instance = new RouteService();
            }
            return RouteService._instance;
        };
        /**
         * Init route service.
         */
        RouteService.prototype.initialize = function () {
            // Get manifest
            var manifest = cfield_service_5.cFieldService.getInstance().CFieldManifest;
            if (manifest && manifest.menu) {
                this.initRouteAndMainMenu(manifest.menu);
            }
            else {
                this._error("cField manifest error: manifest or manifest.menu are invalid (manifest=" + manifest + ", manifest.menu=" + manifest.menu);
            }
        };
        /**
         * Init the service in accord with manifest file.
         * @param menuArray Array of menu items.
         */
        RouteService.prototype.initRouteAndMainMenu = function (menuArray) {
            var cFieldServiceInstance = cfield_service_5.cFieldService.getInstance();
            var isDefaultRouteInit = false;
            var _loop_2 = function (menuItem) {
                if (!menuItem.idRoute) {
                    this_1._log("idRoute=" + menuItem.idRoute + " is not valid. Skipped");
                    return "continue";
                }
                var componentName;
                if (menuItem.viewUrl) {
                    if (menuItem.viewUrl.endsWith(".html") || menuItem.viewUrl.endsWith(".htm")) {
                        componentName = menuItem.idRoute + "-component";
                        if (menuItem.viewModelUrl) {
                            ko.components.register(componentName, {
                                viewModel: { require: menuItem.viewModelUrl },
                                template: { require: "text!" + "/" + menuItem.viewUrl }
                            });
                        }
                        else {
                            ko.components.register(componentName, {
                                template: { require: "text!" + "/" + menuItem.viewUrl }
                            });
                        }
                    }
                    else {
                        this_1._log("Route service: viewUlr=" + menuItem.viewUrl + " not accepted");
                        return "continue";
                    }
                }
                else {
                    if (menuItem.componentName) {
                        componentName = menuItem.componentName;
                    }
                    else {
                        this_1._log("Route service: componentName not accepted");
                        return "continue";
                    }
                }
                // Add route
                crossroads.addRoute(menuItem.idRoute, function () {
                    this._componentName(componentName);
                }.bind(this_1));
                if (!isDefaultRouteInit) {
                    this_1._defaultRoute = menuItem.idRoute;
                    isDefaultRouteInit = true;
                }
                // Add menu
                this_1._routeEntryInfoArray.push(new RouteEntryInfo(menuItem.idRoute, menuItem.componentName, menuItem.icon, menuItem.iconPath, menuItem.displayName, menuItem.translationKey));
            };
            var this_1 = this;
            for (var _i = 0, menuArray_1 = menuArray; _i < menuArray_1.length; _i++) {
                var menuItem = menuArray_1[_i];
                _loop_2(menuItem);
            }
            // Setup route
            this.setupRoute();
        };
        /**
         * Setup the routing.
         */
        RouteService.prototype.setupRoute = function () {
            crossroads.bypassed.add(function (request) {
                this._log(request + " is a dead link");
            });
            // Listen to hash changes. This version use JQuery because the window.dispatchEvent version doesn't work on IE 11.
            $(window).on("hashchange", function (e) {
                var route = "/";
                var hash = window.location.hash;
                if (hash.length == 0) {
                    route = this._defaultRoute;
                }
                else if (hash.length > 0) {
                    // Take the final hash part
                    route = hash.split("#").pop();
                    // Remove query string part
                    route = route.split("?")[0];
                }
                crossroads.parse(route);
            }.bind(this)).trigger("hashchange");
        };
        /** Singleton instance. */
        RouteService._instance = null;
        return RouteService;
    }(base_service_12.BaseService));
    exports.RouteService = RouteService;
    var RouteEntryInfo = /** @class */ (function () {
        function RouteEntryInfo(_idRoute, _componentName, _icon, _iconPath, _displayName, _translationKey) {
            this._idRoute = _idRoute;
            this._componentName = _componentName;
            this._icon = _icon;
            this._iconPath = _iconPath;
            this._displayName = _displayName;
            this._translationKey = _translationKey;
        }
        Object.defineProperty(RouteEntryInfo.prototype, "IdRoute", {
            get: function () { return this._idRoute; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RouteEntryInfo.prototype, "ComponentName", {
            get: function () { return this._componentName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RouteEntryInfo.prototype, "Icon", {
            get: function () { return this._icon; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RouteEntryInfo.prototype, "IconPath", {
            get: function () { return this._iconPath; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RouteEntryInfo.prototype, "DisplayName", {
            get: function () { return this._displayName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RouteEntryInfo.prototype, "TranslationKey", {
            get: function () { return this._translationKey; },
            enumerable: true,
            configurable: true
        });
        return RouteEntryInfo;
    }());
});
define("cfield.app", ["require", "exports", "knockout", "service/cfield.service", "service/route.service", "common/loggable-entity"], function (require, exports, ko, cfield_service_6, route_service_1, loggable_entity_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var MainViewComponentName = "main-view-component";
    /**
     * Main cField app
     */
    var CfieldApp = /** @class */ (function (_super) {
        __extends(CfieldApp, _super);
        function CfieldApp(baseUrl) {
            var _this = _super.call(this) || this;
            // Fix path
            if (baseUrl === "/")
                baseUrl = "";
            // Base path of the views of cField
            _this.viewBasePageUrl = baseUrl + "/view/page";
            // Register KO components
            _this.registerComponents();
            // Bootstrap knockout.js
            ko.applyBindings();
            // Init cFieldService
            var cFieldServiceInstance = cfield_service_6.cFieldService.buildInstance(baseUrl);
            // Wait cField is booted
            var bootCfieldPromise = cFieldServiceInstance.StateMonitor.BootPromise;
            bootCfieldPromise
                .then(function () {
                // Init router
                var routeService = route_service_1.RouteService.getInstance();
                routeService.initialize();
            })
                .catch(function (reason) {
                _this._error("Boot failed");
            });
            return _this;
        }
        CfieldApp.prototype.registerComponents = function () {
            var htmlTemplate = '<div data-bind=\'component: {name: "' + cfield_service_6.ContainerComponentName +
                '", params:{ subComponentName: "' + MainViewComponentName +
                '"}}\'"></div>';
            ko.components.register("main-component", { template: htmlTemplate });
            ko.components.register(MainViewComponentName, { require: "viewModel/main.viewmodel" });
            ko.components.register("alarm-history-page", { require: "viewModel/alarm-history.viewmodel" });
            ko.components.register("log-page", { require: "viewModel/log.viewmodel" });
            ko.components.register("parameter-page", { require: "viewModel/parameter.viewmodel" });
            ko.components.register("pgd1-page", { require: "viewModel/pgd1.viewmodel" });
            ko.components.register("system-network-page", { require: "viewModel/system-network.viewmodel" });
            ko.components.register("unit-page", { require: "viewModel/unit.viewmodel" });
            ko.components.register("upload-page", { require: "viewModel/upload.viewmodel" });
            ko.components.register("variable-page", { require: "viewModel/variable.viewmodel" });
        };
        /**
         * Get instance of cField app in accord with singleton patter.
         * @param baseUrl Base path of c.Field app.
         * @return an instance of cField service.
         */
        CfieldApp.getInstance = function (baseUrl) {
            if (CfieldApp._instance === null) {
                if (baseUrl) {
                    CfieldApp._instance = new CfieldApp(baseUrl);
                }
                else {
                    CfieldApp._instance = new CfieldApp("");
                }
            }
            return CfieldApp._instance;
        };
        /** Singleton instance. */
        CfieldApp._instance = null;
        return CfieldApp;
    }(loggable_entity_6.LoggableEntity));
    exports.CfieldApp = CfieldApp;
});
define("common/color", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ColorGenerator = /** @class */ (function () {
        function ColorGenerator() {
        }
        ColorGenerator.GetColor = function () {
            var color = this._colorArray[this._index];
            this._index++;
            if (this._index == this._colorArray.length) {
                this._index = 0;
            }
            return color;
        };
        ColorGenerator._colorArray = [
            "#82E0AA",
            "#D98880",
            "#F1C40F",
            "#5DADE2",
            "#D7BDE2",
            "#AED6F1",
            "#FAD7A0",
            "#7DCEA0",
            "#E59866",
            "#EC7063",
            "#34495E",
            "#3498DB" // blue
        ];
        ColorGenerator._index = 0;
        return ColorGenerator;
    }());
    exports.ColorGenerator = ColorGenerator;
});
define("common/error", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ErrorCodeConverter = /** @class */ (function () {
        function ErrorCodeConverter() {
        }
        ErrorCodeConverter.ErrorUpgradeCodeToText = function (codeText) {
            var code = parseInt(codeText);
            switch (code) {
                case -5:
                    return "Wrong sign";
                case -6:
                    return "No memory";
                case -11:
                    return "Busy";
                case -12:
                    return "No AP1";
                case -1.:
                    return "No upload";
                case -18:
                    return "Wrong file";
                default:
                    "";
            }
        };
        return ErrorCodeConverter;
    }());
    exports.ErrorCodeConverter = ErrorCodeConverter;
});
define("common/file-util", ["require", "exports", "file-saver"], function (require, exports, fileSaver) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var FileUtil = /** @class */ (function () {
        function FileUtil() {
        }
        /**
         * https://github.com/eligrey/FileSaver.js
         */
        FileUtil.IsBlobSupported = function () {
            try {
                var isFileSaverSupported = !!new Blob();
            }
            catch (e) {
                alert('File save not supported by your browser');
                return false;
            }
            return true;
        };
        FileUtil.SaveTextFileOnDisk = function (filename, body) {
            if (FileUtil.IsBlobSupported() == false) {
                return false;
            }
            var blob = new Blob([body], { type: "text/plain;charset=utf-8" });
            if (blob) {
                // TODO: if I delete the next row the saveAs function is not loaded...
                var workaround = fileSaver;
                window.saveAs(blob, filename);
            }
        };
        return FileUtil;
    }());
    exports.FileUtil = FileUtil;
});
define("component/viewModel/item/alert-item.viewmodel", ["require", "exports", "knockout", "service/translator.service"], function (require, exports, ko, translator_service_12) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var NotUpdateViewModel = /** @class */ (function () {
        function NotUpdateViewModel(_variableName, _setValue, _serverResponse) {
            this._variableName = _variableName;
            this._setValue = _setValue;
            this._serverResponse = _serverResponse;
        }
        Object.defineProperty(NotUpdateViewModel.prototype, "VariableName", {
            get: function () { return this._variableName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NotUpdateViewModel.prototype, "SetValue", {
            get: function () { return this._setValue; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NotUpdateViewModel.prototype, "ServerResponse", {
            get: function () { return this._serverResponse; },
            enumerable: true,
            configurable: true
        });
        return NotUpdateViewModel;
    }());
    exports.NotUpdateViewModel = NotUpdateViewModel;
    var NotResetAlarmViewModel = /** @class */ (function () {
        function NotResetAlarmViewModel(_alarmName, _translationKey, _starttime) {
            this._alarmName = _alarmName;
            this._translationKey = _translationKey;
            this._starttime = _starttime;
            this._alarmDescription = ko.observable(translator_service_12.TranslatorService.getInstance().t(translator_service_12.TranslatorService.VariableTranslationNs + this._translationKey, this._translationKey));
        }
        Object.defineProperty(NotResetAlarmViewModel.prototype, "AlarmName", {
            get: function () { return this._alarmName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NotResetAlarmViewModel.prototype, "AlarmDescription", {
            get: function () {
                return this._alarmDescription;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NotResetAlarmViewModel.prototype, "Starttime", {
            get: function () { return this._starttime; },
            enumerable: true,
            configurable: true
        });
        NotResetAlarmViewModel.prototype.translate = function () {
            this._alarmDescription(translator_service_12.TranslatorService.getInstance().t(translator_service_12.TranslatorService.VariableTranslationNs + this._translationKey, this._translationKey));
        };
        return NotResetAlarmViewModel;
    }());
    exports.NotResetAlarmViewModel = NotResetAlarmViewModel;
    var InvalidUpdateViewModel = /** @class */ (function () {
        function InvalidUpdateViewModel(_variableName, _editedValue) {
            this._variableName = _variableName;
            this._editedValue = _editedValue;
        }
        Object.defineProperty(InvalidUpdateViewModel.prototype, "VariableName", {
            get: function () { return this._variableName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(InvalidUpdateViewModel.prototype, "VariableEditValue", {
            get: function () { return this._editedValue; },
            enumerable: true,
            configurable: true
        });
        return InvalidUpdateViewModel;
    }());
    exports.InvalidUpdateViewModel = InvalidUpdateViewModel;
});
define("component/viewModel/alert.viewmodel", ["require", "exports", "knockout", "service/translator.service"], function (require, exports, ko, translator_service_13) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var BaseAlertCssStyle = /** @class */ (function () {
        function BaseAlertCssStyle() {
        }
        BaseAlertCssStyle.AlertSuccessCssClass = "alert-success";
        BaseAlertCssStyle.AlertInfoCssClass = "alert-info";
        BaseAlertCssStyle.AlertWarningCssClass = "alert-warning";
        BaseAlertCssStyle.AlertErrorCssClass = "alert-danger";
        return BaseAlertCssStyle;
    }());
    exports.BaseAlertCssStyle = BaseAlertCssStyle;
    var ParameterAlertState;
    (function (ParameterAlertState) {
        ParameterAlertState[ParameterAlertState["Invisible"] = 0] = "Invisible";
        ParameterAlertState[ParameterAlertState["Success"] = 1] = "Success";
        ParameterAlertState[ParameterAlertState["Info"] = 2] = "Info";
        ParameterAlertState[ParameterAlertState["Error"] = 3] = "Error";
        ParameterAlertState[ParameterAlertState["Waiting"] = 4] = "Waiting";
        ParameterAlertState[ParameterAlertState["UpdateFailed"] = 5] = "UpdateFailed";
        ParameterAlertState[ParameterAlertState["UpdateInvalid"] = 6] = "UpdateInvalid";
        ParameterAlertState[ParameterAlertState["UpdateFailedAndInvalid"] = 7] = "UpdateFailedAndInvalid";
        ParameterAlertState[ParameterAlertState["NothingToUpdate"] = 8] = "NothingToUpdate";
    })(ParameterAlertState = exports.ParameterAlertState || (exports.ParameterAlertState = {}));
    var BaseAlert = /** @class */ (function () {
        function BaseAlert() {
            this._alertErrorDetailMsg = ko.observable();
        }
        Object.defineProperty(BaseAlert.prototype, "AlertErrorDetailMsg", {
            get: function () { return this._alertErrorDetailMsg; },
            enumerable: true,
            configurable: true
        });
        return BaseAlert;
    }());
    var ParameterAlertViewModel = /** @class */ (function (_super) {
        __extends(ParameterAlertViewModel, _super);
        function ParameterAlertViewModel() {
            var _this = _super.call(this) || this;
            _this._state = ko.observable(ParameterAlertState.Invisible);
            _this._isAlertVisible = ko.computed({
                read: function () {
                    return _this._state() !== ParameterAlertState.Invisible;
                }
            });
            _this._alertCssClass = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case ParameterAlertState.Success:
                            return BaseAlertCssStyle.AlertSuccessCssClass;
                        case ParameterAlertState.Error:
                            return BaseAlertCssStyle.AlertErrorCssClass;
                        case ParameterAlertState.Info:
                            return BaseAlertCssStyle.AlertInfoCssClass;
                        default:
                            return BaseAlertCssStyle.AlertWarningCssClass;
                    }
                }
            });
            _this._alertTitle1 = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case ParameterAlertState.Success:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "operationSuccessfullyCompleted");
                        case ParameterAlertState.Info:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "setParameterWorkflowInfo");
                        case ParameterAlertState.Waiting:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "waiting");
                        case ParameterAlertState.UpdateFailed:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "someParameterAreNotUpdate");
                        case ParameterAlertState.UpdateFailedAndInvalid:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "someParameterAreNotUpdate");
                        case ParameterAlertState.UpdateInvalid:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "someParameterAreInvalid");
                        case ParameterAlertState.NothingToUpdate:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "nothingToUpdate");
                        case ParameterAlertState.Error:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "operationFailed");
                    }
                }
            });
            _this._alertTitle2 = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case ParameterAlertState.UpdateFailedAndInvalid:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "someParameterAreInvalid");
                    }
                }
            });
            _this._alertIsErrorDetailVisible = ko.computed({
                read: function () {
                    return _this._state() === ParameterAlertState.Error;
                }
            });
            _this._alertIsUpdateFailedTableVisible = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case ParameterAlertState.UpdateFailed:
                        case ParameterAlertState.UpdateFailedAndInvalid:
                            return true;
                        default:
                            return false;
                    }
                }
            });
            _this._alertIsUpdateInvalidTableVisible = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case ParameterAlertState.UpdateInvalid:
                            return true;
                        default:
                            return false;
                    }
                }
            });
            _this._alertVariableNameHeader = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case ParameterAlertState.UpdateFailed:
                        case ParameterAlertState.UpdateFailedAndInvalid:
                        case ParameterAlertState.UpdateInvalid:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "name");
                    }
                }
            });
            _this._alertSetValueHeader = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case ParameterAlertState.UpdateFailed:
                        case ParameterAlertState.UpdateFailedAndInvalid:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "valueItryToSet");
                    }
                }
            });
            _this._alertServerResponseHeader = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case ParameterAlertState.UpdateFailed:
                        case ParameterAlertState.UpdateFailedAndInvalid:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "valueAfterTheUpdate");
                    }
                }
            });
            _this._alertNotUpdatedArray = ko.observableArray();
            _this._alertEditValueHeader = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case ParameterAlertState.UpdateInvalid:
                        case ParameterAlertState.UpdateFailedAndInvalid:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "invalidValue");
                    }
                }
            });
            _this._alertInvalidUpdateArray = ko.observableArray();
            return _this;
        }
        Object.defineProperty(ParameterAlertViewModel.prototype, "State", {
            get: function () { return this._state; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "IsAlertVisible", {
            get: function () { return this._isAlertVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertCssClass", {
            get: function () { return this._alertCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertTitle1", {
            get: function () { return this._alertTitle1; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertTitle2", {
            get: function () { return this._alertTitle2; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertIsErrorDetailVisible", {
            get: function () { return this._alertIsErrorDetailVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertIsUpdateFailedTableVisible", {
            get: function () { return this._alertIsUpdateFailedTableVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertIsUpdateInvalidTableVisible", {
            get: function () { return this._alertIsUpdateInvalidTableVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertVariableNametHeader", {
            get: function () { return this._alertVariableNameHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertSetValueHeader", {
            get: function () { return this._alertSetValueHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertServerResponseHeader", {
            get: function () { return this._alertServerResponseHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertNotUpdatedArray", {
            get: function () { return this._alertNotUpdatedArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertEditValueHeader", {
            get: function () { return this._alertEditValueHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterAlertViewModel.prototype, "AlertInvalidUpdateArray", {
            get: function () { return this._alertInvalidUpdateArray; },
            enumerable: true,
            configurable: true
        });
        ParameterAlertViewModel.prototype.translate = function () {
            this._state.notifySubscribers();
        };
        return ParameterAlertViewModel;
    }(BaseAlert));
    exports.ParameterAlertViewModel = ParameterAlertViewModel;
    var AlarmAlertState;
    (function (AlarmAlertState) {
        AlarmAlertState[AlarmAlertState["Invisible"] = 0] = "Invisible";
        AlarmAlertState[AlarmAlertState["Success"] = 1] = "Success";
        AlarmAlertState[AlarmAlertState["Error"] = 2] = "Error";
        AlarmAlertState[AlarmAlertState["Waiting"] = 3] = "Waiting";
        AlarmAlertState[AlarmAlertState["ResetFailed"] = 4] = "ResetFailed";
    })(AlarmAlertState = exports.AlarmAlertState || (exports.AlarmAlertState = {}));
    var AlarmAlertViewModel = /** @class */ (function (_super) {
        __extends(AlarmAlertViewModel, _super);
        function AlarmAlertViewModel() {
            var _this = _super.call(this) || this;
            _this._state = ko.observable(AlarmAlertState.Invisible);
            _this._isAlertVisible = ko.computed({
                read: function () {
                    return _this._state() !== AlarmAlertState.Invisible;
                }
            });
            _this._alertCssClass = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case AlarmAlertState.Success:
                            return BaseAlertCssStyle.AlertSuccessCssClass;
                        case AlarmAlertState.Error:
                            return BaseAlertCssStyle.AlertErrorCssClass;
                        default:
                            return BaseAlertCssStyle.AlertWarningCssClass;
                    }
                }
            });
            _this._alertTitle = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case AlarmAlertState.Success:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "operationSuccessfullyCompleted");
                        case AlarmAlertState.Waiting:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "waiting");
                        case AlarmAlertState.ResetFailed:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "someAlarmAreNotReset");
                        case AlarmAlertState.Error:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "operationFailed");
                    }
                }
            });
            _this._alertIsErrorDetailVisible = ko.computed({
                read: function () {
                    return _this._state() === AlarmAlertState.Error;
                }
            });
            _this._alertIsResetFailedTableVisible = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case AlarmAlertState.ResetFailed:
                            return true;
                        default:
                            return false;
                    }
                }
            });
            _this._alertAlarmNameHeader = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case AlarmAlertState.ResetFailed:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "name");
                    }
                }
            });
            _this._alertAlarmDescriptionHeader = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case AlarmAlertState.ResetFailed:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "description");
                    }
                }
            });
            _this._alertStartimeHeader = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case AlarmAlertState.ResetFailed:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "starttime");
                    }
                }
            });
            _this._alertNotResetAlarmArray = ko.observableArray();
            return _this;
        }
        Object.defineProperty(AlarmAlertViewModel.prototype, "State", {
            get: function () { return this._state; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmAlertViewModel.prototype, "IsAlertVisible", {
            get: function () { return this._isAlertVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmAlertViewModel.prototype, "AlertCssClass", {
            get: function () { return this._alertCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmAlertViewModel.prototype, "AlertTitle", {
            get: function () { return this._alertTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmAlertViewModel.prototype, "AlertIsErrorDetailVisible", {
            get: function () { return this._alertIsErrorDetailVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmAlertViewModel.prototype, "AlertIsResetFailedTableVisible", {
            get: function () { return this._alertIsResetFailedTableVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmAlertViewModel.prototype, "AlertAlarmNameHeader", {
            get: function () { return this._alertAlarmNameHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmAlertViewModel.prototype, "AlertAlarmDescriptionHeader", {
            get: function () { return this._alertAlarmDescriptionHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmAlertViewModel.prototype, "AlertStartimeHeader", {
            get: function () { return this._alertStartimeHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmAlertViewModel.prototype, "AlertNotResetAlarmArray", {
            get: function () { return this._alertNotResetAlarmArray; },
            enumerable: true,
            configurable: true
        });
        AlarmAlertViewModel.prototype.translate = function () {
            this._state.notifySubscribers();
        };
        return AlarmAlertViewModel;
    }(BaseAlert));
    exports.AlarmAlertViewModel = AlarmAlertViewModel;
    var LogAlertState;
    (function (LogAlertState) {
        LogAlertState[LogAlertState["Invisible"] = 0] = "Invisible";
        LogAlertState[LogAlertState["NoData"] = 1] = "NoData";
        LogAlertState[LogAlertState["Error"] = 2] = "Error";
    })(LogAlertState = exports.LogAlertState || (exports.LogAlertState = {}));
    var LogAlertViewModel = /** @class */ (function (_super) {
        __extends(LogAlertViewModel, _super);
        function LogAlertViewModel() {
            var _this = _super.call(this) || this;
            _this._state = ko.observable(LogAlertState.Invisible);
            _this._isAlertVisible = ko.computed({
                read: function () {
                    return _this._state() !== LogAlertState.Invisible;
                }
            });
            _this._alertCssClass = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case LogAlertState.Error:
                            return BaseAlertCssStyle.AlertErrorCssClass;
                        case LogAlertState.NoData:
                            return BaseAlertCssStyle.AlertWarningCssClass;
                    }
                }
            });
            _this._alertTitle = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case LogAlertState.Error:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "operationFailed");
                        case LogAlertState.NoData:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "noData");
                    }
                }
            });
            _this._alertIsErrorDetailVisible = ko.computed({
                read: function () {
                    return _this._state() === LogAlertState.Error;
                }
            });
            return _this;
        }
        Object.defineProperty(LogAlertViewModel.prototype, "State", {
            get: function () { return this._state; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogAlertViewModel.prototype, "IsAlertVisible", {
            get: function () { return this._isAlertVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogAlertViewModel.prototype, "AlertCssClass", {
            get: function () { return this._alertCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogAlertViewModel.prototype, "AlertTitle", {
            get: function () { return this._alertTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogAlertViewModel.prototype, "AlertIsErrorDetailVisible", {
            get: function () { return this._alertIsErrorDetailVisible; },
            enumerable: true,
            configurable: true
        });
        LogAlertViewModel.prototype.translate = function () {
            this._state.notifySubscribers();
        };
        return LogAlertViewModel;
    }(BaseAlert));
    exports.LogAlertViewModel = LogAlertViewModel;
    var IoDirectionAlertState;
    (function (IoDirectionAlertState) {
        IoDirectionAlertState[IoDirectionAlertState["Invisible"] = 0] = "Invisible";
        IoDirectionAlertState[IoDirectionAlertState["BootFailed"] = 1] = "BootFailed";
    })(IoDirectionAlertState = exports.IoDirectionAlertState || (exports.IoDirectionAlertState = {}));
    var IoDirectionViewModel = /** @class */ (function (_super) {
        __extends(IoDirectionViewModel, _super);
        function IoDirectionViewModel() {
            var _this = _super.call(this) || this;
            _this._state = ko.observable(IoDirectionAlertState.Invisible);
            _this._isAlertVisible = ko.computed({
                read: function () {
                    return _this._state() !== IoDirectionAlertState.Invisible;
                }
            });
            _this._alertCssClass = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case IoDirectionAlertState.BootFailed:
                            return BaseAlertCssStyle.AlertErrorCssClass;
                    }
                }
            });
            _this._alertTitle = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case IoDirectionAlertState.BootFailed:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "loadDataFailed");
                    }
                }
            });
            _this._alertIsErrorDetailVisible = ko.computed({
                read: function () {
                    return _this._state() === IoDirectionAlertState.BootFailed;
                }
            });
            return _this;
        }
        Object.defineProperty(IoDirectionViewModel.prototype, "State", {
            get: function () { return this._state; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoDirectionViewModel.prototype, "IsAlertVisible", {
            get: function () { return this._isAlertVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoDirectionViewModel.prototype, "AlertCssClass", {
            get: function () { return this._alertCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoDirectionViewModel.prototype, "AlertTitle", {
            get: function () { return this._alertTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoDirectionViewModel.prototype, "AlertIsErrorDetailVisible", {
            get: function () { return this._alertIsErrorDetailVisible; },
            enumerable: true,
            configurable: true
        });
        IoDirectionViewModel.prototype.translate = function () {
            this._state.notifySubscribers();
        };
        return IoDirectionViewModel;
    }(BaseAlert));
    exports.IoDirectionViewModel = IoDirectionViewModel;
    var SystemAlertState;
    (function (SystemAlertState) {
        SystemAlertState[SystemAlertState["Invisible"] = 0] = "Invisible";
        SystemAlertState[SystemAlertState["BootFailed"] = 1] = "BootFailed";
    })(SystemAlertState = exports.SystemAlertState || (exports.SystemAlertState = {}));
    var SystemAlertViewModel = /** @class */ (function (_super) {
        __extends(SystemAlertViewModel, _super);
        function SystemAlertViewModel() {
            var _this = _super.call(this) || this;
            _this._state = ko.observable(SystemAlertState.Invisible);
            _this._isAlertVisible = ko.computed({
                read: function () {
                    return _this._state() !== SystemAlertState.Invisible;
                }
            });
            _this._alertCssClass = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case SystemAlertState.BootFailed:
                            return BaseAlertCssStyle.AlertErrorCssClass;
                    }
                }
            });
            _this._alertTitle = ko.computed({
                read: function () {
                    switch (_this._state()) {
                        case SystemAlertState.BootFailed:
                            return translator_service_13.TranslatorService.getInstance().t(translator_service_13.TranslatorService.CfieldTranslationNs + "loadDataFailed");
                    }
                }
            });
            _this._alertIsErrorDetailVisible = ko.computed({
                read: function () {
                    return _this._state() === SystemAlertState.BootFailed;
                }
            });
            return _this;
        }
        Object.defineProperty(SystemAlertViewModel.prototype, "State", {
            get: function () { return this._state; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemAlertViewModel.prototype, "IsAlertVisible", {
            get: function () { return this._isAlertVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemAlertViewModel.prototype, "AlertCssClass", {
            get: function () { return this._alertCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemAlertViewModel.prototype, "AlertTitle", {
            get: function () { return this._alertTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemAlertViewModel.prototype, "AlertIsErrorDetailVisible", {
            get: function () { return this._alertIsErrorDetailVisible; },
            enumerable: true,
            configurable: true
        });
        SystemAlertViewModel.prototype.translate = function () {
            this._state.notifySubscribers();
        };
        return SystemAlertViewModel;
    }(BaseAlert));
    exports.SystemAlertViewModel = SystemAlertViewModel;
});
define("component/viewModel/settings-viewmodel", ["require", "exports", "knockout", "common/loggable-entity", "service/storage.service", "service/translator.service"], function (require, exports, ko, loggable_entity_7, storage_service_3, translator_service_14) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    // TODO: include CRC and XML item (now they aren't validate)
    var SettingsViewModel = /** @class */ (function (_super) {
        __extends(SettingsViewModel, _super);
        function SettingsViewModel(_settingRefreshView) {
            var _this = _super.call(this) || this;
            _this._settingRefreshView = _settingRefreshView;
            _this._settingsTitle = ko.observable();
            _this._refreshIntervalSecondsTitle = ko.observable();
            _this._refreshIntervalSecondsValue = ko.observable();
            _this._exportSeparatorCharTitle = ko.observable();
            _this._exportSeparatorCharValue = ko.observable();
            _this._saveTitle = ko.observable();
            _this._cancelTitle = ko.observable();
            // Init
            var storageService = storage_service_3.StorageService.getInstance();
            switch (_this._settingRefreshView) {
                case SettingsRefreshViewId.Alarm:
                    _this._refreshIntervalSecondsValue(storageService.AlarmRefreshIntevalSeconds + "");
                    break;
                case SettingsRefreshViewId.Variable:
                    _this._refreshIntervalSecondsValue(storageService.VariableRefreshIntevalSeconds + "");
                    break;
            }
            _this._exportSeparatorCharValue(storageService.ExportCharSeparator);
            _this.translate();
            return _this;
        }
        Object.defineProperty(SettingsViewModel.prototype, "RefreshIntervalSecondsTitle", {
            get: function () { return this._refreshIntervalSecondsTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SettingsViewModel.prototype, "RefreshIntervalSecondsValue", {
            get: function () { return this._refreshIntervalSecondsValue; },
            set: function (value) {
                this._refreshIntervalSecondsValue = value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SettingsViewModel.prototype, "SettingsTitle", {
            get: function () { return this._settingsTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SettingsViewModel.prototype, "SaveTitle", {
            get: function () { return this._saveTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SettingsViewModel.prototype, "CancelTitle", {
            get: function () { return this._cancelTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SettingsViewModel.prototype, "ExportSeparatorCharTitle", {
            get: function () { return this._exportSeparatorCharTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SettingsViewModel.prototype, "ExportSeparatorCharValue", {
            get: function () { return this._exportSeparatorCharValue; },
            set: function (value) {
                this._exportSeparatorCharValue = value;
            },
            enumerable: true,
            configurable: true
        });
        SettingsViewModel.prototype.save = function () {
            var storageService = storage_service_3.StorageService.getInstance();
            // Overwrite properties
            // Refresh
            var valueNumber = parseInt(this._refreshIntervalSecondsValue());
            if (typeof valueNumber !== "undefined" && valueNumber !== null && isNaN(valueNumber) === false && valueNumber >= 1) {
                switch (this._settingRefreshView) {
                    case SettingsRefreshViewId.Alarm:
                        storageService.AlarmRefreshIntevalSeconds = valueNumber;
                        break;
                    case SettingsRefreshViewId.Variable:
                        storageService.VariableRefreshIntevalSeconds = valueNumber;
                        break;
                }
            }
            else {
                this._log("Skip the saving of refresh interval. Value is not valid. Value=" + this._refreshIntervalSecondsValue());
            }
            // Character separator
            var text = this._exportSeparatorCharValue();
            if (typeof text !== "undefined" && text !== null && text.length !== 0) {
                storageService.ExportCharSeparator = text;
            }
            else {
                this._log("Skip the saving of export characther sepatator. Value is not valid. Value=" + this._exportSeparatorCharValue());
            }
            // Save
            storageService.saveSettings();
        };
        SettingsViewModel.prototype.translate = function () {
            var translator = translator_service_14.TranslatorService.getInstance().t;
            this._settingsTitle(translator(translator_service_14.TranslatorService.CfieldTranslationNs + "settings"));
            this._refreshIntervalSecondsTitle(translator(translator_service_14.TranslatorService.CfieldTranslationNs + "refreshIntervalSeconds"));
            this._exportSeparatorCharTitle(translator(translator_service_14.TranslatorService.CfieldTranslationNs + "exportSeparatorChar"));
            this._saveTitle(translator(translator_service_14.TranslatorService.CfieldTranslationNs + "save"));
            this._cancelTitle(translator(translator_service_14.TranslatorService.CfieldTranslationNs + "cancel"));
        };
        return SettingsViewModel;
    }(loggable_entity_7.LoggableEntity));
    exports.SettingsViewModel = SettingsViewModel;
    var SettingsRefreshViewId;
    (function (SettingsRefreshViewId) {
        SettingsRefreshViewId[SettingsRefreshViewId["Alarm"] = 0] = "Alarm";
        SettingsRefreshViewId[SettingsRefreshViewId["Variable"] = 1] = "Variable";
    })(SettingsRefreshViewId = exports.SettingsRefreshViewId || (exports.SettingsRefreshViewId = {}));
});
define("component/viewModel/alarm-history-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/alarm.service", "service/storage.service", "service/timer.service", "service/translator.service", "component/viewModel/base-table-component.viewmodel", "component/viewModel/alert.viewmodel", "component/viewModel/settings-viewmodel", "component/viewModel/item/alarm-item.viewmodel", "component/viewModel/item/alert-item.viewmodel", "common/file-util"], function (require, exports, ko, cfield_service_7, alarm_service_2, storage_service_4, timer_service_4, translator_service_15, base_table_component_viewmodel_2, alert_viewmodel_1, settings_viewmodel_1, alarm_item_viewmodel_1, alert_item_viewmodel_1, file_util_1) {
    "use strict";
    var TableId = "#alarm-history-component-table";
    /**
     * Alarm component.
     */
    var AlarmHistoryComponentViewModel = /** @class */ (function (_super) {
        __extends(AlarmHistoryComponentViewModel, _super);
        function AlarmHistoryComponentViewModel(hideExtraToolbar) {
            var _this = _super.call(this, hideExtraToolbar, true, true, true) || this;
            /**
             * Refresh the alarms tables with the new values.
             * @param alarmsViewModelArray List of the viewmodels used for the records of the table.
             */
            _this.buildAlarmTable = function (alarmsViewModelArray) {
                $(function () {
                    $(TableId).bootstrapTable({
                        data: alarmsViewModelArray,
                        //height: $(window).height(), // FIX
                        icons: base_table_component_viewmodel_2.BaseTableComponentViewModel.toolbarIconPak,
                        onToggle: function (isCardView) {
                            this._enableCardView = isCardView;
                        }.bind(this),
                        // Insert a button in the toolbar with same style
                        onPostBody: function () {
                            if (this._isToolbarInit === false) {
                                // Inject component without container. In this way I can use default css for toolbar
                                var toolbar_1 = $("alarm-history-component div.columns.columns-right.btn-group.pull-right");
                                var $newToolbarItem = $("<!-- ko component: 'alarm-history-toolbar-component' --><!-- /ko -->");
                                toolbar_1.prepend($newToolbarItem);
                                ko.applyBindings({}, $newToolbarItem[0]);
                                this._isToolbarInit = true;
                            }
                        }.bind(this),
                        // Refresh 
                        onRefresh: function () {
                            this.refreshAlarm(true);
                        }.bind(this),
                        // Locale
                        locale: translator_service_15.TranslatorService.getInstance().CurrentLanguage()
                    });
                }.bind(_this));
                // Register callback for table events.
                _this.registerTableCallback();
                // Now view is initialized
                _this._isAlarmInit(true);
            };
            // Init
            _this._alarmsViewModel = [];
            _this._isAlarmInit = ko.observable(false);
            _this._isResetAlarmEnabled = ko.observable(false);
            _this._isToolbarInit = false;
            _this._startTimeHeader = ko.observable();
            _this._stopTimeHeader = ko.observable();
            _this._idHeader = ko.observable();
            _this._nameHeader = ko.observable();
            _this._descriptionHeader = ko.observable();
            _this._var1Header = ko.observable();
            _this._var2Header = ko.observable();
            _this._actionHeader = ko.observable();
            _this._alarmAlertViewModel = new alert_viewmodel_1.AlarmAlertViewModel();
            _this._isExportAlarmEnabled = ko.observable();
            _this._exportAlarmTitle = ko.observable();
            _this._settingsViewModel = new settings_viewmodel_1.SettingsViewModel(settings_viewmodel_1.SettingsRefreshViewId.Alarm);
            _this._warningTitle = ko.observable();
            _this._confirmWipeMsg = ko.observable();
            _this._warningYesTitle = ko.observable();
            _this._warningNoTitle = ko.observable();
            _this._stopRefreshAlarmFlag = false;
            // Evaluete the state of cField service
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "IsAlarmInit", {
            get: function () {
                return this._isAlarmInit;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "AlarmsViewModel", {
            get: function () {
                return this._alarmsViewModel;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "IsResetAlarmEnabled", {
            get: function () { return this._isResetAlarmEnabled; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "StartTimeHeader", {
            get: function () { return this._startTimeHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "StopTimeHeader", {
            get: function () { return this._stopTimeHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "IdHeader", {
            get: function () { return this._idHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "NameHeader", {
            get: function () { return this._nameHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "DescriptionHeader", {
            get: function () { return this._descriptionHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "Var1Header", {
            get: function () { return this._var1Header; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "Var2Header", {
            get: function () { return this._var2Header; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "ActionHeader", {
            get: function () { return this._actionHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "IsAlertVisible", {
            get: function () { return this._alarmAlertViewModel.IsAlertVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "AlertCssClass", {
            get: function () { return this._alarmAlertViewModel.AlertCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "AlertTitle", {
            get: function () { return this._alarmAlertViewModel.AlertTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "AlertIsErrorDetailVisible", {
            get: function () { return this._alarmAlertViewModel.AlertIsErrorDetailVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "AlertErrorDetailMsg", {
            get: function () { return this._alarmAlertViewModel.AlertErrorDetailMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "AlertIsResetFailedTableVisible", {
            get: function () { return this._alarmAlertViewModel.AlertIsResetFailedTableVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "AlertAlarmNameHeader", {
            get: function () { return this._alarmAlertViewModel.AlertAlarmNameHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "AlertAlarmDescriptionHeader", {
            get: function () { return this._alarmAlertViewModel.AlertAlarmDescriptionHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "AlertStartimeHeader", {
            get: function () { return this._alarmAlertViewModel.AlertStartimeHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "AlertNotResetAlarmArray", {
            get: function () { return this._alarmAlertViewModel.AlertNotResetAlarmArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "ExportAlarmTitle", {
            get: function () { return this._exportAlarmTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "IsExportAlarmEnabled", {
            get: function () { return this._isExportAlarmEnabled; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "SettingsTitle", {
            get: function () { return this._settingsViewModel.SettingsTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "SettingsRefreshInterval", {
            get: function () { return this._settingsViewModel.RefreshIntervalSecondsTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "SettingsSave", {
            get: function () { return this._settingsViewModel.SaveTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "SettingsCancel", {
            get: function () { return this._settingsViewModel.CancelTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "RefreshIntervalSecondsValue", {
            get: function () { return this._settingsViewModel.RefreshIntervalSecondsValue; },
            set: function (value) { this._settingsViewModel.RefreshIntervalSecondsValue = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "ExportSettingsSeparatorChar", {
            get: function () { return this._settingsViewModel.ExportSeparatorCharTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "ExportSettingsSeparatorCharValue", {
            get: function () { return this._settingsViewModel.ExportSeparatorCharValue; },
            set: function (value) { this._settingsViewModel.ExportSeparatorCharValue = value; },
            enumerable: true,
            configurable: true
        });
        AlarmHistoryComponentViewModel.prototype.save = function () {
            this._settingsViewModel.save();
        };
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "WarningTitle", {
            get: function () { return this._warningTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "ConfirmWipeMsg", {
            get: function () { return this._confirmWipeMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "WarningYesTitle", {
            get: function () { return this._warningYesTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryComponentViewModel.prototype, "WarningNoTitle", {
            get: function () { return this._warningNoTitle; },
            enumerable: true,
            configurable: true
        });
        /**
         * Last code to dispose components. Called by framework.
         */
        AlarmHistoryComponentViewModel.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            // Stop refresh alarm task
            this._stopRefreshAlarmFlag = true;
            if (this._refreshAlarmIntervalId) {
                clearTimeout(this._refreshAlarmIntervalId);
            }
            // Dispose cached viewmodel
            this._cFieldServiceInstance.disposeAlarmHistoryComponentViewModelCached();
        };
        // override
        AlarmHistoryComponentViewModel.prototype.onChangeCFieldState = function (newState) {
            _super.prototype.onChangeCFieldState.call(this, newState);
            // Manage ad-hoc state
            switch (newState) {
                case cfield_service_7.CfieldServiceState.RefreshConfig:
                    // Clear state 
                    this._alarmsViewModel = [];
                    this._alarmAlertViewModel.IsAlertVisible(false);
                    this._isResetAlarmEnabled(false);
                    this._alarmAlertViewModel.AlertNotResetAlarmArray.removeAll();
                    this._isAlarmInit(false);
                    this._isExportAlarmEnabled(false);
                    this._isToolbarInit = false;
                    this._stopRefreshAlarmFlag = true;
                    if (this._refreshAlarmIntervalId) {
                        clearTimeout(this._refreshAlarmIntervalId);
                    }
                    // Destroy view 
                    $(TableId).bootstrapTable("destroy");
                    break;
            }
        };
        // override
        AlarmHistoryComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            var _this = this;
            // Init the header of the table with the translation
            this.initTableHeader();
            this.extraTranslation();
            alarm_service_2.AlarmService.getInstance().getAlarms()
                .then(function (alarmsResult) {
                // Build view model
                _this._alarmsViewModel = _this.buildAlarmViewModels(alarmsResult.AlarmSequence);
                _this._rawAlarmData = alarmsResult.RawData;
                // Build table
                _this.buildAlarmTable(_this._alarmsViewModel);
                // Set auto-refresh
                _this._refreshAlarmIntervalId = setTimeout(function () {
                    this.setTimeoutRefreshView();
                }.bind(_this), storage_service_4.StorageService.getInstance().AlarmRefreshIntevalSeconds * 1000);
            })
                .catch(function (reason) {
                _this.showError(reason);
                _this._stopRefreshAlarmFlag = false;
            });
        };
        // override
        AlarmHistoryComponentViewModel.prototype.onChangeScreenSize = function (isMobileScreen) {
            if (isMobileScreen != this._enableCardView) {
                // Change layout of table
                $(TableId).bootstrapTable("toggleView");
            }
        };
        // override
        AlarmHistoryComponentViewModel.prototype.onChangeLanguage = function (newState) {
            _super.prototype.onChangeLanguage.call(this, newState);
            if (newState === translator_service_15.TranslatorEvent.LanguageChanged) {
                this.translate();
            }
        };
        // override
        AlarmHistoryComponentViewModel.prototype.onChangeTimezone = function (newTimeZone) {
            var timerHelper = timer_service_4.TimerService.getInstance();
            for (var _i = 0, _a = this._alarmsViewModel; _i < _a.length; _i++) {
                var alarmViewModel = _a[_i];
                alarmViewModel.StartTime = timerHelper.dateToString(alarmViewModel.StateBundle.StartDate, alarmViewModel.StartTimeTzOffsetMinutes);
                alarmViewModel.StopTime = timerHelper.dateToString(alarmViewModel.StateBundle.StopDate, alarmViewModel.StopTimeTzOffsetMinutes);
            }
            // Refresh table
            $(TableId).bootstrapTable("load", this._alarmsViewModel);
        };
        /**
         * Build view model for the alarms.
         * @param currentAlarms. List of <code>AlarmSequence</code> get by CGI.
         * @return the view model for the alarms.
         */
        AlarmHistoryComponentViewModel.prototype.buildAlarmViewModels = function (currentAlarms) {
            var currentAlarmsViewModel = [];
            var atLeastActiveAlarm = false;
            for (var _i = 0, currentAlarms_1 = currentAlarms; _i < currentAlarms_1.length; _i++) {
                var currentAlarm = currentAlarms_1[_i];
                if (!currentAlarm.hasStart() && AlarmHistoryComponentViewModel.SkipStopEventsWithoutStart) {
                    continue;
                }
                // Build viewmodel
                currentAlarmsViewModel.push(new alarm_item_viewmodel_1.AlarmItemViewModel(currentAlarm));
                if (currentAlarm._isStillActive)
                    atLeastActiveAlarm = true;
            }
            this._isResetAlarmEnabled(atLeastActiveAlarm);
            this._isExportAlarmEnabled(currentAlarmsViewModel.length > 0 ? true : false);
            return currentAlarmsViewModel;
        };
        /**
         * Refresh alarms.
         */
        AlarmHistoryComponentViewModel.prototype.refreshAlarm = function (forced) {
            var _this = this;
            alarm_service_2.AlarmService.getInstance().getAlarms()
                .then(function (alarmsResult) {
                // Refresh table
                _this.refreshTable(alarmsResult.AlarmSequence);
                _this._rawAlarmData = alarmsResult.RawData;
                // Warn user
                if (forced === true) {
                    _this._alarmAlertViewModel.State(alert_viewmodel_1.AlarmAlertState.Success);
                }
            });
        };
        /**
         * Callback used by table.
         */
        AlarmHistoryComponentViewModel.prototype.registerTableCallback = function () {
            // Reset single alarm
            window["alarmHistoryOperationEvents"] = {
                "click .reset-alarm": function (e, value, alarmItemViewModel, index) {
                    var _this = this;
                    var resultPromise = alarm_service_2.AlarmService.getInstance().resetAlarm(alarmItemViewModel.Id);
                    resultPromise
                        .then(function (alarmsResult) {
                        // Compare
                        _this.compareResetResult(alarmsResult.AlarmSequence, [alarmItemViewModel]);
                        // Update table
                        _this.refreshTable(alarmsResult.AlarmSequence);
                        _this._rawAlarmData = alarmsResult.RawData;
                    })
                        .catch(function (reason) {
                        _this.showError(reason);
                    });
                }.bind(this)
            };
        };
        /**
         * Refresh the alarm table. It doesn't register callback for table events (e.g. refresh data, change record color...).
         * @param alarmSequence List of alarms.
         */
        AlarmHistoryComponentViewModel.prototype.refreshTable = function (alarmSequence) {
            // Build view model
            this._alarmsViewModel = this.buildAlarmViewModels(alarmSequence);
            // Refresh table
            $(TableId).bootstrapTable("load", this._alarmsViewModel);
        };
        AlarmHistoryComponentViewModel.prototype.resetAllAlarms = function () {
            var _this = this;
            var resultPromise = alarm_service_2.AlarmService.getInstance().resetAllAlarms();
            resultPromise
                .then(function (alarmsResult) {
                // Compare
                _this.compareResetResult(alarmsResult.AlarmSequence, _this._alarmsViewModel);
                // Update table
                _this.refreshTable(alarmsResult.AlarmSequence);
                _this._rawAlarmData = alarmsResult.RawData;
            })
                .catch(function (reason) {
                _this.showError(reason);
            });
        };
        AlarmHistoryComponentViewModel.prototype.wipeAlarms = function () {
            var _this = this;
            alarm_service_2.AlarmService.getInstance().wipeAlarms()
                .then(function (alarmsResult) {
                // Update table
                _this.refreshTable(alarmsResult.AlarmSequence);
                _this._rawAlarmData = alarmsResult.RawData;
                // Warn user
                _this._alarmAlertViewModel.State(alert_viewmodel_1.AlarmAlertState.Success);
            })
                .catch(function (reason) {
                _this.showError(reason);
            });
        };
        AlarmHistoryComponentViewModel.prototype.compareResetResult = function (alarmSequence, alarmItemViewModelArray) {
            // Compare result
            this._alarmAlertViewModel.AlertNotResetAlarmArray.removeAll();
            if (alarmSequence && alarmSequence.length) {
                for (var _i = 0, alarmSequence_1 = alarmSequence; _i < alarmSequence_1.length; _i++) {
                    var alarmItem = alarmSequence_1[_i];
                    for (var _a = 0, alarmItemViewModelArray_1 = alarmItemViewModelArray; _a < alarmItemViewModelArray_1.length; _a++) {
                        var alarmItemViewModel = alarmItemViewModelArray_1[_a];
                        if (alarmItem.getAlarmId() == alarmItemViewModel.Id
                            // anything involving '=' should use the '+' prefix it will then compare the dates' millisecond values
                            && +alarmItem.getStartTime() === +alarmItemViewModel.StateBundle.StartDate
                            // Check if it is still active
                            && alarmItem._isStillActive) {
                            // Reset failed, warn user
                            this._alarmAlertViewModel.AlertNotResetAlarmArray.push(new alert_item_viewmodel_1.NotResetAlarmViewModel(alarmItemViewModel.Name, alarmItemViewModel.TranslationKey, alarmItemViewModel.StartTime));
                        }
                    }
                }
            }
            // Manage alert
            if (this._alarmAlertViewModel.AlertNotResetAlarmArray().length > 0) {
                this._alarmAlertViewModel.State(alert_viewmodel_1.AlarmAlertState.ResetFailed);
            }
            else {
                this._alarmAlertViewModel.State(alert_viewmodel_1.AlarmAlertState.Success);
            }
        };
        AlarmHistoryComponentViewModel.prototype.exportAlarm = function () {
            var exportDataString;
            if (this._rawAlarmData) {
                // Replace char separator of original cgi result with the char saved in the settings
                exportDataString = this._rawAlarmData.replace(/,/g, storage_service_4.StorageService.getInstance().ExportCharSeparator);
            }
            else {
                exportDataString = "";
            }
            file_util_1.FileUtil.SaveTextFileOnDisk("alarm.csv", exportDataString);
        };
        AlarmHistoryComponentViewModel.prototype.closeAlert = function () {
            this._alarmAlertViewModel.State(alert_viewmodel_1.AlarmAlertState.Invisible);
        };
        // Refresh
        AlarmHistoryComponentViewModel.prototype.setTimeoutRefreshView = function () {
            this.refreshAlarm(false);
            if (this._stopRefreshAlarmFlag === false) {
                this._refreshAlarmIntervalId = setTimeout(function () {
                    this.setTimeoutRefreshView();
                }.bind(this), storage_service_4.StorageService.getInstance().AlarmRefreshIntevalSeconds * 1000);
            }
        };
        AlarmHistoryComponentViewModel.prototype.showError = function (reason) {
            this._log(reason);
            this._alarmAlertViewModel.AlertErrorDetailMsg(reason);
            this._alarmAlertViewModel.State(alert_viewmodel_1.AlarmAlertState.Error);
        };
        // Translation
        AlarmHistoryComponentViewModel.prototype.extraTranslation = function () {
            var translator = translator_service_15.TranslatorService.getInstance().t;
            this._exportAlarmTitle(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "export"));
            this._warningTitle(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "warning"));
            this._confirmWipeMsg(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "confirmWipeMsg"));
            this._warningYesTitle(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "yes"));
            this._warningNoTitle(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "no"));
            this._settingsViewModel.translate();
            if (this.AlertNotResetAlarmArray().length > 0) {
                // Translate
                for (var _i = 0, _a = this.AlertNotResetAlarmArray(); _i < _a.length; _i++) {
                    var alarmAlertViewModel = _a[_i];
                    alarmAlertViewModel.translate();
                }
                // Force refresh
                this.AlertNotResetAlarmArray.notifySubscribers();
            }
        };
        /**
         * Refresh translation of table in accord with current language.
         */
        AlarmHistoryComponentViewModel.prototype.translate = function () {
            var translatorService = translator_service_15.TranslatorService.getInstance();
            var translator = translatorService.t;
            var table = $(TableId);
            table.bootstrapTable("changeLocale", translatorService.CurrentLanguage());
            $(TableId).bootstrapTable("changeTitle", {
                StartTime: translator(translator_service_15.TranslatorService.CfieldTranslationNs + "starttime"),
                StopTime: translator(translator_service_15.TranslatorService.CfieldTranslationNs + "stoptime"),
                Id: translator(translator_service_15.TranslatorService.CfieldTranslationNs + "id"),
                Name: translator(translator_service_15.TranslatorService.CfieldTranslationNs + "name"),
                Description: translator(translator_service_15.TranslatorService.CfieldTranslationNs + "description"),
                Var1: translator(translator_service_15.TranslatorService.CfieldTranslationNs + "sample1"),
                Var2: translator(translator_service_15.TranslatorService.CfieldTranslationNs + "sample2"),
                Actions: translator(translator_service_15.TranslatorService.CfieldTranslationNs + "actions"),
                _callbackBeforeUpdateBodyOnRefreshTranslation: function () {
                    // Refresh view model
                    for (var _i = 0, _a = this._alarmsViewModel; _i < _a.length; _i++) {
                        var alarmViewModel = _a[_i];
                        alarmViewModel.translate();
                    }
                    // Reset toolbar state
                    this._isToolbarInit = false;
                }.bind(this)
            });
            this.extraTranslation();
            this._alarmAlertViewModel.translate();
        };
        AlarmHistoryComponentViewModel.prototype.initTableHeader = function () {
            var translator = translator_service_15.TranslatorService.getInstance().t;
            this._startTimeHeader(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "starttime"));
            this._stopTimeHeader(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "stoptime"));
            this._idHeader(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "id"));
            this._nameHeader(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "name"));
            this._descriptionHeader(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "description"));
            this._var1Header(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "sample1"));
            this._var2Header(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "sample2"));
            this._actionHeader(translator(translator_service_15.TranslatorService.CfieldTranslationNs + "actions"));
        };
        // Skip alarm if there isn't start event
        AlarmHistoryComponentViewModel.SkipStopEventsWithoutStart = true;
        AlarmHistoryComponentViewModel.WarningTitleDefault = "Warning";
        AlarmHistoryComponentViewModel.ConfirmWipeMsgDefault = "Do you want to delete the history of alarms?";
        AlarmHistoryComponentViewModel.WarningYesTitleDefault = "Yes";
        AlarmHistoryComponentViewModel.WarningNoTitleDefault = "No";
        return AlarmHistoryComponentViewModel;
    }(base_table_component_viewmodel_2.BaseTableComponentViewModel));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var hideExtraToolbar = (params && params.hideExtraToolbar) || false;
                var newViewModel = new AlarmHistoryComponentViewModel(hideExtraToolbar);
                // Cache
                cfield_service_7.cFieldService.getInstance().AlarmHistoryComponentViewModelCached = newViewModel;
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_7.cFieldService.getInstance().BaseComponentViewUrl + "/alarm-history.html"
        },
        AlarmHistoryComponentViewModel: AlarmHistoryComponentViewModel
    };
    return objectToExport;
});
define("component/viewModel/base-toolbar.viewmodel", ["require", "exports", "knockout", "service/translator.service", "common/loggable-entity"], function (require, exports, ko, translator_service_16, loggable_entity_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var SettingsModalId = "#settingsDialog";
    var SaveToolbarViewModel = /** @class */ (function (_super) {
        __extends(SaveToolbarViewModel, _super);
        function SaveToolbarViewModel(_isSaveButtonEnabled) {
            var _this = _super.call(this) || this;
            _this._isSaveButtonEnabled = _isSaveButtonEnabled;
            _this._saveDesc = ko.observable();
            return _this;
        }
        Object.defineProperty(SaveToolbarViewModel.prototype, "SaveDesc", {
            get: function () { return this._saveDesc; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SaveToolbarViewModel.prototype, "IsSaveButtonEnabled", {
            get: function () { return this._isSaveButtonEnabled; },
            enumerable: true,
            configurable: true
        });
        // Translation
        SaveToolbarViewModel.prototype.translate = function () {
            var translator = translator_service_16.TranslatorService.getInstance().t;
            this._saveDesc(translator(translator_service_16.TranslatorService.CfieldTranslationNs + "saveIntoController"));
        };
        return SaveToolbarViewModel;
    }(loggable_entity_8.LoggableEntity));
    exports.SaveToolbarViewModel = SaveToolbarViewModel;
    var SettingsToolbarComponentViewModel = /** @class */ (function () {
        function SettingsToolbarComponentViewModel() {
            this._settingsTooltip = ko.observable();
        }
        Object.defineProperty(SettingsToolbarComponentViewModel.prototype, "SettingsTooltip", {
            get: function () { return this._settingsTooltip; },
            enumerable: true,
            configurable: true
        });
        ;
        SettingsToolbarComponentViewModel.prototype.openSettingsDialog = function () {
            $(SettingsModalId).modal("show");
        };
        // Translation
        SettingsToolbarComponentViewModel.prototype.translate = function () {
            var translator = translator_service_16.TranslatorService.getInstance().t;
            this._settingsTooltip(translator(translator_service_16.TranslatorService.CfieldTranslationNs + "settings"));
        };
        return SettingsToolbarComponentViewModel;
    }());
    exports.SettingsToolbarComponentViewModel = SettingsToolbarComponentViewModel;
});
define("component/viewModel/alarm-history-toolbar-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/translator.service", "component/viewModel/base-toolbar.viewmodel"], function (require, exports, ko, cfield_service_8, translator_service_17, base_toolbar_viewmodel_1) {
    "use strict";
    var ConfirmWipeModalId = "#alarmHistoryConfirmWipe";
    /**
     * View model for the extra elements of alarm toolbar (the elements not included in bootstrap-table library).
     */
    var AlarmHistoryToolbarViewModel = /** @class */ (function (_super) {
        __extends(AlarmHistoryToolbarViewModel, _super);
        function AlarmHistoryToolbarViewModel(alarmHistoryComponentViewModel) {
            var _this = _super.call(this) || this;
            _this._cFieldServiceInstance = cfield_service_8.cFieldService.getInstance();
            _this._alarmHistoryComponentViewModel = alarmHistoryComponentViewModel;
            _this._enableResetAll = ko.computed({
                read: function () {
                    return _this._alarmHistoryComponentViewModel.IsResetAlarmEnabled();
                }
            });
            _this._resetAllTooltip = ko.observable();
            _this._wipeTooltip = ko.observable();
            // Translate tooltip of toolbar.
            _this.translate();
            return _this;
        }
        Object.defineProperty(AlarmHistoryToolbarViewModel.prototype, "ResetAllTooltip", {
            get: function () { return this._resetAllTooltip; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(AlarmHistoryToolbarViewModel.prototype, "EnableResetAll", {
            get: function () { return this._enableResetAll; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AlarmHistoryToolbarViewModel.prototype, "WipeTooltip", {
            get: function () { return this._wipeTooltip; },
            enumerable: true,
            configurable: true
        });
        ;
        AlarmHistoryToolbarViewModel.prototype.resetAllAlarms = function () {
            this._alarmHistoryComponentViewModel.resetAllAlarms();
        };
        AlarmHistoryToolbarViewModel.prototype.openWipeAlarmDialog = function () {
            $(ConfirmWipeModalId).modal("show");
        };
        // Translation
        AlarmHistoryToolbarViewModel.prototype.translate = function () {
            _super.prototype.translate.call(this);
            var translator = translator_service_17.TranslatorService.getInstance().t;
            this._resetAllTooltip(translator(translator_service_17.TranslatorService.CfieldTranslationNs + "resetAll"));
            this._wipeTooltip(translator(translator_service_17.TranslatorService.CfieldTranslationNs + "clear"));
        };
        return AlarmHistoryToolbarViewModel;
    }(base_toolbar_viewmodel_1.SettingsToolbarComponentViewModel));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                // This component depend on alarm-history-component. Get it
                var alarmComponentViewModel = cfield_service_8.cFieldService.getInstance().AlarmHistoryComponentViewModelCached;
                if (!alarmComponentViewModel) {
                    throw "The cached alarmHistoryComponentViewModel is null";
                }
                return new AlarmHistoryToolbarViewModel(alarmComponentViewModel);
            }
        },
        template: {
            require: "text!" + cfield_service_8.cFieldService.getInstance().BaseComponentViewUrl + "/alarm-history-toolbar.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/base-upload-component.viewmodel", ["require", "exports", "knockout", "component/viewModel/base-component.viewmodel", "service/translator.service"], function (require, exports, ko, base_component_viewmodel_3, translator_service_18) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var BaseUploadComponentViewModel = /** @class */ (function (_super) {
        __extends(BaseUploadComponentViewModel, _super);
        function BaseUploadComponentViewModel(hideExtraToolbar) {
            var _this = _super.call(this, hideExtraToolbar, false, true, false) || this;
            _this.IsUploadButtonEnabled = ko.observable(false);
            _this.InstructionMessage = ko.observable("");
            _this.IsProgressBarVisible = ko.observable(false);
            _this.IsResultMsgVisible = ko.observable(false);
            _this._abortTitle = ko.observable();
            return _this;
        }
        Object.defineProperty(BaseUploadComponentViewModel.prototype, "AbortTitle", {
            get: function () { return this._abortTitle; },
            enumerable: true,
            configurable: true
        });
        /**
         * Common code to init the component.
         */
        BaseUploadComponentViewModel.prototype.defaultOnSuccessfulInitializedConf = function (idUploadInput) {
            // Enable bootstrap-filestyle
            $(idUploadInput).filestyle({
                icon: false,
                buttonName: "btn-primary"
            });
        };
        // Translation
        BaseUploadComponentViewModel.prototype.translate = function () {
            var translator = translator_service_18.TranslatorService.getInstance().t;
            this._abortTitle(translator(translator_service_18.TranslatorService.CfieldTranslationNs + "abort"));
        };
        BaseUploadComponentViewModel.AbortTitleDefault = "Abort";
        return BaseUploadComponentViewModel;
    }(base_component_viewmodel_3.BaseComponentViewModel));
    exports.BaseUploadComponentViewModel = BaseUploadComponentViewModel;
});
define("component/viewModel/category-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/translator.service", "component/viewModel/base-component.viewmodel"], function (require, exports, ko, cfield_service_9, translator_service_19, base_component_viewmodel_4) {
    "use strict";
    var CategoryComponentViewModel = /** @class */ (function (_super) {
        __extends(CategoryComponentViewModel, _super);
        function CategoryComponentViewModel(params) {
            var _this = _super.call(this, true, false, true, false) || this;
            _this._categoryViewModelProvider = params.parentViewmodel;
            _this._unselectAllCategoriesTitle = ko.observable();
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        Object.defineProperty(CategoryComponentViewModel.prototype, "Categories", {
            // Category viewmodels
            get: function () { return this._categoryViewModelProvider.Categories; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CategoryComponentViewModel.prototype, "UnselectAllCategoriesTitle", {
            get: function () { return this._unselectAllCategoriesTitle; },
            enumerable: true,
            configurable: true
        });
        // override
        CategoryComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            _super.prototype.onSuccessfulInitializedConf.call(this);
            this.translateCategoryComponent();
        };
        // override
        CategoryComponentViewModel.prototype.onChangeLanguage = function (newState) {
            _super.prototype.onChangeLanguage.call(this, newState);
            if (newState === translator_service_19.TranslatorEvent.LanguageChanged) {
                this.translateCategoryComponent();
            }
        };
        CategoryComponentViewModel.prototype.translateCategoryComponent = function () {
            var translator = translator_service_19.TranslatorService.getInstance().t;
            this._unselectAllCategoriesTitle(translator(translator_service_19.TranslatorService.CfieldTranslationNs + "unselectAllCategories"));
        };
        CategoryComponentViewModel.prototype.UnselectAllCategoriesClick = function () {
            this._categoryViewModelProvider.UnselectAllCategories();
        };
        return CategoryComponentViewModel;
    }(base_component_viewmodel_4.BaseComponentViewModel));
    // Export for knockout component
    var objectToExport = {
        viewModel: CategoryComponentViewModel,
        template: {
            require: "text!" + cfield_service_9.cFieldService.getInstance().BaseComponentViewUrl + "/category.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/container-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/login.service", "service/translator.service", "service/version/version-helper", "common/validator"], function (require, exports, ko, cfield_service_10, login_service_3, translator_service_20, version_helper_3, validator_20) {
    "use strict";
    var ContainerComponentViewModel = /** @class */ (function () {
        function ContainerComponentViewModel(subComponentName, showExtraToolbar) {
            var _this = this;
            var cFieldServiceInstance = cfield_service_10.cFieldService.getInstance();
            this._isAgentSupported = version_helper_3.VersionHelper.IsAgentSupported();
            this._agentNotSupportedMsg = ko.observable();
            this._viewState = ko.observable(ViewState.NotInit);
            this._isCfieldInitialized = ko.pureComputed({
                read: function () {
                    return cFieldServiceInstance.StateMonitor.CfieldStateObservable() === cfield_service_10.CfieldServiceState.Initialized;
                }
            });
            this._isCfieldBootSuccess = ko.pureComputed({
                read: function () {
                    return _this._viewState() === ViewState.BootSuccess;
                }
            });
            this._isCfieldError = ko.pureComputed({
                read: function () {
                    var state = _this._viewState();
                    return state === ViewState.GenericError || state === ViewState.CrcChanged || state === ViewState.BootFailed;
                }
            });
            this._errorMsg = ko.pureComputed({
                read: function () {
                    var translator = translator_service_20.TranslatorService.getInstance().t;
                    switch (_this._viewState()) {
                        case ViewState.BootFailed:
                            return validator_20.Validator.isDefined(translator) ?
                                translator(translator_service_20.TranslatorService.CfieldTranslationNs + "cfieldBootFailed") :
                                "c.field boot error. See console (F12) for details";
                        case ViewState.CrcChanged:
                            return validator_20.Validator.isDefined(translator) ?
                                translator(translator_service_20.TranslatorService.CfieldTranslationNs + "configurationChanged") :
                                "The configuration of device is suddenly changed. Reload the page when the installation of application is finished";
                        case ViewState.GenericError:
                            return validator_20.Validator.isDefined(translator) ?
                                translator(translator_service_20.TranslatorService.CfieldTranslationNs + "genericError") :
                                "Errore generico. Dettagli salvati nella console (premere F12).";
                        case ViewState.NotInit:
                            return "";
                    }
                    return "";
                }
            });
            // Set to false. In this way no view is shown before the cField boot promise is resolved/rejected
            this._isLoginEnabled = ko.pureComputed({
                read: function () {
                    return login_service_3.LoginService.getInstance().IsLoginEnabled();
                }
            });
            this._isLoginRequired = ko.pureComputed({
                read: function () {
                    return login_service_3.LoginService.getInstance().IsLoginRequired();
                }
            });
            this._subComponentName = subComponentName;
            this._hideExtraToolbar = showExtraToolbar;
            cFieldServiceInstance.StateMonitor.BootPromise
                .then(function () {
                _this._viewState(ViewState.BootSuccess);
                _this.setAgentNotSupportedMsgTranslation();
            })
                .catch(function (reason) {
                _this._viewState(ViewState.BootFailed);
                _this.setAgentNotSupportedMsgTranslation();
            });
            // Set listener to monitor cFieldState
            this._registeredObserverArray = [];
            this._registeredObserverArray.push(cfield_service_10.cFieldService.getInstance().StateMonitor.CfieldStateObservable.subscribe(this.onChangeCFieldState, this));
        }
        Object.defineProperty(ContainerComponentViewModel.prototype, "IsAgentSupported", {
            get: function () { return this._isAgentSupported; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContainerComponentViewModel.prototype, "AgentNotSupportedMsg", {
            get: function () { return this._agentNotSupportedMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContainerComponentViewModel.prototype, "IsCfieldInitialized", {
            get: function () { return this._isCfieldInitialized; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContainerComponentViewModel.prototype, "IsCfieldBootSuccess", {
            get: function () { return this._isCfieldBootSuccess; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContainerComponentViewModel.prototype, "IsCfieldError", {
            get: function () { return this._isCfieldError; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContainerComponentViewModel.prototype, "ErrorMsg", {
            get: function () { return this._errorMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContainerComponentViewModel.prototype, "IsLoginEnabled", {
            get: function () { return this._isLoginEnabled; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContainerComponentViewModel.prototype, "IsLoginRequired", {
            get: function () { return this._isLoginRequired; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContainerComponentViewModel.prototype, "SubComponentName", {
            get: function () { return this._subComponentName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContainerComponentViewModel.prototype, "HideExtraToolbar", {
            get: function () { return this._hideExtraToolbar; },
            enumerable: true,
            configurable: true
        });
        ContainerComponentViewModel.prototype.dispose = function () {
            // Unregister observers
            for (var _i = 0, _a = this._registeredObserverArray; _i < _a.length; _i++) {
                var observer = _a[_i];
                observer.dispose();
            }
        };
        ContainerComponentViewModel.prototype.onChangeCFieldState = function (newState) {
            var translator = translator_service_20.TranslatorService.getInstance().t;
            switch (newState) {
                case cfield_service_10.CfieldServiceState.RefreshConfig:
                    this._viewState(ViewState.NotInit);
                    break;
                case cfield_service_10.CfieldServiceState.CrcChanged:
                    this._viewState(ViewState.CrcChanged);
                    // Uncomment if you want to reload the page
                    // setTimeout(function() {
                    //     // In cField web application, two contairs are instantiated at the same time. Then you must avoid to re-init cField service twice
                    //     if (cFieldService.getInstance().StateMonitor.CfieldStateObservable() === CfieldServiceState.CrcChanged) {
                    //         // I'm the first component that knockout.js has update
                    //         // Refresh
                    //         let cFieldServiceInstance = cFieldService.getInstance();
                    //         cFieldServiceInstance.releaseResourceOnLogout();
                    //         cFieldServiceInstance.initService();
                    //     }  
                    //     // Update layout
                    //     this._isDeviceConfigurationSuddenlyChanged(false);
                    // }.bind(this), 10000);
                    break;
                case cfield_service_10.CfieldServiceState.Error:
                    this._viewState(ViewState.GenericError);
                    break;
            }
        };
        // Translation
        ContainerComponentViewModel.prototype.setAgentNotSupportedMsgTranslation = function () {
            if (this._isAgentSupported === false) {
                // Get translator
                var translator = translator_service_20.TranslatorService.getInstance().t;
                this._agentNotSupportedMsg(translator(translator_service_20.TranslatorService.CfieldTranslationNs + "browserNotSupported"));
            }
        };
        return ContainerComponentViewModel;
    }());
    var ViewState;
    (function (ViewState) {
        ViewState[ViewState["NotInit"] = 0] = "NotInit";
        ViewState[ViewState["BootSuccess"] = 1] = "BootSuccess";
        ViewState[ViewState["BootFailed"] = 2] = "BootFailed";
        ViewState[ViewState["GenericError"] = 3] = "GenericError";
        ViewState[ViewState["CrcChanged"] = 4] = "CrcChanged";
    })(ViewState || (ViewState = {}));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var newViewModel = new ContainerComponentViewModel(params.subComponentName, params.hideExtraToolbar);
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_10.cFieldService.getInstance().BaseComponentViewUrl + "/container.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/init-progress-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "component/viewModel/base-error-component.viewmodel", "service/translator.service"], function (require, exports, ko, cfield_service_11, base_error_component_viewmodel_2, translator_service_21) {
    "use strict";
    var InitProgressViewModel = /** @class */ (function (_super) {
        __extends(InitProgressViewModel, _super);
        function InitProgressViewModel() {
            var _this = _super.call(this) || this;
            /**
             * Observer of cField service state.
             */
            _this.cfieldServiceStateObserver = function (newState) {
                switch (newState) {
                    case cfield_service_11.CfieldServiceState.DeviceConfigOnDownloadConfig:
                        _this._progressDetailState = ProgressDetailState.Downloading;
                        break;
                    case cfield_service_11.CfieldServiceState.DeviceConfigOnBuildConfig:
                        _this._progressDetailState = ProgressDetailState.Building;
                        break;
                    case cfield_service_11.CfieldServiceState.CFieldVersionNotCompatible:
                        if (_this._canOverrideLoadingFailedMessage) {
                            _this._progressDetailState = ProgressDetailState.NotCompatible;
                            _this._isError(true);
                            _this._canOverrideLoadingFailedMessage = false;
                        }
                        break;
                    case cfield_service_11.CfieldServiceState.CfieldNotSupported:
                        if (_this._canOverrideLoadingFailedMessage) {
                            _this._progressDetailState = ProgressDetailState.NotSupported;
                            _this._isError(true);
                            _this._canOverrideLoadingFailedMessage = false;
                        }
                        break;
                    case cfield_service_11.CfieldServiceState.Error:
                        if (_this._canOverrideLoadingFailedMessage) {
                            _this._progressDetailState = ProgressDetailState.GenericError;
                            _this._isError(true);
                            _this._canOverrideLoadingFailedMessage = false;
                        }
                        break;
                }
                _this.translate();
            };
            _this.ProgressDetail = ko.observable();
            _this._isError = ko.observable(false);
            _this._canOverrideLoadingFailedMessage = true;
            var cFieldServiceInstance = cfield_service_11.cFieldService.getInstance();
            // Register a cField state observer
            _this._observerArray = [];
            _this._observerArray.push(cFieldServiceInstance.StateMonitor.CfieldStateObservable.subscribe(_this.cfieldServiceStateObserver, _this));
            // Callback invoked when boot is completed
            cFieldServiceInstance.StateMonitor.BootPromise
                .then(function () {
                _this.translate();
                cfield_service_11.cFieldService.getInstance().initService();
            })
                .catch(function (reason) {
                _this._log(reason);
            });
            return _this;
        }
        Object.defineProperty(InitProgressViewModel.prototype, "IsError", {
            get: function () { return this._isError; },
            enumerable: true,
            configurable: true
        });
        InitProgressViewModel.prototype.dispose = function () {
            for (var _i = 0, _a = this._observerArray; _i < _a.length; _i++) {
                var observer = _a[_i];
                observer.dispose();
            }
        };
        InitProgressViewModel.prototype.translate = function () {
            var translator = translator_service_21.TranslatorService.getInstance().t;
            this.ProgressDetail(this.getProgressDetailsTranslation(translator));
        };
        InitProgressViewModel.prototype.getProgressDetailsTranslation = function (translator) {
            switch (this._progressDetailState) {
                case ProgressDetailState.Downloading:
                    return translator(translator_service_21.TranslatorService.CfieldTranslationNs + "downloading");
                case ProgressDetailState.Building:
                    return translator(translator_service_21.TranslatorService.CfieldTranslationNs + "building");
                case ProgressDetailState.NotCompatible:
                    return translator(translator_service_21.TranslatorService.CfieldTranslationNs + "cFieldNotCompatible");
                case ProgressDetailState.NotSupported:
                    return translator(translator_service_21.TranslatorService.CfieldTranslationNs + "cFieldNotSupported");
                case ProgressDetailState.GenericError:
                    return translator(translator_service_21.TranslatorService.CfieldTranslationNs + "genericError");
            }
            return "";
        };
        return InitProgressViewModel;
    }(base_error_component_viewmodel_2.BaseErrorComponentViewModel));
    var ProgressDetailState;
    (function (ProgressDetailState) {
        ProgressDetailState[ProgressDetailState["Downloading"] = 0] = "Downloading";
        ProgressDetailState[ProgressDetailState["Building"] = 1] = "Building";
        ProgressDetailState[ProgressDetailState["NotCompatible"] = 2] = "NotCompatible";
        ProgressDetailState[ProgressDetailState["NotSupported"] = 3] = "NotSupported";
        ProgressDetailState[ProgressDetailState["GenericError"] = 4] = "GenericError";
    })(ProgressDetailState || (ProgressDetailState = {}));
    // Export for knockout component
    var objectToExport = {
        viewModel: InitProgressViewModel,
        template: {
            require: "text!" + cfield_service_11.cFieldService.getInstance().BaseComponentViewUrl + "/init-progress.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/io-direction-type-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/device-config.service", "service/translator.service", "service/variable.service", "dataModel/io.model", "common/data-structure", "common/validator", "component/viewModel/base-table-component.viewmodel", "component/viewModel/base-variable-header-component.viewmodel", "component/viewModel/item/io-pin.viewmodel", "component/viewModel/alert.viewmodel"], function (require, exports, ko, cfield_service_12, device_config_service_7, translator_service_22, variable_service_4, io_model_2, data_structure_5, validator_21, base_table_component_viewmodel_3, base_variable_header_component_viewmodel_2, io_pin_viewmodel_1, alert_viewmodel_2) {
    "use strict";
    var IoDirectionTypeComponentViewModel = /** @class */ (function (_super) {
        __extends(IoDirectionTypeComponentViewModel, _super);
        function IoDirectionTypeComponentViewModel(hideExtraToolbar) {
            var _this = _super.call(this, hideExtraToolbar, false, true, false) || this;
            _this.toolbarHeaders = [];
            // ViewModels
            _this._analogInputItemViewModelArray = [];
            _this._analogOutputItemViewModelArray = [];
            _this._digitalInputItemViewModelArray = [];
            _this._digitalOutputItemViewModelArray = [];
            _this._isIoDirectionTypeInit = ko.observable(false);
            _this.AnalogInputTitleHeader = ko.observable();
            _this.AnalogOutputTitleHeader = ko.observable();
            _this.DigitalInputTitleHeader = ko.observable();
            _this.DigitalOutputTitleHeader = ko.observable();
            _this.PinHeader = ko.observable();
            _this.ChannelTypeHeader = ko.observable();
            _this.ToogleTableDesc = ko.observable();
            _this.ColumnDesc = ko.observable();
            // Toolbar entries
            _this.toolbarHeaders.push(new ToolbarItemCheckboxViewModel(IoDirectionTypeComponentViewModel.PinNameId, _this.PinHeader, true));
            _this.toolbarHeaders.push(new ToolbarItemCheckboxViewModel(IoDirectionTypeComponentViewModel.ChannelTypeId, _this.ChannelTypeHeader, true));
            _this.toolbarHeaders.push(new ToolbarItemCheckboxViewModel(IoDirectionTypeComponentViewModel.VariableNameId, _this.VariableNameHeader, false));
            _this.toolbarHeaders.push(new ToolbarItemCheckboxViewModel(IoDirectionTypeComponentViewModel.VariableDescriptionId, _this.VariableDescriptionHeader, true));
            _this.toolbarHeaders.push(new ToolbarItemCheckboxViewModel(IoDirectionTypeComponentViewModel.VariableCurrentValueId, _this.VariableValueHeader, true));
            _this.toolbarHeaders.push(new ToolbarItemCheckboxViewModel(IoDirectionTypeComponentViewModel.VariableMinValue, _this.VariableMinValueHeader, false));
            _this.toolbarHeaders.push(new ToolbarItemCheckboxViewModel(IoDirectionTypeComponentViewModel.VariableMaxValue, _this.VariableMaxValueHeader, false));
            _this._ioDirectionAlertViewModel = new alert_viewmodel_2.IoDirectionViewModel();
            _this._isRefreshVariableEnable = true;
            // Evaluete the state of cField service
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        Object.defineProperty(IoDirectionTypeComponentViewModel.prototype, "IsIoDirectionTypeInit", {
            get: function () { return this._isIoDirectionTypeInit; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoDirectionTypeComponentViewModel.prototype, "IsAlertVisible", {
            get: function () { return this._ioDirectionAlertViewModel.IsAlertVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoDirectionTypeComponentViewModel.prototype, "AlertCssClass", {
            get: function () { return this._ioDirectionAlertViewModel.AlertCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoDirectionTypeComponentViewModel.prototype, "AlertTitle", {
            get: function () { return this._ioDirectionAlertViewModel.AlertTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoDirectionTypeComponentViewModel.prototype, "AlertIsErrorDetailVisible", {
            get: function () { return this._ioDirectionAlertViewModel.AlertIsErrorDetailVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(IoDirectionTypeComponentViewModel.prototype, "AlertErrorDetailMsg", {
            get: function () { return this._ioDirectionAlertViewModel.AlertErrorDetailMsg; },
            enumerable: true,
            configurable: true
        });
        IoDirectionTypeComponentViewModel.prototype.closeAlert = function () {
            this._ioDirectionAlertViewModel.State(alert_viewmodel_2.IoDirectionAlertState.Invisible);
        };
        // override
        IoDirectionTypeComponentViewModel.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            // Stop refresh variable task
            this._isRefreshVariableEnable = false;
            if (validator_21.Validator.isDefined(this._refreshVariableTimeoutId)) {
                clearTimeout(this._refreshVariableTimeoutId);
            }
        };
        // override
        IoDirectionTypeComponentViewModel.prototype.onChangeCFieldState = function (newState) {
            _super.prototype.onChangeCFieldState.call(this, newState);
            // Manage ad-hoc state
            switch (newState) {
                case cfield_service_12.CfieldServiceState.RefreshConfig:
                    // Clear state 
                    this._analogInputItemViewModelArray = [];
                    this._analogOutputItemViewModelArray = [];
                    this._digitalInputItemViewModelArray = [];
                    this._digitalOutputItemViewModelArray = [];
                    this._ioDirectionAlertViewModel.State(alert_viewmodel_2.IoDirectionAlertState.Invisible);
                    this._isIoDirectionTypeInit(false);
                    //  Destroy view 
                    $(IoDirectionTypeComponentViewModel.IdAnalogInputTable).bootstrapTable("destroy");
                    $(IoDirectionTypeComponentViewModel.IdDigitalInputTable).bootstrapTable("destroy");
                    $(IoDirectionTypeComponentViewModel.IdAnalogOutputTable).bootstrapTable("destroy");
                    $(IoDirectionTypeComponentViewModel.IdDigitalOutputTable).bootstrapTable("destroy");
                    break;
            }
        };
        // override
        IoDirectionTypeComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            var _this = this;
            // Init translation
            this.translate();
            // Get IO configuration
            var ioConfigurationPromise = device_config_service_7.DeviceConfigService.getInstance().getIoConfiguration();
            ioConfigurationPromise
                .then(function (ioConfiguration) {
                _this._ioConfiguration = ioConfiguration;
                _this.buildView(ioConfiguration);
                // Manage refresh of view
                _this._idsToUpdate = _this.initVariableToMonitor(ioConfiguration);
                _this._refreshVariableTimeoutId = setTimeout(function () {
                    // Rescheduling
                    this.setTimeoutRefreshView();
                }.bind(_this), 1000);
            })
                .catch(function (reason) {
                _this.showError(reason, alert_viewmodel_2.IoDirectionAlertState.BootFailed);
            });
        };
        // override
        IoDirectionTypeComponentViewModel.prototype.onChangeLanguage = function (newState) {
            _super.prototype.onChangeLanguage.call(this, newState);
            if (newState === translator_service_22.TranslatorEvent.LanguageChanged) {
                // Update labels
                this.translate();
                // Update tables
                this.refreshTableTranslation(IoDirectionTypeComponentViewModel.IdAnalogInputTable);
                this.refreshTableTranslation(IoDirectionTypeComponentViewModel.IdDigitalInputTable);
                this.refreshTableTranslation(IoDirectionTypeComponentViewModel.IdAnalogOutputTable);
                this.refreshTableTranslation(IoDirectionTypeComponentViewModel.IdDigitalOutputTable);
            }
        };
        // override
        // state must be not mandatory to due the ovveride construct
        IoDirectionTypeComponentViewModel.prototype.showError = function (reason, state) {
            this._log(reason);
            this._ioDirectionAlertViewModel.AlertErrorDetailMsg(reason);
            this._ioDirectionAlertViewModel.State(state);
        };
        IoDirectionTypeComponentViewModel.prototype.buildView = function (ioConfiguration) {
            this.buildViewModel(ioConfiguration);
            this.buildTables();
            // Toolbar event
            $("#io-direction-type-component-toogle-tables").off('click').on('click', function () {
                $(IoDirectionTypeComponentViewModel.IdAnalogInputTable).bootstrapTable("toggleView");
                $(IoDirectionTypeComponentViewModel.IdDigitalInputTable).bootstrapTable("toggleView");
                $(IoDirectionTypeComponentViewModel.IdAnalogOutputTable).bootstrapTable("toggleView");
                $(IoDirectionTypeComponentViewModel.IdDigitalOutputTable).bootstrapTable("toggleView");
            });
            // Prevent the clousere of drop menu when user click on checkbox
            $("#io-direction-type-component-tables-show-columns-list").click(function (e) {
                e.stopPropagation();
            });
        };
        IoDirectionTypeComponentViewModel.prototype.buildViewModel = function (ioConfiguration) {
            var mainVariableDictionary = device_config_service_7.DeviceConfigService.getInstance().VariablesDictionary;
            // Build viewmodels
            // Universal
            for (var _i = 0, _a = ioConfiguration.IoUniversalArray; _i < _a.length; _i++) {
                var pin = _a[_i];
                if (pin.IsActive === true) {
                    var variable = mainVariableDictionary[pin.VariableId];
                    if (!validator_21.Validator.isDefined(variable)) {
                        this._log("IoDirectionTypeComponentViewModel buildViewModel(): variable not found. Id=" + pin.VariableId);
                        continue;
                    }
                    var pinViewModel = new io_pin_viewmodel_1.IoPinViewModel(pin.PinDisplayName, pin.ChannelTypeKey, variable._currentValue, variable);
                    switch (pin.Direction) {
                        case io_model_2.IoDirection.AIN:
                            this._analogInputItemViewModelArray.push(pinViewModel);
                            break;
                        case io_model_2.IoDirection.AOUT:
                            this._analogOutputItemViewModelArray.push(pinViewModel);
                            break;
                        case io_model_2.IoDirection.DI:
                        case io_model_2.IoDirection.DI_COUNT:
                        case io_model_2.IoDirection.DI_FREQ:
                            this._digitalInputItemViewModelArray.push(pinViewModel);
                            break;
                        case io_model_2.IoDirection.DOUT:// impossible case
                            this._digitalOutputItemViewModelArray.push(pinViewModel);
                            break;
                    }
                }
            }
            // AOUT
            for (var _b = 0, _c = ioConfiguration.IoAnalogOutputArray; _b < _c.length; _b++) {
                var pin = _c[_b];
                if (pin.IsActive === true) {
                    var variable = mainVariableDictionary[pin.VariableId];
                    this._analogOutputItemViewModelArray.push(new io_pin_viewmodel_1.IoPinViewModel(pin.PinDisplayName, pin.ChannelTypeKey, variable._currentValue, variable));
                }
            }
            // DI
            for (var _d = 0, _e = ioConfiguration.IoDigitalInputArray; _d < _e.length; _d++) {
                var pin = _e[_d];
                if (pin.IsActive === true) {
                    var variable = mainVariableDictionary[pin.VariableId];
                    this._digitalInputItemViewModelArray.push(new io_pin_viewmodel_1.IoPinViewModel(pin.PinDisplayName, pin.ChannelTypeKey, variable._currentValue, variable));
                }
            }
            // DOUT
            for (var _f = 0, _g = ioConfiguration.IoDigitalOutputArray; _f < _g.length; _f++) {
                var pin = _g[_f];
                if (pin.IsActive === true) {
                    var variable = mainVariableDictionary[pin.VariableId];
                    this._digitalOutputItemViewModelArray.push(new io_pin_viewmodel_1.IoPinViewModel(pin.PinDisplayName, pin.ChannelTypeKey, variable._currentValue, variable));
                }
            }
        };
        IoDirectionTypeComponentViewModel.prototype.buildTables = function () {
            this.buildTable(IoDirectionTypeComponentViewModel.IdAnalogInputTable, this._analogInputItemViewModelArray);
            this.buildTable(IoDirectionTypeComponentViewModel.IdDigitalInputTable, this._digitalInputItemViewModelArray);
            this.buildTable(IoDirectionTypeComponentViewModel.IdAnalogOutputTable, this._analogOutputItemViewModelArray);
            this.buildTable(IoDirectionTypeComponentViewModel.IdDigitalOutputTable, this._digitalOutputItemViewModelArray);
            this._isIoDirectionTypeInit(true);
        };
        IoDirectionTypeComponentViewModel.prototype.buildTable = function (idTable, pinViewModels) {
            $(function () {
                $(idTable).bootstrapTable({
                    data: pinViewModels,
                    icons: base_table_component_viewmodel_3.BaseTableComponentViewModel.toolbarIconPak,
                    onToggle: function (isCardView) {
                        this._enableCardView = isCardView;
                    }.bind(this),
                    // Locale
                    locale: translator_service_22.TranslatorService.getInstance().CurrentLanguage()
                });
            }.bind(this));
        };
        IoDirectionTypeComponentViewModel.prototype.setTimeoutRefreshView = function () {
            var _this = this;
            if (this._isRefreshVariableEnable === true) {
                this.refreshVariableValue()
                    .then(function () {
                    _this._refreshVariableTimeoutId = setTimeout(function () {
                        // Rescheduling
                        this.setTimeoutRefreshView();
                    }.bind(_this), 1000);
                })
                    .catch(function (reason) {
                    _this.showError(reason);
                });
            }
        };
        IoDirectionTypeComponentViewModel.prototype.initVariableToMonitor = function (ioConfiguration) {
            var idSet = new data_structure_5.StringSet();
            if (validator_21.Validator.isDefined(ioConfiguration)) {
                if (validator_21.Validator.isDefined(ioConfiguration.IoUniversalArray)) {
                    for (var _i = 0, _a = ioConfiguration.IoUniversalArray; _i < _a.length; _i++) {
                        var pin = _a[_i];
                        if (validator_21.Validator.isDefined(pin)) {
                            if (validator_21.Validator.isDefined(pin.VariableId)) {
                                idSet.addNoConflict(pin.VariableId);
                            }
                            if (validator_21.Validator.isDefined(pin.MaxVariableId)) {
                                idSet.addNoConflict(pin.VariableId);
                            }
                            if (validator_21.Validator.isDefined(pin.MinVariableId)) {
                                idSet.addNoConflict(pin.VariableId);
                            }
                        }
                    }
                    for (var _b = 0, _c = ioConfiguration.IoAnalogOutputArray; _b < _c.length; _b++) {
                        var pin = _c[_b];
                        if (validator_21.Validator.isDefined(pin) && validator_21.Validator.isDefined(pin.VariableId)) {
                            idSet.addNoConflict(pin.VariableId);
                        }
                    }
                    for (var _d = 0, _e = ioConfiguration.IoDigitalInputArray; _d < _e.length; _d++) {
                        var pin = _e[_d];
                        if (validator_21.Validator.isDefined(pin) && validator_21.Validator.isDefined(pin.VariableId)) {
                            idSet.addNoConflict(pin.VariableId);
                        }
                    }
                    for (var _f = 0, _g = ioConfiguration.IoDigitalOutputArray; _f < _g.length; _f++) {
                        var pin = _g[_f];
                        if (validator_21.Validator.isDefined(pin) && validator_21.Validator.isDefined(pin.VariableId)) {
                            idSet.addNoConflict(pin.VariableId);
                        }
                    }
                }
            }
            return idSet.getValues();
        };
        IoDirectionTypeComponentViewModel.prototype.refreshVariableValue = function () {
            var _this = this;
            if (validator_21.Validator.isDefined(this._idsToUpdate) && this._idsToUpdate.length > 0) {
                return variable_service_4.VariableService.getInstance().getVar(this._idsToUpdate).then(function () {
                    if (validator_21.Validator.isDefined(_this._ioConfiguration)) {
                        // Inefficient but IO are few. TODO: refactor
                        // Clear state
                        _this._analogInputItemViewModelArray = [];
                        _this._analogOutputItemViewModelArray = [];
                        _this._digitalInputItemViewModelArray = [];
                        _this._digitalOutputItemViewModelArray = [];
                        // Rebuilb viewmodel
                        _this.buildViewModel(_this._ioConfiguration);
                        // Refresh data
                        $(IoDirectionTypeComponentViewModel.IdAnalogInputTable).bootstrapTable("load", _this._analogInputItemViewModelArray);
                        $(IoDirectionTypeComponentViewModel.IdAnalogOutputTable).bootstrapTable("load", _this._analogOutputItemViewModelArray);
                        $(IoDirectionTypeComponentViewModel.IdDigitalInputTable).bootstrapTable("load", _this._digitalInputItemViewModelArray);
                        $(IoDirectionTypeComponentViewModel.IdDigitalOutputTable).bootstrapTable("load", _this._digitalOutputItemViewModelArray);
                    }
                })
                    .catch(function (reason) {
                    _this.showError(reason);
                });
            }
            else {
                return Promise.reject("No data to refresh");
            }
        };
        // override
        IoDirectionTypeComponentViewModel.prototype.translate = function () {
            var translator = translator_service_22.TranslatorService.getInstance().t;
            this.AnalogInputTitleHeader(translator(translator_service_22.TranslatorService.CfieldTranslationNs + "analogInput"));
            this.AnalogOutputTitleHeader(translator(translator_service_22.TranslatorService.CfieldTranslationNs + "analogOutput"));
            this.DigitalInputTitleHeader(translator(translator_service_22.TranslatorService.CfieldTranslationNs + "digitalInput"));
            this.DigitalOutputTitleHeader(translator(translator_service_22.TranslatorService.CfieldTranslationNs + "digitalOutput"));
            this.PinHeader(translator(translator_service_22.TranslatorService.CfieldTranslationNs + "pin"));
            this.ChannelTypeHeader(translator(translator_service_22.TranslatorService.CfieldTranslationNs + "channelType"));
            this.ToogleTableDesc(this.getToolbarTranslation(function (locale) { return locale.formatToggle(); }));
            this.ColumnDesc(this.getToolbarTranslation(function (locale) { return locale.formatColumns(); }));
        };
        IoDirectionTypeComponentViewModel.prototype.refreshTableTranslation = function (tableId) {
            var translatorService = translator_service_22.TranslatorService.getInstance();
            var translator = translatorService.t;
            var table = $(tableId);
            table.bootstrapTable("changeLocale", translatorService.CurrentLanguage());
            table.bootstrapTable("changeTitle", {
                PinName: translator(translator_service_22.TranslatorService.CfieldTranslationNs + "pin"),
                ChannelType: translator(translator_service_22.TranslatorService.CfieldTranslationNs + "channelType"),
                VariableName: translator(translator_service_22.TranslatorService.CfieldTranslationNs + "name"),
                VariableDescription: translator(translator_service_22.TranslatorService.CfieldTranslationNs + "description"),
                VariableCurrentValue: translator(translator_service_22.TranslatorService.CfieldTranslationNs + "value"),
                _callbackBeforeUpdateBodyOnRefreshTranslation: function () {
                    // Refresh view models
                    for (var _i = 0, _a = this._analogInputItemViewModelArray; _i < _a.length; _i++) {
                        var pinViewModel = _a[_i];
                        pinViewModel.translate();
                    }
                    for (var _b = 0, _c = this._digitalInputItemViewModelArray; _b < _c.length; _b++) {
                        var pinViewModel = _c[_b];
                        pinViewModel.translate();
                    }
                    for (var _d = 0, _e = this._analogOutputItemViewModelArray; _d < _e.length; _d++) {
                        var pinViewModel = _e[_d];
                        pinViewModel.translate();
                    }
                    for (var _f = 0, _g = this._digitalOutputItemViewModelArray; _f < _g.length; _f++) {
                        var pinViewModel = _g[_f];
                        pinViewModel.translate();
                    }
                }.bind(this)
            });
        };
        IoDirectionTypeComponentViewModel.prototype.getToolbarTranslation = function (callback) {
            var bootstrapTablesLocale = $.fn.bootstrapTable.locales;
            var currentLanguage = translator_service_22.TranslatorService.getInstance().CurrentLanguage();
            if (bootstrapTablesLocale && currentLanguage) {
                var lang = $.fn.bootstrapTable.locales[currentLanguage];
                if (validator_21.Validator.isDefined(lang)) {
                    var translation = $.fn.bootstrapTable.locales[currentLanguage].formatToggle();
                    if (translation)
                        return translation;
                }
            }
            return "";
        };
        // Id of the table
        IoDirectionTypeComponentViewModel.IdAnalogInputTable = "#io-analog-input-table";
        IoDirectionTypeComponentViewModel.IdDigitalInputTable = "#io-digital-input-table";
        IoDirectionTypeComponentViewModel.IdAnalogOutputTable = "#io-analog-output-table";
        IoDirectionTypeComponentViewModel.IdDigitalOutputTable = "#io-digital-output-table";
        // Field names (used to identify the column to show/hide)
        IoDirectionTypeComponentViewModel.PinNameId = "PinName";
        IoDirectionTypeComponentViewModel.ChannelTypeId = "ChannelType";
        IoDirectionTypeComponentViewModel.VariableNameId = "VariableName";
        IoDirectionTypeComponentViewModel.VariableDescriptionId = "VariableDescription";
        IoDirectionTypeComponentViewModel.VariableCurrentValueId = "VariableCurrentValue";
        IoDirectionTypeComponentViewModel.VariableMinValue = "MinVariableValueUom";
        IoDirectionTypeComponentViewModel.VariableMaxValue = "MaxVariableValueUom";
        return IoDirectionTypeComponentViewModel;
    }(base_variable_header_component_viewmodel_2.BaseVariableHeaderComponentViewModel));
    /**
     * Viewmodel for the top toolbar
     */
    var ToolbarItemCheckboxViewModel = /** @class */ (function () {
        function ToolbarItemCheckboxViewModel(_field, Text, isChecked) {
            this._field = _field;
            this.Text = Text;
            var initValue = false;
            if (validator_21.Validator.isDefined(isChecked)) {
                initValue = isChecked;
            }
            this.IsChecked = ko.observable(initValue);
        }
        ToolbarItemCheckboxViewModel.prototype.showHideColumn = function () {
            var bootstrapTableMethodCommand;
            // Knockout.js has already update the IsChecked value (due to binding). Then use inverse logic
            if (this.IsChecked()) {
                bootstrapTableMethodCommand = "showColumn";
            }
            else {
                bootstrapTableMethodCommand = "hideColumn";
            }
            $(IoDirectionTypeComponentViewModel.IdAnalogInputTable).bootstrapTable(bootstrapTableMethodCommand, this._field);
            $(IoDirectionTypeComponentViewModel.IdAnalogOutputTable).bootstrapTable(bootstrapTableMethodCommand, this._field);
            // Not enable min and max for digital table
            if (this._field !== "MinVariableValueUom" && this._field !== "MaxVariableValueUom") {
                $(IoDirectionTypeComponentViewModel.IdDigitalInputTable).bootstrapTable(bootstrapTableMethodCommand, this._field);
                $(IoDirectionTypeComponentViewModel.IdDigitalOutputTable).bootstrapTable(bootstrapTableMethodCommand, this._field);
            }
            return true;
        };
        return ToolbarItemCheckboxViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var hideExtraToolbar = (params && params.hideExtraToolbar) || false;
                var newViewModel = new IoDirectionTypeComponentViewModel(hideExtraToolbar);
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_12.cFieldService.getInstance().BaseComponentViewUrl + "/io-direction-type.html"
        }
    };
    return objectToExport;
});
define("service/device-log.service", ["require", "exports", "service/cfield.service", "service/base.service", "service/device-config.service", "service/timer.service", "common/file-util", "dataModel/log.model"], function (require, exports, cField_service_3, base_service_13, device_config_service_8, timer_service_5, file_util_2, log_model_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var DeviceLogService = /** @class */ (function (_super) {
        __extends(DeviceLogService, _super);
        function DeviceLogService() {
            var _this = _super.call(this) || this;
            if (DeviceLogService._instance) {
                _this._error(base_service_13.BaseService._singletonConstructorErrorMsg);
                return _this;
            }
            _this._deviceEngine = cField_service_3.cFieldService.getInstance().DeviceEngine;
            return _this;
        }
        DeviceLogService.getInstance = function () {
            if (DeviceLogService._instance === null) {
                DeviceLogService._instance = new DeviceLogService();
            }
            return DeviceLogService._instance;
        };
        DeviceLogService.prototype.getAvailableLogs = function () {
            return new Promise(function (resolve, reject) {
                $.ajax({
                    type: "POST",
                    url: '/getlogids.csv',
                    success: function (data) {
                        if (data) {
                            var availableLogEntry = $.parseJSON(data);
                            // Update log definition model
                            var logDefinitionModel = device_config_service_8.DeviceConfigService.getInstance().LogDefinitionModel;
                            var periodicLogMap = logDefinitionModel.PeriodicLogMap;
                            var eventLogMap = logDefinitionModel.EventLogMap;
                            for (var _i = 0, availableLogEntry_1 = availableLogEntry; _i < availableLogEntry_1.length; _i++) {
                                var logEntry = availableLogEntry_1[_i];
                                var periodicLogItem = periodicLogMap[logEntry.name];
                                if (typeof periodicLogItem !== "undefined" && periodicLogItem != null) {
                                    periodicLogItem.Id = logEntry.id;
                                }
                                else {
                                    var eventLogItem = eventLogMap[logEntry.name];
                                    if (typeof eventLogItem !== "undefined" && eventLogItem != null) {
                                        eventLogItem.Id = logEntry.id;
                                    }
                                }
                            }
                            resolve(availableLogEntry);
                        }
                        else {
                            resolve([]);
                        }
                    },
                    error: function (jqXHR, textStatus, errorThrown) {
                        reject("getlogids.csv fails: " + textStatus + "  " + errorThrown);
                    },
                    cache: false,
                    timeout: 10000
                });
            });
        };
        DeviceLogService.prototype.getLogData = function (id, startTime, stopTime, timezone) {
            return new Promise(function (resolve, reject) {
                // Compute date format
                var dateFormat = timezone === timer_service_5.Timezone.Local ? "utc" : "device";
                this._deviceEngine.getLogs(id, startTime, stopTime, dateFormat, function (result, loadedData) {
                    if (result === true) {
                        var logModel = this.buildLogModel(id, startTime, stopTime, timezone, loadedData);
                        resolve(logModel);
                    }
                    else {
                        reject("DeviceLogService. GetLog() is failed");
                    }
                }.bind(this));
            }.bind(this));
        };
        DeviceLogService.prototype.buildLogModel = function (id, startTime, stopTime, timezone, data) {
            var logModel = new log_model_2.LogModel(id, startTime, stopTime, timezone, data);
            if (data) {
                var labels = [];
                // Parse
                var rows = $.csv.toArrays(data);
                // At least one record in addition of labels
                if (rows.length > 1) {
                    var headersRow = rows[0];
                    // First row contains headers...
                    for (var i = 0; i < headersRow.length; i++) {
                        labels.push(headersRow[i]);
                    }
                    logModel.LabelArray = headersRow;
                    // Extract offset of timezone
                    var firstRow = rows[1];
                    var stringDate = firstRow[DeviceLogService.CsvDateOffset];
                    var offset = timer_service_5.TimerService.getMinuteOffset(stringDate);
                    logModel.OriginalOffsetMinuteFromUtc = offset;
                    // Scan data
                    for (var i = 1; i < rows.length; i++) {
                        var currentRow = rows[i];
                        if (currentRow.length !== headersRow.length) {
                            continue;
                        }
                        var date = new Date(currentRow[DeviceLogService.CsvDateOffset]);
                        var dataMap = {};
                        for (var j = DeviceLogService.CsvDataOffset; j < currentRow.length; j++) {
                            dataMap[headersRow[j]] = currentRow[j];
                        }
                        logModel.addLogItem(new log_model_2.LogItemModel(date, currentRow[DeviceLogService.CsvEventOffset], dataMap));
                    }
                }
            }
            return logModel;
        };
        DeviceLogService.prototype.exportLog = function (logModel) {
            return file_util_2.FileUtil.SaveTextFileOnDisk("logs.csv", logModel.RawData);
        };
        DeviceLogService._instance = null;
        // Offset of csv columns
        DeviceLogService.CsvDateOffset = 0;
        DeviceLogService.CsvEventOffset = 1;
        DeviceLogService.CsvDataOffset = 2;
        return DeviceLogService;
    }(base_service_13.BaseService));
    exports.DeviceLogService = DeviceLogService;
});
define("component/viewModel/item/log-chart.viewmodel", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var LogChartViewModel = /** @class */ (function () {
        function LogChartViewModel(_label, _data, _serieColor, _serieOptions) {
            this._label = _label;
            this._data = _data;
            this._serieColor = _serieColor;
            this._serieOptions = _serieOptions;
        }
        Object.defineProperty(LogChartViewModel.prototype, "LabelArray", {
            get: function () { return this._label; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogChartViewModel.prototype, "Data", {
            get: function () { return this._data; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogChartViewModel.prototype, "ColorArray", {
            get: function () { return this._serieColor; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogChartViewModel.prototype, "SerieOptions", {
            get: function () { return this._serieOptions; },
            enumerable: true,
            configurable: true
        });
        return LogChartViewModel;
    }());
    exports.LogChartViewModel = LogChartViewModel;
});
define("component/viewModel/item/period-item.viewmodel", ["require", "exports", "service/translator.service"], function (require, exports, translator_service_23) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var PeriodItemViewModel = /** @class */ (function () {
        function PeriodItemViewModel(_periodItem, _periodTranslationKey) {
            this._periodItem = _periodItem;
            this._periodTranslationKey = _periodTranslationKey;
        }
        Object.defineProperty(PeriodItemViewModel.prototype, "PeriodItem", {
            get: function () { return this._periodItem; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(PeriodItemViewModel.prototype, "Description", {
            get: function () {
                return translator_service_23.TranslatorService.getInstance().t(translator_service_23.TranslatorService.CfieldTranslationNs + this._periodTranslationKey);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(PeriodItemViewModel.prototype, "MomentJsPeriodType", {
            get: function () {
                switch (this._periodItem) {
                    case PeriodItem.Days:
                        return "days";
                    case PeriodItem.Weeks:
                        return "weeks";
                    case PeriodItem.Months:
                        return "months";
                }
            },
            enumerable: true,
            configurable: true
        });
        return PeriodItemViewModel;
    }());
    exports.PeriodItemViewModel = PeriodItemViewModel;
    var PeriodItem;
    (function (PeriodItem) {
        PeriodItem[PeriodItem["Days"] = 0] = "Days";
        PeriodItem[PeriodItem["Weeks"] = 1] = "Weeks";
        PeriodItem[PeriodItem["Months"] = 2] = "Months";
    })(PeriodItem = exports.PeriodItem || (exports.PeriodItem = {}));
});
define("component/viewModel/log-component.viewmodel", ["require", "exports", "knockout", "resize-sensor", "dygraphs", "service/cfield.service", "service/device-config.service", "service/device-log.service", "service/timer.service", "service/translator.service", "service/variable.service", "component/viewModel/base-component.viewmodel", "component/viewModel/alert.viewmodel", "component/viewModel/item/log-definition-item.viewmodel", "component/viewModel/item/log-chart.viewmodel", "component/viewModel/item/period-item.viewmodel", "common/view", "common/data-structure", "common/color"], function (require, exports, ko, ResizeSensor, Dygraphs, cfield_service_13, device_config_service_9, device_log_service_1, timer_service_6, translator_service_24, variable_service_5, base_component_viewmodel_5, alert_viewmodel_3, log_definition_item_viewmodel_2, log_chart_viewmodel_1, period_item_viewmodel_1, view_4, data_structure_6, color_1) {
    "use strict";
    var PeriodicLogDefinitionTable = "#log-component-periodic-table";
    var EventLogDefinitionTable = "#log-component-event-table";
    var LogComponentViewModel = /** @class */ (function (_super) {
        __extends(LogComponentViewModel, _super);
        function LogComponentViewModel(hideExtraToolbar) {
            var _this = _super.call(this, true, true, true, true) || this;
            _this.PeriodItemViewModelArray = [
                new period_item_viewmodel_1.PeriodItemViewModel(period_item_viewmodel_1.PeriodItem.Days, "days"),
                new period_item_viewmodel_1.PeriodItemViewModel(period_item_viewmodel_1.PeriodItem.Weeks, "weeks"),
                new period_item_viewmodel_1.PeriodItemViewModel(period_item_viewmodel_1.PeriodItem.Months, "months"),
            ];
            // Definition of logs
            _this._periodicLogDefinitionItemViewModel = [];
            _this._eventLogDefinitionItemViewModel = [];
            _this._isFirstLogOpened = false;
            _this.getLog = function () {
                _this._logTopAlertViewModel.State(alert_viewmodel_3.LogAlertState.Invisible);
                _this._logBottomAlertViewModel.State(alert_viewmodel_3.LogAlertState.Invisible);
                var periodicLogDefinitionItemViewModelArray = $(PeriodicLogDefinitionTable).bootstrapTable("getSelections");
                var eventLogDefinitionItemViewModelArray = $(EventLogDefinitionTable).bootstrapTable("getSelections");
                var baseLogDefinitionModel;
                // Validate params
                var logId;
                if (periodicLogDefinitionItemViewModelArray.length > 0) {
                    baseLogDefinitionModel = periodicLogDefinitionItemViewModelArray[0];
                    logId = baseLogDefinitionModel.LogId;
                }
                else if (eventLogDefinitionItemViewModelArray.length > 0) {
                    baseLogDefinitionModel = eventLogDefinitionItemViewModelArray[0];
                    logId = baseLogDefinitionModel.LogId;
                }
                if (typeof logId !== "undefined" && logId != null) {
                    var startDate = _this._startDate();
                    var stopDate = _this._stopDate();
                    var timezone = timer_service_6.TimerService.getInstance().CurrentTimezoneObservable();
                    _this._selectBaseDefinitionModel = baseLogDefinitionModel;
                    // Update Ui
                    _this._isLoadingLog(true);
                    _this._isChartLoaded(false);
                    _this._isChartContainterVisible(true);
                    // Scroll at the bottom of the page
                    _this.scrollToBottomPage();
                    // Get log
                    var logModelPromise = device_log_service_1.DeviceLogService.getInstance().getLogData(logId, startDate, stopDate, timezone);
                    logModelPromise
                        .then(function (logModel) {
                        if (logModel.LogItemArray.length == 0) {
                            _this._logBottomAlertViewModel.State(alert_viewmodel_3.LogAlertState.NoData);
                            _this._isLoadingLog(false);
                            _this._isChartLoaded(false);
                            _this._isChartContainterVisible(false);
                            return;
                        }
                        _this._isChartLoaded(true);
                        // Build VM
                        _this._chartViewModel = _this.buildChartViewModel(logModel, baseLogDefinitionModel);
                        // Draw graph
                        _this.plotChart(_this._chartViewModel);
                        // Cache log model
                        _this._logModel = logModel;
                        // Adjust view
                        _this._isLoadingLog(false);
                        var element = document.getElementById('log-component-chart-container');
                        if (element) {
                            // Resize
                            _this._chartAreaSizeListener = new ResizeSensor(element, function () {
                                if (this._chartReference)
                                    this._chartReference.resize();
                            }.bind(_this));
                            // Scroll at the bottom of the page
                            _this.scrollToBottomPage();
                        }
                    })
                        .catch(function (reason) {
                        _this.showError(reason);
                        _this._isChartContainterVisible(false);
                    });
                }
                else {
                    _this.showError("LogComponentViewModel buildDefinitionLogViewModels(): log id not valid " + logId);
                    _this._isChartContainterVisible(false);
                }
            };
            // Init
            _this._isLogViewInit = ko.observable(false);
            _this._isLogDefined = ko.observable(false);
            _this._atLeastAPeriodicLog = ko.observable(true);
            _this._atLeastAEventLog = ko.observable(true);
            _this._availableLogArray = ko.observableArray();
            _this._noLogAvailableText = ko.observable();
            _this._periodicLogsTitle = ko.observable();
            _this._eventLogsTitle = ko.observable();
            _this._intervalTitle = ko.observable();
            _this._starttimeTitle = ko.observable();
            _this._stoptimeTitle = ko.observable();
            _this._logNameTitle = ko.observable();
            _this._recordNumberTitle = ko.observable();
            _this._loadTitle = ko.observable();
            _this._exportTitle = ko.observable();
            _this._intervalSecondTitle = ko.observable();
            _this._variableNameTitle = ko.observable();
            _this._isVisibleTitle = ko.observable();
            _this._axisTitle = ko.observable();
            _this._colorTitle = ko.observable();
            _this._styleTitle = ko.observable();
            _this._periodTitle = ko.observable();
            _this._quickLoadTitle = ko.observable();
            _this._agoTitle = ko.observable();
            _this._invalidDateMsg = ko.observable();
            _this._isChartContainterVisible = ko.observable(false);
            _this._enableLoadLog = ko.observable(false);
            _this._isLoadingLog = ko.observable(false);
            _this._isChartLoaded = ko.observable(false);
            _this._isChartDirty = ko.observable(false);
            _this._isChartDirty.extend({ notify: "always" });
            _this._observerIdArray = [_this._isChartDirty.subscribe(_this.updateChart, _this)];
            _this._logTopAlertViewModel = new alert_viewmodel_3.LogAlertViewModel();
            _this._logBottomAlertViewModel = new alert_viewmodel_3.LogAlertViewModel();
            var currentDate = new Date();
            _this.IntervalDateNumber = ko.observable(1);
            _this._selectedPeriodItemViewModel = _this.PeriodItemViewModelArray[0];
            var now = new Date(currentDate.getTime());
            var startDateMoment = moment(now).subtract(1, 'days');
            var stopDateMoment = moment(now);
            _this._startDate = ko.observable(startDateMoment.toDate());
            _this._stopDate = ko.observable(stopDateMoment.toDate());
            _this._isInvalidDate = ko.observable(false);
            // Set listeners to manage when user uses quick load date function
            _this._observerIdArray.push(_this.IntervalDateNumber.subscribe(_this.onChangePeriod, _this));
            $('#intervalPeriod').on("changed.bs.select", function (e, clickedIndex, newValue, oldValue) {
                this._selectedPeriodItemViewModel = this.PeriodItemViewModelArray[clickedIndex];
                this.onChangePeriod();
            }.bind(_this));
            // Set listerners to validate the date
            _this._observerIdArray.push(_this._startDate.subscribe(_this.onChangeDate, _this));
            _this._observerIdArray.push(_this._stopDate.subscribe(_this.onChangeDate, _this));
            // Evaluete the state of cField service
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        Object.defineProperty(LogComponentViewModel.prototype, "IsLogViewInit", {
            get: function () { return this._isLogViewInit; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "IsLogDefined", {
            get: function () { return this._isLogDefined; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "AvailableLogArray", {
            get: function () { return this._availableLogArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "AtLeastAPeriodicLog", {
            get: function () { return this._atLeastAPeriodicLog; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "AtLeastAEventLog", {
            get: function () { return this._atLeastAEventLog; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "NoLogAvailableText", {
            get: function () { return this._noLogAvailableText; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "PeriodicLogsTitle", {
            get: function () { return this._periodicLogsTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "EventLogsTitle", {
            get: function () { return this._eventLogsTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "IntervalTitle", {
            get: function () { return this._intervalTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "StarttimeTitle", {
            get: function () { return this._starttimeTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "StoptimeTitle", {
            get: function () { return this._stoptimeTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "LogNameTitle", {
            get: function () { return this._logNameTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "RecordNumberTitle", {
            get: function () { return this._recordNumberTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "IntervalSecondTitle", {
            get: function () { return this._intervalSecondTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "LoadTitle", {
            get: function () { return this._loadTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "ExportTitle", {
            get: function () { return this._exportTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "VariableNameTitle", {
            get: function () { return this._variableNameTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "IsVisibleTitle", {
            get: function () { return this._isVisibleTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "AxisTitle", {
            get: function () { return this._axisTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "ColorTitle", {
            get: function () { return this._colorTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "StyleTitle", {
            get: function () { return this._styleTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "PeriodTitle", {
            get: function () { return this._periodTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "QuickLoadTitle", {
            get: function () { return this._quickLoadTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "AgoTitle", {
            get: function () { return this._agoTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "StartDate", {
            get: function () { return this._startDate; },
            set: function (value) { this._startDate = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "StopDate", {
            get: function () { return this._stopDate; },
            set: function (value) { this._stopDate = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "IsInvalidDate", {
            get: function () { return this._isInvalidDate; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "InvalidDateMsg", {
            get: function () { return this._invalidDateMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "IsChartContainterVisible", {
            get: function () { return this._isChartContainterVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "EnableLoadLog", {
            get: function () { return this._enableLoadLog; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "IsLoadingLog", {
            get: function () { return this._isLoadingLog; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "IsChartLoaded", {
            get: function () { return this._isChartLoaded; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "IsTopAlertVisible", {
            get: function () { return this._logTopAlertViewModel.IsAlertVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "TopAlertCssClass", {
            get: function () { return this._logTopAlertViewModel.AlertCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "TopAlertTitle", {
            get: function () { return this._logTopAlertViewModel.AlertTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "TopAlertIsErrorDetailVisible", {
            get: function () { return this._logTopAlertViewModel.AlertIsErrorDetailVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "TopAlertErrorDetailMsg", {
            get: function () { return this._logTopAlertViewModel.AlertErrorDetailMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "IsBottomAlertVisible", {
            get: function () { return this._logBottomAlertViewModel.IsAlertVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "BottomAlertCssClass", {
            get: function () { return this._logBottomAlertViewModel.AlertCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "BottomAlertTitle", {
            get: function () { return this._logBottomAlertViewModel.AlertTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "BottomAlertIsErrorDetailVisible", {
            get: function () { return this._logBottomAlertViewModel.AlertIsErrorDetailVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LogComponentViewModel.prototype, "BottomAlertErrorDetailMsg", {
            get: function () { return this._logBottomAlertViewModel.AlertErrorDetailMsg; },
            enumerable: true,
            configurable: true
        });
        // override
        LogComponentViewModel.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            if (this._chartAreaSizeListener) {
                this._chartAreaSizeListener.detach();
            }
            for (var _i = 0, _a = this._observerIdArray; _i < _a.length; _i++) {
                var observer = _a[_i];
                observer.dispose();
            }
        };
        LogComponentViewModel.prototype.onChangePeriod = function () {
            // Validate
            if (this._selectedPeriodItemViewModel) {
                var startDateMoment = moment(this._stopDate()).subtract(this.IntervalDateNumber(), this._selectedPeriodItemViewModel.MomentJsPeriodType);
                this._startDate(startDateMoment.toDate());
            }
        };
        LogComponentViewModel.prototype.onChangeDate = function () {
            // Validate
            if (this._startDate() > this._stopDate()) {
                this._isInvalidDate(true);
            }
            else {
                this._isInvalidDate(false);
            }
        };
        // override
        LogComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            var _this = this;
            // 1) Update variables used in the log definition
            var initPromiste = this.updateLogVariable();
            initPromiste
                .then(function () {
                // 2) Get id of log
                return _this.getAvailableLog();
            })
                .then(function () {
                // 3) Build view
                _this.buildDefinitionLogView();
                _this.IsLogDefined(_this._periodicLogDefinitionItemViewModel.length > 0 || _this._eventLogDefinitionItemViewModel.length > 0);
                _this._isLogViewInit(true);
            })
                .catch(function (reason) {
                _this.showError(reason);
            });
            this.translate();
        };
        // override
        LogComponentViewModel.prototype.onChangeCFieldState = function (newState) {
            _super.prototype.onChangeCFieldState.call(this, newState);
            // Manage ad-hoc state
            switch (newState) {
                case cfield_service_13.CfieldServiceState.RefreshConfig:
                    // Clear state
                    this._isLogViewInit(false);
                    this._atLeastAPeriodicLog(true);
                    this._atLeastAEventLog(true);
                    this._isChartContainterVisible(false);
                    this._enableLoadLog(false);
                    this._periodicLogDefinitionItemViewModel = [];
                    this._isLoadingLog(false);
                    this._isChartLoaded(false);
                    this._isChartDirty(false);
                    this._logTopAlertViewModel.State(alert_viewmodel_3.LogAlertState.Invisible);
                    this._logBottomAlertViewModel.State(alert_viewmodel_3.LogAlertState.Invisible);
                    break;
            }
        };
        // override
        LogComponentViewModel.prototype.onChangeLanguage = function (newState) {
            _super.prototype.onChangeLanguage.call(this, newState);
            if (newState === translator_service_24.TranslatorEvent.LanguageChanged) {
                $("#startDate").data("DateTimePicker").locale(translator_service_24.TranslatorService.getInstance().CurrentLanguage());
                $("#stopDate").data("DateTimePicker").locale(translator_service_24.TranslatorService.getInstance().CurrentLanguage());
                this.refreshTableTranslation();
                this.translate();
                // Expand log definition if needed
                var isFound = false;
                var n = this._periodicLogDefinitionItemViewModel.length;
                for (var i = 0; i < n; i++) {
                    if (this._periodicLogDefinitionItemViewModel[i].IsSelected === true) {
                        $(PeriodicLogDefinitionTable).bootstrapTable("check", i);
                        isFound = true;
                        break;
                    }
                }
                if (isFound == false) {
                    n = this._eventLogDefinitionItemViewModel.length;
                    for (var i = 0; i < n; i++) {
                        if (this._eventLogDefinitionItemViewModel[i].IsSelected === true) {
                            $(EventLogDefinitionTable).bootstrapTable("check", i);
                        }
                    }
                }
            }
        };
        // override
        LogComponentViewModel.prototype.onChangeTimezone = function (newTimeZone) {
            // The data of log must be refreshed
            this._isChartLoaded(false);
        };
        LogComponentViewModel.prototype.getAvailableLog = function () {
            return device_log_service_1.DeviceLogService.getInstance().getAvailableLogs();
        };
        /**
         * Update the link to variable value used by log (e.g. in the periodic olo the "sampling time (s)" can be bound to variable
         */
        LogComponentViewModel.prototype.updateLogVariable = function () {
            var variableDictionary = device_config_service_9.DeviceConfigService.getInstance().VariablesDictionary;
            var logDefinitionModel = device_config_service_9.DeviceConfigService.getInstance().LogDefinitionModel;
            var peridiocLogMap = logDefinitionModel.PeriodicLogMap;
            var eventLogMap = logDefinitionModel.EventLogMap;
            var variableIdToUpdateSet = new data_structure_6.StringSet();
            for (var logName in peridiocLogMap) {
                var periodicLog = peridiocLogMap[logName];
                // This is the variable with sampling time. It can be link to variable
                var sampleVariable = variableDictionary[periodicLog.SampleVariableId];
                if (sampleVariable) {
                    if (sampleVariable._familyType === device_config_service_9.DeviceConfigService.VarTypeVariable) {
                        variableIdToUpdateSet.addNoConflict(sampleVariable._id);
                    }
                }
                else {
                    this._log("LogComponentViewModel updateLogVariable(): variable " + sampleVariable + " not found in the dictionary");
                }
            }
            // Not used
            // for (let logName in eventLogMap) {}
            if (variableIdToUpdateSet.Count > 0) {
                return variable_service_5.VariableService.getInstance().getVar(variableIdToUpdateSet.getValues());
            }
            else {
                return Promise.resolve();
            }
        };
        LogComponentViewModel.prototype.buildDefinitionLogView = function () {
            this.buildDefinitionLogViewModels();
            this.buildDefinitionLogTables();
        };
        LogComponentViewModel.prototype.buildDefinitionLogViewModels = function () {
            var variableDictionary = device_config_service_9.DeviceConfigService.getInstance().VariablesDictionary;
            var logDefinitionModel = device_config_service_9.DeviceConfigService.getInstance().LogDefinitionModel;
            var peridiocLogMap = logDefinitionModel.PeriodicLogMap;
            var eventLogMap = logDefinitionModel.EventLogMap;
            var atLeastAPeriodicLog = false;
            var atLeastAEventLog = false;
            for (var logName in peridiocLogMap) {
                var periodicLog = peridiocLogMap[logName];
                var sampleVariable = variableDictionary[periodicLog.SampleVariableId];
                if (sampleVariable) {
                    this._periodicLogDefinitionItemViewModel.push(new log_definition_item_viewmodel_2.PeriodicLogDefinitionItemViewModel(periodicLog.Id, periodicLog.Name, periodicLog.RecordNumber, sampleVariable._currentValue, this.buildSerieArray(periodicLog, variableDictionary), this));
                    atLeastAPeriodicLog = true;
                }
                else {
                    this._log("LogComponentViewModel buildDefinitionLogViewModels(): variable " + sampleVariable + " is skipped");
                }
            }
            for (var logName in eventLogMap) {
                var eventLog = eventLogMap[logName];
                this._eventLogDefinitionItemViewModel.push(new log_definition_item_viewmodel_2.EventLogDefinitionItemViewModel(eventLog.Id, eventLog.Name, eventLog.RecordNumber, this.buildSerieArray(eventLog, variableDictionary), this));
                atLeastAEventLog = true;
            }
            this._atLeastAPeriodicLog(atLeastAPeriodicLog);
            this._atLeastAEventLog(atLeastAEventLog);
        };
        /**
         * Build serie of log
         */
        LogComponentViewModel.prototype.buildSerieArray = function (baseLogDefinitionModel, variableDictionary) {
            var serieArray = [];
            var variableIdArray = baseLogDefinitionModel.LoggedVariableIdArray;
            var translator = translator_service_24.TranslatorService.getInstance().t;
            if (variableIdArray && variableIdArray.length) {
                for (var _i = 0, variableIdArray_1 = variableIdArray; _i < variableIdArray_1.length; _i++) {
                    var varId = variableIdArray_1[_i];
                    var variable = variableDictionary[varId];
                    if (variable) {
                        // Create item
                        var logSerieItemViewModel = new log_definition_item_viewmodel_2.LogSerieItemViewModel(variable._name, translator(translator_service_24.TranslatorService.VariableTranslationNs + variable._name, { defaultValue: variable._name || "" }), true, log_definition_item_viewmodel_2.YAxisType.Primary, color_1.ColorGenerator.GetColor(), log_definition_item_viewmodel_2.ChartLineStyle.Line);
                        // Add observer
                        this._observerIdArray.push(logSerieItemViewModel.OnChange.subscribe(function () {
                            this._isChartDirty(true);
                        }.bind(this), this));
                        // Save
                        serieArray.push(logSerieItemViewModel);
                    }
                    else {
                        _super.prototype.showError.call(this, "LogComponentViewModel buildDefinitionLogViewModels(): variable " + varId + " not found in the dictionary");
                    }
                }
            }
            return serieArray;
        };
        LogComponentViewModel.prototype.buildDefinitionLogTables = function () {
            window["logComponentSerieTableFormatter"] = function (index, row, $element) {
                this.buildSerieLogTables($element.html("<table id='log-component-serie-table'></table>").find("table"), row);
            }.bind(this);
            var isPeriodicTableRendered = false;
            var isEventTableRendered = false;
            // Periodic log
            $(function () {
                $(PeriodicLogDefinitionTable).bootstrapTable({
                    data: this._periodicLogDefinitionItemViewModel,
                    locale: translator_service_24.TranslatorService.getInstance().CurrentLanguage(),
                    onPostBody: function () {
                        isPeriodicTableRendered = true;
                        this.openFirstItemOfDefinitionLogTables(isPeriodicTableRendered, isEventTableRendered, this._periodicLogDefinitionItemViewModel, this._eventLogDefinitionItemViewModel);
                    }.bind(this)
                });
            }.bind(this));
            $(PeriodicLogDefinitionTable).on("check.bs.table", function (event, rowViewModel, $tr) {
                // TODO: improve (inefficient)
                $(EventLogDefinitionTable).bootstrapTable("uncheckAll");
                $(EventLogDefinitionTable).bootstrapTable("load", this._eventLogDefinitionItemViewModel);
                $(PeriodicLogDefinitionTable).bootstrapTable("collapseAllRows");
                $(PeriodicLogDefinitionTable).bootstrapTable("expandRow", $tr.data("index"));
            }.bind(this));
            // Event log
            $(function () {
                $(EventLogDefinitionTable).bootstrapTable({
                    data: this._eventLogDefinitionItemViewModel,
                    locale: translator_service_24.TranslatorService.getInstance().CurrentLanguage(),
                    onPostBody: function () {
                        isEventTableRendered = true;
                        this.openFirstItemOfDefinitionLogTables(isPeriodicTableRendered, isEventTableRendered, this._periodicLogDefinitionItemViewModel, this._eventLogDefinitionItemViewModel);
                    }.bind(this)
                });
            }.bind(this));
            $(EventLogDefinitionTable).on("check.bs.table", function (event, rowViewModel, $tr) {
                // TODO: improve (inefficient)
                $(PeriodicLogDefinitionTable).bootstrapTable("uncheckAll");
                $(PeriodicLogDefinitionTable).bootstrapTable("load", this._periodicLogDefinitionItemViewModel);
                $(EventLogDefinitionTable).bootstrapTable("collapseAllRows");
                $(EventLogDefinitionTable).bootstrapTable("expandRow", $tr.data("index"));
            }.bind(this));
        };
        LogComponentViewModel.prototype.openFirstItemOfDefinitionLogTables = function (isPeriodicTableRendered, isEventTableRendered, periodicLogDefinitionItemViewModelArray, eventLogDefinitionItemViewModelArray) {
            if (this._isFirstLogOpened === false && isPeriodicTableRendered && isEventTableRendered) {
                if (periodicLogDefinitionItemViewModelArray.length > 0) {
                    this._isFirstLogOpened = true;
                    $(PeriodicLogDefinitionTable).bootstrapTable('check', 0);
                }
                else if (eventLogDefinitionItemViewModelArray.length > 0) {
                    this._isFirstLogOpened = true;
                    $(EventLogDefinitionTable).bootstrapTable('check', 0);
                }
            }
        };
        LogComponentViewModel.prototype.buildSerieLogTables = function ($el, baseLogDefinitionModel) {
            // Don't use Knockout component here because it introduces a big delay
            var translator = translator_service_24.TranslatorService.getInstance().t;
            // Columns
            var columns = [
                { field: "SerieDescription", title: translator(translator_service_24.TranslatorService.CfieldTranslationNs + "description") },
                { field: "IsSerieVisible", title: translator(translator_service_24.TranslatorService.CfieldTranslationNs + "isVisible"),
                    formatter: "serieVisibleCheckboxFormatter", events: "serieVisibleCheckboxFormatterEvent", align: "center",
                    width: 80 },
                { field: "AxisName", title: translator(translator_service_24.TranslatorService.CfieldTranslationNs + "axis"),
                    formatter: "serieSelectAxisFormatter", events: "serieSelectAxisFormatterEvent", width: 115, halign: "center" },
                { field: "ColorName", title: translator(translator_service_24.TranslatorService.CfieldTranslationNs + "color"),
                    formatter: "serieColorPickerFormatter", events: "serieColorPickerFormatterEvent", align: "center", width: 80 },
                { field: "ChartLineStyleName", title: translator(translator_service_24.TranslatorService.CfieldTranslationNs + "style"),
                    formatter: "serieSelectStyleFormatter", events: "serieSelectStyleFormatterEvent", width: 85, halign: "center" },
            ];
            $el.bootstrapTable({
                columns: columns,
                data: baseLogDefinitionModel.LogSerieArray,
                onPostBody: function () {
                    $(".selectpicker").selectpicker("show");
                    //(<any>$(".colorpicker-component")).colorpicker();
                    view_4.View.InitColorPicker();
                }
            });
        };
        LogComponentViewModel.prototype.updateChart = function (value) {
            if (this._isChartLoaded()) {
                if (value && this._logModel && this._selectBaseDefinitionModel) {
                    // refresh viewmodel
                    this.refreshChartViewModel();
                    // Update chart
                    this.updatePlotChart(this._chartViewModel);
                }
                else {
                    this._log("Log view updateChart(): no valid logModel and selectBaseDefinitionModel");
                }
            }
        };
        LogComponentViewModel.prototype.scrollToBottomPage = function () {
            var mainCfieldWrapper = $("#page-content-wrapper");
            if (mainCfieldWrapper) {
                mainCfieldWrapper.scrollTop($("#page-content-wrapper")[0].scrollHeight);
            }
        };
        LogComponentViewModel.prototype.exportData = function () {
            if (this._logModel) {
                device_log_service_1.DeviceLogService.getInstance().exportLog(this._logModel);
            }
        };
        LogComponentViewModel.prototype.buildChartViewModel = function (logModel, baseLogDefinitionItemViewModel) {
            // Build a map from variable (log) name to log serie
            var mapVariableToVariableSerieViewModel = {};
            for (var _i = 0, _a = baseLogDefinitionItemViewModel.LogSerieArray; _i < _a.length; _i++) {
                var serie = _a[_i];
                if (serie.IsSerieVisible) {
                    mapVariableToVariableSerieViewModel[serie.SerieName] = serie;
                }
            }
            // "Copy" the array;
            var logItemArray = logModel.LogItemArray.slice();
            // Order by date
            logItemArray.sort(function (a, b) {
                return (a.LogDate > b.LogDate) ? 1 : -1;
            });
            // Get the order used to retrieve map entry
            var firstLogItem = logItemArray[0];
            var propertyEntryArray = [];
            for (var property in firstLogItem.LogValueMap) {
                // Check if variable is needed
                if (mapVariableToVariableSerieViewModel[property]) {
                    propertyEntryArray.push(property);
                }
            }
            // Order
            propertyEntryArray.sort(function (a, b) {
                return a > b ? 1 : -1;
            });
            // Manage local/remote date
            var useLocalDate = logModel.Timezone === timer_service_6.Timezone.Local;
            var remoteMinuteOffsetUtc;
            if (!useLocalDate) {
                if (logItemArray.length > 0) {
                    remoteMinuteOffsetUtc = logModel.OriginalOffsetMinuteFromUtc;
                }
            }
            // Build VM
            var data = [];
            var i = 0;
            for (var _b = 0, logItemArray_1 = logItemArray; _b < logItemArray_1.length; _b++) {
                var logItem = logItemArray_1[_b];
                // Build record
                var record = [];
                if (useLocalDate) {
                    record.push(logItem.LogDate);
                }
                else {
                    // Dygraphjs seems not support the offeset on the date. Insert a fake data
                    var minuteOffset = remoteMinuteOffsetUtc + (new Date()).getTimezoneOffset();
                    record.push(timer_service_6.TimerService.addHour(logItem.LogDate, minuteOffset));
                }
                for (var _c = 0, propertyEntryArray_1 = propertyEntryArray; _c < propertyEntryArray_1.length; _c++) {
                    var property = propertyEntryArray_1[_c];
                    var value = parseFloat(logItem.LogValueMap[property]);
                    record.push(value);
                }
                data.push(record);
            }
            // Build label, color, primary/secondary axis
            var labelArray = ["Time"];
            var colorLabelArray = [];
            var serieOption = {};
            for (var _d = 0, propertyEntryArray_2 = propertyEntryArray; _d < propertyEntryArray_2.length; _d++) {
                var variableLabel = propertyEntryArray_2[_d];
                labelArray.push(variableLabel);
                var serieItemViewModel = mapVariableToVariableSerieViewModel[variableLabel];
                colorLabelArray.push(serieItemViewModel.ColorName);
                if (serieItemViewModel.YAxisType === log_definition_item_viewmodel_2.YAxisType.Secondary) {
                    // Move to second axis
                    serieOption[serieItemViewModel.SerieName] = { axis: "y2" };
                }
                else {
                    serieOption[serieItemViewModel.SerieName] = { axis: "y1" };
                }
                // Set type of line
                if (serieItemViewModel.ChartLineStyle === log_definition_item_viewmodel_2.ChartLineStyle.Step) {
                    if (serieOption[serieItemViewModel.SerieName]) {
                        var optionsForSerie = serieOption[serieItemViewModel.SerieName];
                        optionsForSerie["stepPlot"] = true;
                    }
                    else {
                        serieOption[serieItemViewModel.SerieName] = { stepPlot: true };
                    }
                }
            }
            return new log_chart_viewmodel_1.LogChartViewModel(labelArray, data, colorLabelArray, serieOption);
        };
        LogComponentViewModel.prototype.refreshChartViewModel = function () {
            // TODO: improve
            this._chartViewModel = this.buildChartViewModel(this._logModel, this._selectBaseDefinitionModel);
        };
        LogComponentViewModel.prototype.plotChart = function (logChartViewModel) {
            this._chartReference = new Dygraphs(// TODO: fix import
            // containing div
            document.getElementById("log-chart"), logChartViewModel.Data, {
                labels: logChartViewModel.LabelArray,
                colors: logChartViewModel.ColorArray,
                series: logChartViewModel.SerieOptions,
                labelsKMB: true,
                legend: "always",
                labelsDiv: document.getElementById("log-chart-legend"),
                drawGapEdgePoints: true,
                showRangeSelector: true,
                highlightSeriesOpts: {
                    strokeWidth: 3,
                    strokeBorderWidth: 1,
                    highlightCircleSize: 5
                },
                highlightCircleSize: 2,
                strokeWidth: 1,
                strokeBorderWidth: 1,
            });
        };
        LogComponentViewModel.prototype.updatePlotChart = function (logChartViewModel) {
            this._chartReference.updateOptions({
                file: logChartViewModel.Data,
                labels: logChartViewModel.LabelArray,
                colors: logChartViewModel.ColorArray,
                series: logChartViewModel.SerieOptions
            });
        };
        LogComponentViewModel.prototype.closeTopAlert = function () {
            this._logTopAlertViewModel.State(alert_viewmodel_3.LogAlertState.Invisible);
        };
        LogComponentViewModel.prototype.closeBottomAlert = function () {
            this._logBottomAlertViewModel.State(alert_viewmodel_3.LogAlertState.Invisible);
        };
        LogComponentViewModel.prototype.showError = function (reason) {
            this._log(reason);
            this._logTopAlertViewModel.AlertErrorDetailMsg(reason);
            this._logTopAlertViewModel.State(alert_viewmodel_3.LogAlertState.Error);
        };
        // Traslation
        LogComponentViewModel.prototype.translate = function () {
            var translator = translator_service_24.TranslatorService.getInstance().t;
            this._noLogAvailableText(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "noLogAvailable"));
            this._periodicLogsTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "periodicLog"));
            this._eventLogsTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "eventLog"));
            this._intervalTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "interval"));
            this._starttimeTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "starttime"));
            this._stoptimeTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "stoptime"));
            this._loadTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "load"));
            this._exportTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "export"));
            this._logNameTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "name"));
            this._recordNumberTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "recordNumber"));
            this._intervalSecondTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "intervalSecond"));
            this._variableNameTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "name"));
            this._isVisibleTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "isVisible"));
            this._axisTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "axis"));
            this._colorTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "color"));
            this._styleTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "style"));
            this._periodTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "period"));
            this._quickLoadTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "quickLoadDate"));
            this._agoTitle(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "ago"));
            this._invalidDateMsg(translator(translator_service_24.TranslatorService.CfieldTranslationNs + "invalidDate"));
        };
        LogComponentViewModel.prototype.refreshTableTranslation = function () {
            var translatorService = translator_service_24.TranslatorService.getInstance();
            var translator = translatorService.t;
            // Periodic table
            var table = $(PeriodicLogDefinitionTable);
            table.bootstrapTable("changeLocale", translatorService.CurrentLanguage());
            $(PeriodicLogDefinitionTable).bootstrapTable("changeTitle", {
                LogName: translator(translator_service_24.TranslatorService.CfieldTranslationNs + "name"),
                LogNrecord: translator(translator_service_24.TranslatorService.CfieldTranslationNs + "recordNumber"),
                LogSampledVariableValue: translator(translator_service_24.TranslatorService.CfieldTranslationNs + "intervalSecond")
            });
            // Event log
            table = $(EventLogDefinitionTable);
            table.bootstrapTable("changeLocale", translatorService.CurrentLanguage());
            $(EventLogDefinitionTable).bootstrapTable("changeTitle", {
                LogName: translator(translator_service_24.TranslatorService.CfieldTranslationNs + "name"),
                LogNrecord: translator(translator_service_24.TranslatorService.CfieldTranslationNs + "recordNumber")
            });
        };
        return LogComponentViewModel;
    }(base_component_viewmodel_5.BaseComponentViewModel));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var hideExtraToolbar = (params && params.hideExtraToolbar) || false;
                var newViewModel = new LogComponentViewModel(hideExtraToolbar);
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_13.cFieldService.getInstance().BaseComponentViewUrl + "/log.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/login-form-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/login.service", "service/translator.service", "component/viewModel/base-error-component.viewmodel"], function (require, exports, ko, cfield_service_14, login_service_4, translator_service_25, base_error_component_viewmodel_3) {
    "use strict";
    /**
     * Common utility for the components that require login.
     */
    var LoginFormComponentViewModel = /** @class */ (function (_super) {
        __extends(LoginFormComponentViewModel, _super);
        /**
         * Init component.
         * @param cFieldService CField service instance.
         */
        function LoginFormComponentViewModel() {
            var _this = _super.call(this) || this;
            _this._usernamePlaceholder = ko.observable();
            _this._passwordPlaceholder = ko.observable();
            _this._username = ko.observable();
            _this._password = ko.observable();
            _this._login = ko.observable();
            _this._loginFailedMsg = ko.observable();
            _this._loginFailedMsgState = ko.observable(LoginMsgFailedState.NoError);
            _this._isLoginFailedMsgVisible = ko.pureComputed({
                read: function () {
                    return _this._loginFailedMsgState() !== LoginMsgFailedState.NoError;
                }
            });
            var cFieldServiceInstance = cfield_service_14.cFieldService.getInstance();
            _this._logoPath = cFieldServiceInstance.BaseUrl + "/assets/img/login.svg";
            cFieldServiceInstance.StateMonitor.BootPromise
                .then(function () {
                _this.translate();
            })
                .catch(function (reason) {
                _this._log(reason);
            });
            return _this;
        }
        Object.defineProperty(LoginFormComponentViewModel.prototype, "UsernamePlaceholder", {
            get: function () { return this._usernamePlaceholder; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(LoginFormComponentViewModel.prototype, "PasswordPlaceholder", {
            get: function () { return this._passwordPlaceholder; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(LoginFormComponentViewModel.prototype, "Login", {
            get: function () { return this._login; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(LoginFormComponentViewModel.prototype, "Username", {
            get: function () { return this._username; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(LoginFormComponentViewModel.prototype, "Password", {
            get: function () { return this._password; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LoginFormComponentViewModel.prototype, "LoginFailedMsg", {
            get: function () { return this._loginFailedMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LoginFormComponentViewModel.prototype, "IsLoginFailedMsgVisible", {
            get: function () { return this._isLoginFailedMsgVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LoginFormComponentViewModel.prototype, "LogoPath", {
            get: function () { return this._logoPath; },
            enumerable: true,
            configurable: true
        });
        /**
         * Dispose.
         */
        LoginFormComponentViewModel.prototype.dispose = function () {
            if (this._cFieldServiceInitSubscription) {
                this._cFieldServiceInitSubscription.dispose();
            }
        };
        LoginFormComponentViewModel.prototype.translate = function () {
            var translator = translator_service_25.TranslatorService.getInstance().t;
            this._usernamePlaceholder(translator(translator_service_25.TranslatorService.CfieldTranslationNs + "username"));
            this._passwordPlaceholder(translator(translator_service_25.TranslatorService.CfieldTranslationNs + "password"));
            this._login(translator(translator_service_25.TranslatorService.CfieldTranslationNs + "login"));
            this._loginFailedMsg(this.getLoginMsgFailedTranslation(translator));
        };
        /**
         * Log in.
         */
        LoginFormComponentViewModel.prototype.onClickLogin = function () {
            var _this = this;
            var username = this._username();
            var password = this._password();
            var translator = translator_service_25.TranslatorService.getInstance().t;
            if (!username && !password) {
                this._loginFailedMsgState(LoginMsgFailedState.InvalidCredential);
                this._loginFailedMsg(this.getLoginMsgFailedTranslation(translator));
                return;
            }
            if (!username) {
                this._loginFailedMsgState(LoginMsgFailedState.EmptyUsername);
                this._loginFailedMsg(this.getLoginMsgFailedTranslation(translator));
                return;
            }
            if (!password) {
                this._loginFailedMsgState(LoginMsgFailedState.EmptyPassword);
                this._loginFailedMsg(this.getLoginMsgFailedTranslation(translator));
                return;
            }
            var loginPromise = login_service_4.LoginService.getInstance().login(this._username(), this._password());
            loginPromise
                .then(function (account) {
                // Clear state (in case of logout)
                _this.clearState();
                // Don't initialize cField service here. The init-progress viemodel will init cField when it will be bound
                // cFieldService.getInstance().initService();
            })
                .catch(function (reason) {
                var translator = translator_service_25.TranslatorService.getInstance().t;
                _this._loginFailedMsgState(LoginMsgFailedState.LoginFailed);
                _this._loginFailedMsg(_this.getLoginMsgFailedTranslation(translator));
                _this._log(reason);
            });
        };
        LoginFormComponentViewModel.prototype.clearState = function () {
            this._username("");
            this._password("");
            this._loginFailedMsgState(LoginMsgFailedState.NoError);
            this._loginFailedMsg("");
        };
        LoginFormComponentViewModel.prototype.getLoginMsgFailedTranslation = function (translator) {
            switch (this._loginFailedMsgState()) {
                case LoginMsgFailedState.NoError:
                    return "";
                case LoginMsgFailedState.InvalidCredential:
                    return translator(translator_service_25.TranslatorService.CfieldTranslationNs + "invalidCredential");
                case LoginMsgFailedState.EmptyUsername:
                    return translator(translator_service_25.TranslatorService.CfieldTranslationNs + "emptyUsername");
                case LoginMsgFailedState.EmptyPassword:
                    return translator(translator_service_25.TranslatorService.CfieldTranslationNs + "emptyPassword");
                case LoginMsgFailedState.LoginFailed:
                    return translator(translator_service_25.TranslatorService.CfieldTranslationNs + "loginFailed");
            }
            return "";
        };
        return LoginFormComponentViewModel;
    }(base_error_component_viewmodel_3.BaseErrorComponentViewModel));
    var LoginMsgFailedState;
    (function (LoginMsgFailedState) {
        LoginMsgFailedState[LoginMsgFailedState["NoError"] = 0] = "NoError";
        LoginMsgFailedState[LoginMsgFailedState["InvalidCredential"] = 1] = "InvalidCredential";
        LoginMsgFailedState[LoginMsgFailedState["EmptyUsername"] = 2] = "EmptyUsername";
        LoginMsgFailedState[LoginMsgFailedState["EmptyPassword"] = 3] = "EmptyPassword";
        LoginMsgFailedState[LoginMsgFailedState["LoginFailed"] = 4] = "LoginFailed";
    })(LoginMsgFailedState || (LoginMsgFailedState = {}));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var newViewModel = new LoginFormComponentViewModel();
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_14.cFieldService.getInstance().BaseComponentViewUrl + "/login-form.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/network-component.viewmodel", ["require", "exports", "service/cfield.service", "component/viewModel/base-component.viewmodel"], function (require, exports, cfield_service_15, base_component_viewmodel_6) {
    "use strict";
    var NetworkComponentViewModel = /** @class */ (function (_super) {
        __extends(NetworkComponentViewModel, _super);
        function NetworkComponentViewModel(hideExtraToolbar) {
            var _this = _super.call(this, hideExtraToolbar, false, false, false) || this;
            // Evaluete the state of cField service
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        /**
         * Init. Now <code>cField</code> service is initialized.
         */
        // override
        NetworkComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            _super.prototype.dispose.call(this);
            // Test
            // Simulate state
            $("#FieldBus").addClass("running");
            $("#BMS").addClass("error");
            $("#FieldBus").click(function (event) {
                $('#myModal').modal('toggle');
                event.preventDefault();
            });
            $("#BMS").click(function (event) {
                $('#myModal').modal('toggle');
                event.preventDefault();
            });
        };
        return NetworkComponentViewModel;
    }(base_component_viewmodel_6.BaseComponentViewModel));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var hideExtraToolbar = (params && params.hideExtraToolbar) || false;
                var newViewModel = new NetworkComponentViewModel(hideExtraToolbar);
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_15.cFieldService.getInstance().BaseComponentViewUrl + "/network.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/parameter-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/device-config.service", "service/import-export.service", "service/translator.service", "service/variable.service", "component/viewModel/base-table-component.viewmodel", "component/viewModel/base-category-component.viewmodel", "component/viewModel/item/variable-item.viewmodel", "component/viewModel/item/alert-item.viewmodel", "component/viewModel/alert.viewmodel", "dataModel/datatype.model", "common/data-structure", "common/converter", "common/file-util", "common/validator", "component/presenter/variable-item.presenter", "component/presenter/update-item-set"], function (require, exports, ko, cfield_service_16, device_config_service_10, import_export_service_2, translator_service_26, variable_service_6, base_table_component_viewmodel_4, base_category_component_viewmodel_2, variable_item_viewmodel_3, alert_item_viewmodel_2, alert_viewmodel_4, datatype_model_11, data_structure_7, converter_5, file_util_3, validator_22, variable_item_presenter_1, update_item_set_1) {
    "use strict";
    var ParameterComponentViewModel = /** @class */ (function (_super) {
        __extends(ParameterComponentViewModel, _super);
        function ParameterComponentViewModel(hideExtraToolbar) {
            var _this = _super.call(this, hideExtraToolbar, true, true, false, ParameterComponentViewModel.IdTable) || this;
            // Item view models
            _this._parameterItemViewModel = [];
            // Flag current value/imported value
            _this._sourceParamToExport = datatype_model_11.SourceValue.Current;
            // Init
            _this._variableDictionary = device_config_service_10.DeviceConfigService.getInstance().VariablesDictionary;
            _this._isParameterInit = ko.observable(false);
            _this.VariableInitValueHeader = ko.observable();
            _this.VariableDevValueHeader = ko.observable();
            _this.VariableCurrentValueHeader = ko.observable();
            _this.VariableImportedValueHeader = ko.observable();
            _this._parameterAlert = new alert_viewmodel_4.ParameterAlertViewModel();
            _this._isFilterImportedValueActive = ko.observable(false);
            _this._isImportParameterDone = ko.observable(false);
            _this._updateItemSetHelper = new update_item_set_1.UpdateItemSetHelper(new data_structure_7.CustomMap(), ko.observable());
            _this._isSaveRequested = ko.pureComputed({
                read: function () {
                    return _this._updateItemSetHelper.NumberOfPendingUpdate() > 0 || _this._isImportParameterDone() === true;
                }
            });
            _this._isToolbarInit = false;
            // Evaluete the state of cField service
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        Object.defineProperty(ParameterComponentViewModel.prototype, "IsParameterInit", {
            get: function () {
                return this._isParameterInit;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "IsFilterImportedValueActive", {
            get: function () { return this._isFilterImportedValueActive; },
            enumerable: true,
            configurable: true
        });
        ParameterComponentViewModel.prototype.IsImportParameterDone = function () { return this._isImportParameterDone(); };
        Object.defineProperty(ParameterComponentViewModel.prototype, "IsSaveRequested", {
            get: function () { return this._isSaveRequested; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "IsAlertVisible", {
            get: function () { return this._parameterAlert.IsAlertVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertCssClass", {
            get: function () { return this._parameterAlert.AlertCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertTitle1", {
            get: function () { return this._parameterAlert.AlertTitle1; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertTitle2", {
            get: function () { return this._parameterAlert.AlertTitle2; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertIsErrorDetailVisible", {
            get: function () { return this._parameterAlert.AlertIsErrorDetailVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertErrorDetailMsg", {
            get: function () { return this._parameterAlert.AlertErrorDetailMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertIsUpdateFailedTableVisible", {
            get: function () { return this._parameterAlert.AlertIsUpdateFailedTableVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertIsUpdateInvalidTableVisible", {
            get: function () { return this._parameterAlert.AlertIsUpdateInvalidTableVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertVariableNameHeader", {
            get: function () { return this._parameterAlert.AlertVariableNametHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertSetValueHeader", {
            get: function () { return this._parameterAlert.AlertSetValueHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertServerResponseHeader", {
            get: function () { return this._parameterAlert.AlertServerResponseHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertNotUpdatedArray", {
            get: function () { return this._parameterAlert.AlertNotUpdatedArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertEditValueHeader", {
            get: function () { return this._parameterAlert.AlertEditValueHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterComponentViewModel.prototype, "AlertInvalidUpdateArray", {
            get: function () { return this._parameterAlert.AlertInvalidUpdateArray; },
            enumerable: true,
            configurable: true
        });
        // override
        ParameterComponentViewModel.prototype.onChangeCFieldState = function (newState) {
            _super.prototype.onChangeCFieldState.call(this, newState);
            // Manage ad-hoc state
            switch (newState) {
                case cfield_service_16.CfieldServiceState.RefreshConfig:
                    // Clear state 
                    this._parameterItemViewModel = [];
                    this._isParameterInit(false);
                    this._isToolbarInit = false;
                    this._parameterAlert.IsAlertVisible(false);
                    this._parameterAlert.AlertNotUpdatedArray.removeAll();
                    this._parameterAlert.AlertInvalidUpdateArray.removeAll();
                    this._isFilterImportedValueActive(false);
                    // Destroy view 
                    $(ParameterComponentViewModel.IdTable).bootstrapTable("destroy");
                    break;
            }
        };
        // override
        ParameterComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            var _this = this;
            _super.prototype.onSuccessfulInitializedConf.call(this);
            // Init the header of the table with the translation
            this.translate();
            var variableService = variable_service_6.VariableService.getInstance();
            // 1) Refresh link to variable min/max limits. TODO: filter only relevant variables
            var initPromise = variableService.updateVariableLimits(true);
            initPromise
                .then(function () {
                // 2) Get params
                return variableService.getParameters();
            })
                .then(function (devParamsContainerArray) {
                // 3) Build view
                // Build view model
                _this._parameterItemViewModel = _this.buildParameterItemViewModel(devParamsContainerArray);
                // Build table
                _this.buildParameterTable(_this._parameterItemViewModel);
            })
                .catch(function (reason) {
                _this.showError(reason);
            });
        };
        // override
        ParameterComponentViewModel.prototype.onChangeScreenSize = function (isMobileScreen) {
            if (isMobileScreen != this._enableCardView) {
                // Change layout of table
                $(ParameterComponentViewModel.IdTable).bootstrapTable("toggleView");
            }
        };
        // override
        ParameterComponentViewModel.prototype.onChangeLanguage = function (newState) {
            _super.prototype.onChangeLanguage.call(this, newState);
            if (newState === translator_service_26.TranslatorEvent.LanguageChanged) {
                this.refreshTranslation();
            }
        };
        ParameterComponentViewModel.prototype.buildParameterItemViewModel = function (devParamContainerArray) {
            var parameterItemViewModelArray = [];
            for (var _i = 0, devParamContainerArray_1 = devParamContainerArray; _i < devParamContainerArray_1.length; _i++) {
                var item = devParamContainerArray_1[_i];
                // Build viewmodel
                var viewModel = new variable_item_viewmodel_3.ParameterItemViewModel(this, item.DevParamValue, item.Variable, item.Variable._initValue, item.Variable._devValue, new variable_item_presenter_1.RowStyleWithMemoryEditableVariableItemPresenter(this._updateItemSetHelper), new variable_item_presenter_1.RowStyleEditableVariableItemPresenter(), this.getCatagories(item.Variable));
                parameterItemViewModelArray.push(viewModel);
            }
            return parameterItemViewModelArray;
        };
        ParameterComponentViewModel.prototype.buildParameterTable = function (parameterItemViewModelArray) {
            $(function () {
                $(ParameterComponentViewModel.IdTable).bootstrapTable({
                    data: parameterItemViewModelArray,
                    icons: base_table_component_viewmodel_4.BaseTableComponentViewModel.toolbarIconPak,
                    onToggle: function (isCardView) {
                        this._enableCardView = isCardView;
                    }.bind(this),
                    // Refresh 
                    onRefresh: function () {
                        this.refreshParameterValues();
                    }.bind(this),
                    // Insert a button in the toolbar with same style
                    onPostBody: function () {
                        if (this._isToolbarInit === false) {
                            // Inject component without container. In this way I can use default css for toolbar
                            // TODO: improve the searching
                            var toolbar_2 = $("parameter-component div.columns.columns-right.btn-group.pull-right");
                            var $newToolbarItem = $("<!-- ko component: 'parameter-toolbar-component' --><!-- /ko -->");
                            toolbar_2.prepend($newToolbarItem);
                            ko.applyBindings({}, $newToolbarItem[0]);
                            this._isToolbarInit = true;
                        }
                    }.bind(this),
                    // Locale
                    locale: translator_service_26.TranslatorService.getInstance().CurrentLanguage()
                });
            }.bind(this));
            // Now view is initialized
            this._isParameterInit(true);
        };
        ParameterComponentViewModel.prototype.refreshParameterValues = function () {
            var _this = this;
            $(ParameterComponentViewModel.IdTable).bootstrapTable("showLoading");
            var variableService = variable_service_6.VariableService.getInstance();
            var initPromise = variableService.updateVariableLimits(true);
            initPromise
                .then(function () {
                return variableService.getParameters();
            })
                .then(function (devParamsContainerArray) {
                if (devParamsContainerArray && devParamsContainerArray.length > 0) {
                    // Refresh viewmodel
                    _this.refreshViewModel(devParamsContainerArray);
                    $(ParameterComponentViewModel.IdTable).bootstrapTable("load", _this._parameterItemViewModel);
                    $(ParameterComponentViewModel.IdTable).bootstrapTable("hideLoading");
                    _this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.Success);
                }
            })
                .catch(function (reason) {
                _this.showError(reason);
            });
        };
        ParameterComponentViewModel.prototype.refreshViewModel = function (devParamsContainerArray) {
            // Build a map with result
            var idToDevParamContainerDictionary = {};
            for (var _i = 0, devParamsContainerArray_1 = devParamsContainerArray; _i < devParamsContainerArray_1.length; _i++) {
                var item = devParamsContainerArray_1[_i];
                idToDevParamContainerDictionary[item.Variable._id] = item;
            }
            // Refresh view model
            for (var _a = 0, _b = this._parameterItemViewModel; _a < _b.length; _a++) {
                var parameterViewModel = _b[_a];
                var parameterItem = idToDevParamContainerDictionary[parameterViewModel.Variable._id];
                if (!validator_22.Validator.isDefined(parameterItem)) {
                    this.showError("ParameterComponentViewModel refreshViewModel(). Id not found");
                    continue;
                }
                // Min, max and current value can be changed
                parameterViewModel.refresh(true, parameterItem.DevParamValue, parameterItem.Variable._initValue, parameterItem.Variable._devValue);
            }
        };
        ParameterComponentViewModel.prototype.importParametersFromFile = function (file) {
            var _this = this;
            var importedDevParamsPromise = import_export_service_2.ImportExportService.getInstance().parseDevParamsFile(file);
            variable_service_6.VariableService.getInstance().getParameters(importedDevParamsPromise)
                .then(function (devParamContainerArray) {
                // Build map
                var importedParameterMap = new data_structure_7.CustomMap();
                if (validator_22.Validator.isDefined(devParamContainerArray)) {
                    for (var _i = 0, devParamContainerArray_2 = devParamContainerArray; _i < devParamContainerArray_2.length; _i++) {
                        var param = devParamContainerArray_2[_i];
                        importedParameterMap.addNoConflict(param.DevParam.Variable, param);
                    }
                }
                // Update view model
                _this.refreshViewModelWithImportedValue(importedParameterMap);
                _this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.Success);
            })
                .catch(function (reason) {
                _this.showError(reason);
            });
        };
        /**
         * Refresh table with imported values from file
         */
        ParameterComponentViewModel.prototype.refreshViewModelWithImportedValue = function (importedParameterMap) {
            var elaboratedParameterSet = new data_structure_7.StringSet();
            // Update view model
            for (var _i = 0, _a = this._parameterItemViewModel; _i < _a.length; _i++) {
                var parameterViewModel = _a[_i];
                // Reset import value
                parameterViewModel.resetImportedValue();
                var importedParameter = importedParameterMap.Set[parameterViewModel.VariableName];
                if (validator_22.Validator.isDefined(importedParameter)) {
                    // Overwrite
                    parameterViewModel.VariableImportedValue = importedParameter.DevParamValue;
                    elaboratedParameterSet.addNoConflict(parameterViewModel.VariableName);
                }
                else {
                    parameterViewModel.VariableImportedValue = null;
                }
            }
            // Compute not elaborated parameters
            var notElaboratedParameters = [];
            var elaboratedParameterNameMap = elaboratedParameterSet.Set;
            var importedParameters = importedParameterMap.getValues();
            for (var _b = 0, importedParameters_1 = importedParameters; _b < importedParameters_1.length; _b++) {
                var parameter = importedParameters_1[_b];
                if (!elaboratedParameterSet.contains(parameter.DevParam.Variable)) {
                    notElaboratedParameters.push(parameter);
                }
            }
            // Update error message
            this._parameterAlert.AlertInvalidUpdateArray.removeAll();
            for (var _c = 0, notElaboratedParameters_1 = notElaboratedParameters; _c < notElaboratedParameters_1.length; _c++) {
                var parameter = notElaboratedParameters_1[_c];
                this._parameterAlert.AlertInvalidUpdateArray.push(new alert_item_viewmodel_2.InvalidUpdateViewModel(parameter.DevParam.Variable, parameter.DevParam.DefaultValue));
            }
            if (this._parameterAlert.AlertInvalidUpdateArray().length > 0) {
                this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.UpdateFailedAndInvalid);
            }
            // Manage state
            this._isImportParameterDone(true);
            this._sourceParamToExport = datatype_model_11.SourceValue.Imported;
            // Refresh table
            $(ParameterComponentViewModel.IdTable).bootstrapTable("load", this._parameterItemViewModel);
            $(ParameterComponentViewModel.IdTable).bootstrapTable("showColumn", "VariableImportedValue");
        };
        /**
         * Send CGI to set parameters.
         */
        ParameterComponentViewModel.prototype.setDevParams = function () {
            var _this = this;
            // Warn user
            this._parameterAlert.AlertInvalidUpdateArray.removeAll();
            this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.Waiting);
            // Get params to set
            var devParamsToSetArray;
            switch (this._sourceParamToExport) {
                case datatype_model_11.SourceValue.Current:
                    devParamsToSetArray = this.getCurrentOrEditParameter();
                    break;
                case datatype_model_11.SourceValue.Imported:
                    devParamsToSetArray = this.getImportedParameter();
                    break;
            }
            if (devParamsToSetArray.length > 0) {
                // Set params
                var resultPromise = this.sendSetParamsCgi(devParamsToSetArray)
                    .then(function (returnCode) {
                    if (validator_22.Validator.isDefined(returnCode) && returnCode.result == '0') {
                        // Get params
                        return variable_service_6.VariableService.getInstance().getParameters();
                    }
                    else {
                        return Promise.reject("Error code: " + returnCode.result + " Try to reboot the device");
                    }
                })
                    .then(function (devParamContainer) {
                    // Compare
                    _this.compareParams(devParamsToSetArray, devParamContainer);
                })
                    .catch(function (reason) {
                    _this.showError(reason);
                });
            }
            else {
                // Warn user
                this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.NothingToUpdate);
            }
        };
        /**
         * @return Parameters from edit or current value of viewmodel item.
         */
        ParameterComponentViewModel.prototype.getCurrentOrEditParameter = function (forceCurrentParams) {
            var devParamToSetArray = [];
            var datatypeDictionary = datatype_model_11.DataTypeUtil.getInstance().DataTypeDictionary;
            // Extract all variables
            for (var _i = 0, _a = this._parameterItemViewModel; _i < _a.length; _i++) {
                var item = _a[_i];
                var viewmodel = item;
                var variable = item.Variable;
                var id = item.Variable._id;
                if (item.IsVariableModified && !item.IsVariableEditedValid) {
                    this._parameterAlert.AlertInvalidUpdateArray.push(new alert_item_viewmodel_2.InvalidUpdateViewModel(item.VariableName, item.VariableEditedValue));
                    continue;
                }
                var newValue = viewmodel.VariableEditedValue;
                if (validator_22.Validator.isDefined(newValue) && !forceCurrentParams) {
                    if (typeof newValue === "boolean") {
                        newValue = converter_5.Converter.BoolToBoolText(newValue);
                    }
                }
                else {
                    // Value round to default precision
                    //newValue = viewmodel.VariableCurrentValue;
                    newValue = viewmodel.VariableFullPrecisionValue;
                    if (typeof newValue === "boolean") {
                        newValue = converter_5.Converter.BoolToBoolText(newValue);
                    }
                }
                if (validator_22.Validator.isDefined(variable) && validator_22.Validator.isDefined(viewmodel)) {
                    // Build DevParam object
                    devParamToSetArray.push(new import_export_service_2.DevParamToSet(variable._name, variable._description, datatypeDictionary[variable._dataType], newValue));
                }
                else {
                    this.showError("ParameterComponentViewModel getCurrentOrEditParameter(): variable not found (id=" + id + ")");
                }
            }
            return devParamToSetArray;
        };
        /**
         * @return Parameters from imported paramater value of view model.
         */
        ParameterComponentViewModel.prototype.getImportedParameter = function () {
            var devParamToSetArray = [];
            var datatypeDictionary = datatype_model_11.DataTypeUtil.getInstance().DataTypeDictionary;
            // Extract all variables
            for (var _i = 0, _a = this._parameterItemViewModel; _i < _a.length; _i++) {
                var item = _a[_i];
                var viewmodel = item;
                var variable = item.Variable;
                var id = item.Variable._id;
                if (!item.IsVariableImportedValueSet) {
                    this._parameterAlert.AlertInvalidUpdateArray.push(new alert_item_viewmodel_2.InvalidUpdateViewModel(item.VariableName, "-")); // TODO: better description
                    continue;
                }
                else if (!item.IsVariableImportedValueValid) {
                    this._parameterAlert.AlertInvalidUpdateArray.push(new alert_item_viewmodel_2.InvalidUpdateViewModel(item.VariableName, item.VariableImportedValue));
                    continue;
                }
                var newValue = viewmodel.VariableImportedValue;
                if (validator_22.Validator.isDefined(newValue)) {
                    if (typeof newValue === "boolean") {
                        newValue = converter_5.Converter.BoolToBoolText(newValue);
                    }
                }
                if (validator_22.Validator.isDefined(variable) && validator_22.Validator.isDefined(viewmodel)) {
                    // Build DevParam object
                    devParamToSetArray.push(new import_export_service_2.DevParamToSet(variable._name, variable._description, datatypeDictionary[variable._dataType], newValue));
                }
                else {
                    this.showError("ParameterComponentViewModel getImportedParameter(): variable not found (id=" + id + ")");
                }
            }
            return devParamToSetArray;
        };
        ParameterComponentViewModel.prototype.sendSetParamsCgi = function (devParamToSetArray) {
            // Build text of the file
            var text = import_export_service_2.ImportExportService.getInstance().writeDevParamText(devParamToSetArray, false);
            // Send with CGI
            return variable_service_6.VariableService.getInstance().setDevVars(text);
        };
        ParameterComponentViewModel.prototype.compareParams = function (setParamArray, readParamAfterSetArray) {
            this._parameterAlert.AlertNotUpdatedArray.removeAll();
            // Build map 
            var readParamMap = {};
            for (var _i = 0, readParamAfterSetArray_1 = readParamAfterSetArray; _i < readParamAfterSetArray_1.length; _i++) {
                var devParamContainterItem = readParamAfterSetArray_1[_i];
                readParamMap[devParamContainterItem.DevParam.Variable] = devParamContainterItem;
            }
            // Scan response
            for (var _a = 0, setParamArray_1 = setParamArray; _a < setParamArray_1.length; _a++) {
                var setParam = setParamArray_1[_a];
                // Extract value to set
                var readParam = readParamMap[setParam.Variable];
                if (validator_22.Validator.isDefined(readParam)) {
                    // Compare value
                    var getValue = readParam.DevParamValue;
                    var setValue = converter_5.Converter.ConvertValueInAccordWithDataType(readParam.Variable._dataType, setParam.DefaultValue);
                    var areEqual = void 0;
                    if (datatype_model_11.DataTypeUtil.IsNumericDataType(readParam.Variable._dataType) && getValue !== "" && setValue !== "") {
                        getValue = Number(getValue);
                        setValue = Number(setValue);
                        areEqual = validator_22.Validator.CompareValues(getValue, setValue, variable_item_viewmodel_3.ParameterItemViewModel.numberMaxVariableAccuracy);
                    }
                    else {
                        areEqual = getValue == setValue;
                    }
                    if (!areEqual) {
                        // Insert into error list
                        this._parameterAlert.AlertNotUpdatedArray.push(new alert_item_viewmodel_2.NotUpdateViewModel(readParam.Variable._name, setParam.DefaultValue, getValue.toString()));
                    }
                }
                else {
                    this.showError("ParameterComponent compareParams(). The parameter with name = " + readParam.Variable._name +
                        " is not been found in the set of written parameters.");
                }
            }
            // Update viewmodel
            this.refreshViewModel(readParamAfterSetArray);
            // Manage alert
            if (this._parameterAlert.AlertNotUpdatedArray().length == 0 && this._parameterAlert.AlertInvalidUpdateArray().length == 0) {
                this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.Success);
            }
            else if (this._parameterAlert.AlertNotUpdatedArray().length > 0 && this._parameterAlert.AlertInvalidUpdateArray().length > 0) {
                this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.UpdateFailedAndInvalid);
            }
            else if (this._parameterAlert.AlertNotUpdatedArray().length > 0 && this._parameterAlert.AlertInvalidUpdateArray().length == 0) {
                this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.UpdateFailed);
            }
            else {
                this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.UpdateInvalid);
            }
            // Clear state
            this._updateItemSetHelper.clear();
            this._isImportParameterDone(false);
            this._sourceParamToExport = datatype_model_11.SourceValue.Current;
            // Refresh table
            $(ParameterComponentViewModel.IdTable).bootstrapTable("load", this._parameterItemViewModel);
        };
        /**
         * Export parameters to file
         */
        ParameterComponentViewModel.prototype.saveDevParams = function () {
            // Get params to set
            var devParamsToSetArray = this.getCurrentOrEditParameter(true);
            if (devParamsToSetArray.length > 0) {
                // Build text of the file
                var text = import_export_service_2.ImportExportService.getInstance().writeDevParamText(devParamsToSetArray, true);
                // Save
                file_util_3.FileUtil.SaveTextFileOnDisk("DevParams.txt", text);
            }
        };
        /**
         * Select the records with current value equal to imported value
         */
        ParameterComponentViewModel.prototype.filterCurrentNotEqualToImportedValues = function () {
            var isFilterActive = !(this._isFilterImportedValueActive());
            // Switch filter
            this._isFilterImportedValueActive(isFilterActive);
            // Filter
            var filter = isFilterActive ? { IsCurrentNotEqualToImportedValue: true } : {};
            $(ParameterComponentViewModel.IdTable).bootstrapTable("filterBy", filter);
        };
        ParameterComponentViewModel.prototype.getInfo = function () {
            this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.Info);
        };
        ParameterComponentViewModel.prototype.showError = function (reason) {
            this._log(reason);
            this._parameterAlert.AlertErrorDetailMsg(reason);
            this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.Error);
        };
        ParameterComponentViewModel.prototype.closeAlert = function () {
            this._parameterAlert.State(alert_viewmodel_4.ParameterAlertState.Invisible);
        };
        // Translations
        ParameterComponentViewModel.prototype.translate = function () {
            var translator = translator_service_26.TranslatorService.getInstance().t;
            this.VariableInitValueHeader(translator(translator_service_26.TranslatorService.CfieldTranslationNs + "init"));
            this.VariableDevValueHeader(translator(translator_service_26.TranslatorService.CfieldTranslationNs + "dev"));
            this.VariableCurrentValueHeader(translator(translator_service_26.TranslatorService.CfieldTranslationNs + "currentVal"));
            this.VariableImportedValueHeader(translator(translator_service_26.TranslatorService.CfieldTranslationNs + "imported"));
            this._parameterAlert.translate();
        };
        /**
         * Refresh translation of table in accord with current language.
         */
        ParameterComponentViewModel.prototype.refreshTranslation = function () {
            var translatorService = translator_service_26.TranslatorService.getInstance();
            var translator = translatorService.t;
            var table = $(ParameterComponentViewModel.IdTable);
            table.bootstrapTable("changeLocale", translatorService.CurrentLanguage());
            $(ParameterComponentViewModel.IdTable).bootstrapTable("changeTitle", {
                VariableName: translator(translator_service_26.TranslatorService.CfieldTranslationNs + "name"),
                VariableDescription: translator(translator_service_26.TranslatorService.CfieldTranslationNs + "description"),
                CategoryArray: translator(translator_service_26.TranslatorService.CfieldTranslationNs + "categories"),
                VariableType: translator(translator_service_26.TranslatorService.CfieldTranslationNs + "type"),
                VariableUoM: translator(translator_service_26.TranslatorService.CfieldTranslationNs + "UoM"),
                VariableMinValue: translator(translator_service_26.TranslatorService.CfieldTranslationNs + "min"),
                VariableMaxValue: translator(translator_service_26.TranslatorService.CfieldTranslationNs + "max"),
                VariableInitValue: translator(translator_service_26.TranslatorService.CfieldTranslationNs + "init"),
                VariableDevValue: translator(translator_service_26.TranslatorService.CfieldTranslationNs + "dev"),
                VariableCurrentValue: translator(translator_service_26.TranslatorService.CfieldTranslationNs + "currentVal"),
                VariableImportedValue: translator(translator_service_26.TranslatorService.CfieldTranslationNs + "imported"),
                _callbackBeforeUpdateBodyOnRefreshTranslation: function () {
                    // Reset toolbar state
                    this._isToolbarInit = false;
                }.bind(this)
            });
            // Alert
            this._parameterAlert.translate();
        };
        ParameterComponentViewModel.IdTable = "#parameter-component-table";
        return ParameterComponentViewModel;
    }(base_category_component_viewmodel_2.BaseCategoryComponentViewModel));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var hideExtraToolbar = (params && params.hideExtraToolbar) || false;
                var newViewModel = new ParameterComponentViewModel(hideExtraToolbar);
                // Cache
                cfield_service_16.cFieldService.getInstance().ParameterComponenetViewModelCached = newViewModel;
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_16.cFieldService.getInstance().BaseComponentViewUrl + "/parameter.html"
        },
        ParameterComponentViewModel: ParameterComponentViewModel
    };
    return objectToExport;
});
define("component/viewModel/parameter-toolbar-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/translator.service", "component/viewModel/base-toolbar.viewmodel", "common/validator"], function (require, exports, ko, cfield_service_17, translator_service_27, base_toolbar_viewmodel_2, validator_23) {
    "use strict";
    var ImportDevParamFormId = "#parameter-component-import-params-form";
    var ParameterToolbarComponentViewModel = /** @class */ (function (_super) {
        __extends(ParameterToolbarComponentViewModel, _super);
        function ParameterToolbarComponentViewModel(_parameterComponentViewModel) {
            var _this = _super.call(this, ko.pureComputed({
                read: function () {
                    return _this._parameterComponentViewModel.IsSaveRequested();
                }
            })) || this;
            _this._parameterComponentViewModel = _parameterComponentViewModel;
            _this._isImportParameterDone = ko.pureComputed({
                read: function () {
                    return !_this._parameterComponentViewModel.IsImportParameterDone();
                }
            });
            _this._isFilterIconVisible = ko.pureComputed({
                read: function () {
                    return !_this._parameterComponentViewModel.IsFilterImportedValueActive();
                }
            });
            _this._importDesc = ko.observable();
            _this._exportDesc = ko.observable();
            _this._copyValueDesc = ko.observable();
            _this._infoTooltip = ko.observable();
            _this._filterImportedCurrentValuesDesc = ko.observable();
            _this._removeFilterImportedCurrentValuesDesc = ko.observable();
            _this._filterDesc = ko.pureComputed({
                read: function () {
                    if (_this._parameterComponentViewModel.IsFilterImportedValueActive()) {
                        return _this._removeFilterImportedCurrentValuesDesc();
                    }
                    else {
                        return _this._filterImportedCurrentValuesDesc();
                    }
                }
            });
            // Translate tooltip of toolbar.
            _this.translate();
            // Register listener
            _this.importParameterListener();
            return _this;
        }
        Object.defineProperty(ParameterToolbarComponentViewModel.prototype, "IsImportParameterDone", {
            get: function () { return this._isImportParameterDone; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterToolbarComponentViewModel.prototype, "IsFilterIconVisible", {
            get: function () { return this._isFilterIconVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterToolbarComponentViewModel.prototype, "ImportDesc", {
            get: function () { return this._importDesc; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterToolbarComponentViewModel.prototype, "ExportDesc", {
            get: function () { return this._exportDesc; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterToolbarComponentViewModel.prototype, "CopyValueDesc", {
            get: function () { return this._copyValueDesc; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterToolbarComponentViewModel.prototype, "FilterDesc", {
            get: function () { return this._filterDesc; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterToolbarComponentViewModel.prototype, "FilterImportedCurrentValuesDesc", {
            get: function () { return this._filterImportedCurrentValuesDesc; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterToolbarComponentViewModel.prototype, "RemoveFilterImportedCurrentValuesDesc", {
            get: function () { return this._removeFilterImportedCurrentValuesDesc; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ParameterToolbarComponentViewModel.prototype, "InfoTooltip", {
            get: function () { return this._infoTooltip; },
            enumerable: true,
            configurable: true
        });
        ParameterToolbarComponentViewModel.prototype.getInfo = function () {
            this._parameterComponentViewModel.getInfo();
        };
        ParameterToolbarComponentViewModel.prototype.setParams = function () {
            this._parameterComponentViewModel.setDevParams();
        };
        ParameterToolbarComponentViewModel.prototype.saveDevParams = function () {
            this._parameterComponentViewModel.saveDevParams();
        };
        ParameterToolbarComponentViewModel.prototype.filterCurrentNotEqualToImportedValues = function () {
            this._parameterComponentViewModel.filterCurrentNotEqualToImportedValues();
        };
        /**
         * Translate tooltip of toolbar.
         */
        ParameterToolbarComponentViewModel.prototype.translate = function () {
            _super.prototype.translate.call(this);
            // Init toolbar description
            var translator = translator_service_27.TranslatorService.getInstance().t;
            this._importDesc(translator(translator_service_27.TranslatorService.CfieldTranslationNs + "importRecipe"));
            this._exportDesc(translator(translator_service_27.TranslatorService.CfieldTranslationNs + "exportRecipe"));
            this._copyValueDesc(translator(translator_service_27.TranslatorService.CfieldTranslationNs + "overwriteCurrentValueWithImported"));
            this._filterImportedCurrentValuesDesc(translator(translator_service_27.TranslatorService.CfieldTranslationNs + "filterImportedCurrentValues"));
            this._removeFilterImportedCurrentValuesDesc(translator(translator_service_27.TranslatorService.CfieldTranslationNs + "removeFilter"));
            this._infoTooltip(translator(translator_service_27.TranslatorService.CfieldTranslationNs + "parameterViewInfo"));
        };
        ParameterToolbarComponentViewModel.prototype.importParameterListener = function () {
            $(ImportDevParamFormId)
                .on("click touchstart", function () {
                $(this).val("");
            })
                .change(function (event) {
                var files = event.target.files;
                if (validator_23.Validator.isDefined(files)) {
                    var file = files[0];
                    if (validator_23.Validator.isDefined(file)) {
                        this._parameterComponentViewModel.importParametersFromFile(file);
                    }
                    else {
                        this._error("ParameterToolbarComponentViewModel importParameterListener(): file is null");
                    }
                }
                else {
                    this._error("ParameterToolbarComponentViewModel importParameterListener(): files are null");
                }
            }.bind(this));
        };
        return ParameterToolbarComponentViewModel;
    }(base_toolbar_viewmodel_2.SaveToolbarViewModel));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                // This component depend on paramter-component. Get it
                var parameterComponentViewModel = cfield_service_17.cFieldService.getInstance().ParameterComponenetViewModelCached;
                if (!parameterComponentViewModel) {
                    throw "The cached parameterComponentViewModel is null";
                }
                return new ParameterToolbarComponentViewModel(parameterComponentViewModel);
            }
        },
        template: {
            require: "text!" + cfield_service_17.cFieldService.getInstance().BaseComponentViewUrl + "/parameter-toolbar.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/pgd1-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "component/viewModel/base-component.viewmodel", "common/validator", "pLan", "jquery-hammer"], function (require, exports, ko, cfield_service_18, base_component_viewmodel_7, validator_24) {
    "use strict";
    var Pgd1ComponentViewModel = /** @class */ (function (_super) {
        __extends(Pgd1ComponentViewModel, _super);
        function Pgd1ComponentViewModel(hideExtraToolbar) {
            var _this = _super.call(this, hideExtraToolbar, false, false, false) || this;
            _this._isOrientationChanged = false;
            // Display stuff
            _this._pixelPerDisplayPixelWidth = 4;
            _this._pixelPerDisplayPixelHeight = 4;
            _this._ledStatus = 0;
            _this._touch_mode = false;
            _this.doOnOrientationChange = function () {
                // Workaround: some mobile browser call this method before the component is rendered (then the size of div are too big). Wait before redraw
                // the canvas. TODO: improve
                setTimeout(function () {
                    this._isOrientationChanged = true;
                    switch (window.orientation) {
                        case -90:
                        case 90:
                            this.initCanvas();
                            this.fillDisplayWithTmpPattern();
                            break;
                        default:
                            this.initCanvas();
                            this.fillDisplayWithTmpPattern();
                            break;
                    }
                }.bind(_this), 100);
            };
            /*
             *   This function is called by the Protocol whenever leds status changes.
             */
            _this.refreshLedStatus = function (ledStatusMask) {
                var changedLed = (_this._ledStatus ^ ledStatusMask);
                if (changedLed !== 0) {
                    for (var i = 0; i < Pgd1ComponentViewModel.MaxLeds; i++) {
                        var ledMask = (1 << i);
                        if (changedLed & ledMask) {
                            switch (i) {
                                case Pgd1ComponentViewModel.AlarmLed:
                                    _this._alarmImagePath(_this.buildLedImageFilePath(Pgd1ComponentViewModel.AlarmLedName, !!(ledStatusMask & ledMask)));
                                    break;
                                case Pgd1ComponentViewModel.PrgLed:
                                    _this._prgImagePath(_this.buildLedImageFilePath(Pgd1ComponentViewModel.PrgLedName, !!(ledStatusMask & ledMask)));
                                    break;
                                case Pgd1ComponentViewModel.BacklightLed:
                                    /* backlight led has a NEGATED logic */
                                    var state = !(ledStatusMask & ledMask);
                                    _this._upImagePath(_this.buildLedImageFilePath(Pgd1ComponentViewModel.UpLedName, state));
                                    _this._enterImagePath(_this.buildLedImageFilePath(Pgd1ComponentViewModel.EnterLedName, state));
                                    _this._downImagePath(_this.buildLedImageFilePath(Pgd1ComponentViewModel.DownLedName, state));
                                    _this._escImagePath(_this.buildLedImageFilePath(Pgd1ComponentViewModel.EscLedName, state));
                                    break;
                            }
                        }
                    }
                    _this._ledStatus = ledStatusMask;
                }
            };
            _this.refreshDisplay = function (delta) {
                // Workaround: this method is called even if the component is disposed
                if (!_this._isRunning) {
                    return;
                }
                var displayBuffer = _this._pLanDisplay.getDisplayBuffer();
                var refreshArea = delta || new DirtyRegion(0, 0, displayBuffer.length - 1, _this._pLanDisplay.getDisplayWidthPixels() - 1);
                var start = new Date().getTime();
                var cv = document.getElementById("lcdCanvas");
                if (!validator_24.Validator.isDefined(cv)) {
                    return;
                }
                var ctx = cv.getContext("2d");
                if (!validator_24.Validator.isDefined(ctx)) {
                    return;
                }
                var startRow;
                var endRow;
                var startColIdx;
                var endColIdx;
                var p1;
                var p2;
                var p3;
                var p4;
                if (_this._isOrientationChanged) {
                    startRow = 0;
                    endRow = _this._pLanDisplay.getDisplayPhysicalRowsCount();
                    startColIdx = 0;
                    endColIdx = _this._pLanDisplay.getDisplayWidthPixels();
                    p1 = 0;
                    p2 = 0;
                    p3 = _this._pixelPerDisplayPixelWidth * endColIdx;
                    p4 = _this._pixelPerDisplayPixelHeight * _this._pLanDisplay.getDisplayHeightPixels();
                    _this._isOrientationChanged = false;
                }
                else {
                    startRow = refreshArea.getMinRowIdx();
                    endRow = refreshArea.getMaxRowIdx() + 1;
                    startColIdx = refreshArea.getMinColOffset();
                    endColIdx = refreshArea.getMaxColOffset() + 1;
                    p1 = startColIdx * _this._pixelPerDisplayPixelWidth;
                    p2 = (startRow * _this._pLanDisplay.getPixelsPerPhysicalRow()) * _this._pixelPerDisplayPixelHeight;
                    p3 = (endColIdx - startColIdx) * _this._pixelPerDisplayPixelWidth;
                    p4 = (endRow - startRow) * _this._pLanDisplay.getPixelsPerPhysicalRow() * _this._pixelPerDisplayPixelHeight;
                }
                ctx.fillStyle = Pgd1ComponentViewModel.PixelColorOff;
                ctx.fillRect(p1, p2, p3, p4);
                for (var i = startRow; i < endRow; i++) {
                    var row = displayBuffer[i];
                    for (var j = startColIdx; j < endColIdx; j++) {
                        var colData = row[j];
                        for (var k = 0; k < _this._pLanDisplay.getPixelsPerPhysicalRow(); k++) {
                            var mask = (1 << k);
                            var imgToDraw = void 0;
                            if ((colData & mask) !== 0) {
                                imgToDraw = _this._imagePixelOn;
                            }
                            else {
                                imgToDraw = _this._imagePixelOff;
                            }
                            ctx.drawImage(imgToDraw, j * _this._pixelPerDisplayPixelWidth, (i * _this._pLanDisplay.getPixelsPerPhysicalRow() + k) * _this._pixelPerDisplayPixelHeight);
                        }
                    }
                }
            };
            _this._isRunning = true;
            _this._isPgd1Init = ko.observable(false);
            _this._alarmImagePath = ko.observable(_this.buildLedImageFilePath(Pgd1ComponentViewModel.AlarmLedName, false));
            _this._upImagePath = ko.observable(_this.buildLedImageFilePath(Pgd1ComponentViewModel.UpLedName, false));
            _this._prgImagePath = ko.observable(_this.buildLedImageFilePath(Pgd1ComponentViewModel.PrgLedName, false));
            _this._enterImagePath = ko.observable(_this.buildLedImageFilePath(Pgd1ComponentViewModel.EnterLedName, false));
            _this._escImagePath = ko.observable(_this.buildLedImageFilePath(Pgd1ComponentViewModel.EscLedName, false));
            _this._downImagePath = ko.observable(_this.buildLedImageFilePath(Pgd1ComponentViewModel.DownLedName, false));
            _this._isAlarmButtonPressed = ko.observable(false);
            _this._isUpButtonPressed = ko.observable(false);
            _this._isPrgButtonPressed = ko.observable(false);
            _this._isEnterButtonPressed = ko.observable(false);
            _this._isEscButtonPressed = ko.observable(false);
            _this._isDownButtonPressed = ko.observable(false);
            _this._isShortLongButtonUp = ko.observable(true);
            _this._isShortButtonDown = ko.observable(false);
            _this._isLongButtonDown = ko.observable(false);
            // Evaluete the state of cField service
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "IsPgd1Init", {
            /**
             * Get the <code>_isPgd1Init</code> field.
             * @return the <code>_isPgd1Init</code> field.
             */
            get: function () {
                return this._isPgd1Init;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "AlarmImagePath", {
            get: function () {
                return this._alarmImagePath;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "IsAlarmButtonPressed", {
            get: function () {
                return this._isAlarmButtonPressed;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "UpImagePath", {
            get: function () {
                return this._upImagePath;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "IsUpButtonPressed", {
            get: function () {
                return this._isUpButtonPressed;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "PrgImagePath", {
            get: function () {
                return this._prgImagePath;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "IsPrgButtonPressed", {
            get: function () {
                return this._isPrgButtonPressed;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "EnterImagePath", {
            get: function () {
                return this._enterImagePath;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "IsEnterButtonPressed", {
            get: function () {
                return this._isEnterButtonPressed;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "EscImagePath", {
            get: function () {
                return this._escImagePath;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "IsEscButtonPressed", {
            get: function () {
                return this._isEscButtonPressed;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "DownImagePath", {
            get: function () {
                return this._downImagePath;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "IsDownButtonPressed", {
            get: function () {
                return this._isDownButtonPressed;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "IsShortLongButtonUp", {
            get: function () {
                return this._isShortLongButtonUp;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "IsShortButtonDown", {
            get: function () {
                return this._isShortButtonDown;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Pgd1ComponentViewModel.prototype, "IsLongButtonDown", {
            get: function () {
                return this._isLongButtonDown;
            },
            enumerable: true,
            configurable: true
        });
        Pgd1ComponentViewModel.prototype.dispose = function () {
            this._isRunning = false;
            if (this._pLanProtocol != null) {
                this._pLanProtocol.stopProtocol();
            }
        };
        /**
         * Init. Now <code>cField</code> service is initialized.
         */
        // override
        Pgd1ComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            _super.prototype.dispose.call(this);
            this._isPgd1Init(true);
            setTimeout(function () {
                this._pLanDisplay = new Display();
                this._pLanProtocol = new Protocol(this._pLanDisplay, {
                    refreshDisplay: this.refreshDisplay,
                    refreshLedStatus: this.refreshLedStatus,
                });
                this._multiKeyMode = Pgd1ComponentViewModel.MkModeNone;
                this.initMultiKeys();
                this._isReadOnly = false;
                this.initCanvas();
                this.fillDisplayWithTmpPattern();
                this._pLanProtocol.startProtocol();
                var _loop_3 = function (i) {
                    $("#" + Pgd1ComponentViewModel.ButtonDefault[i].name + "Button").hammer().bind("tap press", function (event) {
                        this.onPress(Pgd1ComponentViewModel.ButtonDefault[i].key, event.type);
                    }.bind(this_2));
                };
                var this_2 = this;
                // Register event listener
                for (var i = 0; i < Pgd1ComponentViewModel.ButtonDefault.length; i++) {
                    _loop_3(i);
                }
                // Orientation change callback
                window.addEventListener("orientationchange", this.doOnOrientationChange);
            }.bind(this), 5);
        };
        Pgd1ComponentViewModel.prototype.initCanvas = function () {
            var cv = document.getElementById("lcdCanvas");
            var ctx = cv.getContext("2d");
            this.findBestRatio();
            this._imagePixelOn = this.createDisplayPixelImage(Pgd1ComponentViewModel.PixelColorOn);
            this._imagePixelOff = this.createDisplayPixelImage(Pgd1ComponentViewModel.PixelColorOff);
        };
        Pgd1ComponentViewModel.prototype.findBestRatio = function () {
            var width = $("#lcdCanvasContainer").outerWidth();
            var ratio = width / this._pLanDisplay.getDisplayWidthPixels();
            var ratioInt = Math.floor(ratio);
            this._pixelPerDisplayPixelWidth = ratioInt;
            this._pixelPerDisplayPixelHeight = ratioInt;
            var cv = document.getElementById("lcdCanvas");
            var ctx = cv.getContext("2d");
            cv.width = this._pixelPerDisplayPixelWidth * this._pLanDisplay.getDisplayWidthPixels();
            cv.height = this._pixelPerDisplayPixelHeight * this._pLanDisplay.getDisplayHeightPixels();
        };
        Pgd1ComponentViewModel.prototype.createDisplayPixelImage = function (color) {
            var cv = document.createElement("canvas");
            cv.width = this._pixelPerDisplayPixelWidth;
            cv.height = this._pixelPerDisplayPixelHeight;
            var ctx = cv.getContext("2d");
            ctx.fillStyle = color;
            ctx.fillRect(0, 0, this._pixelPerDisplayPixelWidth, this._pixelPerDisplayPixelHeight);
            return cv;
        };
        Pgd1ComponentViewModel.prototype.fillDisplayWithTmpPattern = function () {
            var cv = document.getElementById("lcdCanvas");
            var ctx = cv.getContext("2d");
            for (var i = 0; i < this._pLanDisplay.getDisplayHeightPixels(); i++) {
                var comparison = i % 2;
                for (var j = 0; j < this._pLanDisplay.getDisplayWidthPixels(); j++) {
                    var imgToDraw;
                    if (j % 2 === comparison) {
                        imgToDraw = this._imagePixelOn;
                    }
                    else {
                        imgToDraw = this._imagePixelOff;
                    }
                    ctx.drawImage(imgToDraw, j * this._pixelPerDisplayPixelWidth, i * this._pixelPerDisplayPixelHeight);
                }
            }
        };
        Pgd1ComponentViewModel.prototype.initMultiKeys = function () {
            this._multiKeySelect = new Array(Pgd1ComponentViewModel.HardwareKeys.length);
        };
        Pgd1ComponentViewModel.prototype.onPress = function (key, eventType) {
            /* if in readonly mode do not process events... */
            if (this._isReadOnly) {
                return;
            }
            // Check if I am in a multi-click session
            if (this._multiKeyMode != Pgd1ComponentViewModel.MkModeNone) {
                // search current item, if selected deselect it, if not selected select it
                for (var i = 0; i < Pgd1ComponentViewModel.HardwareKeys.length; i++) {
                    if (Pgd1ComponentViewModel.HardwareKeys[i].value == key.value) {
                        if (this._multiKeySelect[i]) {
                            // was selected, deselect it
                            this._multiKeySelect[i] = false;
                            // Remove pressed button style
                            this.setButtonPressed(key.text, false);
                        }
                        else {
                            // was not selected, select it and check if a couple is selected
                            this._multiKeySelect[i] = true;
                            // Add pressed button style
                            this.setButtonPressed(key.text, true);
                        }
                        break;
                    }
                }
            }
            else {
                for (var i = 0; i < Pgd1ComponentViewModel.HardwareKeys.length; i++) {
                    if (Pgd1ComponentViewModel.HardwareKeys[i].value == key.value) {
                        this._multiKeySelect[i] = true;
                        // Add pressed button style
                        this.setButtonPressed(key.text, true);
                        setTimeout(function () {
                            // Check if there is at least two button pressed
                            var foundKeys = this.findKeys();
                            if (foundKeys[0] != null && foundKeys[1] != null) {
                                // Elaborate request (2 button)
                                if (eventType === "tap") {
                                    this._pLanProtocol.keyPressed(foundKeys);
                                    this._pLanProtocol.keyReleased();
                                }
                                else if (eventType === "press") {
                                    this._pLanProtocol.keyLongPressed(foundKeys);
                                    this._pLanProtocol.keyReleased();
                                }
                                this.clearMultikeyChoice();
                                this.resetOnPressState();
                            }
                            else if (foundKeys[0] != null || foundKeys[1] != null) {
                                // Elaborate request (1 button)
                                this._pLanProtocol.keyPressed([key.value]);
                                this._pLanProtocol.keyReleased();
                                this.clearMultikeyChoice();
                                //this.resetOnPressState();
                            }
                        }.bind(this), Pgd1ComponentViewModel.PressTimeoutMs);
                        break;
                    }
                }
            }
        };
        Pgd1ComponentViewModel.prototype.onRelease = function (name, context) {
            if (this._multiKeyMode != Pgd1ComponentViewModel.MkModeNone) {
                this.checkMultipleKeySelection();
                this.resetOnPressState();
            }
            else {
                this.setButtonPressed(name, false);
            }
        };
        Pgd1ComponentViewModel.prototype.findKeys = function () {
            // Check if there is at least two button pressed
            var key1Val = null;
            var key2Val = null;
            for (var i = 0; i < Pgd1ComponentViewModel.HardwareKeys.length; i++) {
                if (this._multiKeySelect[i]) {
                    if (key1Val == null) {
                        key1Val = Pgd1ComponentViewModel.HardwareKeys[i].value;
                    }
                    else {
                        key2Val = Pgd1ComponentViewModel.HardwareKeys[i].value;
                        // found 2, send'em now!
                        break;
                    }
                }
            }
            return [key1Val, key2Val];
        };
        Pgd1ComponentViewModel.prototype.setButtonPressed = function (name, value) {
            switch (name) {
                case Keys.KEY_ALARM.text:
                    this._isAlarmButtonPressed(value);
                    break;
                case Keys.KEY_UP.text:
                    this._isUpButtonPressed(value);
                    break;
                case Keys.KEY_PRG.text:
                    this._isPrgButtonPressed(value);
                    break;
                case Keys.KEY_ENTER.text:
                    this._isEnterButtonPressed(value);
                    break;
                case Keys.KEY_ESC.text:
                    this._isEscButtonPressed(value);
                    break;
                case Keys.KEY_DOWN.text:
                    this._isDownButtonPressed(value);
                    break;
            }
        };
        Pgd1ComponentViewModel.prototype.clearMultikeyChoice = function () {
            for (var i = 0; i < Pgd1ComponentViewModel.HardwareKeys.length; i++) {
                this._multiKeySelect[i] = false;
            }
            this._multiKeyMode = Pgd1ComponentViewModel.MkModeNone;
            this._isShortButtonDown(false);
            this._isLongButtonDown(false);
        };
        Pgd1ComponentViewModel.prototype.resetOnPressState = function () {
            this._isAlarmButtonPressed(false);
            this._isUpButtonPressed(false);
            this._isPrgButtonPressed(false);
            this._isEnterButtonPressed(false);
            this._isEscButtonPressed(false);
            this._isDownButtonPressed(false);
        };
        Pgd1ComponentViewModel.prototype.checkMultipleKeySelection = function () {
            // Check if there is at least two button pressed
            var foundKeys = this.findKeys();
            if (foundKeys[0] != null && foundKeys[1] != null) {
                switch (this._multiKeyMode) {
                    case Pgd1ComponentViewModel.MkModeShort:
                        this._pLanProtocol.keyPressed(foundKeys);
                        break;
                    case Pgd1ComponentViewModel.MkModeLong:
                        this._pLanProtocol.keyLongPressed(foundKeys);
                        break;
                }
                this._pLanProtocol.keyReleased();
                this.clearMultikeyChoice();
                this.resetOnPressState();
            }
        };
        Pgd1ComponentViewModel.prototype.onShortBtnClick = function () {
            /* if in readonly mode do not process events... */
            if (this._isReadOnly) {
                return;
            }
            switch (this._multiKeyMode) {
                case Pgd1ComponentViewModel.MkModeShort:
                    // exit multikey procedure
                    this.clearMultikeyChoice();
                    this.resetOnPressState();
                    break;
                case Pgd1ComponentViewModel.MkModeLong:
                    this._isLongButtonDown(false);
                case Pgd1ComponentViewModel.MkModeNone:
                    this._isShortButtonDown(true);
                    this._multiKeyMode = Pgd1ComponentViewModel.MkModeShort;
                    break;
            }
        };
        Pgd1ComponentViewModel.prototype.onLongBtnClick = function () {
            /* if in readonly mode do not process events... */
            if (this._isReadOnly) {
                return;
            }
            switch (this._multiKeyMode) {
                case Pgd1ComponentViewModel.MkModeLong:
                    // exit multikey procedure
                    this.clearMultikeyChoice();
                    this.resetOnPressState();
                    break;
                case Pgd1ComponentViewModel.MkModeShort:
                    this._isShortButtonDown(false);
                case Pgd1ComponentViewModel.MkModeNone:
                    this._isLongButtonDown(true);
                    this._multiKeyMode = Pgd1ComponentViewModel.MkModeLong;
                    break;
            }
        };
        Pgd1ComponentViewModel.prototype.buildLedImageFilePath = function (name, status) {
            return Pgd1ComponentViewModel.WebpgdImgsPath + "/" + name + "_" + (status ? "ON" : "OFF") + ".svg";
        };
        // Leds stuff
        Pgd1ComponentViewModel.MaxLeds = 24;
        Pgd1ComponentViewModel.AlarmLed = 12 - 1; //0-based
        Pgd1ComponentViewModel.PrgLed = 7 - 1; //0-based
        Pgd1ComponentViewModel.BacklightLed = 24 - 1; //0-based
        Pgd1ComponentViewModel.WebpgdImgsPath = "assets/img/pgd1";
        Pgd1ComponentViewModel.AlarmLedName = "alarm";
        Pgd1ComponentViewModel.PrgLedName = "prg";
        Pgd1ComponentViewModel.UpLedName = "up";
        Pgd1ComponentViewModel.EnterLedName = "enter";
        Pgd1ComponentViewModel.DownLedName = "down";
        Pgd1ComponentViewModel.EscLedName = "esc";
        Pgd1ComponentViewModel.PixelColorOn = "#000000";
        Pgd1ComponentViewModel.PixelColorOff = "#6ABCE7";
        Pgd1ComponentViewModel.MkModeNone = 0;
        Pgd1ComponentViewModel.MkModeShort = 1;
        Pgd1ComponentViewModel.MkModeLong = 2;
        Pgd1ComponentViewModel.HardwareKeys = [Keys.KEY_ALARM, Keys.KEY_PRG, Keys.KEY_ESC, Keys.KEY_UP, Keys.KEY_ENTER, Keys.KEY_DOWN];
        Pgd1ComponentViewModel.ButtonDefault = [
            { name: Pgd1ComponentViewModel.AlarmLedName, key: Keys.KEY_ALARM },
            { name: Pgd1ComponentViewModel.PrgLedName, key: Keys.KEY_PRG },
            { name: Pgd1ComponentViewModel.UpLedName, key: Keys.KEY_UP },
            { name: Pgd1ComponentViewModel.EnterLedName, key: Keys.KEY_ENTER },
            { name: Pgd1ComponentViewModel.DownLedName, key: Keys.KEY_DOWN },
            { name: Pgd1ComponentViewModel.EscLedName, key: Keys.KEY_ESC }
        ];
        Pgd1ComponentViewModel.PressTimeoutMs = 100;
        return Pgd1ComponentViewModel;
    }(base_component_viewmodel_7.BaseComponentViewModel));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var hideExtraToolbar = (params && params.hideExtraToolbar) || false;
                var newViewModel = new Pgd1ComponentViewModel(hideExtraToolbar);
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_18.cFieldService.getInstance().BaseComponentViewUrl + "/pgd1.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/serie-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service"], function (require, exports, ko, cfield_service_19) {
    "use strict";
    var SerieComponentViewModel = /** @class */ (function () {
        function SerieComponentViewModel(text) {
            this.Prova = ko.observable(text ? text.text : "");
        }
        return SerieComponentViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        // viewModel: SerieComponentViewModel, 
        viewModel: {
            createViewModel: function (params, componentInfo) {
                return new SerieComponentViewModel(params);
            }
        },
        template: {
            require: "text!" + cfield_service_19.cFieldService.getInstance().BaseComponentViewUrl + "/serie.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/item/system-item.viewmodel", ["require", "exports", "service/translator.service", "common/view", "common/loggable-entity", "common/validator"], function (require, exports, translator_service_28, view_5, loggable_entity_9, validator_25) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var TypeViewModel = /** @class */ (function () {
        function TypeViewModel(_translationKey) {
            this._translationKey = _translationKey;
            this.translate();
        }
        Object.defineProperty(TypeViewModel.prototype, "Type", {
            get: function () { return this._type; },
            enumerable: true,
            configurable: true
        });
        TypeViewModel.prototype.translate = function () {
            this._type = translator_service_28.TranslatorService.getInstance().t(translator_service_28.TranslatorService.CfieldTranslationNs + this._translationKey);
        };
        return TypeViewModel;
    }());
    var PairItemViewModel = /** @class */ (function (_super) {
        __extends(PairItemViewModel, _super);
        function PairItemViewModel(translationKey, _value) {
            var _this = _super.call(this, translationKey) || this;
            _this._value = _value;
            return _this;
        }
        Object.defineProperty(PairItemViewModel.prototype, "Value", {
            get: function () { return this._value; },
            enumerable: true,
            configurable: true
        });
        return PairItemViewModel;
    }(TypeViewModel));
    exports.PairItemViewModel = PairItemViewModel;
    var ScaledValueItemViewModel = /** @class */ (function (_super) {
        __extends(ScaledValueItemViewModel, _super);
        function ScaledValueItemViewModel(translationKey, value, scale, _suffixTranslationKey) {
            var _this = _super.call(this, translationKey) || this;
            _this._suffixTranslationKey = _suffixTranslationKey;
            _this._noAvailable = false;
            try {
                _this._value = view_5.DataFormatter.FloatToString(value / scale + "", 1);
                if (validator_25.Validator.isDefined(_suffixTranslationKey)) {
                    _this._value += " " + translator_service_28.TranslatorService.getInstance().t(translator_service_28.TranslatorService.CfieldTranslationNs + _this._suffixTranslationKey);
                    ;
                }
            }
            catch (e) {
                ScaledValueItemViewModel._logger._log("Invalid cast. value=" + value + "\n " + e);
            }
            return _this;
        }
        Object.defineProperty(ScaledValueItemViewModel.prototype, "NoAvailable", {
            get: function () { return this._noAvailable; },
            set: function (value) { this._noAvailable = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ScaledValueItemViewModel.prototype, "Value", {
            get: function () {
                if (validator_25.Validator.isDefined(this._noAvailable) && this._noAvailable === true) {
                    return "-";
                }
                return this._value;
            },
            enumerable: true,
            configurable: true
        });
        // Init common logger for class
        ScaledValueItemViewModel._logger = new loggable_entity_9.LoggableEntityStatic("ScaledValueItemViewModel");
        return ScaledValueItemViewModel;
    }(TypeViewModel));
    exports.ScaledValueItemViewModel = ScaledValueItemViewModel;
    var MemoryItemViewModel = /** @class */ (function (_super) {
        __extends(MemoryItemViewModel, _super);
        function MemoryItemViewModel(translationKey, value, free, scale, noAvailable) {
            var _this = _super.call(this, translationKey, value, scale) || this;
            try {
                _this._free = view_5.DataFormatter.FloatToString(free / scale + "", 1);
                if (validator_25.Validator.isDefined(noAvailable) && noAvailable === true) {
                    _this.NoAvailable = noAvailable;
                }
            }
            catch (e) {
                ScaledValueItemViewModel._logger._log("Invalid cast. free=" + free + "\n " + e);
            }
            return _this;
        }
        Object.defineProperty(MemoryItemViewModel.prototype, "Free", {
            get: function () {
                if (validator_25.Validator.isDefined(this.NoAvailable) && this.NoAvailable === true) {
                    return "-";
                }
                return this._free;
            },
            enumerable: true,
            configurable: true
        });
        return MemoryItemViewModel;
    }(ScaledValueItemViewModel));
    exports.MemoryItemViewModel = MemoryItemViewModel;
});
define("component/viewModel/system-network-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/device-config.service", "service/info.service", "service/translator.service", "component/viewModel/base-component.viewmodel", "component/viewModel/alert.viewmodel", "component/viewModel/item/system-item.viewmodel", "dataModel/network-info.model", "dataModel/system-info.model", "common/validator"], function (require, exports, ko, cfield_service_20, device_config_service_11, info_service_2, translator_service_29, base_component_viewmodel_8, alert_viewmodel_5, system_item_viewmodel_1, network_info_model_2, system_info_model_3, validator_26) {
    "use strict";
    // Query id
    var ModalSaveNetworkParamId = "#modalConfirmNetworkUpdate";
    var InfoCompomentSystemTableId = "#info-component-system-table";
    var InfoComponentMemoryTable = "#info-component-memory-table";
    var InfoComponentFilesystemTable = "#info-component-filesystem-table";
    var InfoComponentApplicationTableId = "#info-component-application-table";
    //const InfoComponentApplicationMemoryTableId = "#info-component-application-memory-table";
    var SystemNetworkComponentViewModel = /** @class */ (function (_super) {
        __extends(SystemNetworkComponentViewModel, _super);
        function SystemNetworkComponentViewModel(hideExtraToolbar) {
            var _this = _super.call(this, hideExtraToolbar, false, true, false) || this;
            // Data model
            _this._systemInfo = null;
            _this._networkInfo = null;
            _this._applicationInfo = null;
            // Viewmodels
            _this._memoryInfoViewModel = [];
            _this._fileSystemInfoFileSystemInfo = [];
            _this._applicationMemoryViewModel = [];
            _this._applicationCycleTimeViewModelArray = [];
            _this.buildSystemInfoTable = function (systemInfo) {
                // Prepare mac address
                var mac = [];
                if (validator_26.Validator.isDefined(systemInfo.MacAddress) && validator_26.Validator.isDefined(systemInfo.MacAddress.length)) {
                    for (var i = 0; i < systemInfo.MacAddress.length; i += 2) {
                        mac.push(systemInfo.MacAddress.substr(i, 2));
                        if (i + 2 < systemInfo.MacAddress.length) {
                            mac.push("-");
                        }
                    }
                }
                var formattedMac = mac.join("");
                var translator = translator_service_29.TranslatorService.getInstance().t;
                // Prepare plugins		
                var pluginsArray = [];
                if (validator_26.Validator.isDefined(systemInfo.HasBacnetServerClientLicense) && systemInfo.HasBacnetServerClientLicense) {
                    pluginsArray.push(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "bacnetClientServer"));
                }
                if (validator_26.Validator.isDefined(systemInfo.HasBacnetServerLicense) && systemInfo.HasBacnetServerLicense) {
                    pluginsArray.push(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "bacnetServer"));
                }
                if (validator_26.Validator.isDefined(systemInfo.HasSnmpLicense) && systemInfo.HasSnmpLicense) {
                    pluginsArray.push(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "snmp"));
                }
                if (validator_26.Validator.isDefined(systemInfo.HasNetLicense) && systemInfo.HasNetLicense) {
                    pluginsArray.push(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "net"));
                }
                if (validator_26.Validator.isDefined(systemInfo.HasDigitalSignature) && systemInfo.HasDigitalSignature) {
                    pluginsArray.push(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "digitalSignature"));
                }
                var plugins;
                if (pluginsArray.length > 0) {
                    plugins = pluginsArray.join(", ");
                }
                else {
                    plugins = pluginsArray[0];
                }
                var systemInfoViewModel = [];
                systemInfoViewModel.push(new SystemInfoItemViewModel(_this.SystemBootTitle, systemInfo.BootVersion, false));
                systemInfoViewModel.push(new SystemInfoItemViewModel(_this.SystemOsTitle, systemInfo.OsVersion, false));
                systemInfoViewModel.push(new SystemInfoItemViewModel(_this.SystemCoreTypeTitle, systemInfo.CoreType, false));
                systemInfoViewModel.push(new SystemInfoItemViewModel(_this.SystemPcoTypeTitle, system_info_model_3.SystemInfoUtil.GetPcoTypeDescription(systemInfo.PcoType), false));
                systemInfoViewModel.push(new SystemInfoItemViewModel(_this.SystemBoardTypeTitle, system_info_model_3.SystemInfoUtil.GetBoardTypeDescription(systemInfo.PcoType, systemInfo.BoardType), false));
                systemInfoViewModel.push(new SystemInfoItemViewModel(_this.SystemMacTitle, validator_26.Validator.isDefined(formattedMac) ? formattedMac : systemInfo.MacAddress, false));
                systemInfoViewModel.push(new SystemInfoItemViewModel(_this.SystemUidTitle, systemInfo.Uid, false));
                systemInfoViewModel.push(new SystemInfoItemViewModel(_this.SystemTeraTitle, systemInfo.Tera, false));
                systemInfoViewModel.push(new SystemInfoItemViewModel(_this.SystemPluginsTitle, plugins, false));
                // TODO: two alternative: 1) write all records of table into html OR 2) build table dinamically
                // let vm = new SystemInfoItemViewModel(this.ApplicationTitle(), "", true, "<system-network-detail-app-component></system-network-detail-app-component>");
                // cFieldService.getInstance().registerTestComponent("system-network-detail-app-component", vm, "test.html")
                // systemInfoViewModel.push(vm);
                $(function () {
                    $(InfoCompomentSystemTableId).bootstrapTable({
                        data: systemInfoViewModel,
                    });
                }.bind(_this));
            };
            _this.buildMemoryInfoTable = function (memoryInfo) {
                _this._fileSystemInfoFileSystemInfo = [
                    new system_item_viewmodel_1.MemoryItemViewModel("ram", memoryInfo.Size, memoryInfo.Free, 1024 * 1024)
                ];
                $(function () {
                    $(InfoComponentMemoryTable).bootstrapTable({
                        data: this._fileSystemInfoFileSystemInfo,
                        locale: translator_service_29.TranslatorService.getInstance().CurrentLanguage()
                    });
                }.bind(_this));
            };
            _this.buildFileSystemInfoTable = function (fileSystemInfoFileSystemInfo) {
                var isMsd0Available = validator_26.Validator.isDefined(fileSystemInfoFileSystemInfo.Msd0Info.Size)
                    && fileSystemInfoFileSystemInfo.Msd0Info.Size != 0;
                var isMmc0Available = validator_26.Validator.isDefined(fileSystemInfoFileSystemInfo.Mcc0Info.Size)
                    && fileSystemInfoFileSystemInfo.Mcc0Info.Size != 0;
                _this._memoryInfoViewModel = [
                    new system_item_viewmodel_1.MemoryItemViewModel("nand0", fileSystemInfoFileSystemInfo.Nand0Info.Size, fileSystemInfoFileSystemInfo.Nand0Info.Free, 1024),
                    new system_item_viewmodel_1.MemoryItemViewModel("nand1", fileSystemInfoFileSystemInfo.Nand1Info.Size, fileSystemInfoFileSystemInfo.Nand1Info.Free, 1024),
                    new system_item_viewmodel_1.MemoryItemViewModel("msd0", fileSystemInfoFileSystemInfo.Msd0Info.Size, fileSystemInfoFileSystemInfo.Msd0Info.Free, 1024, !isMsd0Available),
                    new system_item_viewmodel_1.MemoryItemViewModel("mmc0", fileSystemInfoFileSystemInfo.Mcc0Info.Size, fileSystemInfoFileSystemInfo.Mcc0Info.Free, 1024, !isMmc0Available)
                ];
                $(function () {
                    $(InfoComponentFilesystemTable).bootstrapTable({
                        data: this._memoryInfoViewModel,
                        locale: translator_service_29.TranslatorService.getInstance().CurrentLanguage()
                    });
                }.bind(_this));
            };
            _this.buildApplicationInfoTable = function (applicationInfo, applicationCycleInfo) {
                var applicationInfoViewModel = [
                    new system_item_viewmodel_1.PairItemViewModel("name", applicationInfo.SoftwareName),
                    new system_item_viewmodel_1.PairItemViewModel("version", applicationInfo.SoftwareVersion),
                    new system_item_viewmodel_1.PairItemViewModel("configuration", applicationInfo.ConfigurationName),
                    new system_item_viewmodel_1.ScaledValueItemViewModel("currentCycleTime", applicationCycleInfo.Current, 1, "ms"),
                    new system_item_viewmodel_1.ScaledValueItemViewModel("maxCycleTime", applicationCycleInfo.Max, 1, "ms")
                ];
                $(function () {
                    $(InfoComponentApplicationTableId).bootstrapTable({
                        data: applicationInfoViewModel
                    });
                }.bind(_this));
            };
            _this._isSystemNetworkApplicationInit = ko.observable(false);
            // Labels
            _this.SystemTitle = ko.observable();
            // Network labels
            _this.NetworkTitle = ko.observable();
            _this.NetworkSaveTitle = ko.observable();
            _this._networkIpOffTitle = ko.observable();
            _this._networkIpManualTitle = ko.observable();
            _this._networkHostnameTitle = ko.observable();
            _this._networkInvalidHostnameMsg = ko.observable();
            _this._networkMdnsTitle = ko.observable();
            _this._networkInvalidAddressMsg = ko.observable();
            _this._networkIpv4IpLabelTitle = ko.observable();
            _this._networkIpv4AutoTitle = ko.observable();
            _this._networkIpv4IpTitle = ko.observable();
            _this._networkIpv4MaskTitle = ko.observable();
            _this._networkIpGatewayTitle = ko.observable();
            _this._networkIpDnsTitle = ko.observable();
            _this.CanShowIpv6Section = ko.observable(false);
            _this._networkIpv6SlaacTitle = ko.observable();
            _this._networkIpv6IpLabelTitle = ko.observable();
            _this._networkIpv6LinkLocalTitle = ko.observable();
            _this._networkIpv6IpTitle = ko.observable();
            _this.ApplicationTitle = ko.observable();
            _this.MemoryTitle = ko.observable();
            _this.TypeTitle = ko.observable();
            _this.FreeMemoryTitle = ko.observable();
            _this.UsedMemoryTitle = ko.observable();
            _this.SizeMemoryTitle = ko.observable();
            _this.MBTitle = ko.observable();
            _this.CycleTimeTitle = ko.observable();
            _this.MsTitle = ko.observable();
            // Values
            _this.NetworkHostnameValue = ko.observable();
            _this.NetworkIsMdnsActiveValue = ko.observable();
            _this.NetworkIpv4IpValue = ko.observable();
            _this.NetworkIpv4MaskValue = ko.observable();
            _this.NetworkIpv4GatewayValue = ko.observable();
            _this.NetworkIpv4DnsValue = ko.observable();
            _this._networkIpv4Modes = ko.observableArray([]);
            _this.NetworkIpv4SelectedMode = ko.observable();
            _this.NetworkIpv4IsManualViewExpanded = ko.pureComputed({
                read: function () {
                    var value = _this.NetworkIpv4SelectedMode();
                    return validator_26.Validator.isDefined(value) ? (value.Mode === Ipv4Mode.Manual || value.Mode === Ipv4Mode.Auto) : false;
                }
            });
            _this._networkIpv4CanEditValue = ko.pureComputed({
                read: function () {
                    var value = _this.NetworkIpv4SelectedMode();
                    return validator_26.Validator.isDefined(value) ? value.Mode === Ipv4Mode.Manual : false;
                }
            });
            _this._networkIpv6LinkLocalValue = ko.observable();
            _this.NetworkIpv6IpValue = ko.observable();
            _this.NetworkIpv6GatewayValue = ko.observable();
            _this.NetworkIpv6DnsValue = ko.observable();
            _this._networkIpv6Modes = ko.observableArray([]);
            _this.NetworkIpv6SelectedMode = ko.observable();
            _this.NetworkIpv6IsManualViewExpanded = ko.pureComputed({
                read: function () {
                    var value = _this.NetworkIpv6SelectedMode();
                    return validator_26.Validator.isDefined(value) ? (value.Mode === Ipv6Mode.Manual || value.Mode === Ipv6Mode.Slaac) : false;
                }
            });
            _this._networkIpv6CanEditValue = ko.pureComputed({
                read: function () {
                    var value = _this.NetworkIpv6SelectedMode();
                    return validator_26.Validator.isDefined(value) ? value.Mode === Ipv6Mode.Manual : false;
                }
            });
            // Validation
            _this.IsHostnameValid = ko.pureComputed({
                read: function () {
                    var value = _this.NetworkHostnameValue();
                    if (validator_26.Validator.isDefined(value)) {
                        if (value !== "") {
                            return validator_26.Validator.ValidateHostname(value);
                        }
                    }
                    return true;
                }
            });
            _this.NetworkHostnameTooltip = ko.pureComputed({
                read: function () {
                    return _this.IsHostnameValid() ?
                        "" : _this.NetworkInvalidHostnameMsg();
                }
            });
            _this.NetworkIpv4IsInvalidIp = ko.observable();
            _this._registeredObserverArray.push(_this.NetworkIpv4IpValue.subscribe(function (newValue) {
                this.NetworkIpv4IsInvalidIp(!(validator_26.Validator.isDefined(newValue) && validator_26.Validator.ValidateIpv4(newValue)));
            }.bind(_this)));
            _this.NetworkIpv4IpTooltip = ko.pureComputed({
                read: function () {
                    return _this.NetworkIpv4IsInvalidIp() ?
                        _this.NetworkInvalidAddressMsg() : validator_26.Validator.Ipv4ToString(_this.NetworkIpv4IpValue());
                }
            });
            _this.NetworkIpv4IsInvalidMask = ko.observable();
            _this._registeredObserverArray.push(_this.NetworkIpv4MaskValue.subscribe(function (newValue) {
                this.NetworkIpv4IsInvalidMask(!(validator_26.Validator.isDefined(newValue) && validator_26.Validator.ValidateIpv4(newValue)));
            }.bind(_this)));
            _this.NetworkIpv4MaskTooltip = ko.pureComputed({
                read: function () {
                    return _this.NetworkIpv4IsInvalidMask() ?
                        _this.NetworkInvalidAddressMsg() : validator_26.Validator.Ipv4ToString(_this.NetworkIpv4MaskValue());
                }
            });
            _this.NetworkIpv4IsInvalidGateway = ko.observable();
            _this._registeredObserverArray.push(_this.NetworkIpv4GatewayValue.subscribe(function (newValue) {
                this.NetworkIpv4IsInvalidGateway(!(validator_26.Validator.isDefined(newValue) && validator_26.Validator.ValidateIpv4(newValue)));
            }.bind(_this)));
            _this.NetworkIpv4GatewayTooltip = ko.pureComputed({
                read: function () {
                    return _this.NetworkIpv4IsInvalidGateway() ?
                        _this.NetworkInvalidAddressMsg() : validator_26.Validator.Ipv4ToString(_this.NetworkIpv4GatewayValue());
                }
            });
            _this.NetworkIpv4IsInvalidDns = ko.observable();
            _this._registeredObserverArray.push(_this.NetworkIpv4DnsValue.subscribe(function (newValue) {
                this.NetworkIpv4IsInvalidDns(!(validator_26.Validator.isDefined(newValue) && validator_26.Validator.ValidateIpv4(newValue)));
            }.bind(_this)));
            _this.NetworkIpv4DnsTooltip = ko.pureComputed({
                read: function () {
                    return _this.NetworkIpv4IsInvalidDns() ?
                        _this.NetworkInvalidAddressMsg() : validator_26.Validator.Ipv4ToString(_this.NetworkIpv4DnsValue());
                }
            });
            _this.NetworkIpv6IsInvalidIp = ko.observable();
            _this._registeredObserverArray.push(_this.NetworkIpv6IpValue.subscribe(function (newValue) {
                this.NetworkIpv6IsInvalidIp(!(validator_26.Validator.isDefined(this.NetworkIpv6IpValue())
                    && validator_26.Validator.ValidateIpv6(this.NetworkIpv6IpValue())));
            }.bind(_this)));
            _this.NetworkIpv6IpTooltip = ko.pureComputed({
                read: function () {
                    return _this.NetworkIpv6IsInvalidIp() ?
                        _this.NetworkInvalidAddressMsg() : validator_26.Validator.Ipv6ToString(_this.NetworkIpv6IpValue());
                }
            });
            _this.NetworkIpv6IsInvalidGateway = ko.observable();
            _this._registeredObserverArray.push(_this.NetworkIpv6GatewayValue.subscribe(function (newValue) {
                this.NetworkIpv6IsInvalidGateway(!(validator_26.Validator.isDefined(newValue) && validator_26.Validator.ValidateIpv6(newValue)));
            }.bind(_this)));
            _this.NetworkIpv6GatewayTooltip = ko.pureComputed({
                read: function () {
                    return _this.NetworkIpv6IsInvalidGateway() ?
                        _this.NetworkInvalidAddressMsg() : validator_26.Validator.Ipv6ToString(_this.NetworkIpv6GatewayValue());
                }
            });
            _this.NetworkIpv6IsInvalidDns = ko.observable();
            _this._registeredObserverArray.push(_this.NetworkIpv6DnsValue.subscribe(function (newValue) {
                this.NetworkIpv6IsInvalidDns(!(validator_26.Validator.isDefined(newValue) && validator_26.Validator.ValidateIpv6(newValue)));
            }.bind(_this)));
            _this.NetworkIpv6DnsTooltip = ko.pureComputed({
                read: function () {
                    return _this.NetworkIpv6IsInvalidDns() ?
                        _this.NetworkInvalidAddressMsg() : validator_26.Validator.Ipv6ToString(_this.NetworkIpv6DnsValue());
                }
            });
            _this.ModalTitle = ko.observable();
            _this.ModalYesTitle = ko.observable();
            _this.ModalNoTitle = ko.observable();
            _this.ModalCloseTitle = ko.observable();
            _this._systemAlertViewModel = new alert_viewmodel_5.SystemAlertViewModel();
            _this.IsModalConfirmSavingEnabled = ko.observable(true);
            // Evaluete the state of cField service
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "IsSystemNetworkApplicationInit", {
            /**
             * Get the <code>_isSystemNetworkInit</code> field.
             * @return the <code>_isSystemNetworkInit</code> field.
             */
            get: function () {
                return this._isSystemNetworkApplicationInit;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkHostnameTitle", {
            get: function () { return this._networkHostnameTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkMdnsTitle", {
            get: function () { return this._networkMdnsTitle; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkInvalidHostnameMsg", {
            get: function () { return this._networkInvalidHostnameMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpGatewayTitle", {
            get: function () { return this._networkIpGatewayTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpDnsTitle", {
            get: function () { return this._networkIpDnsTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkInvalidAddressMsg", {
            get: function () { return this._networkInvalidAddressMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpv4IpLabelTitle", {
            get: function () { return this._networkIpv4IpLabelTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpv4IpTitle", {
            get: function () { return this._networkIpv4IpTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpv4MaskTitle", {
            get: function () { return this._networkIpv4MaskTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpv6IpLabelTitle", {
            get: function () { return this._networkIpv6IpLabelTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpv6LinkLocalTitle", {
            get: function () { return this._networkIpv6LinkLocalTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpv6IpTitle", {
            get: function () { return this._networkIpv6IpTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpv4Modes", {
            get: function () { return this._networkIpv4Modes; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpv4CanEditValue", {
            get: function () { return this._networkIpv4CanEditValue; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpv6LinkLocalValue", {
            get: function () { return this._networkIpv6LinkLocalValue; },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpv6Modes", {
            get: function () { return this._networkIpv6Modes; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "NetworkIpv6CanEditValue", {
            get: function () { return this._networkIpv6CanEditValue; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "IsAlertVisible", {
            get: function () { return this._systemAlertViewModel.IsAlertVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "AlertCssClass", {
            get: function () { return this._systemAlertViewModel.AlertCssClass; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "AlertTitle", {
            get: function () { return this._systemAlertViewModel.AlertTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "AlertIsErrorDetailVisible", {
            get: function () { return this._systemAlertViewModel.AlertIsErrorDetailVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemNetworkComponentViewModel.prototype, "AlertErrorDetailMsg", {
            get: function () { return this._systemAlertViewModel.AlertErrorDetailMsg; },
            enumerable: true,
            configurable: true
        });
        SystemNetworkComponentViewModel.prototype.closeAlert = function () {
            this._systemAlertViewModel.State(alert_viewmodel_5.SystemAlertState.Invisible);
        };
        // override
        SystemNetworkComponentViewModel.prototype.onChangeCFieldState = function (newState) {
            _super.prototype.onChangeCFieldState.call(this, newState);
            // Manage ad-hoc state
            switch (newState) {
                case cfield_service_20.CfieldServiceState.RefreshConfig:
                    // Clear state 
                    this._memoryInfoViewModel = [];
                    this._fileSystemInfoFileSystemInfo = [];
                    this._applicationMemoryViewModel = [];
                    this._applicationCycleTimeViewModelArray = [];
                    this._systemAlertViewModel.State(alert_viewmodel_5.SystemAlertState.Invisible);
                    this._isSystemNetworkApplicationInit(false);
                    //  Destroy view 
                    $(InfoCompomentSystemTableId).bootstrapTable("destroy");
                    $(InfoComponentMemoryTable).bootstrapTable("destroy");
                    $(InfoComponentFilesystemTable).bootstrapTable("destroy");
                    $(InfoComponentApplicationTableId).bootstrapTable("destroy");
                    //(<any>$(InfoComponentApplicationMemoryTableId)).bootstrapTable("destroy");
                    break;
            }
        };
        SystemNetworkComponentViewModel.prototype.onChangeLanguage = function (newState) {
            this.translate();
            var translatorService = translator_service_29.TranslatorService.getInstance();
            var translator = translatorService.t;
            // Destroy
            $(InfoCompomentSystemTableId).bootstrapTable("destroy");
            $(InfoComponentApplicationTableId).bootstrapTable("destroy");
            // Rebuild table
            this.buildSystemInfoTable(this._systemInfo);
            this.buildApplicationInfoTable(this._applicationInfo, this._systemInfo.AppInfo.AppCycleTymeInfo);
            // For the table with the header, I refresh the header and the viewmodel
            var table = $(InfoComponentMemoryTable);
            table.bootstrapTable("changeLocale", translatorService.CurrentLanguage());
            table.bootstrapTable("changeTitle", {
                Type: this.TypeTitle(),
                Value: this.SizeMemoryTitle(),
                Free: this.FreeMemoryTitle(),
                _callbackBeforeUpdateBodyOnRefreshTranslation: function () {
                    // Refresh view models
                    for (var _i = 0, _a = this._memoryInfoViewModel; _i < _a.length; _i++) {
                        var item = _a[_i];
                        item.translate();
                    }
                }.bind(this)
            });
            table = $(InfoComponentFilesystemTable);
            table.bootstrapTable("changeLocale", translatorService.CurrentLanguage());
            table.bootstrapTable("changeTitle", {
                Type: this.TypeTitle(),
                Value: this.SizeMemoryTitle(),
                Free: this.FreeMemoryTitle(),
                _callbackBeforeUpdateBodyOnRefreshTranslation: function () {
                    // Refresh view models
                    for (var _i = 0, _a = this._fileSystemInfoFileSystemInfo; _i < _a.length; _i++) {
                        var item = _a[_i];
                        item.translate();
                    }
                }.bind(this)
            });
            // table = (<any>$(InfoComponentApplicationMemoryTableId));
            // table.bootstrapTable("changeLocale", translatorService.CurrentLanguage());
            // table.bootstrapTable("changeTitle", {
            // 	Type: this.TypeTitle(),
            // 	Value: this.UsedMemoryTitle(),
            // 	_callbackBeforeUpdateBodyOnRefreshTranslation: function() { 
            // 		// Refresh view models
            // 		for (let item of this._applicationMemoryViewModel) {
            // 			item.translate();
            // 		}
            // 	}.bind(this)
            // });
        };
        /**
         * Init alarms view model. Now <code>cField</code> service is initialized.
         */
        SystemNetworkComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            var _this = this;
            // Init the header of the table with the translation
            this.translate();
            var infoService = info_service_2.InfoService.getInstance();
            var systemInfoPromise = infoService.getSystemInfo();
            systemInfoPromise
                .then(function (systemInfo) {
                _this._systemInfo = systemInfo;
                _this.buildSystemInfoTable(systemInfo);
            })
                .then(function () {
                return infoService.getNetworkInfo();
            })
                .then(function (networkInfo) {
                _this._networkInfo = networkInfo;
                // Build viewmodel
                _this.buildNetworkViewModel(networkInfo);
                // Build table
                _this.buildMemoryInfoTable(_this._systemInfo.MemoryInfo);
                _this.buildFileSystemInfoTable(_this._systemInfo.FileSystemInfo);
                _this._applicationInfo = device_config_service_11.DeviceConfigService.getInstance().ApplicationInfo;
                _this.buildApplicationInfoTable(_this._applicationInfo, _this._systemInfo.AppInfo.AppCycleTymeInfo);
                //this.buildApplicationMemoryTable(this._systemInfo.AppInfo.AppMemoryInfo);
                // Now view is initialized
                _this._isSystemNetworkApplicationInit(true);
                // Init bootstrap-select
                $(".selectpicker").selectpicker();
            })
                .catch(function (reason) {
                _this.showError(reason, alert_viewmodel_5.SystemAlertState.BootFailed);
            });
        };
        SystemNetworkComponentViewModel.prototype.buildNetworkViewModel = function (networkInfo) {
            if (validator_26.Validator.isDefined(networkInfo)) {
                this.NetworkHostnameValue(("" + networkInfo.Hostname).slice(0));
                this.NetworkIsMdnsActiveValue(networkInfo.IsMdnsActive === true);
                this.NetworkIpv4IpValue(("" + networkInfo.Ipv4Ip).slice(0));
                this.NetworkIpv4MaskValue(("" + networkInfo.Ipv4Mask).slice(0));
                this.NetworkIpv4GatewayValue(("" + networkInfo.Ipv4Gateway).slice(0));
                this.NetworkIpv4DnsValue(("" + networkInfo.Ipv4Dns).slice(0));
                // Init select
                this._networkIpv4Modes.push(new Ipv4ModeViewModel(Ipv4Mode.Auto, this._networkIpv4AutoTitle()));
                this._networkIpv4Modes.push(new Ipv4ModeViewModel(Ipv4Mode.Manual, this._networkIpManualTitle()));
                this._networkIpv4Modes.push(new Ipv4ModeViewModel(Ipv4Mode.Off, this._networkIpOffTitle()));
                // Manage selection
                if (!networkInfo.Ipv4IsEnable) {
                    this.NetworkIpv4SelectedMode(this._networkIpv4Modes()[2]);
                }
                else if (networkInfo.Ipv4IsDhcpActive) {
                    this.NetworkIpv4SelectedMode(this._networkIpv4Modes()[0]);
                }
                else {
                    this.NetworkIpv4SelectedMode(this._networkIpv4Modes()[1]);
                }
                if (validator_26.Validator.isDefined(this._systemInfo.HasNetLicense) && this._systemInfo.HasNetLicense) {
                    this.CanShowIpv6Section(true);
                    this._networkIpv6LinkLocalValue(("" + networkInfo.Ipv6LinkLocal).slice(0));
                    this.NetworkIpv6IpValue(("" + networkInfo.Ipv6Ip).slice(0));
                    this.NetworkIpv6GatewayValue(("" + networkInfo.Ipv6Gateway).slice(0));
                    this.NetworkIpv6DnsValue(("" + networkInfo.Ipv6Dns).slice(0));
                    // Init select
                    this._networkIpv6Modes.push(new Ipv6ModeViewModel(Ipv6Mode.Slaac, this._networkIpv6SlaacTitle()));
                    this._networkIpv6Modes.push(new Ipv6ModeViewModel(Ipv6Mode.Manual, this._networkIpManualTitle()));
                    this._networkIpv6Modes.push(new Ipv6ModeViewModel(Ipv6Mode.Off, this._networkIpOffTitle()));
                    // Manage selection
                    if (!networkInfo.Ipv6IsEnable) {
                        this.NetworkIpv6SelectedMode(this._networkIpv6Modes()[2]);
                    }
                    else if (networkInfo.Ipv6IsDhcpActive) {
                        this.NetworkIpv6SelectedMode(this._networkIpv6Modes()[0]);
                    }
                    else {
                        this.NetworkIpv6SelectedMode(this._networkIpv6Modes()[1]);
                    }
                }
                else {
                    this.CanShowIpv6Section(false);
                }
            }
        };
        // override
        // state must be not mandatory to due the ovveride construct
        SystemNetworkComponentViewModel.prototype.showError = function (reason, state) {
            this._log(reason);
            this._systemAlertViewModel.AlertErrorDetailMsg(reason);
            this._systemAlertViewModel.State(state);
        };
        // private buildApplicationMemoryTable = (applicationMemoryInfo: AppMemoryInfo) => {
        // 	this._applicationMemoryViewModel = [
        // 		new ScaledValueItemViewModel("ram", applicationMemoryInfo.Ram, 1),
        // 		new ScaledValueItemViewModel("rom", applicationMemoryInfo.Rom, 1),
        // 		new ScaledValueItemViewModel("code", applicationMemoryInfo.Code, 1),
        // 		new ScaledValueItemViewModel("data", applicationMemoryInfo.Data, 1),
        // 		new ScaledValueItemViewModel("virtualMachine", applicationMemoryInfo.Vm, 1),
        // 		new ScaledValueItemViewModel("chng", applicationMemoryInfo.Chng, 1),
        // 		new ScaledValueItemViewModel("conf", applicationMemoryInfo.Conf, 1),
        // 		new ScaledValueItemViewModel("prot", applicationMemoryInfo.Prot, 1)
        // 	];
        // 	$(function() {
        // 		(<any>$(InfoComponentApplicationMemoryTableId)).bootstrapTable({
        // 			data: this._applicationMemoryViewModel,
        // 			locale: TranslatorService.getInstance().CurrentLanguage()
        // 		});
        // 	}.bind(this));
        // }
        SystemNetworkComponentViewModel.prototype.saveNetworkCommonParam = function () {
            var translator = translator_service_29.TranslatorService.getInstance().t;
            // Validate
            var hostname = this.NetworkHostnameValue();
            if (validator_26.Validator.isDefined(hostname) && hostname.length <= 32 && validator_26.Validator.isDefined(this.NetworkIsMdnsActiveValue())) {
                this.ModalTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "confirmNetworkSaving"));
                this.ModalYesTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "yes"));
                this.ModalNoTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "no"));
                this._networkSavingType = NetworkSavingType.Common;
                this.IsModalConfirmSavingEnabled(true);
            }
            else {
                this.ModalTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "invalidParameters"));
                this.ModalCloseTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "close"));
                this.IsModalConfirmSavingEnabled(false);
                this._log("SystemNetworkComponentViewModel saveNetworkCommonParam(): hostname=" + hostname + ", mdns=" + this.NetworkIsMdnsActiveValue());
            }
            // Show modal
            $(ModalSaveNetworkParamId).modal("show");
        };
        SystemNetworkComponentViewModel.prototype.saveNetworkIpv4Param = function () {
            var translator = translator_service_29.TranslatorService.getInstance().t;
            // Validate
            if (this.NetworkIpv4SelectedMode().Mode === Ipv4Mode.Off
                // ipv4 auto
                || this.NetworkIpv4SelectedMode().Mode === Ipv4Mode.Auto
                // ipv4 manual with valid ip
                || this.NetworkIpv4IsInvalidIp() === false && this.NetworkIpv4IsInvalidMask() === false
                    && this.NetworkIpv4IsInvalidGateway() === false && this.NetworkIpv4IsInvalidDns() === false) {
                this.ModalTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "confirmNetworkSaving"));
                this.ModalYesTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "yes"));
                this.ModalNoTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "no"));
                this._networkSavingType = NetworkSavingType.Ipv4;
                this.IsModalConfirmSavingEnabled(true);
            }
            else {
                this.ModalTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "invalidParameters"));
                this.ModalCloseTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "close"));
                this.IsModalConfirmSavingEnabled(false);
                this._log("SystemNetworkComponentViewModel saveNetworkIpv4Param(): ip=" + this.NetworkIpv4IpValue() +
                    ", mask=" + this.NetworkIpv4MaskValue() +
                    ", gateway=" + this.NetworkIpv4GatewayValue() +
                    ", dns=" + this.NetworkIpv4DnsValue());
            }
            // Show modal
            $(ModalSaveNetworkParamId).modal("show");
        };
        SystemNetworkComponentViewModel.prototype.saveNetworkIpv6Param = function () {
            var translator = translator_service_29.TranslatorService.getInstance().t;
            // Validate
            if (this.NetworkIpv6SelectedMode().Mode === Ipv6Mode.Off
                // ipv6 slaac
                || this.NetworkIpv6SelectedMode().Mode === Ipv6Mode.Slaac
                // ipv6 manual with valid ip
                || this.NetworkIpv6IsInvalidIp() === false && this.NetworkIpv6IsInvalidGateway() === false
                    && this.NetworkIpv6IsInvalidDns() === false) {
                this.ModalTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "confirmNetworkSaving"));
                this.ModalYesTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "yes"));
                this.ModalNoTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "no"));
                this._networkSavingType = NetworkSavingType.Ipv6;
                this.IsModalConfirmSavingEnabled(true);
            }
            else {
                this.ModalTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "invalidParameters"));
                this.ModalCloseTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "close"));
                this.IsModalConfirmSavingEnabled(false);
                this._log("SystemNetworkComponentViewModel saveNetworkIpv6Param(): ip=" + this.NetworkIpv6IpValue() +
                    ", gateway=" + this.NetworkIpv6GatewayValue() +
                    ", dns=" + this.NetworkIpv6DnsValue());
            }
            // Show modal
            $(ModalSaveNetworkParamId).modal("show");
        };
        SystemNetworkComponentViewModel.prototype.updateNetworkParams = function () {
            switch (this._networkSavingType) {
                case NetworkSavingType.Common:
                    info_service_2.InfoService.getInstance().setNetworkCommonParam(this.getNetworkCommonParamToUpdate());
                    break;
                case NetworkSavingType.Ipv4:
                    info_service_2.InfoService.getInstance().setNetworkIpv4Param(this.getNetworkIpv4ParamToUpdate());
                    break;
                case NetworkSavingType.Ipv6:
                    info_service_2.InfoService.getInstance().setNetworkIpv6Param(this.getNetworkIpv6ParamToUpdate());
                    break;
            }
            // Close model
            $(ModalSaveNetworkParamId).modal("hide");
            // Refresh
            setTimeout(function () {
                location.reload();
            }, 250);
        };
        SystemNetworkComponentViewModel.prototype.getNetworkCommonParamToUpdate = function () {
            // Return only changed values
            var hostname;
            if (this.NetworkHostnameValue() !== this._networkInfo.Hostname) {
                hostname = this.NetworkHostnameValue();
            }
            var isMsdnActive;
            if (this.NetworkIsMdnsActiveValue() !== this._networkInfo.IsMdnsActive) {
                isMsdnActive = this.NetworkIsMdnsActiveValue();
            }
            return new network_info_model_2.NetworkCommonInfo(hostname, isMsdnActive);
        };
        SystemNetworkComponentViewModel.prototype.getNetworkIpv4ParamToUpdate = function () {
            // ipv4 off
            if (this.NetworkIpv4SelectedMode().Mode === Ipv4Mode.Off) {
                return new network_info_model_2.NetworkIpv4Info(false);
            }
            // ipv4 auto
            if (this.NetworkIpv4SelectedMode().Mode === Ipv4Mode.Auto) {
                return new network_info_model_2.NetworkIpv4Info(true, true);
            }
            // ipv4 manual with valid ip. Set all values even if they aren't changed
            var ip = validator_26.Validator.Ipv4ToString(this.NetworkIpv4IpValue());
            var mask = validator_26.Validator.Ipv4ToString(this.NetworkIpv4MaskValue());
            var gateway = validator_26.Validator.Ipv4ToString(this.NetworkIpv4GatewayValue());
            var dns = validator_26.Validator.Ipv4ToString(this.NetworkIpv4DnsValue());
            // Replace . with :
            if (validator_26.Validator.isDefined(ip)) {
                ip = ip.replace(/\./g, ":");
            }
            if (validator_26.Validator.isDefined(mask)) {
                mask = mask.replace(/\./g, ":");
            }
            if (validator_26.Validator.isDefined(gateway)) {
                gateway = gateway.replace(/\./g, ":");
            }
            if (validator_26.Validator.isDefined(dns)) {
                dns = dns.replace(/\./g, ":");
            }
            return new network_info_model_2.NetworkIpv4Info(true, false, ip, mask, gateway, dns);
        };
        SystemNetworkComponentViewModel.prototype.getNetworkIpv6ParamToUpdate = function () {
            // ipv6 off
            if (this.NetworkIpv6SelectedMode().Mode === Ipv6Mode.Off) {
                return new network_info_model_2.NetworkIpv6Info(false);
            }
            // ipv6 slaac
            if (this.NetworkIpv6SelectedMode().Mode === Ipv6Mode.Slaac) {
                return new network_info_model_2.NetworkIpv6Info(true, true);
            }
            // ipv6 manual with valid ip. Set all values even if they aren't changed
            var ip = validator_26.Validator.Ipv6ToString(this.NetworkIpv6IpValue());
            var gateway = validator_26.Validator.Ipv6ToString(this.NetworkIpv6GatewayValue());
            var dns = validator_26.Validator.Ipv6ToString(this.NetworkIpv6DnsValue());
            return new network_info_model_2.NetworkIpv6Info(true, false, ip, gateway, dns);
        };
        SystemNetworkComponentViewModel.prototype.translate = function () {
            var translator = translator_service_29.TranslatorService.getInstance().t;
            this.SystemTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "system"));
            this.SystemBootTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "boot");
            this.SystemOsTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "os");
            this.SystemCoreTypeTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "coreType");
            this.SystemBoardTypeTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "boardType");
            this.SystemPcoTypeTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "pcoType");
            this.SystemMacTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "mac");
            this.SystemUidTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "uid");
            this.SystemTeraTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "tera");
            this.SystemPluginsTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "plugins");
            this.MemoryTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "memory"));
            this.TypeTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "type"));
            this.MBTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "MB"));
            this.FreeMemoryTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "free") + " [" + this.MBTitle() + "]");
            this.UsedMemoryTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "used") + " [" + this.MBTitle() + "]");
            this.SizeMemoryTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "size") + " [" + this.MBTitle() + "]");
            this.NetworkTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "network"));
            this.NetworkSaveTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "save"));
            this._networkIpManualTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "manual"));
            this._networkIpOffTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "off"));
            this._networkHostnameTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "hostname"));
            this._networkInvalidHostnameMsg(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "invalidHostname"));
            this._networkMdnsTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "mdns"));
            this._networkInvalidAddressMsg(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "invalidAddress"));
            this._networkIpv4IpLabelTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "ipv4"));
            this._networkIpv4AutoTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "auto"));
            this._networkIpv4IpTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "ip"));
            this._networkIpv4MaskTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "mask"));
            this._networkIpGatewayTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "gateway"));
            this._networkIpDnsTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "dns"));
            this._networkIpv4Modes.notifySubscribers();
            this._networkIpv6IpLabelTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "ipv6"));
            this._networkIpv6SlaacTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "slaac"));
            this._networkIpv6LinkLocalTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "linkLocal"));
            this._networkIpv6IpTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "publicIp"));
            this._networkIpv6Modes.notifySubscribers();
            this.ApplicationTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "application"));
            this.ApplicationNameTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "name");
            this.ApplicationVersionTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "version");
            this.ApplicationConfigurationNameTitle = translator(translator_service_29.TranslatorService.CfieldTranslationNs + "configuration");
            this.CycleTimeTitle(translator(translator_service_29.TranslatorService.CfieldTranslationNs + "cycleTime"));
            this.MsTitle(("[" + translator(translator_service_29.TranslatorService.CfieldTranslationNs + "ms")) + "]");
        };
        return SystemNetworkComponentViewModel;
    }(base_component_viewmodel_8.BaseComponentViewModel));
    var SystemInfoItemViewModel = /** @class */ (function () {
        function SystemInfoItemViewModel(_key, _value, _withDetails, _componentName) {
            this._key = _key;
            this._value = _value;
            this._withDetails = _withDetails;
            this._componentName = _componentName;
        }
        Object.defineProperty(SystemInfoItemViewModel.prototype, "Key", {
            get: function () { return this._key; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfoItemViewModel.prototype, "Value", {
            get: function () { return this._value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfoItemViewModel.prototype, "WithDetails", {
            get: function () { return this._withDetails; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemInfoItemViewModel.prototype, "Component", {
            get: function () { return this._componentName; },
            enumerable: true,
            configurable: true
        });
        return SystemInfoItemViewModel;
    }());
    var Ipv4Mode;
    (function (Ipv4Mode) {
        Ipv4Mode[Ipv4Mode["Auto"] = 0] = "Auto";
        Ipv4Mode[Ipv4Mode["Manual"] = 1] = "Manual";
        Ipv4Mode[Ipv4Mode["Off"] = 2] = "Off";
    })(Ipv4Mode || (Ipv4Mode = {}));
    var Ipv6Mode;
    (function (Ipv6Mode) {
        Ipv6Mode[Ipv6Mode["Slaac"] = 0] = "Slaac";
        Ipv6Mode[Ipv6Mode["Manual"] = 1] = "Manual";
        Ipv6Mode[Ipv6Mode["Off"] = 2] = "Off";
    })(Ipv6Mode || (Ipv6Mode = {}));
    var Ipv4ModeViewModel = /** @class */ (function () {
        function Ipv4ModeViewModel(_mode, _title) {
            this._mode = _mode;
            this._title = _title;
        }
        Object.defineProperty(Ipv4ModeViewModel.prototype, "Mode", {
            get: function () { return this._mode; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Ipv4ModeViewModel.prototype, "Title", {
            get: function () { return this._title; },
            enumerable: true,
            configurable: true
        });
        return Ipv4ModeViewModel;
    }());
    var Ipv6ModeViewModel = /** @class */ (function () {
        function Ipv6ModeViewModel(_mode, _title) {
            this._mode = _mode;
            this._title = _title;
        }
        Object.defineProperty(Ipv6ModeViewModel.prototype, "Mode", {
            get: function () { return this._mode; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Ipv6ModeViewModel.prototype, "Title", {
            get: function () { return this._title; },
            enumerable: true,
            configurable: true
        });
        return Ipv6ModeViewModel;
    }());
    var NetworkSavingType;
    (function (NetworkSavingType) {
        NetworkSavingType[NetworkSavingType["Common"] = 0] = "Common";
        NetworkSavingType[NetworkSavingType["Ipv4"] = 1] = "Ipv4";
        NetworkSavingType[NetworkSavingType["Ipv6"] = 2] = "Ipv6";
    })(NetworkSavingType || (NetworkSavingType = {}));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var hideExtraToolbar = (params && params.hideExtraToolbar) || false;
                var newViewModel = new SystemNetworkComponentViewModel(hideExtraToolbar);
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_20.cFieldService.getInstance().BaseComponentViewUrl + "/system-network.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/upgrade-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/translator.service", "component/viewModel/base-upload-component.viewmodel", "common/error"], function (require, exports, ko, cfield_service_21, translator_service_30, base_upload_component_viewmodel_1, error_1) {
    "use strict";
    var UpgradeComponentViewModel = /** @class */ (function (_super) {
        __extends(UpgradeComponentViewModel, _super);
        function UpgradeComponentViewModel(hideExtraToolbar) {
            var _this = _super.call(this, hideExtraToolbar) || this;
            /**
             * Polling the server to check if upgrade is completed.
             */
            _this.pollingServerCallback = function () {
                $.ajax({
                    cache: false,
                    url: "/upgrade.cgi",
                    dataType: "json",
                    success: function (data) {
                        if (data.status != 5 && data.status != 10) {
                            clearInterval(this._pollingIntervalId);
                        }
                        if (data.status == 1 || data.status == 2 || data.status == 3 || data.status == 4 || data.status == 9) {
                            /* Restore deny button, which may have been hidden by previous execution, or hide it if useless */
                            if (data.status == 9) {
                                this.IsAbortButtonVisible(true);
                            }
                            else if (data.status == 1 || data.status == 2 || data.status == 3 || data.status == 4) {
                                this.IsAbortButtonVisible(false);
                            }
                            // Manage state
                            switch (data.status) {
                                case 1:
                                    this._upgradeState(UpgradeState.Failed);
                                    break;
                                case 3:
                                    this._upgradeState(UpgradeState.Successfully_Reboot_Need);
                                    break;
                                case 4:
                                    this._upgradeState(UpgradeState.Successfully_Restart_Need);
                                    break;
                                case 9:
                                    this._upgradeState(UpgradeState.RequestConfirm);
                                    break;
                            }
                        }
                    }.bind(this),
                    error: function (jqXHR, textStatus, errorThrown) {
                        this._log(errorThrown);
                        this.cFieldServiceInstance.LogService("");
                    }.bind(this)
                });
            }.bind(_this);
            _this._upgradeTitle = ko.observable();
            _this._upgradeState = ko.observable(UpgradeState.NotStarted);
            _this.ResultMsg = ko.observable("");
            _this.InstructionModalMessage = ko.observable("");
            _this.IsAbortButtonVisible = ko.observable(false);
            _this._yesTitle = ko.observable();
            _this._noTitle = ko.observable();
            _this.ProgressBarClass = ko.pureComputed({
                read: function () {
                    var state = _this._upgradeState();
                    switch (state) {
                        case UpgradeState.NotStarted:
                        case UpgradeState.Started:
                            return "progress-bar progress-bar-striped active";
                        case UpgradeState.UploadComplete:
                        case UpgradeState.RequestConfirm:
                            return "progress-bar progress-bar-warning";
                        case UpgradeState.UpgradeStart:
                            return "progress-bar progress-bar-warning progress-bar-striped active";
                        case UpgradeState.Successfully:
                        case UpgradeState.Successfully_Reboot_Need:
                        case UpgradeState.Successfully_Restart_Need:
                            return "progress-bar progress-bar-success";
                        case UpgradeState.Failed:
                        case UpgradeState.UploadError:
                            return "progress-bar progress-bar-danger";
                    }
                }
            });
            // Subscriber to upgrade state
            _this._upgradeStateSubscription = _this._upgradeState.subscribe(_this.upgradeResultMsgTranslation, _this);
            // Evaluete the state of cField service
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        Object.defineProperty(UpgradeComponentViewModel.prototype, "UpgradeTitle", {
            get: function () { return this._upgradeTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UpgradeComponentViewModel.prototype, "YesTitle", {
            get: function () { return this._yesTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UpgradeComponentViewModel.prototype, "NoTitle", {
            get: function () { return this._noTitle; },
            enumerable: true,
            configurable: true
        });
        /**
         * Dispose
         */
        UpgradeComponentViewModel.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            // Eventually abort upload. If the request it's already started you can stop it (asyn process)
            if (this._upgradeState() == UpgradeState.Started && this._cachedJqXHR) {
                this._cachedJqXHR.abort();
            }
            // Unregister observer
            this._upgradeStateSubscription.dispose();
        };
        // override
        UpgradeComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            _super.prototype.defaultOnSuccessfulInitializedConf.call(this, "#upgradeBrowserFile");
            // Set a listener on file changed
            $(UpgradeComponentViewModel.InputId).change(function (e) {
                var fileName = $(UpgradeComponentViewModel.InputId).val();
                if (fileName) {
                    this.IsUploadButtonEnabled(true);
                    // Clear view state
                    this.IsProgressBarVisible(false);
                    this.IsResultMsgVisible(false);
                }
                else {
                    this.IsUploadButtonEnabled(false);
                }
            }.bind(this));
            // Enable jquery.form
            $("#upgrade-form").ajaxForm({
                beforeSend: function (jqXHR, options) {
                    this.IsProgressBarVisible(true);
                    this.IsResultMsgVisible(false);
                    this._upgradeState(UpgradeState.Started);
                    this._cachedJqXHR = jqXHR;
                }.bind(this),
                error: function () {
                    this._upgradeState(UpgradeState.Failed);
                }.bind(this),
                complete: function (xhr) {
                    this.onAjaxComplete(xhr);
                }.bind(this)
            });
            // Init translation
            this.updateTranslation();
        };
        // override
        UpgradeComponentViewModel.prototype.onChangeLanguage = function (newState) {
            if (newState === translator_service_30.TranslatorEvent.LanguageChanged) {
                this.updateTranslation();
            }
        };
        /**
         * Manage ajax complete event.
         * @param xhr <code>xhr</code> of the request.
         */
        UpgradeComponentViewModel.prototype.onAjaxComplete = function (xhr) {
            var translator = translator_service_30.TranslatorService.getInstance().t;
            var response = xhr.responseText;
            if (!response) {
                this._upgradeState(UpgradeState.Failed);
                return;
            }
            var data = jQuery.parseJSON(xhr.responseText);
            this.IsResultMsgVisible(true);
            if (data && data.error === 0) {
                this._upgradeState(UpgradeState.UploadComplete);
                // Show modal
                $(UpgradeComponentViewModel.UpgradeModalId).modal("show");
                // Callback
                this._pollingIntervalId = window.setInterval(this.pollingServerCallback, 500);
            }
            else {
                this._errorCode = data.error;
                this._upgradeState(UpgradeState.UploadError);
                // try to reset state
                this.cgiUpgrade(false);
            }
        };
        /**
         * Ok click listener button.
         */
        UpgradeComponentViewModel.prototype.upgConfirm = function () {
            this._pollingIntervalId = window.setInterval(this.pollingServerCallback, 500);
            this._upgradeState(UpgradeState.UpgradeStart);
            this.cgiUpgrade(true);
        };
        /**
         * Abort click listener button.
         */
        UpgradeComponentViewModel.prototype.upgDeny = function () {
            this.cgiUpgrade(false);
            //$.get("/upgrade.cgi?confirm=0", function(){});
            this._upgradeState(UpgradeState.UpgradeAborted);
            this.IsAbortButtonVisible(false);
            this.IsProgressBarVisible(false);
            // dismiss dialog
            $(UpgradeComponentViewModel.UpgradeModalId).modal("hide");
        };
        UpgradeComponentViewModel.prototype.cgiUpgrade = function (isConfirm) {
            var flagString;
            if (isConfirm) {
                flagString = "1";
            }
            else {
                flagString = "0";
            }
            $.ajax({
                cache: false,
                url: "/upgrade.cgi?confirm=" + flagString,
                error: function (jqXHR, textStatus, errorThrown) {
                    this._log(errorThrown);
                    this.cFieldServiceInstance.LogService("");
                }.bind(this)
            });
        };
        /**
         * Update translation.
         */
        UpgradeComponentViewModel.prototype.updateTranslation = function () {
            var translator = translator_service_30.TranslatorService.getInstance().t;
            this._upgradeTitle(translator(translator_service_30.TranslatorService.CfieldTranslationNs + "upgrade"));
            this._yesTitle(translator(translator_service_30.TranslatorService.CfieldTranslationNs + "yes"));
            this._noTitle(translator(translator_service_30.TranslatorService.CfieldTranslationNs + "no"));
            this.InstructionMessage(translator(translator_service_30.TranslatorService.CfieldTranslationNs + "upgradeInstruction"));
            this.InstructionModalMessage(translator(translator_service_30.TranslatorService.CfieldTranslationNs + "upgradeAppAskConfirm"));
            this.setResultMessageTranslation(translator);
            _super.prototype.translate.call(this);
        };
        /**
         * Call to update result message.
         */
        UpgradeComponentViewModel.prototype.updateResultMsgTranslation = function () {
            this.setResultMessageTranslation(translator_service_30.TranslatorService.getInstance().t);
        };
        UpgradeComponentViewModel.prototype.setResultMessageTranslation = function (translator) {
            switch (this._upgradeState()) {
                case UpgradeState.Successfully_Reboot_Need:
                    this.ResultMsg(translator(translator_service_30.TranslatorService.CfieldTranslationNs + "rebootDevice"));
                    break;
                case UpgradeState.Successfully_Restart_Need:
                    this.ResultMsg(translator(translator_service_30.TranslatorService.CfieldTranslationNs + "restartDevice"));
                    break;
                case UpgradeState.Successfully:
                    this.ResultMsg(translator(translator_service_30.TranslatorService.CfieldTranslationNs + "operationSuccessfullyCompleted"));
                    break;
                case UpgradeState.UploadError:
                case UpgradeState.Failed:
                    var message = translator(translator_service_30.TranslatorService.CfieldTranslationNs + "operationFailed");
                    if (this._errorCode) {
                        message = message + " (" + this._errorCode + ") - " + error_1.ErrorCodeConverter.ErrorUpgradeCodeToText(this._errorCode);
                    }
                    this.ResultMsg(message);
                    break;
                default:
                    this.ResultMsg("");
            }
        };
        /**
         * Callback to update result message.
         */
        UpgradeComponentViewModel.prototype.upgradeResultMsgTranslation = function () {
            this.setResultMessageTranslation(translator_service_30.TranslatorService.getInstance().t);
        };
        /** Input id. */
        UpgradeComponentViewModel.InputId = "#upgradeBrowserFile";
        /** Modal id. */
        UpgradeComponentViewModel.UpgradeModalId = "#modalUpgrade";
        return UpgradeComponentViewModel;
    }(base_upload_component_viewmodel_1.BaseUploadComponentViewModel));
    var UpgradeState;
    (function (UpgradeState) {
        UpgradeState[UpgradeState["NotStarted"] = 0] = "NotStarted";
        UpgradeState[UpgradeState["RequestConfirm"] = 1] = "RequestConfirm";
        UpgradeState[UpgradeState["Started"] = 2] = "Started";
        UpgradeState[UpgradeState["UploadComplete"] = 3] = "UploadComplete";
        UpgradeState[UpgradeState["UploadError"] = 4] = "UploadError";
        UpgradeState[UpgradeState["UpgradeStart"] = 5] = "UpgradeStart";
        UpgradeState[UpgradeState["UpgradeAborted"] = 6] = "UpgradeAborted";
        UpgradeState[UpgradeState["Successfully"] = 7] = "Successfully";
        UpgradeState[UpgradeState["Successfully_Reboot_Need"] = 8] = "Successfully_Reboot_Need";
        UpgradeState[UpgradeState["Successfully_Restart_Need"] = 9] = "Successfully_Restart_Need";
        UpgradeState[UpgradeState["Failed"] = 10] = "Failed";
    })(UpgradeState || (UpgradeState = {}));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var hideExtraToolbar = (params && params.hideExtraToolbar) || false;
                var newViewModel = new UpgradeComponentViewModel(hideExtraToolbar);
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_21.cFieldService.getInstance().BaseComponentViewUrl + "/upgrade.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/upload-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "component/viewModel/base-upload-component.viewmodel", "service/translator.service"], function (require, exports, ko, cfield_service_22, base_upload_component_viewmodel_2, translator_service_31) {
    "use strict";
    var UploadComponentViewModel = /** @class */ (function (_super) {
        __extends(UploadComponentViewModel, _super);
        /**
         * Init component.
         */
        function UploadComponentViewModel(hideExtraToolbar) {
            var _this = _super.call(this, hideExtraToolbar) || this;
            /** Explicit abort. */
            _this._explicitAbort = false;
            _this._uploadTitle = ko.observable();
            _this._uploadState = ko.observable(UploadState.NotStarted);
            _this.ResultMsg = ko.observable("");
            _this.IsAbortButtonVisible = ko.observable(false);
            _this.ProgressBarClass = ko.pureComputed({
                read: function () {
                    var state = _this._uploadState();
                    switch (state) {
                        case UploadState.NotStarted:
                        case UploadState.Started:
                            return "progress-bar progress-bar-striped active";
                        case UploadState.Successfully:
                            return "progress-bar progress-bar-success";
                        case UploadState.Failed:
                            return "progress-bar progress-bar-danger";
                    }
                }
            });
            // Register an observer
            _this._uploadStateSubscription = _this._uploadState.subscribe(_this.updateResultMsgTranslation, _this);
            // Evaluete the state of cField service
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        Object.defineProperty(UploadComponentViewModel.prototype, "UploadTitle", {
            get: function () { return this._uploadTitle; },
            enumerable: true,
            configurable: true
        });
        /**
         * Dispose.
         */
        UploadComponentViewModel.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            // Eventually abort upload.
            this.abortUpload();
            // Unregister observer
            this._uploadStateSubscription.dispose();
        };
        /**
         * Abort upload.
         */
        UploadComponentViewModel.prototype.abortUpload = function () {
            // If the request it's already started you can stop it (asyn process)
            if (this._uploadState() == UploadState.Started && this._cachedJqXHR) {
                this._cachedJqXHR.abort();
            }
        };
        UploadComponentViewModel.prototype.abortUploadExplicit = function () {
            this._explicitAbort = true;
            this.abortUpload();
        };
        // override
        UploadComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            _super.prototype.defaultOnSuccessfulInitializedConf.call(this, UploadComponentViewModel.InputId);
            // Set a listener on file changed
            $(UploadComponentViewModel.InputId).change(function (e) {
                var fileName = $(UploadComponentViewModel.InputId).val();
                if (fileName) {
                    this.IsUploadButtonEnabled(true);
                    // Clear view state
                    this.IsProgressBarVisible(false);
                    this.IsResultMsgVisible(false);
                    this.abortUpload();
                }
                else {
                    this.IsUploadButtonEnabled(false);
                }
            }.bind(this));
            // Enable jquery.form
            $("#upload-form").ajaxForm({
                beforeSend: function (jqXHR, options) {
                    this.IsProgressBarVisible(true);
                    this.IsResultMsgVisible(false);
                    this._uploadState(UploadState.Started);
                    this.IsAbortButtonVisible(true);
                    this._cachedJqXHR = jqXHR;
                }.bind(this),
                error: function () {
                    if (this._explicitAbort) {
                        this._explicitAbort = false;
                        this._uploadState(UploadState.NotStarted);
                        this.IsProgressBarVisible(false);
                        this.IsResultMsgVisible(false);
                    }
                    else {
                        this._uploadState(UploadState.Failed);
                    }
                }.bind(this),
                complete: function (xhr) {
                    this.onAjaxComplete(xhr);
                }.bind(this)
            });
            // Init instruction
            this.updateTranslation();
        };
        // override
        UploadComponentViewModel.prototype.onChangeLanguage = function (newState) {
            if (newState === translator_service_31.TranslatorEvent.LanguageChanged) {
                this.updateTranslation();
            }
        };
        /**
         * Manage ajax complete event.
         * @param xhr <code>xhr</code> of the request.
         */
        UploadComponentViewModel.prototype.onAjaxComplete = function (xhr) {
            this.IsAbortButtonVisible(false);
            var translator = translator_service_31.TranslatorService.getInstance().t;
            var response = xhr.responseText;
            if (!response) {
                this._uploadState(UploadState.Failed);
                return;
            }
            var data = jQuery.parseJSON(xhr.responseText);
            this.IsResultMsgVisible(true);
            if (data && data.result === 1) {
                this._uploadState(UploadState.Successfully);
            }
            else {
                this._uploadState(UploadState.Failed);
            }
        };
        /**
         * Update translation.
         */
        UploadComponentViewModel.prototype.updateTranslation = function () {
            var translator = translator_service_31.TranslatorService.getInstance().t;
            this._uploadTitle(translator(translator_service_31.TranslatorService.CfieldTranslationNs + "upload"));
            this.InstructionMessage(translator(translator_service_31.TranslatorService.CfieldTranslationNs + "uploadInstruction"));
            this.setResultMessageTranslation(translator);
            _super.prototype.translate.call(this);
        };
        UploadComponentViewModel.prototype.setResultMessageTranslation = function (translator) {
            switch (this._uploadState()) {
                case UploadState.Failed:
                    this.ResultMsg(translator(translator_service_31.TranslatorService.CfieldTranslationNs + "operationFailed"));
                    break;
                case UploadState.Successfully:
                    this.ResultMsg(translator(translator_service_31.TranslatorService.CfieldTranslationNs + "operationSuccessfullyCompleted"));
                    break;
                default:
                    this.ResultMsg("");
                    break;
            }
        };
        /**
         * Callback to update result message.
         */
        UploadComponentViewModel.prototype.updateResultMsgTranslation = function () {
            this.setResultMessageTranslation(translator_service_31.TranslatorService.getInstance().t);
        };
        /** Input id. */
        UploadComponentViewModel.InputId = "#uploadBrowserFile";
        return UploadComponentViewModel;
    }(base_upload_component_viewmodel_2.BaseUploadComponentViewModel));
    var UploadState;
    (function (UploadState) {
        UploadState[UploadState["NotStarted"] = 0] = "NotStarted";
        UploadState[UploadState["Started"] = 1] = "Started";
        UploadState[UploadState["Successfully"] = 2] = "Successfully";
        UploadState[UploadState["Failed"] = 3] = "Failed";
    })(UploadState || (UploadState = {}));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var hideExtraToolbar = (params && params.hideExtraToolbar) || false;
                var newViewModel = new UploadComponentViewModel(hideExtraToolbar);
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_22.cFieldService.getInstance().BaseComponentViewUrl + "/upload.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/item/serie-chart-viewmodel", ["require", "exports", "knockout", "service/variable.service", "component/viewModel/item/log-definition-item.viewmodel", "common/data-structure", "common/color", "common/loggable-entity"], function (require, exports, ko, variable_service_7, log_definition_item_viewmodel_3, data_structure_8, color_2, loggable_entity_10) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ChartOptions = /** @class */ (function () {
        function ChartOptions(_labelArray, _isVisibleArray, _colorArray, _yAxisTypeArray, _chartLineStyleType, _data) {
            this._labelArray = _labelArray;
            this._isVisibleArray = _isVisibleArray;
            this._colorArray = _colorArray;
            this._yAxisTypeArray = _yAxisTypeArray;
            this._chartLineStyleType = _chartLineStyleType;
            this._data = _data;
        }
        Object.defineProperty(ChartOptions.prototype, "LabelArray", {
            get: function () {
                var labelArray = this._labelArray.slice();
                labelArray.unshift("Time");
                return labelArray;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ChartOptions.prototype, "IsVisibleArray", {
            get: function () { return this._isVisibleArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ChartOptions.prototype, "ColorArray", {
            get: function () { return this._colorArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ChartOptions.prototype, "Data", {
            get: function () { return this._data; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ChartOptions.prototype, "SerieOptions", {
            get: function () {
                // Build serie options
                var serieOption = {};
                for (var i = 0; i < this._isVisibleArray.length; i++) {
                    var serieName = this._labelArray[i];
                    var yAxisType = this._yAxisTypeArray[i];
                    var chartLineStyle = this._chartLineStyleType[i];
                    if (yAxisType === log_definition_item_viewmodel_3.YAxisType.Secondary) {
                        // Move to second axis
                        serieOption[serieName] = { axis: "y2" };
                    }
                    else {
                        serieOption[serieName] = { axis: "y1" };
                    }
                    if (chartLineStyle === log_definition_item_viewmodel_3.ChartLineStyle.Step) {
                        if (serieOption[serieName]) {
                            // There is also other properties. Add it
                            var optionsForSerie = serieOption[serieName];
                            optionsForSerie["stepPlot"] = true;
                        }
                        else {
                            // No other properties. Create id
                            serieOption[serieName] = { stepPlot: true };
                        }
                    }
                }
                return serieOption;
            },
            enumerable: true,
            configurable: true
        });
        return ChartOptions;
    }());
    exports.ChartOptions = ChartOptions;
    var SerieChartViewModel = /** @class */ (function () {
        function SerieChartViewModel() {
            this._idToIndexMap = new data_structure_8.CustomMap();
            this._variableSerieItemViewModelArray = [];
            this._data = [];
            this._registeredObserverArray = [];
            this._isDirty = ko.observable();
            this._isDirty.extend({ notify: "always" });
            this._isDataDirty = ko.observable();
            this._isDataDirty.extend({ notify: "always" });
        }
        Object.defineProperty(SerieChartViewModel.prototype, "VariableSerieItemViewModelArray", {
            get: function () { return this._variableSerieItemViewModelArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SerieChartViewModel.prototype, "ChartOptions", {
            get: function () {
                var labelArray = [];
                var isVisibleArray = [];
                var colorArray = [];
                var yAxisTypeArray = [];
                var chartLineStyleTypeArray = [];
                for (var _i = 0, _a = this._variableSerieItemViewModelArray; _i < _a.length; _i++) {
                    var serie = _a[_i];
                    labelArray.push(serie.SerieName);
                    isVisibleArray.push(serie.IsSerieVisible);
                    colorArray.push(serie.ColorName);
                    yAxisTypeArray.push(serie.YAxisType);
                    chartLineStyleTypeArray.push(serie.ChartLineStyle);
                }
                return new ChartOptions(labelArray, isVisibleArray, colorArray, yAxisTypeArray, chartLineStyleTypeArray, this._data);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SerieChartViewModel.prototype, "Data", {
            get: function () { return this._data; },
            enumerable: true,
            configurable: true
        });
        SerieChartViewModel.prototype.resetData = function () { this._data = []; };
        Object.defineProperty(SerieChartViewModel.prototype, "IsDirty", {
            get: function () { return this._isDirty; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SerieChartViewModel.prototype, "IsDataDirty", {
            get: function () { return this._isDataDirty; },
            enumerable: true,
            configurable: true
        });
        SerieChartViewModel.prototype.dispose = function () {
            for (var _i = 0, _a = this._registeredObserverArray; _i < _a.length; _i++) {
                var observer = _a[_i];
                observer.dispose();
            }
        };
        SerieChartViewModel.prototype.addDefault = function (id, name) {
            this.add(id, name, true, color_2.ColorGenerator.GetColor(), log_definition_item_viewmodel_3.YAxisType.Primary, log_definition_item_viewmodel_3.ChartLineStyle.Line, true);
        };
        /**
         *
         * @param id
         * @param name
         * @param isVisible
         * @param color
         * @param yAxisType
         * @param style
         * @param canNotifyIsDirtyProperty
         * @return <code>true</code> if the add isn't skipped
         */
        SerieChartViewModel.prototype.add = function (id, name, isVisible, color, yAxisType, chartLineStyle, canNotifyIsDirtyProperty) {
            // Skip if variable is already monitored
            if (this._idToIndexMap.contains(id))
                return false;
            // Add serie
            var index = this._variableSerieItemViewModelArray.length;
            var serie = new log_definition_item_viewmodel_3.VariableSerieItemViewModel(id, name, "", isVisible, yAxisType, color, chartLineStyle);
            this._variableSerieItemViewModelArray.push(serie);
            this._idToIndexMap.addNoConflict(id, index);
            if (this._data && this._data.length > 0) {
                // Check mismatch between the number of labels and data columns
                var firstRow = this._data[0];
                if (this._variableSerieItemViewModelArray.length + 1 !== firstRow.length) {
                    // Add a null value column to all data
                    for (var _i = 0, _a = this._data; _i < _a.length; _i++) {
                        var record = _a[_i];
                        record.push(null);
                    }
                }
            }
            // Register an observer
            this._registeredObserverArray.push(serie.OnChange.subscribe(function () {
                this._isDataDirty(true);
            }.bind(this), this));
            if (canNotifyIsDirtyProperty) {
                // Refresh data
                this._isDirty.notifySubscribers();
            }
            return true;
        };
        // TODO check size of arrays
        SerieChartViewModel.prototype.addMultiple = function (idArray, nameArray, isVisibleArray, colorArray, yAxisTypeArray, styleArray) {
            var atLeastOneAdd = false;
            for (var i = 0; i < idArray.length; i++) {
                var result = this.add(idArray[i], nameArray[i], isVisibleArray[i], colorArray[i], yAxisTypeArray[i], styleArray[i], false);
                atLeastOneAdd = atLeastOneAdd || result;
            }
            if (atLeastOneAdd) {
                // Refresh data
                this._isDirty.notifySubscribers();
            }
        };
        // Improve performance
        SerieChartViewModel.prototype.remove = function (id) {
            // Search index
            var index;
            if (this._idToIndexMap.contains(id)) {
                index = this._idToIndexMap.Set[id];
            }
            else {
                // Not found
                SerieChartViewModel._logger._log("SerieChartViewModelHelper remove(): the id=" + id + " is not been found");
                return;
            }
            // Remove
            this._variableSerieItemViewModelArray.splice(index, 1);
            for (var i = 0; i < this._data.length; i++) {
                this._data[i].splice(index, 1);
            }
            // Rebuild id map
            this._idToIndexMap = new data_structure_8.CustomMap();
            var set = this._idToIndexMap.Set;
            for (var i = 0; i < this._variableSerieItemViewModelArray.length; i++) {
                var serie = this._variableSerieItemViewModelArray[i];
                set[serie.Id] = i;
            }
            // Refresh data
            this._isDirty.notifySubscribers();
        };
        SerieChartViewModel.prototype.pollingData = function () {
            var _this = this;
            return variable_service_7.VariableService.getInstance().getVar(this._idToIndexMap.getKeys())
                .then(function (result) {
                if (result._isSuccess) {
                    if (result._deviceVariableArray && result._deviceVariableArray.length > 0) {
                        // Build new record
                        var data = new Array(_this._idToIndexMap.Count + 1);
                        // Position 0 for the date
                        // Not support for local/remote date
                        data[0] = new Date();
                        for (var _i = 0, _a = result._deviceVariableArray; _i < _a.length; _i++) {
                            var variable = _a[_i];
                            var id = variable.getId();
                            if (typeof id !== "undefined" && id !== null) {
                                var index = _this._idToIndexMap.Set[id] + 1; // Position 0 is reserved for the date
                                if (typeof index !== "undefined" && index !== null) {
                                    // Parse number
                                    var value = new Number(variable.getValue());
                                    if (value != NaN) {
                                        // Save value
                                        data[index] = value.valueOf();
                                    }
                                    else {
                                        SerieChartViewModel._logger._log("SerieChartViewModelHelper pollingData(): invalid number ("
                                            + variable.getValue() + ") for variable (" + variable.getName() + ")");
                                    }
                                }
                                else {
                                    SerieChartViewModel._logger._log("SerieChartViewModelHelper pollingData(): id of response is not been found");
                                }
                            }
                            else {
                                SerieChartViewModel._logger._log("SerieChartViewModelHelper pollingData(): found an invalid id from getVar");
                            }
                        }
                        // Save record
                        _this._data.push(data);
                        // Not allocate too much memory
                        if (_this._data.length > 1000) {
                            _this._data.shift();
                        }
                        // Refresh data
                        _this._isDataDirty.notifySubscribers();
                    }
                    else {
                        SerieChartViewModel._logger._log("SerieChartViewModelHelper pollingData(): no data");
                    }
                }
                else {
                    return Promise.reject("SerieChartViewModelHelper pollingData(): getVar is failed!");
                }
            });
        };
        // Init common logger for class
        SerieChartViewModel._logger = new loggable_entity_10.LoggableEntityStatic(new SerieChartViewModel());
        return SerieChartViewModel;
    }());
    exports.SerieChartViewModel = SerieChartViewModel;
});
define("component/viewModel/variable-component.viewmodel", ["require", "exports", "knockout", "dygraphs", "service/cfield.service", "service/device-config.service", "service/storage.service", "service/translator.service", "service/variable.service", "component/viewModel/base-table-component.viewmodel", "component/viewModel/base-category-component.viewmodel", "component/viewModel/settings-viewmodel", "component/viewModel/item/variable-item.viewmodel", "component/viewModel/item/log-definition-item.viewmodel", "component/viewModel/item/serie-chart-viewmodel", "component/presenter/variable-item.presenter", "dataModel/datatype.model", "common/data-structure", "common/view", "common/string-util", "common/converter", "common/color", "common/file-util", "common/data-structure", "common/validator", "common/style", "component/presenter/update-item-set"], function (require, exports, ko, Dygraphs, cfield_service_23, device_config_service_12, storage_service_5, translator_service_32, variable_service_8, base_table_component_viewmodel_5, base_category_component_viewmodel_3, settings_viewmodel_2, variable_item_viewmodel_4, log_definition_item_viewmodel_4, serie_chart_viewmodel_1, variable_item_presenter_2, datatype_model_12, data_structure_9, view_6, string_util_3, converter_6, color_3, file_util_4, data_structure_10, validator_27, style_3, update_item_set_2) {
    "use strict";
    var TableJqueryId = "#variable-view-component-table";
    var SerieTableJqueryId = "#variable-view-serie-table";
    var PlotChartModalJqueryId = "#chartDialog";
    var ChartModalContainerClass = "chartContainerDialog";
    // Template
    var SaveIconTemplate = '<i class="fa fa-download action" aria-hidden="false" title="%s"></i>'; // Don't remove action class
    // Style
    var AlertStateSuccessClass = "alert-success";
    var AlertStateWarningClass = "alert-warning";
    var AlertStateErrorClass = "alert-danger";
    var VariableComponentViewModel = /** @class */ (function (_super) {
        __extends(VariableComponentViewModel, _super);
        function VariableComponentViewModel(hideExtraToolbar) {
            var _this = _super.call(this, hideExtraToolbar, true, true, false, TableJqueryId) || this;
            // VM
            _this._variableViewModelArray = [];
            _this._isUpdateEnabled = true;
            _this._isTableInEditMode = false;
            _this._isVariableViewInit = ko.observable(false);
            _this._isToolbarInit = false;
            _this._actionHeader = ko.observable();
            _this._plotHeader = ko.observable();
            _this._setValueHeader = ko.observable();
            _this._isRetainHeader = ko.observable();
            _this._saveTooltip = ko.observable();
            _this._serverResponseHeader = ko.observable();
            _this._valueThatItryToSetHeader = ko.observable();
            _this._valueAfterTheUpdateHeader = ko.observable();
            _this._updateInfoMsg = ko.observable();
            _this._isUpdateInfoVisible = ko.observable(false);
            _this._isUpdateErrorInfoDetailVisible = ko.observable(false);
            _this._isUpdateWarningInfoDetailVisible = ko.observable(false);
            _this._failedSetVarItemArray = ko.observableArray();
            _this._notUpdatedSetVarItemArray = ko.observableArray();
            _this._updateItemSetHelper = new update_item_set_2.UpdateItemSetHelper(new VariableItemViewModelMap(), ko.observable(0));
            _this.IsUpdatesPending = ko.pureComputed({
                read: function () {
                    return _this._updateItemSetHelper.NumberOfPendingUpdate() > 0;
                }
            });
            _this._alertState = ko.observable();
            _this._updateInfoMsgState = UpdateInfoState.Success;
            _this._settingsViewModel = new settings_viewmodel_2.SettingsViewModel(settings_viewmodel_2.SettingsRefreshViewId.Variable);
            _this._isPlotModalVisible = ko.observable(false);
            _this._serieChartViewModel = new serie_chart_viewmodel_1.SerieChartViewModel();
            _this._isSelectedAtLeastOneChartVariable = ko.observable(false);
            _this._selectAtLeastOneChartVariableMsg = ko.observable();
            _this._liveTitle = ko.observable();
            _this._resetZoomTitle = ko.observable();
            _this._exportTitle = ko.observable();
            _this._isRefreshVariableEnable = true;
            _this._isChartLiveEnabled = false;
            _this._isSelectedSerieSetChange = ko.observable();
            _this._isSelectedSerieSetChange.extend({ notify: "always" });
            _this._registeredObserverArray.push(_this._isSelectedSerieSetChange.subscribe(function (variableItemViewModel) {
                this.updateSerieViewModel(variableItemViewModel);
                if (this._isPlotModalVisible()) {
                    this.updateChart();
                }
            }.bind(_this)));
            _this.registerChartDialogLogic();
            // Evaluete the state of cField service
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        Object.defineProperty(VariableComponentViewModel.prototype, "IsVariableViewInit", {
            get: function () { return this._isVariableViewInit; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "ActionHeader", {
            get: function () { return this._actionHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "PlotHeader", {
            get: function () { return this._plotHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "SetValueHeader", {
            get: function () { return this._setValueHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "IsRetainHeader", {
            get: function () { return this._isRetainHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "ServerResponseHeader", {
            get: function () { return this._serverResponseHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "ValueThatItryToSetHeader", {
            get: function () { return this._valueThatItryToSetHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "ValueAfterTheUpdateHeader", {
            get: function () { return this._valueAfterTheUpdateHeader; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "UpdateInfoMsg", {
            get: function () { return this._updateInfoMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "IsUpdateInfoVisible", {
            get: function () { return this._isUpdateInfoVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "IsUpdateErrorInfoDetailsVisible", {
            get: function () { return this._isUpdateErrorInfoDetailVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "IsUpdateWarningInfoDetailsVisible", {
            get: function () { return this._isUpdateWarningInfoDetailVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "IsTableInEditMode", {
            set: function (value) { this._isTableInEditMode = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "AlertState", {
            get: function () { return this._alertState; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "FailedSetVarItemArray", {
            get: function () { return this._failedSetVarItemArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "NotUpdatedSetVarItemArray", {
            get: function () { return this._notUpdatedSetVarItemArray; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "SettingsTitle", {
            get: function () { return this._settingsViewModel.SettingsTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "SettingsRefreshInterval", {
            get: function () { return this._settingsViewModel.RefreshIntervalSecondsTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "SettingsSave", {
            get: function () { return this._settingsViewModel.SaveTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "SettingsCancel", {
            get: function () { return this._settingsViewModel.CancelTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "RefreshIntervalSecondsValue", {
            get: function () { return this._settingsViewModel.RefreshIntervalSecondsValue; },
            set: function (value) { this._settingsViewModel.RefreshIntervalSecondsValue = value; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "ExportSettingsSeparatorChar", {
            get: function () { return this._settingsViewModel.ExportSeparatorCharTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "ExportSettingsSeparatorCharValue", {
            get: function () { return this._settingsViewModel.ExportSeparatorCharValue; },
            set: function (value) { this._settingsViewModel.ExportSeparatorCharValue = value; },
            enumerable: true,
            configurable: true
        });
        VariableComponentViewModel.prototype.save = function () {
            this._settingsViewModel.save();
        };
        Object.defineProperty(VariableComponentViewModel.prototype, "IsPlotModalVisible", {
            get: function () { return this._isPlotModalVisible; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "IsSelectedAtLeastOneChartVariable", {
            get: function () { return this._isSelectedAtLeastOneChartVariable; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "SelectAtLeastOneChartVariableMsg", {
            get: function () { return this._selectAtLeastOneChartVariableMsg; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "LiveTitle", {
            get: function () { return this._liveTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "ExportTitle", {
            get: function () { return this._exportTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "ResetZoomTitle", {
            get: function () { return this._resetZoomTitle; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "SaveTooltip", {
            get: function () { return this._saveTooltip; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "IsSelectedSerieChange", {
            get: function () { return this._isSelectedSerieSetChange; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableComponentViewModel.prototype, "JquerySelectorModalMinimize", {
            get: function () {
                var query = PlotChartModalJqueryId + " .modalMinimize";
                var jQuerySelector = $(query);
                return jQuerySelector;
            },
            enumerable: true,
            configurable: true
        });
        // override
        VariableComponentViewModel.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            // Stop refresh variable task
            this._isRefreshVariableEnable = false;
            if (this._refreshVariableTimeoutId) {
                clearTimeout(this._refreshVariableTimeoutId);
            }
            // Eventually stop live task
            this._isChartLiveEnabled = false;
            if (this._refreshChartTimeoutId) {
                clearTimeout(this._refreshChartTimeoutId);
            }
            // Dispose viewmodel helper
            this._serieChartViewModel.dispose();
            // Dispose cached viewmodel
            this._cFieldServiceInstance.disposeVariableComponentViewModelCached();
        };
        // override
        VariableComponentViewModel.prototype.onChangeCFieldState = function (newState) {
            _super.prototype.onChangeCFieldState.call(this, newState);
            // Manage ad-hoc state
            switch (newState) {
                case cfield_service_23.CfieldServiceState.RefreshConfig:
                    // Clear state 
                    this._isUpdateEnabled = true;
                    this._isTableInEditMode = false;
                    this._variableViewModelArray = [];
                    this._isUpdateInfoVisible(false);
                    this._isUpdateErrorInfoDetailVisible(false);
                    this._isUpdateWarningInfoDetailVisible(false);
                    this._updateInfoMsg("");
                    this._isToolbarInit = false;
                    this._isVariableViewInit(false);
                    this._updateItemSetHelper.clear();
                    this._failedSetVarItemArray.removeAll();
                    this._notUpdatedSetVarItemArray.removeAll();
                    this._updateInfoMsgState = UpdateInfoState.Success;
                    this._isPlotModalVisible(false);
                    this._isSelectedAtLeastOneChartVariable(false);
                    this._serieChartViewModel = new serie_chart_viewmodel_1.SerieChartViewModel();
                    //  Destroy view
                    $(TableJqueryId).bootstrapTable("destroy");
                    break;
                case cfield_service_23.CfieldServiceState.CrcChanged:
                    // Disable all updates
                    this._isUpdateEnabled = false;
                    break;
            }
        };
        // override
        VariableComponentViewModel.prototype.onSuccessfulInitializedConf = function () {
            var _this = this;
            _super.prototype.onSuccessfulInitializedConf.call(this);
            // Init translation
            this.translate();
            this.initHeader();
            var deviceConfigService = device_config_service_12.DeviceConfigService.getInstance();
            var updateAllVariableValues = variable_service_8.VariableService.getInstance().getAllVar();
            updateAllVariableValues
                .then(function (response) {
                if (_this._isUpdateEnabled == true && _this._isTableInEditMode === false && _this._isChartLiveEnabled === false) {
                    var variables = response._deviceVariableArray;
                    if (response._isSuccess == false || variables.length == 0) {
                        _this._log("VariableComponentViewModel onSuccessfulInitializedConf(): update all variables promise return empty/partial empty set");
                        _this.setAlertError();
                    }
                    _this.buildViewModel(deviceConfigService.VariablesDictionary);
                    _this.buildTable();
                    _this._isVariableViewInit(true);
                    // Set auto-refresh
                    _this._refreshVariableTimeoutId = setTimeout(function () {
                        this.setTimeoutRefreshView();
                    }.bind(_this), storage_service_5.StorageService.getInstance().VariableRefreshIntevalSeconds * 1000);
                }
            })
                .catch(function (reason) {
                _this._log(reason);
                _this.setAlertError();
            });
        };
        // override
        VariableComponentViewModel.prototype.onChangeScreenSize = function (isMobileScreen) {
            if (isMobileScreen != this._enableCardView) {
                // Change layout of table
                $(TableJqueryId).bootstrapTable("toggleView");
            }
        };
        // override
        VariableComponentViewModel.prototype.onChangeLanguage = function (newState) {
            _super.prototype.onChangeLanguage.call(this, newState);
            if (newState === translator_service_32.TranslatorEvent.LanguageChanged) {
                this.translate();
                // Update tables
                this.refreshTableTranslation();
            }
        };
        // Refresh
        VariableComponentViewModel.prototype.setTimeoutRefreshView = function () {
            var _this = this;
            if (this._isTableInEditMode === false && this._isChartLiveEnabled === false) {
                this.refreshVariables(false, true)
                    .then(function () {
                    // Rescheduling
                    if (_this._isRefreshVariableEnable == true) {
                        _this._refreshVariableTimeoutId = setTimeout(function () {
                            this.setTimeoutRefreshView();
                        }.bind(_this), storage_service_5.StorageService.getInstance().VariableRefreshIntevalSeconds * 1000);
                    }
                })
                    .catch(function (reason) {
                    _this._log("VariableComponentViewModel setTimeoutRefreshView() failed: " + reason);
                });
            }
        };
        VariableComponentViewModel.prototype.initHeader = function () {
            var translator = translator_service_32.TranslatorService.getInstance().t;
            this._actionHeader(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "actions"));
            this._plotHeader(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "plot"));
        };
        VariableComponentViewModel.prototype.buildViewModel = function (mainVariableDictionary) {
            for (var variableProperty in mainVariableDictionary) {
                var variable = mainVariableDictionary[variableProperty];
                if (variable._scope && (variable._scope._webRead === true || variable._scope._webWrite === true)) {
                    this._variableViewModelArray.push(new VariableItemViewModel(variable._currentValue, variable, new VariableStyleValidator(this._updateItemSetHelper, this), this.getCatagories(variable), this));
                }
            }
        };
        VariableComponentViewModel.prototype.buildTable = function () {
            $(function () {
                $(TableJqueryId).bootstrapTable({
                    data: this._variableViewModelArray,
                    icons: base_table_component_viewmodel_5.BaseTableComponentViewModel.toolbarIconPak,
                    onToggle: function (isCardView) {
                        this._enableCardView = isCardView;
                    }.bind(this),
                    onPostBody: function () {
                        if (this._isToolbarInit === false) {
                            // Inject component without container. In this way I can use default css for toolbar
                            var toolbar_3 = $("variable-component div.columns.columns-right.btn-group.pull-right");
                            var $newToolbarItem = $("<!-- ko component: 'variable-toolbar-component' --><!-- /ko -->");
                            toolbar_3.prepend($newToolbarItem);
                            ko.applyBindings({}, $newToolbarItem[0]);
                            this._isToolbarInit = true;
                        }
                    }.bind(this),
                    onRefresh: function () {
                        this.refreshVariables(true, false);
                    }.bind(this),
                    // Locale
                    locale: translator_service_32.TranslatorService.getInstance().CurrentLanguage()
                });
            }.bind(this));
            // TODO: solution more efficient
            // Filter table
            // let categoryIdQuery: string[] = [];
            // for (let activeCategoryViewModel of this.ListOfActiveCategories) {
            //  	categoryIdQuery.push(activeCategoryViewModel.Id);
            // }
            // (<any>$(TableJqueryId)).bootstrapTable("arrayFilterBy", { 
            // 	// "Categories" is the property of EditableVariableItemViewModel
            // 	CategoryId: categoryIdQuery });
            // Action formatter
            window["variableViewOperationFormatter"] = function (value, variable, index) {
                if (variable.IsVariableToUpdate == true) {
                    var html = string_util_3.StringUtil.sprintf(SaveIconTemplate, this._saveTooltip());
                    return html;
                }
                else
                    return "";
            }.bind(this);
            // Action event
            window["variableViewOperationFormatterEvents"] = {
                "click .action": function (e, value, variableViewModel, index) {
                    this.setVar(variableViewModel);
                }.bind(this)
            };
            // Plot event
            window["variableViewPlotFormatterEvents"] = {
                "click input:checkbox": function (e, value, variableViewModel, index) {
                    view_6.Formatter.GetInputHtml(e, this);
                    var inputBox = view_6.Formatter.GetInputHtml(e, this);
                    // Get value
                    var newValue = inputBox.checked;
                    // Update vm
                    variableViewModel.IsPlotSelected = newValue;
                }
            };
        };
        VariableComponentViewModel.prototype.refreshVariables = function (resetEditing, isSilentUpdate) {
            var _this = this;
            var updateAllVariableValuesPromise = variable_service_8.VariableService.getInstance().getAllVar();
            updateAllVariableValuesPromise.then(function (response) {
                if (_this._isUpdateEnabled == true &&
                    ((_this._isTableInEditMode === false && _this._isChartLiveEnabled === false) || isSilentUpdate === false)) {
                    var variableArray = response._deviceVariableArray;
                    if (response._isSuccess == false || variableArray.length == 0) {
                        _this._log("VariableComponentViewModel refreshVariables(): update all variables promise return empty/partial empty set");
                        _this.setAlertError();
                    }
                    var mainVariableDictionary = device_config_service_12.DeviceConfigService.getInstance().VariablesDictionary;
                    // Refresh view model
                    for (var _i = 0, _a = _this._variableViewModelArray; _i < _a.length; _i++) {
                        var variableViewModel = _a[_i];
                        // Variable dictionary is already update
                        var variable = variableViewModel.Variable;
                        variableViewModel.refresh(resetEditing, variable._currentValue);
                    }
                    if (resetEditing) {
                        _this._updateItemSetHelper.clear();
                    }
                    // Success
                    if (isSilentUpdate == false) {
                        _this.setSuccessAlert();
                    }
                    // Refresh table
                    $(TableJqueryId).bootstrapTable("load", _this._variableViewModelArray);
                }
            })
                .catch(function (reason) {
                _this._log("VariableComponentViewModel refreshVariables() failed: " + reason);
                _this.setAlertError();
            });
            return updateAllVariableValuesPromise;
        };
        VariableComponentViewModel.prototype.setVarOnClickCommand = function () {
            var pendingUpdates = this._updateItemSetHelper.PendingUpdates;
            if (validator_27.Validator.isDefined(pendingUpdates) && pendingUpdates.length > 0) {
                this.setVar(pendingUpdates);
            }
        };
        /**
         * Algorithm:
         * 1- Send 'setVar'
         * 2- Seng 'getVar'
         * 3- compare values
         * @param inputViewModel
         */
        VariableComponentViewModel.prototype.setVar = function (inputViewModel) {
            var _this = this;
            var variableArray = [];
            var variableIdArray = [];
            var newValueArray = [];
            // Get array
            var variableItemViewModels = data_structure_10.ArrayUtil.GetArray(inputViewModel);
            if (validator_27.Validator.isDefined(variableItemViewModels)) {
                for (var _i = 0, variableItemViewModels_1 = variableItemViewModels; _i < variableItemViewModels_1.length; _i++) {
                    var variableItemViewModel = variableItemViewModels_1[_i];
                    // Validate
                    if (validator_27.Validator.isDefined(variableItemViewModel.VariableEditedValue)) {
                        variableArray.push(variableItemViewModel.Variable);
                        variableIdArray.push(variableItemViewModel.Variable._id);
                        if (typeof (variableItemViewModel.VariableEditedValue) === "boolean") {
                            // variable is a boolean
                            newValueArray.push(converter_6.Converter.BoolToIntText(variableItemViewModel.VariableEditedValue));
                        }
                        else {
                            newValueArray.push(variableItemViewModel.VariableEditedValue);
                        }
                    }
                }
            }
            else {
                this._log("VariableComponentViewModel setVar(): input data is null");
                return;
            }
            if (variableArray.length == 0) {
                this._log("VariableComponentViewModel setVar(): no data to set");
                return;
            }
            // Map Id-> value to set. Use in case of error
            var idToValueToSetDictionary = {};
            for (var i = 0; i < variableIdArray.length; i++) {
                idToValueToSetDictionary[variableIdArray[i]] = newValueArray[i];
            }
            var skipThenChain = false;
            var setVarResponse;
            // 1-Set var
            var setVarPromise = variable_service_8.VariableService.getInstance().setVar(variableArray, newValueArray);
            setVarPromise
                .then(function (setVarResponseObject) {
                if (_this._isUpdateEnabled == true) {
                    var setVariableArrayResponse = setVarResponseObject._deviceVariableArray;
                    setVarResponse = setVariableArrayResponse;
                    if (setVarResponseObject._isSuccess == false || variableArray.length == 0) {
                        _this._log("VariableComponentViewModel refreshVariables(): update all variables promise return empty/partial empty set");
                        _this.setAlertError();
                    }
                    if (setVarResponse.length == 0) {
                        _this._log("variableComponentViewModel setVar(). The response of setVar is empty");
                        _this.setAlertError();
                        skipThenChain = true;
                    }
                    else {
                        // Retrieve the value update by application
                        return new Promise(function (resolve, reject) {
                            setTimeout(function () {
                                // 2-Get var
                                var getVarPromise = variable_service_8.VariableService.getInstance().getVar(variableIdArray);
                                getVarPromise
                                    .then(function (getSetVarResponse) {
                                    resolve(getSetVarResponse);
                                })
                                    .catch(function (reason) {
                                    reject("GetVar failed: " + reason);
                                });
                            }, 500);
                        });
                    }
                }
            })
                .then(function (getVarResponseObject) {
                if (_this._isUpdateEnabled == true) {
                    var updatedVarResponse = getVarResponseObject._deviceVariableArray;
                    if (skipThenChain == false) {
                        // Build maps
                        var setVarResponseDictionary = {};
                        if (setVarResponse) {
                            for (var _i = 0, setVarResponse_1 = setVarResponse; _i < setVarResponse_1.length; _i++) {
                                var deviceVariable = setVarResponse_1[_i];
                                setVarResponseDictionary[deviceVariable.getId()] = deviceVariable;
                            }
                        }
                        // Manage status
                        var errorDeviceVariableArray = [];
                        var notUpdateDeviceVariableArray = [];
                        var updatedVariableSet = new data_structure_9.StringSet();
                        var variableDictionary = device_config_service_12.DeviceConfigService.getInstance().VariablesDictionary;
                        for (var _a = 0, updatedVarResponse_1 = updatedVarResponse; _a < updatedVarResponse_1.length; _a++) {
                            var variable = updatedVarResponse_1[_a];
                            var currentId = variable.getId();
                            if (!validator_27.Validator.isDefined(currentId)) {
                                _this._log("VariableComponentViewModel setVar(): variable not found (no id)");
                                continue;
                            }
                            // Cache in the set
                            updatedVariableSet.addNoConflict(variable.getId());
                            var setVarVariableResponse = setVarResponseDictionary[currentId];
                            if (validator_27.Validator.isDefined(setVarVariableResponse)) {
                                if (setVarVariableResponse.getStatus() === "OK") {
                                    var setValueString = setVarVariableResponse.getValue();
                                    var getValueString = variable.getValue();
                                    var areEqual = void 0;
                                    var currentVariableDefinition = variableDictionary[currentId];
                                    if (validator_27.Validator.isDefined(currentVariableDefinition)) {
                                        if (datatype_model_12.DataTypeUtil.IsNumericDataType(currentVariableDefinition._dataType) && setValueString !== "" && getValueString !== "") {
                                            // Compare setVar response with getVar response
                                            var getValue = Number(getValueString);
                                            var setValue = Number(setValueString);
                                            areEqual = validator_27.Validator.CompareValues(setValue, getValue, variable_item_viewmodel_4.EditableVariableItemViewModel.numberMaxVariableAccuracy);
                                        }
                                        else {
                                            areEqual = getValueString == setValueString;
                                        }
                                        if (!areEqual) {
                                            // Insert into error list
                                            var originalVariableValue = _this.getOriginalVariableValue(setVarVariableResponse, idToValueToSetDictionary);
                                            if (originalVariableValue != null) {
                                                notUpdateDeviceVariableArray.push(new DeviceVariableContainer(setVarVariableResponse, variable, originalVariableValue));
                                            }
                                        }
                                    }
                                    else {
                                        _this._log("VariableComponentViewModel setVar(): variable definition not found. Id=" + currentId);
                                    }
                                }
                                else {
                                    var originalVariableValue = _this.getOriginalVariableValue(setVarVariableResponse, idToValueToSetDictionary); // check
                                    if (typeof (originalVariableValue) !== "undefined" && originalVariableValue != null) {
                                        errorDeviceVariableArray.push(new DeviceVariableContainer(setVarVariableResponse, variable, originalVariableValue));
                                    }
                                    else {
                                        _this._log("VariableComponentViewModel setVar(): the set variable with id =" + setVarVariableResponse.getId() + " isn't been found in the set variables or is " + originalVariableValue);
                                    }
                                }
                            }
                            else {
                                _this._log("VariableComponentViewModel setVar(): the variable with id =" + variable.getId() + " respond null in the getVar");
                            }
                        }
                        // Manage Ui alert
                        _this.setAlertMsgInfo(notUpdateDeviceVariableArray, errorDeviceVariableArray);
                        // Refresh viewmodel
                        var deviceConfigService = device_config_service_12.DeviceConfigService.getInstance();
                        var mainVariableDictionary = deviceConfigService.VariablesDictionary;
                        for (var _b = 0, _c = _this._variableViewModelArray; _b < _c.length; _b++) {
                            var variableViewModel = _c[_b];
                            if (updatedVariableSet.contains(variableViewModel.Variable._id)) {
                                var variable = variableViewModel.Variable;
                                variableViewModel.refresh(true, variable._currentValue);
                            }
                        }
                        $(TableJqueryId).bootstrapTable("load", _this._variableViewModelArray);
                        _this._updateItemSetHelper.clear();
                    }
                }
            })
                .catch(function (reason) {
                // Show error
                var msg = _this._updateInfoMsg();
                _this._updateInfoMsg(msg + reason);
                _this._alertState(AlertStateErrorClass);
                _this._isUpdateErrorInfoDetailVisible(false);
                _this._isUpdateWarningInfoDetailVisible(false);
                _this._isUpdateInfoVisible(true);
                _this._updateItemSetHelper.clear();
            });
        };
        VariableComponentViewModel.prototype.getOriginalVariableValue = function (setVarResponse, IdToValueToSetDictionary) {
            var originalVariableValue = IdToValueToSetDictionary[setVarResponse.getId()];
            if (validator_27.Validator.isDefined(originalVariableValue)) {
                return originalVariableValue;
            }
            else {
                this._log("VariableComponentViewModel setVar(): the set variable with id =" + setVarResponse.getId() + " isn't been found in the set variables or is " + originalVariableValue);
                return null;
            }
        };
        // Alert section
        /**
         * Set the result message after a setVar()
         * @param notUpdateDeviceVariableArray
         * @param errorDeviceVariableArray
         */
        VariableComponentViewModel.prototype.setAlertMsgInfo = function (notUpdateDeviceVariableArray, errorDeviceVariableArray) {
            // Reset state
            this._failedSetVarItemArray.removeAll();
            this._notUpdatedSetVarItemArray.removeAll();
            if (errorDeviceVariableArray.length > 0) {
                // Error
                this._alertState(AlertStateErrorClass);
                // Ad hoc manage of UI
                this._updateInfoMsg(translator_service_32.TranslatorService.getInstance().t(translator_service_32.TranslatorService.CfieldTranslationNs + "error"));
                for (var _i = 0, errorDeviceVariableArray_1 = errorDeviceVariableArray; _i < errorDeviceVariableArray_1.length; _i++) {
                    var variable = errorDeviceVariableArray_1[_i];
                    this._failedSetVarItemArray.push(new FailedSetVarItemViewModel(variable._setVarResponse.getName(), variable._valueToSet, variable._setVarResponse.getStatus()));
                }
                this._isUpdateErrorInfoDetailVisible(true);
                if (notUpdateDeviceVariableArray.length > 0) {
                    for (var _a = 0, notUpdateDeviceVariableArray_1 = notUpdateDeviceVariableArray; _a < notUpdateDeviceVariableArray_1.length; _a++) {
                        var variable = notUpdateDeviceVariableArray_1[_a];
                        this._notUpdatedSetVarItemArray.push(new NotUpdatedSetVarItemViewModel(variable._setVarResponse.getName(), variable._valueToSet, variable._getVarResponse.getValue()));
                    }
                    this._isUpdateWarningInfoDetailVisible(true);
                }
                this._updateInfoMsgState = UpdateInfoState.Error;
            }
            else if (notUpdateDeviceVariableArray.length > 0) {
                // Warning
                this._alertState(AlertStateWarningClass);
                // Ad hoc manage of UI
                this._updateInfoMsg(translator_service_32.TranslatorService.getInstance().t(translator_service_32.TranslatorService.CfieldTranslationNs + "someVariableAreNotUpdate"));
                for (var _b = 0, notUpdateDeviceVariableArray_2 = notUpdateDeviceVariableArray; _b < notUpdateDeviceVariableArray_2.length; _b++) {
                    var variable = notUpdateDeviceVariableArray_2[_b];
                    this._notUpdatedSetVarItemArray.push(new NotUpdatedSetVarItemViewModel(variable._setVarResponse.getName(), variable._valueToSet, variable._getVarResponse.getValue()));
                }
                this._isUpdateWarningInfoDetailVisible(true);
                this._updateInfoMsgState = UpdateInfoState.SomeVariableNotUpdated;
            }
            else {
                // Success
                this.setSuccessAlert();
            }
            this._isUpdateInfoVisible(true);
        };
        VariableComponentViewModel.prototype.setSuccessAlert = function () {
            this._updateInfoMsgState = UpdateInfoState.Success;
            this._alertState(AlertStateSuccessClass);
            this._updateInfoMsg(translator_service_32.TranslatorService.getInstance().t(translator_service_32.TranslatorService.CfieldTranslationNs + "operationSuccessfullyCompleted"));
            this._isUpdateErrorInfoDetailVisible(false);
            this._isUpdateWarningInfoDetailVisible(false);
        };
        /**
         * Set the warning message if some update must be set.
         */
        VariableComponentViewModel.prototype.setAlertMsgPenging = function () {
            if (this._updateItemSetHelper.NumberOfPendingUpdate() > 0) {
                this._updateInfoMsgState = UpdateInfoState.PendingChanges;
                var text = translator_service_32.TranslatorService.getInstance().t(translator_service_32.TranslatorService.CfieldTranslationNs + "numberPendingChanges");
                this._updateInfoMsg(text + " " + this._updateItemSetHelper.NumberOfPendingUpdate());
                this._alertState(AlertStateWarningClass);
                this._isUpdateErrorInfoDetailVisible(false);
                this._isUpdateWarningInfoDetailVisible(false);
                this._isUpdateInfoVisible(true);
            }
            else {
                this._isUpdateInfoVisible(false);
            }
        };
        // Note that the setVar() has its own management of UI and it doesn't call this method
        VariableComponentViewModel.prototype.setAlertError = function () {
            this._updateInfoMsgState = UpdateInfoState.VariableNotUpdated;
            this._updateInfoMsg(translator_service_32.TranslatorService.getInstance().t(translator_service_32.TranslatorService.CfieldTranslationNs + "noUpdateData"));
            this._alertState(AlertStateErrorClass);
            this._isUpdateErrorInfoDetailVisible(false);
            this._isUpdateWarningInfoDetailVisible(false);
            this._isUpdateInfoVisible(true);
        };
        VariableComponentViewModel.prototype.openPlotDialog = function () {
            if (this._serieChartViewModel.VariableSerieItemViewModelArray.length === 0) {
                this._isSelectedAtLeastOneChartVariable(false);
            }
            else {
                this._isSelectedAtLeastOneChartVariable(true);
                // Update viewmodel
                this.updateChartViewModelHelper();
            }
            if (this._isPlotModalVisible()) {
                // Find the 
                view_6.ViewLogic.ModalMinimizeLogic(this.JquerySelectorModalMinimize, ChartModalContainerClass);
                return;
            }
            // Open dialog
            this._isPlotModalVisible(true);
            $(PlotChartModalJqueryId).modal("show");
            // Callback to execute when the modal is opened
            $(PlotChartModalJqueryId).on("shown.bs.modal", function (e) {
                var _this = this;
                if (this._isSelectedAtLeastOneChartVariable()) {
                    // Draw serie
                    this.buildSerieView(this._serieChartViewModel.VariableSerieItemViewModelArray);
                    // Get first sample
                    this._isChartLiveEnabled = true;
                    this._serieChartViewModel.pollingData()
                        .then(function () {
                        // Draw chart (first time)
                        _this.buildLiveChartView(_this._serieChartViewModel.ChartOptions);
                        // Register the callbacks to update the chart
                        _this._registeredObserverArray.push(_this._serieChartViewModel.IsDirty.subscribe(_this.updateChart, _this));
                        _this._registeredObserverArray.push(_this._serieChartViewModel.IsDataDirty.subscribe(_this.updateDataChart, _this));
                        // Start polling
                        _this.setTimeoutRefreshChart();
                    })
                        .catch(function (reason) {
                        _this._log("VariableComponentViewmodel openPlotDialog(). Details: " + reason);
                        _this._isChartLiveEnabled = false;
                    });
                    // Logic of modal
                    view_6.ViewLogic.ModalForceMaximazeClass(this.JquerySelectorModalMinimize, ChartModalContainerClass);
                }
            }.bind(this));
        };
        VariableComponentViewModel.prototype.registerChartDialogLogic = function () {
            $(".modalMinimize").on("click", function () {
                view_6.ViewLogic.ModalMinimizeLogic(this, ChartModalContainerClass);
            });
            $("button[data-dismiss='modal']").click(function () {
                view_6.ViewLogic.ModalCloseMinimizeLogic(this, ChartModalContainerClass);
            });
        };
        VariableComponentViewModel.prototype.setTimeoutRefreshChart = function () {
            var _this = this;
            if (this._isChartLiveEnabled === true) {
                this._serieChartViewModel.pollingData()
                    .then(function () {
                    if (_this._isChartLiveEnabled == true) {
                        _this._refreshChartTimeoutId = setTimeout(function () {
                            this.setTimeoutRefreshChart();
                        }.bind(_this), 1000);
                    }
                    else {
                        _this._log("live refresh disabled");
                    }
                })
                    .catch(function (reason) {
                    _this._log("VariableComponentViewmodel setTimeoutRefreshChart(). Details: " + reason);
                    _this._isChartLiveEnabled = false;
                });
            }
        };
        VariableComponentViewModel.prototype.updateChartViewModelHelper = function () {
            if (this._variableViewModelArray) {
                var idArray = [];
                var nameArray = [];
                var isVisibleArray = [];
                var colorArray = [];
                var yAxisTypeArray = [];
                var chartLineStyleArray = [];
                for (var _i = 0, _a = this._variableViewModelArray; _i < _a.length; _i++) {
                    var variableItemViewModel = _a[_i];
                    if (variableItemViewModel.IsPlotSelected) {
                        idArray.push(variableItemViewModel.Variable._id);
                        nameArray.push(variableItemViewModel.VariableName);
                        isVisibleArray.push(true);
                        colorArray.push(color_3.ColorGenerator.GetColor());
                        yAxisTypeArray.push(log_definition_item_viewmodel_4.YAxisType.Primary);
                        chartLineStyleArray.push(log_definition_item_viewmodel_4.ChartLineStyle.Line);
                    }
                }
                // Add 
                this._serieChartViewModel.addMultiple(idArray, nameArray, isVisibleArray, colorArray, yAxisTypeArray, chartLineStyleArray);
            }
        };
        VariableComponentViewModel.prototype.buildSerieView = function (logSerieItemViewModelArray) {
            $(function () {
                $(SerieTableJqueryId).bootstrapTable({
                    data: logSerieItemViewModelArray,
                    onPostBody: function () {
                        $(".selectpicker").selectpicker("show");
                        view_6.View.InitColorPicker();
                    }
                });
            }.bind(this));
        };
        VariableComponentViewModel.prototype.refreshSerieView = function (logSerieItemViewModelArray) {
            // Destroy and rebuild
            $(SerieTableJqueryId).bootstrapTable("destroy");
            $(function () {
                $(SerieTableJqueryId).bootstrapTable({
                    data: logSerieItemViewModelArray,
                    onPostBody: function () {
                        $(".selectpicker").selectpicker("show");
                        $(".colorpicker-component").colorpicker();
                    }
                });
            }.bind(this));
        };
        VariableComponentViewModel.prototype.buildLiveChartView = function (chartOptions) {
            this._chartReference = new Dygraphs(// TODO: fix import
            document.getElementById("live-chart"), chartOptions.Data, {
                labels: chartOptions.LabelArray,
                colors: chartOptions.ColorArray,
                series: chartOptions.SerieOptions,
                highlightCircleSize: 2,
                strokeWidth: 1,
                strokeBorderWidth: 1,
                highlightSeriesOpts: {
                    strokeWidth: 3,
                    strokeBorderWidth: 1,
                    highlightCircleSize: 5
                }
            });
            var onClickChart = function (event) {
                if (this._chartReference.isSeriesLocked()) {
                    this._chartReference.clearSelection();
                }
                else {
                    this._chartReference.setSelection(this._chartReference.getSelection(), this._chartReference.getHighlightSeries(), true);
                }
            }.bind(this);
            this._chartReference.updateOptions({ clickCallback: onclick }, true);
            //this._chartReference.setSelection(false, 's005');
        };
        VariableComponentViewModel.prototype.updateSerieViewModel = function (variableItemViewModel) {
            if (variableItemViewModel.IsPlotSelected) {
                this._serieChartViewModel.addDefault(variableItemViewModel.Variable._id, variableItemViewModel.VariableName);
            }
            else {
                this._serieChartViewModel.remove(variableItemViewModel.Variable._id);
            }
        };
        VariableComponentViewModel.prototype.updateChart = function () {
            if (this._serieChartViewModel.VariableSerieItemViewModelArray.length === 0) {
                this._isSelectedAtLeastOneChartVariable(false);
            }
            else {
                this._isSelectedAtLeastOneChartVariable(true);
                // Update viewmodel
                this.refreshSerieView(this._serieChartViewModel.VariableSerieItemViewModelArray);
                this.updateDataChart();
            }
        };
        VariableComponentViewModel.prototype.updateDataChart = function () {
            if (this._chartReference && this._isPlotModalVisible()) {
                var chartOptions = this._serieChartViewModel.ChartOptions;
                this._chartReference.updateOptions({
                    file: chartOptions.Data,
                    labels: chartOptions.LabelArray,
                    colors: chartOptions.ColorArray,
                    series: chartOptions.SerieOptions,
                    visibility: chartOptions.IsVisibleArray
                });
            }
        };
        VariableComponentViewModel.prototype.resetZoomChart = function () {
            if (this._chartReference) {
                this._chartReference.updateOptions({
                    dateWindow: null,
                    valueRange: null
                });
            }
        };
        VariableComponentViewModel.prototype.exportLive = function () {
            // Prepare data
            var data = [this._serieChartViewModel.ChartOptions.LabelArray.slice()];
            // overwrite date
            for (var _i = 0, _a = this._serieChartViewModel.Data; _i < _a.length; _i++) {
                var row = _a[_i];
                var record = row.slice();
                // Parse date
                record[0] = new moment(record[0]).format();
                data.push(record);
            }
            var exportData = $.csv.fromArrays(data);
            var exportDataString;
            if (exportData) {
                // Replace char separator of original cgi result with the char saved in the settings
                exportDataString = exportData.replace(/,/g, storage_service_5.StorageService.getInstance().ExportCharSeparator);
            }
            else {
                exportDataString = "";
            }
            // Save
            file_util_4.FileUtil.SaveTextFileOnDisk("live.csv", exportDataString);
        };
        VariableComponentViewModel.prototype.closePlotDialog = function () {
            // Stop polling
            if (this._refreshChartTimeoutId) {
                clearTimeout(this._refreshChartTimeoutId);
                this._isChartLiveEnabled = false;
                this.setTimeoutRefreshView();
            }
            // Reset viewModel state
            this._serieChartViewModel.resetData();
            this._isPlotModalVisible(false);
        };
        // Translation
        VariableComponentViewModel.prototype.translate = function () {
            var translator = translator_service_32.TranslatorService.getInstance().t;
            this._setValueHeader(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "setValue"));
            this._isRetainHeader(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "isRetain"));
            this._saveTooltip(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "saveIntoController"));
            this._serverResponseHeader(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "serverResponse"));
            this._valueThatItryToSetHeader(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "valueItryToSet"));
            this._valueAfterTheUpdateHeader(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "valueAfterTheUpdate"));
            this._selectAtLeastOneChartVariableMsg(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "selectAtLeastOneVariableToPlot"));
            this._liveTitle(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "live"));
            this._resetZoomTitle(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "resetZoom"));
            this._exportTitle(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "export"));
            switch (this._updateInfoMsgState) {
                case UpdateInfoState.Success:
                    this._updateInfoMsg(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "operationSuccessfullyCompleted"));
                    break;
                case UpdateInfoState.PendingChanges:
                    this._updateInfoMsg(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "numberPendingChanges") + " "
                        + this._updateItemSetHelper.NumberOfPendingUpdate());
                    break;
                case UpdateInfoState.SomeVariableNotUpdated:
                    this._updateInfoMsg(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "someVariableAreNotUpdate"));
                    break;
                case UpdateInfoState.VariableNotUpdated:
                    this._updateInfoMsg(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "variableAreNotUpdate"));
                    break;
                case UpdateInfoState.Error:
                    this._updateInfoMsg(translator(translator_service_32.TranslatorService.CfieldTranslationNs + "error"));
                    break;
            }
        };
        VariableComponentViewModel.prototype.refreshTableTranslation = function () {
            var translatorService = translator_service_32.TranslatorService.getInstance();
            var translator = translatorService.t;
            var table = $(TableJqueryId);
            table.bootstrapTable("changeLocale", translatorService.CurrentLanguage());
            table.bootstrapTable("changeTitle", {
                VariableName: translator(translator_service_32.TranslatorService.CfieldTranslationNs + "name"),
                VariableDescription: translator(translator_service_32.TranslatorService.CfieldTranslationNs + "description"),
                VariableType: translator(translator_service_32.TranslatorService.CfieldTranslationNs + "type"),
                VariableCurrentValue: translator(translator_service_32.TranslatorService.CfieldTranslationNs + "value"),
                Actions: translator(translator_service_32.TranslatorService.CfieldTranslationNs + "actions"),
                CategoryArray: translator(translator_service_32.TranslatorService.CfieldTranslationNs + "categories"),
                IsPlotSelected: translator(translator_service_32.TranslatorService.CfieldTranslationNs + "plot"),
                _callbackBeforeUpdateBodyOnRefreshTranslation: function () {
                    // Reset toolbar state
                    this._isToolbarInit = false;
                }.bind(this)
            });
        };
        return VariableComponentViewModel;
    }(base_category_component_viewmodel_3.BaseCategoryComponentViewModel));
    var DeviceVariableContainer = /** @class */ (function () {
        function DeviceVariableContainer(_setVarResponse, _getVarResponse, _valueToSet) {
            this._setVarResponse = _setVarResponse;
            this._getVarResponse = _getVarResponse;
            this._valueToSet = _valueToSet;
        }
        return DeviceVariableContainer;
    }());
    var VariableItemViewModel = /** @class */ (function (_super) {
        __extends(VariableItemViewModel, _super);
        function VariableItemViewModel(variableCurrentValue, variable, editableVariableItemPresenter, categoryViewModels, _variableComponentViewModel) {
            var _this = _super.call(this, variable._currentValue, variable, editableVariableItemPresenter, categoryViewModels) || this;
            _this._variableComponentViewModel = _variableComponentViewModel;
            _this._isSaveIconShow = false;
            _this._isPlotSelected = false;
            return _this;
        }
        Object.defineProperty(VariableItemViewModel.prototype, "IsPlotSelected", {
            get: function () { return this._isPlotSelected; },
            set: function (value) {
                if (value !== this._isPlotSelected) {
                    this._isPlotSelected = value;
                    // Notify main view
                    this._variableComponentViewModel.IsSelectedSerieChange.notifySubscribers(this);
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VariableItemViewModel.prototype, "IsRetain", {
            get: function () { return this._variable._flags._isRetain; },
            enumerable: true,
            configurable: true
        });
        // Override
        VariableItemViewModel.prototype.refresh = function (resetEditing, variableCurrentValue) {
            _super.prototype.refresh.call(this, resetEditing, variableCurrentValue);
            if (resetEditing == true) {
                this._isSaveIconShow = false;
            }
        };
        VariableItemViewModel.prototype.addSaveIcon = function (index) {
            // Naive solution
            if (this._isSaveIconShow === false) {
                $(TableJqueryId).bootstrapTable("updateCell", {
                    index: index,
                    field: "Actions",
                    value: "_",
                    reinit: false
                });
                // Color row
                var row = $("#variable-view-component-table tbody tr[data-index='" + index + "']");
                if (validator_27.Validator.isDefined(row)) {
                    row.addClass(style_3.StyleClass.RemeberToSaveRowClass);
                }
                // Insert save icon
                var cell = row.find("td.actions");
                if (validator_27.Validator.isDefined(cell)) {
                    var saveIconHtml = string_util_3.StringUtil.sprintf(SaveIconTemplate, this._variableComponentViewModel.SaveTooltip());
                    cell.html(saveIconHtml);
                    // Set listener
                    var icon = cell.find(".action");
                    if (validator_27.Validator.isDefined(icon)) {
                        icon.click(function () {
                            this._variableComponentViewModel.setVar(this);
                        }.bind(this));
                    }
                    this._isSaveIconShow = true;
                }
                // Expand column
                //(<any>$(TableJqueryId)).bootstrapTable("showColumn", "Actions");
            }
            // Refresh alert
            this._variableComponentViewModel.setAlertMsgPenging();
        };
        VariableItemViewModel.prototype.removeSaveIcon = function (index) {
            // Naive solution
            if (this._isSaveIconShow === true) {
                $(TableJqueryId).bootstrapTable("updateCell", {
                    index: index,
                    field: "Actions",
                    value: "_",
                    reinit: false
                });
                // Remove color from row
                var row = $("table tbody tr[data-index='" + index + "']");
                row.removeClass(style_3.StyleClass.RemeberToSaveRowClass);
                // Remove save icon
                var cell = row.find("td.actions");
                cell.html("");
                // Refresh alert
                this._variableComponentViewModel.setAlertMsgPenging();
                this._isSaveIconShow = false;
            }
        };
        return VariableItemViewModel;
    }(variable_item_viewmodel_4.EditableVariableItemViewModel));
    var VariableStyleValidator = /** @class */ (function (_super) {
        __extends(VariableStyleValidator, _super);
        function VariableStyleValidator(_updateItemSetHelper, _variableComponentViewModel) {
            var _this = _super.call(this, _updateItemSetHelper) || this;
            _this._variableComponentViewModel = _variableComponentViewModel;
            return _this;
        }
        // override
        VariableStyleValidator.prototype.onModifyValidValue = function (jquerySelector, event, value, variable, index) {
            _super.prototype.onModifyValidValue.call(this, jquerySelector, event, value, variable, index);
            variable.addSaveIcon(index);
        };
        // override
        VariableStyleValidator.prototype.onInvalidValue = function (jquerySelector, event, value, variable, index) {
            _super.prototype.onInvalidValue.call(this, jquerySelector, event, value, variable, index);
            variable.removeSaveIcon(index);
            // Refresh alert
            this._variableComponentViewModel.setAlertMsgPenging();
        };
        // override
        VariableStyleValidator.prototype.onNotModifyValidValue = function (jquerySelector, event, value, variable, index) {
            _super.prototype.onNotModifyValidValue.call(this, jquerySelector, event, value, variable, index);
            variable.removeSaveIcon(index);
        };
        // override
        VariableStyleValidator.prototype.onFocus = function () {
            this._variableComponentViewModel.IsTableInEditMode = true;
        };
        return VariableStyleValidator;
    }(variable_item_presenter_2.RowStyleWithMemoryEditableVariableItemPresenter));
    var VariableItemViewModelMap = /** @class */ (function (_super) {
        __extends(VariableItemViewModelMap, _super);
        function VariableItemViewModelMap() {
            var _this = _super.call(this) || this;
            _this._countVariableValidationError = 0;
            _this._isVariableValidationError = ko.observable(false);
            return _this;
        }
        Object.defineProperty(VariableItemViewModelMap.prototype, "IsVariableValidationError", {
            get: function () { return this._isVariableValidationError; },
            enumerable: true,
            configurable: true
        });
        // override
        VariableItemViewModelMap.prototype.addNoConflict = function (key, value) {
            var result = _super.prototype.addNoConflict.call(this, key, value);
            if (result) {
                if (!value.IsVariableEditedValid) {
                    this._countVariableValidationError++;
                    this._isVariableValidationError(true);
                }
            }
            return result;
        };
        VariableItemViewModelMap.prototype.remove = function (key) {
            var result = _super.prototype.remove.call(this, key);
            if (result) {
                if (!result.IsVariableEditedValid) {
                    this._countVariableValidationError--;
                    if (this._countVariableValidationError == 0) {
                        this._isVariableValidationError(false);
                    }
                    else {
                        this._isVariableValidationError(true);
                    }
                }
            }
            return result;
        };
        return VariableItemViewModelMap;
    }(data_structure_9.CustomMap));
    // Failed setVar item 
    var FailedSetVarItemViewModel = /** @class */ (function () {
        function FailedSetVarItemViewModel(_variableName, _setValue, _serverResponse) {
            this._variableName = _variableName;
            this._setValue = _setValue;
            this._serverResponse = _serverResponse;
        }
        Object.defineProperty(FailedSetVarItemViewModel.prototype, "VariableName", {
            get: function () { return this._variableName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FailedSetVarItemViewModel.prototype, "SetValue", {
            get: function () { return this._setValue; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FailedSetVarItemViewModel.prototype, "ServerResponse", {
            get: function () { return this._serverResponse; },
            enumerable: true,
            configurable: true
        });
        return FailedSetVarItemViewModel;
    }());
    var NotUpdatedSetVarItemViewModel = /** @class */ (function () {
        function NotUpdatedSetVarItemViewModel(_variableName, _setValue, _readValueAfterUpdate) {
            this._variableName = _variableName;
            this._setValue = _setValue;
            this._readValueAfterUpdate = _readValueAfterUpdate;
        }
        Object.defineProperty(NotUpdatedSetVarItemViewModel.prototype, "VariableName", {
            get: function () { return this._variableName; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NotUpdatedSetVarItemViewModel.prototype, "SetValue", {
            get: function () { return this._setValue; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NotUpdatedSetVarItemViewModel.prototype, "ReadValueAfterSet", {
            get: function () { return this._readValueAfterUpdate; },
            enumerable: true,
            configurable: true
        });
        return NotUpdatedSetVarItemViewModel;
    }());
    // Alert info section
    var UpdateInfoState;
    (function (UpdateInfoState) {
        UpdateInfoState[UpdateInfoState["Success"] = 0] = "Success";
        UpdateInfoState[UpdateInfoState["Error"] = 1] = "Error";
        UpdateInfoState[UpdateInfoState["PendingChanges"] = 2] = "PendingChanges";
        UpdateInfoState[UpdateInfoState["SomeVariableNotUpdated"] = 3] = "SomeVariableNotUpdated";
        UpdateInfoState[UpdateInfoState["VariableNotUpdated"] = 4] = "VariableNotUpdated";
    })(UpdateInfoState || (UpdateInfoState = {}));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var hideExtraToolbar = (params && params.hideExtraToolbar) || false;
                var newViewModel = new VariableComponentViewModel(hideExtraToolbar);
                // Cache
                cfield_service_23.cFieldService.getInstance().VariableComponenetViewModelCached = newViewModel;
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_23.cFieldService.getInstance().BaseComponentViewUrl + "/variable.html"
        }
    };
    return objectToExport;
});
define("component/viewModel/variable-toolbar-component.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/translator.service", "component/viewModel/base-toolbar.viewmodel"], function (require, exports, ko, cfield_service_24, translator_service_33, base_toolbar_viewmodel_3) {
    "use strict";
    var VariableToolbarViewModel = /** @class */ (function (_super) {
        __extends(VariableToolbarViewModel, _super);
        function VariableToolbarViewModel(_variableComponentViewModel) {
            var _this = _super.call(this, ko.pureComputed({
                read: function () {
                    return _this._variableComponentViewModel.IsUpdatesPending();
                }
            })) || this;
            _this._variableComponentViewModel = _variableComponentViewModel;
            _this._settingsToolbarComponentViewModel = new base_toolbar_viewmodel_3.SettingsToolbarComponentViewModel();
            _this._plotTooltip = ko.observable();
            // Translate
            _this.translate();
            _this._settingsToolbarComponentViewModel.translate();
            return _this;
        }
        Object.defineProperty(VariableToolbarViewModel.prototype, "SettingsTooltip", {
            get: function () { return this._settingsToolbarComponentViewModel.SettingsTooltip; },
            enumerable: true,
            configurable: true
        });
        VariableToolbarViewModel.prototype.openSettingsDialog = function () {
            this._settingsToolbarComponentViewModel.openSettingsDialog();
        };
        Object.defineProperty(VariableToolbarViewModel.prototype, "PlotTooltip", {
            get: function () { return this._plotTooltip; },
            enumerable: true,
            configurable: true
        });
        VariableToolbarViewModel.prototype.openPlotDialog = function () {
            this._variableComponentViewModel.openPlotDialog();
        };
        VariableToolbarViewModel.prototype.setAllVar = function () {
            this._variableComponentViewModel.setVarOnClickCommand();
        };
        // Translation
        VariableToolbarViewModel.prototype.translate = function () {
            _super.prototype.translate.call(this);
            this._plotTooltip(translator_service_33.TranslatorService.getInstance().t(translator_service_33.TranslatorService.CfieldTranslationNs + "plotTooltip"));
        };
        return VariableToolbarViewModel;
    }(base_toolbar_viewmodel_3.SaveToolbarViewModel));
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                // This component depend on variable-component. Get it
                var variableComponentViewModel = cfield_service_24.cFieldService.getInstance().VariableComponenetViewModelCached;
                if (!variableComponentViewModel) {
                    throw "The cached variableComponentViewModel is null";
                }
                return new VariableToolbarViewModel(variableComponentViewModel);
            }
        },
        template: {
            require: "text!" + cfield_service_24.cFieldService.getInstance().BaseComponentViewUrl + "/variable-toolbar.html"
        }
    };
    return objectToExport;
});
define("dataModel/alarm.model", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    // not used
    var Alarm = /** @class */ (function () {
        function Alarm(rawAlarm, mainVaribleDictionary) {
            this._edge = EdgeConverter.codeToAlarmEdge(rawAlarm._edge);
            this._name = rawAlarm._name;
            this._description = rawAlarm._description;
            this._actIdVar = rawAlarm._actId;
            this._trgIdVar = rawAlarm._trgId;
            this._threId = rawAlarm._threId;
            this._timeId = rawAlarm._timeId;
            this._variableDictionary = mainVaribleDictionary;
        }
        Object.defineProperty(Alarm.prototype, "Edge", {
            get: function () {
                return this._edge;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Alarm.prototype, "Name", {
            get: function () {
                return this._name;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Alarm.prototype, "Description", {
            get: function () {
                return this._description;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Alarm.prototype, "ActVariable", {
            get: function () {
                return this._variableDictionary[this._actIdVar];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Alarm.prototype, "TrgVariable", {
            get: function () {
                return this._variableDictionary[this._trgIdVar];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Alarm.prototype, "ThreVariable", {
            get: function () {
                return this._variableDictionary[this._threId];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Alarm.prototype, "TimeVariable", {
            get: function () {
                return this._variableDictionary[this._timeId];
            },
            enumerable: true,
            configurable: true
        });
        return Alarm;
    }());
    exports.Alarm = Alarm;
    var AlarmEdge;
    (function (AlarmEdge) {
        AlarmEdge[AlarmEdge["Positive"] = 0] = "Positive";
        AlarmEdge[AlarmEdge["Negative"] = 1] = "Negative";
    })(AlarmEdge = exports.AlarmEdge || (exports.AlarmEdge = {}));
    var EdgeConverter = /** @class */ (function () {
        function EdgeConverter() {
        }
        EdgeConverter.codeToAlarmEdge = function (code) {
            return EdgeConverter._dictionary[code.trim()];
        };
        EdgeConverter._dictionary = {
            "0": AlarmEdge.Positive,
            "1": AlarmEdge.Negative
        };
        return EdgeConverter;
    }());
});
define("viewModel/alarm-history.viewmodel", ["require", "exports", "service/cfield.service"], function (require, exports, cfield_service_25) {
    "use strict";
    var AlarmHistoryViewModel = /** @class */ (function () {
        function AlarmHistoryViewModel() {
        }
        return AlarmHistoryViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                return new AlarmHistoryViewModel();
            }
        },
        template: {
            require: "text!" + cfield_service_25.cFieldService.getInstance().BasePageViewUrl + "/alarm-history.html"
        }
    };
    return objectToExport;
});
define("viewModel/log.viewmodel", ["require", "exports", "service/cfield.service"], function (require, exports, cfield_service_26) {
    "use strict";
    var LogViewModel = /** @class */ (function () {
        function LogViewModel() {
        }
        return LogViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                return new LogViewModel();
            }
        },
        template: {
            require: "text!" + cfield_service_26.cFieldService.getInstance().BasePageViewUrl + "/log.html"
        }
    };
    return objectToExport;
});
define("viewModel/main.viewmodel", ["require", "exports", "knockout", "service/cfield.service", "service/route.service", "service/translator.service", "service/timer.service", "component/viewModel/base-component.viewmodel"], function (require, exports, ko, cfield_service_27, route_service_2, translator_service_34, timer_service_7, base_component_viewmodel_9) {
    "use strict";
    var MainViewModel = /** @class */ (function (_super) {
        __extends(MainViewModel, _super);
        function MainViewModel() {
            var _this = _super.call(this, false, true, true, false) || this;
            /** Current active menu item. */
            _this.activeMenuItem = ko.observable();
            /**
             * Change the current active menu item. Note the syntax: in this way the <code>this</code> points to the <code>MainViewModel</code> instance. Otherwise with the classic
             * syntax the <code>this</code> points to the <code>MenuItemViewModel</code> instance.
             * @param item The ckicked menu item.
             * @return <code>true</code>. In this way the click handler doesn't mark the event as resolved and the defaulc click action is taken (see at the end of note 3 of
             * http://knockoutjs.com/documentation/click-binding.html).
             */
            _this.goToItem = function (item) {
                _this.activeMenuItem(item);
                return true;
            };
            // Init 
            _this.isSidebarClosed = ko.observable(_this.IsMobileScreen());
            _this.menuItems = ko.observableArray();
            // Component to load
            _this.observableComponentName = ko.pureComputed({
                read: function () {
                    return route_service_2.RouteService.getInstance().ComponentName();
                }
            });
            // Register UI listener
            _this.initSidebar();
            // Init menu
            _this.initMenu();
            // Init icon texts
            _this._languageDescription = ko.pureComputed({
                read: function () {
                    return translator_service_34.TranslatorService.getInstance().CurrentLanguage();
                }
            });
            _this._timeDescription = ko.pureComputed({
                read: function () {
                    var timezone = timer_service_7.TimerService.getInstance().CurrentTimezoneObservable();
                    switch (timezone) {
                        case timer_service_7.Timezone.Local:
                            return "LOC";
                        case timer_service_7.Timezone.Remote:
                            return "REM";
                    }
                }
            });
            _this._languageTooltip = ko.observable();
            _this._timeTooltip = ko.observable();
            _this._menuTooltip = ko.observable();
            _this._homeTooltip = ko.observable();
            _this._logoutTooltip = ko.observable();
            _this._refreshTooltip = ko.observable();
            // Evaluete the state of cField service
            _this.onChangeCFieldState(_this._cFieldServiceInstance.StateMonitor.CfieldStateObservable());
            return _this;
        }
        Object.defineProperty(MainViewModel.prototype, "HomeTooltip", {
            get: function () { return this._homeTooltip; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MainViewModel.prototype, "MenuTooltip", {
            get: function () { return this._menuTooltip; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MainViewModel.prototype, "LanguageDescription", {
            get: function () { return this._languageDescription; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MainViewModel.prototype, "LanguageTooltip", {
            get: function () { return this._languageTooltip; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MainViewModel.prototype, "TimeDescription", {
            get: function () { return this._timeDescription; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MainViewModel.prototype, "TimeTooltip", {
            get: function () { return this._timeTooltip; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MainViewModel.prototype, "LogoutTooltip", {
            get: function () { return this._logoutTooltip; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MainViewModel.prototype, "RefreshTooltip", {
            get: function () { return this._refreshTooltip; },
            enumerable: true,
            configurable: true
        });
        // override
        MainViewModel.prototype.onSuccessfulInitializedConf = function () {
            this.setTooltipTranslation();
        };
        // override
        MainViewModel.prototype.onChangeLanguage = function (newState) {
            if (newState === translator_service_34.TranslatorEvent.LanguageChanged) {
                this.refreshTranslation();
                this.setTooltipTranslation();
            }
        };
        // override
        MainViewModel.prototype.onChangeScreenSize = function (isMobileScreen) {
            this.isSidebarClosed(isMobileScreen);
        };
        MainViewModel.prototype.initMenu = function () {
            // Init menu
            for (var _i = 0, _a = route_service_2.RouteService.getInstance().RouteEntryInfoArray; _i < _a.length; _i++) {
                var routeInfo = _a[_i];
                var menuItem = new MenuItemViewModel(routeInfo.IdRoute, routeInfo.Icon, routeInfo.IconPath, routeInfo.DisplayName, routeInfo.TranslationKey);
                this.menuItems.push(menuItem);
                if (routeInfo.ComponentName === this.observableComponentName()) {
                    this.activeMenuItem(menuItem);
                }
            }
        };
        /** On click listener to manage the menu (oper or closed) */
        MainViewModel.prototype.manageSidebar = function () {
            this.isSidebarClosed(!this.isSidebarClosed());
        };
        /**
         * Manage the Ui logic.
         */
        MainViewModel.prototype.initSidebar = function () {
            // Init view
            $('#menu ul').hide();
            $('#menu ul').children('.current').parent().show();
            // Manage click of item
            $('#menu li a').click(function () {
                var checkElement = $(this).next();
                if ((checkElement.is('ul')) && (checkElement.is(':visible'))) {
                    return false;
                }
                if ((checkElement.is('ul')) && (!checkElement.is(':visible'))) {
                    $('#menu ul:visible').slideUp('normal');
                    checkElement.slideDown('normal');
                    return false;
                }
            });
            // get a reference to elements 
            var sideWrapper = document.getElementById('sidebar-wrapper');
            var wrapper = document.getElementById('main-component');
        };
        // Translation
        MainViewModel.prototype.refreshTranslation = function () {
            ko.utils.arrayForEach(this.menuItems(), function (item) {
                item.updateTranslation(translator_service_34.TranslatorService.getInstance().t);
            });
        };
        MainViewModel.prototype.setTooltipTranslation = function () {
            var translator = translator_service_34.TranslatorService.getInstance().t;
            this._languageTooltip(translator(translator_service_34.TranslatorService.CfieldTranslationNs + "changeLanguage"));
            this._timeTooltip(translator(translator_service_34.TranslatorService.CfieldTranslationNs + "changeTimezone"));
            this._homeTooltip(translator(translator_service_34.TranslatorService.CfieldTranslationNs + "home"));
            this._menuTooltip(translator(translator_service_34.TranslatorService.CfieldTranslationNs + "menu"));
            this._logoutTooltip(translator(translator_service_34.TranslatorService.CfieldTranslationNs + "logout"));
            this._refreshTooltip(translator(translator_service_34.TranslatorService.CfieldTranslationNs + "refresh"));
        };
        return MainViewModel;
    }(base_component_viewmodel_9.BaseComponentViewModel));
    var MenuItemViewModel = /** @class */ (function () {
        function MenuItemViewModel(_id, _icon, _iconPath, displayStaticName, _translationKey) {
            this._id = _id;
            this._icon = _icon;
            this._iconPath = _iconPath;
            this.displayStaticName = displayStaticName;
            this._translationKey = _translationKey;
            if (typeof (displayStaticName) !== "undefined" || displayStaticName != null) {
                this._displayName = ko.observable(displayStaticName);
            }
            else {
                var translator = translator_service_34.TranslatorService.getInstance().t;
                if (!translator || !this._translationKey) {
                    this._displayName = ko.observable(displayStaticName);
                }
                else {
                    var translation = translator(translator_service_34.TranslatorService.CfieldTranslationNs + this._translationKey);
                    if (translation) {
                        this._displayName = ko.observable(translation);
                    }
                    else {
                        this._displayName = ko.observable("");
                    }
                }
            }
        }
        Object.defineProperty(MenuItemViewModel.prototype, "Id", {
            get: function () { return this._id; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MenuItemViewModel.prototype, "Icon", {
            get: function () { return this._icon; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MenuItemViewModel.prototype, "IconPath", {
            get: function () { return this._iconPath; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MenuItemViewModel.prototype, "IsIconFont", {
            get: function () {
                if (this._icon) {
                    return true;
                }
                else {
                    return false;
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MenuItemViewModel.prototype, "DisplayName", {
            get: function () { return this._displayName; },
            enumerable: true,
            configurable: true
        });
        MenuItemViewModel.prototype.updateTranslation = function (translator) {
            if (this._translationKey) {
                this._displayName(translator(translator_service_34.TranslatorService.CfieldTranslationNs + this._translationKey));
            }
        };
        return MenuItemViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var newViewModel = new MainViewModel();
                return newViewModel;
            }
        },
        template: {
            require: "text!" + cfield_service_27.cFieldService.getInstance().BasePageViewUrl + "/main.html"
        }
    };
    return objectToExport;
});
define("viewModel/network.viewmodel", ["require", "exports", "service/cfield.service"], function (require, exports, cfield_service_28) {
    "use strict";
    var NetworkViewModel = /** @class */ (function () {
        function NetworkViewModel() {
        }
        return NetworkViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                return new NetworkViewModel();
            }
        },
        template: {
            require: "text!" + cfield_service_28.cFieldService.getInstance().BasePageViewUrl + "/network.html"
        }
    };
    return objectToExport;
});
define("viewModel/parameter.viewmodel", ["require", "exports", "service/cfield.service"], function (require, exports, cfield_service_29) {
    "use strict";
    var ParameterViewModel = /** @class */ (function () {
        function ParameterViewModel() {
        }
        return ParameterViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                return new ParameterViewModel();
            }
        },
        template: {
            require: "text!" + cfield_service_29.cFieldService.getInstance().BasePageViewUrl + "/parameter.html"
        }
    };
    return objectToExport;
});
define("viewModel/pgd1.viewmodel", ["require", "exports", "service/cfield.service"], function (require, exports, cfield_service_30) {
    "use strict";
    var Pgd1ViewModel = /** @class */ (function () {
        function Pgd1ViewModel() {
        }
        return Pgd1ViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                return new Pgd1ViewModel();
            }
        },
        template: {
            require: "text!" + cfield_service_30.cFieldService.getInstance().BasePageViewUrl + "/pgd1.html"
        }
    };
    return objectToExport;
});
define("viewModel/system-network.viewmodel", ["require", "exports", "service/cfield.service"], function (require, exports, cfield_service_31) {
    "use strict";
    var SystemNetworkViewModel = /** @class */ (function () {
        function SystemNetworkViewModel() {
        }
        return SystemNetworkViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                return new SystemNetworkViewModel();
            }
        },
        template: {
            require: "text!" + cfield_service_31.cFieldService.getInstance().BasePageViewUrl + "/system-network.html"
        }
    };
    return objectToExport;
});
define("viewModel/unit.viewmodel", ["require", "exports", "service/cfield.service"], function (require, exports, cfield_service_32) {
    "use strict";
    var UnitViewModel = /** @class */ (function () {
        function UnitViewModel() {
        }
        return UnitViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                return new UnitViewModel();
            }
        },
        template: {
            require: "text!" + cfield_service_32.cFieldService.getInstance().BasePageViewUrl + "/unit.html"
        }
    };
    return objectToExport;
});
define("viewModel/upload.viewmodel", ["require", "exports", "service/cfield.service"], function (require, exports, cfield_service_33) {
    "use strict";
    var UploadViewModel = /** @class */ (function () {
        function UploadViewModel() {
        }
        return UploadViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                return new UploadViewModel();
            }
        },
        template: {
            require: "text!" + cfield_service_33.cFieldService.getInstance().BasePageViewUrl + "/upload.html"
        }
    };
    return objectToExport;
});
define("viewModel/variable.viewmodel", ["require", "exports", "service/cfield.service"], function (require, exports, cfield_service_34) {
    "use strict";
    var VariableViewModel = /** @class */ (function () {
        function VariableViewModel() {
        }
        return VariableViewModel;
    }());
    // Export for knockout component
    var objectToExport = {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                return new VariableViewModel();
            }
        },
        template: {
            require: "text!" + cfield_service_34.cFieldService.getInstance().BasePageViewUrl + "/variable.html"
        }
    };
    return objectToExport;
});

//# sourceMappingURL=cfield.app.js.map
